/**
 * Controls the order of ad networks
 */
bento.define('ads/mediator', [
    'bento',
    'bento/utils',
    'ads/admob',
    'ads/heyzap'
], function (
    Bento,
    Utils,
    AdMob,
    HeyZap
) {
    'use strict';
    var canShow = {
        'AdMob': false,
        'HeyZap': false
    },
    networks = {
        'AdMob': AdMob,
        'HeyZap': HeyZap
    };

    return {
        canShow: function () {
            var network;
            for (network in networks) {
                if (!networks.hasOwnProperty(network)) {
                    continue;
                }
                if (networks[network].canShow()) {
                    return true;
                }
            }
            console.log('No ad available at all');
            return false;
        },
        show: function (callback, endCallback, preference) {
            if (preference && networks[preference] && networks[preference].canShow()) {
                console.log('Show preffered network ' + preference);
                networks[preference].show(callback, endCallback);
            } else if (HeyZap.canShow()) {
                // try heyzap first
                console.log('Show HeyZap');
                HeyZap.show(callback, endCallback);
            } else if (AdMob.canShow()) {
                // try Admob next
                console.log('Show AdMob');
                AdMob.show(callback, endCallback);
            }
        }
    };
});