/**
 * Handles the backbutton press on android
 */
bento.define('components/backbutton', [
    'bento',
    'bento/eventsystem',
    'bento/utils',
    'gui/dialog'
], function (
    Bento,
    EventSystem,
    Utils,
    Dialog
) {
    'use strict';
    var didCatch = false,
        onBackButton = function () {
            var oldPauseLevel;
            var currentPauseLevel = Bento.objects.isPaused();
            // pass to Bento EventSystem
            EventSystem.fire('backButton');

            // if no listener caught the event, perform a standard task
            if (!didCatch && window.Cocoon && window.Cocoon.Dialog && currentPauseLevel < Utils.PauseLevels.EXIT) {
                // ask to exit game

                oldPauseLevel = currentPauseLevel;
                Bento.objects.pause(Utils.PauseLevels.EXIT);
                new Dialog({
                    name: 'exitAppStandard',
                    text: Utils.Localization.getText('exit'),
                    pauseLevel: Utils.PauseLevels.EXIT,
                    type: 'yesno',
                    onYes: function () {
                        navigator.app.exitApp();
                    },
                    onNo: function () {},
                    onClose: function () {
                        Bento.objects.pause(oldPauseLevel);
                    }
                });

            }
            didCatch = false;
        };
    // catch backbutton event: prevents default Cordova behavior
    document.addEventListener("backbutton", onBackButton, false);

    return function (onPressed) {
        var viewport = Bento.getViewport(),
            parent = null,
            listener = function () {
                // a listener caught the backbutton event
                didCatch = true;

                // is the parent paused?
                if (parent && Bento.objects.isPaused() > parent.updateWhenPaused) {
                    return;
                }
                // invoke callback
                if (onPressed) {
                    onPressed({
                        entity: parent
                    });
                }
            },
            component = {
                z: -10000,
                name: 'backButton',
                attached: function (data) {
                    parent = data.entity;
                },
                start: function () {
                    // start listening
                    EventSystem.on('backButton', listener);
                },
                destroy: function () {
                    // remove listener
                    EventSystem.off('backButton', listener);
                }
            };

        return component;
    };
});