/**
 * Component that checks if a card was 3D touched
 */
bento.define('components/card3dtouch', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'components/draggable',
    'gui/critterdetails'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween,
    Draggable,
    CritterDetails
) {
    'use strict';
    /**
     * Describe your settings object parameters
     * @param {Object} settings
     * @param {Object} settings.on3DTouch, // callback when 3d touched (after opening details) 
     */
    return function (settings) {
        var viewport = Bento.getViewport();
        var entity;
        var isListening = false;
        var visualTest = function () {
            // for testing purposes: make the card spin
            new Tween({
                from: 0,
                to: Math.PI * 2,
                'in': 60,
                ease: 'linear',
                alpha: 0,
                beta: 0,
                updateWhenPaused: Utils.PauseLevels.ALWAYS,
                'do': function (v, t) {
                    entity.rotation = v;
                },
                onComplete: function () {

                }
            });
        };
        /** 
         * Catch all the pointerUp events to be fired again later
         */
        var pointerUpEvents = [];
        var catchPointerUp = function (evt) {
            // clone event
            var pointerUpEvent = {
                id: evt.id,
                eventType: evt.eventType,
                position: evt.position.clone(),
                localPosition: evt.localPosition.clone(),
                worldPosition: evt.worldPosition.clone()
            };
            pointerUpEvents.push(pointerUpEvent);

        };
        /**
         * 3D touch event
         */
        var critterDetails;
        var onTouch = function (result) {
            var localPos;
            var pauseLevel = Bento.objects.isPaused();
            // pauselevel too high
            if (Bento.objects.isPaused() >= Utils.PauseLevels.DETAILS) {
                return;
            }

            // let's not do this while dragging, unless the card didnt move much
            if (Draggable.isDragging() && Draggable.getDistance().magnitude() > 3) {
                return;
            }

            // card has not been discovered yet
            if (entity.isHidden()) {
                // Bento.audio.playSound('sfx_error');
                return;
            }

            // 3D touch hit the card
            localPos = entity.getLocalPosition(result.worldPosition);
            if (entity.getBoundingBox().hasPosition(localPos)) {
                // show an overlay with the corresponding critter
                // visualTest();

                // catch the all pointerup events, 
                // the rest of the game will miss pointerUp event because it will go into a new pause level
                EventSystem.on('pointerUp', catchPointerUp);

                critterDetails = new CritterDetails({
                    critter: entity.getProperties(),
                    onClose: function () {
                        // fire delayed pointerup events
                        Utils.forEach(pointerUpEvents, function (pointerUpEvent, i, l, breakLoop) {
                            if (pointerUpEvent) {
                                EventSystem.fire('pointerUp', pointerUpEvent);
                            }
                        });
                        // clean up
                        pointerUpEvents = [];
                        EventSystem.off('pointerUp', catchPointerUp);
                    }
                });
                
                if (component.on3DTouch) {
                    component.on3DTouch(critterDetails, pauseLevel);
                }
            }
        };
        var component = {
            name: '3DTouch',
            start: function (data) {
                if (!isListening) {
                    EventSystem.on('3Dtouch', onTouch);
                    isListening = true;
                }
            },
            destroy: function (data) {
                if (isListening) {
                    EventSystem.off('3Dtouch', onTouch);
                    isListening = false;
                }
            },
            attached: function (data) {
                entity = data.entity;
            },
            getCritterDetails: function () {
                return critterDetails;
            },
            on3DTouch: function () {
                // overwrite me
            }
        };
        return component;
    };
});