/**
 * Collision with trigger
 */
bento.define('components/collidewithtrigger', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/components/sprite',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'bento/packedimage',
    'modules/photo'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    Sprite,
    EventSystem,
    Utils,
    Tween,
    PackedImage,
    Photo
) {
    'use strict';
    /**
     * Describe your settings object parameters
     * @param {Object} settings
     */
    return function (settings) {
        var viewport = Bento.getViewport();
        var entity;
        var isBossBattle = false;
        var enemies = [];
        var collisionWithTrigger = {
            name: 'collideWithTrigger',
            attached: function (data) {
                entity = data.entity;
            },
            update: function () {
                // collide with trigger
                entity.collidesWith({
                    family: 'triggers',
                    onCollide: onCollide
                });
            }
        };
        var onCollide = function (other) {
            var exclamation;
            var hash;


            if (other.visible === false) {
                return;
            }
            if (other.name === 'boss') {
                isBossBattle = true;
            }
            if (other.name === 'inactiveBoss') {
                return;
            }
            if (entity.isActivated && !entity.isActivated()) {
                return;
            }
            entity.stopSound();

            if (other.name === 'chest') {
                Utils.Analytics.event('Game', 'battle_with_mimic');
                Utils.Firebase.logEvent('battle_with_mimic');
            }

            enemies = other.getEnemies();

            // show !!!
            exclamation = new Entity({
                z: Utils.Layers.SKY,
                name: 'exclamation',
                position: entity.position.add(new Vector2(-16, -32)),
                originRelative: new Vector2(0.5, 0.5),
                components: [
                    new Sprite({
                        imageName: 'worldmap/attackbubble'
                    })
                ]
            });
            Bento.objects.attach(exclamation);
            Bento.audio.stopAllMusic();

            Bento.audio.playSound('sfx_alert_battle');

            // remember trigger that was touched
            hash = other.getHash();
            Utils.worldMap.collected[hash] = true;
            // Bento.objects.remove(other);
            // remember player position
            Utils.worldMap.position = entity.position.clone();

            Bento.objects.pause();
            // make a little jump
            new Tween({
                from: 48,
                to: 64,
                'in': 10,
                ease: 'squareroot',
                updateWhenPaused: 1,
                'do': function (v, t) {
                    exclamation.position.y = entity.position.y - v;
                },
                onComplete: function () {
                    new Tween({
                        from: 64,
                        to: 48,
                        'in': 40,
                        ease: 'easeOutBounce',
                        updateWhenPaused: 1,
                        'do': function (v, t) {
                            exclamation.position.y = entity.position.y - v;
                        },
                        onComplete: function () {
                            Bento.objects.resume();
                            transition();
                            Bento.audio.playSound('sfx_transition_battle');
                        }
                    });
                }
            });

        };
        /**
         * Transition screen into battle
         */
        var transition = function () {
            var viewport = Bento.getViewport();
            var sprite;
            var entity;
            var scale;
            var fadeSprite = new Sprite({
                imageName: 'fade'
            });
            var fade = new Entity({
                z: 1,
                name: 'fade',
                position: new Vector2(viewport.width / 2, viewport.height / 2),
                originRelative: new Vector2(0.5, 0.5),
                float: true,
                components: [
                    fadeSprite
                ]
            });
            var photo = new Photo({
                scale: 1
            });
            var oldRatio = 0;
            var smear = 0.5;
            Bento.objects.get('mode7', function (mode7) {
                var renderer = mode7.getRenderer();
                renderer.setSize(photo.getCanvas().width, photo.getCanvas().height, false);
            });
            photo.snap();
            Bento.objects.get('mode7', function (mode7) {
                var renderer = mode7.getRenderer();
                renderer.setSize(Bento.getCanvas().width, Bento.getCanvas().height, false);
            });
            Bento.objects.removeAll();

            sprite = new Sprite({
                image: new PackedImage(photo.getCanvas())
            });
            Utils.lastScreenshot = photo.getCanvas();
            entity = new Entity({
                z: 0,
                name: '',
                position: new Vector2(viewport.width / 2, viewport.height / 2),
                originRelative: new Vector2(0.5, 0.5),
                float: true,
                components: [
                    sprite
                ]
            });

            fade.alpha = (0);
            scale = viewport.width / entity.dimension.width;
            entity.scale.x = scale;
            entity.scale.y = scale;

            Bento.objects.attach(entity);
            Bento.objects.attach(fade);

            if (isBossBattle) {
                Bento.audio.playMusic('bgm_boss');
                Bento.audio.setVolume(1, 'bgm_boss');
            } else {
                Bento.audio.playMusic('bgm_battle');
                Bento.audio.setVolume(1, 'bgm_battle');
            }

            // smearing fade doesnt work on android
            // if (Utils.isNativeAndroid()) {
                // fade.alpha = 1;
            // }
            new Tween({
                from: 0,
                to: 1,
                'in': 60,
                ease: 'quadratic',
                'do': function (v, t) {
                    entity.alpha = (smear - v * smear);
                    entity.rotation = v * Math.PI * 3;
                    // entity.rotation += Math.PI / 60;
                    entity.scale.x = scale * (1 + v * 10);
                    entity.scale.y = scale * (1 + v * 10);
                },
                onComplete: function () {
                    new Tween({
                        from: 0,
                        to: 1,
                        'in': 60,
                        ease: 'linear',
                        'do': function (v, t) {
                            fade.alpha = (v);
                        },
                        onComplete: function () {
                            // Bento.screens.show('screens/worldmap', settings.planetData);
                            Bento.screens.show('screens/battle', {
                                planetData: Utils.worldMap.planetData,
                                enemies: enemies,
                                isBossBattle: isBossBattle
                            });
                        }
                    });
                }
            });
        };
        return collisionWithTrigger;
    };
});