/**
 * Component that draws the boundingbox of an entity.
 *
 * ---REMINDER--- Be sure to attach the debug component before you attach the sprite.
 */
bento.define('components/debug', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    Utils,
    Tween
) {
    'use strict';
    var globalHold = null;
    return function (settings) {
        var viewport = Bento.getViewport(),
            drawBoundingBox = settings.drawBoundingBox || true,
            move = settings.move || false,
            fromMid = settings.fromMid || false,
            isHolding = false,
            pointerPos = new Vector2(0, 0),
            holdPos = new Vector2(0, 0),
            component = {
                name: 'debug',
                postDraw: function (data) {
                    var box = data.entity.getBoundingBox();
                    data.renderer.fillRect([0, 0, 1, 0.5], box.x - viewport.x, box.y - viewport.y, box.width, box.height);
                },
                update: function (data) {
                    var entity = data.entity;
                    if (globalHold === entity) {
                        entity.position = pointerPos.add(holdPos);
                    }
                },
                attached: function (data) {
                    var entity = data.entity;
                    if (move) {
                        entity.attach(new Clickable({
                            onClick: function (data) {
                                if (!globalHold || globalHold.z <= entity.z) {
                                    globalHold = entity;
                                } else {
                                    return;
                                }
                                holdPos = entity.position.substract(data.worldPosition);
                                pointerPos = data.worldPosition;
                            },
                            pointerMove: function (data) {
                                if (globalHold === entity) {
                                    pointerPos = data.worldPosition;
                                }
                            },
                            pointerUp: function () {
                                if (globalHold === entity) {
                                    globalHold = null;
                                    if (!fromMid)
                                        console.log(entity.name, 'Vector2(' + Math.floor(entity.position.x) + ', ' + Math.floor(entity.position.y) + ')');
                                    else
                                        console.log(entity.name, 'screenMid.add(Vector2(' + Math.floor(entity.position.x - viewport.width / 2) +
                                            ', ' + Math.floor(entity.position.y - viewport.height / 2) + '))');
                                }
                            },
                            onHoldEnd: function () {}
                        }));
                    }

                    // move self to top
                    entity.moveComponentTo(component, 0);
                }
            };
        return component;
    };
});