/**
 * Depth component simulates depth
 */
bento.define('components/depth', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween
) {
    'use strict';
    /**
     * Describe your settings object parameters
     * @param {Object} settings
     */
    return function (settings) {
        var viewport = Bento.getViewport();
        var viewportBox;
        var entity;
        var mode7;
        var test = true;
        var originalPosition;
        var updatePos = function () {
            var position;
            var diff;
            var worldPos;
            // option 2: move them based on the position
            if (!mode7) {
                mode7 = Bento.objects.get('mode7');
                return;
            }
            position = component.position.subtract(new Vector2(128, 128));
            diff = position.add(new Vector2(32, 0));

            entity.position = mode7.worldToScreen(position);
            // workaround: smoothing of position to counter shaking bug
            // worldPos = mode7.worldToScreen(position);
            // entity.position.x += (worldPos.x - entity.position.x) / 2;
            // entity.position.y += (worldPos.y - entity.position.y) / 2;

            // get scale
            diff = mode7.worldToScreen(diff);

            entity.scale.x = (diff.x - entity.position.x) / 32 * component.scale.x;
            // entity.scale.x = (entity.position.y - viewport.y + 64) / viewport.height * 1.5;
            entity.scale.y = (diff.x - entity.position.x) / 32 * component.scale.y;

        };
        var component = {
            name: 'depth',
            scale: new Vector2(1, 1),
            position: new Vector2(0, 0), // "real" world position, edit this to move the entity around
            start: function (data) {
                originalPosition = entity.position.clone();
                mode7 = Bento.objects.get('mode7');
                this.position = settings.position || entity.position.clone();
                // this.position.x -= 128;
                // this.position.y -= 128;

                // option 1: add them as mesh
                // mode7.makeSprite(position, Bento.assets.getImage('worldmap/ball').image);
                // entity.visible = false;
                updatePos();
            },
            getOriginalPosition: function () {
                return originalPosition;
            },
            destroy: function (data) {},
            update: function () {
                var box = entity.getBoundingBox();
                if (!viewportBox) {
                    viewportBox = viewport.clone().grow(128);
                }
                if (box.intersect(viewport)) {
                    entity.visible = true;
                } else {
                    entity.visible = false;
                }
                if (viewportBox.hasPosition(entity.position) || entity.timer % 10 === 0 || entity.dimension.height > 48) {
                    updatePos();
                }
            },
            draw: function (data) {
                // updatePos();
                viewportBox = null;
            },
            attached: function (data) {
                entity = data.entity;

                // move component to top
                entity.moveComponentTo(component, 0);
            }
        };
        return component;
    };
});