/**
 * Can drop draggables on this
 */
bento.define('components/droptarget', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'bento/components/clickable'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween,
    Clickable
) {
    'use strict';
    /**
     * Describe your settings object parameters
     * @param {Object} settings
     * @param {Function} settings.onDrop
     * @param {Function} settings.onRemoved
     */
    return function (settings) {
        var viewport = Bento.getViewport();
        var entity;
        var clickable = new Clickable({
            pointerDown: function (data) {
            }
        });
        var component = {
            name: 'dropTarget',
            attached: function (data) {
                entity = data.entity;
                entity.family.push('dropTargets');
                entity.attach(clickable);
            },
            start: function () {
                EventSystem.on('dragStart', dragStart);
                EventSystem.on('dragEnd', dragEnd);
            },
            destroy: function () {
                EventSystem.off('dragStart', dragStart);
                EventSystem.off('dragEnd', dragEnd);
            },
            update: function () {
            },
            canDrop: true
        };
        var dragStart = function (data) {
            // ready to receive?
        };
        var dragEnd = function (data) {
            // dropped here?
            if (clickable.isHovering && settings.onDrop) {
                settings.onDrop(data);
                Bento.audio.playSound('sfx_place_card');
            }
        };
        return component;
    };
});