/**
 * Enemy behavior
 */
bento.define('components/enemy', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'bento/gui/counter',
    'entities/battle/critter',
    'entities/card',
    'components/debug',
    'components/walk',
    'gui/damagecounter',
    'gui/hpbar',
    'entities/particles/hit'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    Utils,
    Tween,
    Counter,
    Critter,
    Card,
    Debug,
    Walk,
    DamageCounter,
    HpBar,
    HitParticle
) {
    'use strict';
    /**
     * Describe your settings object parameters
     * @param {Object} properties - critter properties
     * @param {Object} battleManager - reference to battle manager
     */
    return function (properties, battleManager) {
        var viewport = Bento.getViewport();
        var frontCritter = null;
        var enemy;
        var walkComponent;
        var exclamation = null;
        var isReady = false;
        var heartIcon = new Entity({
            z: Utils.Layers.GUI,
            name: 'heartIcon',
            position: new Vector2(-7, -2),
            originRelative: new Vector2(0, 0),
            components: [new Sprite({
                imageName: 'gui/iconheart'
            })]
        });
        var hpBar = new HpBar(properties, true).attach({
            update: function () {
                hpBar.setValue(properties.currentHp);
                // hpBar.position = enemy.position.clone();
                // hpBar.position.x -= enemy.dimension.width / 2;
                // hpBar.position.y -= enemy.dimension.height + 16;

                hpBar.position = enemy.position.clone();
                hpBar.position.x -= enemy.dimension.width / 2;
                hpBar.position.y = battleManager.getGroundY() + 4;
            }
        });
        var turnCounter = new Counter({
            z: Utils.Layers.GUI,
            name: 'damageCounter',
            image: Bento.assets.getImage('gui/numberstiny'),
            frameWidth: 12,
            frameHeight: 14,
            spacing: new Vector2(-4, 0),
            // position: settings.position
        }).attach({
            update: function () {
                turnCounter.setValue(component.turn || 0);
                turnCounter.position = enemy.position.clone();
                turnCounter.position.y = battleManager.getGroundY() + 8;
            }
        });
        // enemy hits frontCritter
        var component = {
            name: 'enemyBehavior',
            turn: 0,
            attached: function (data) {
                enemy = data.entity;
                walkComponent = enemy.getComponent('walk');
                // this.turn = enemy.getProperty('turnsToAttack') || 3;

            },
            // whether the enemy is ready attack
            ready: function (value) {
                isReady = value;
            },
            start: function (data) {
                // Bento.objects.attach(hpBar);
                // Bento.objects.attach(turnCounter);
            },
            destroy: function () {
                // Bento.objects.remove(turnCounter);
                // Bento.objects.remove(hpBar);
            },
            update: function () {
                var frontEnemy = battleManager.getFrontEnemy();
                frontCritter = battleManager.getFrontCritter();
                if (frontCritter && enemy === frontEnemy && component.turn === 0) {
                    component.turn = battleManager.isTwitterBattle ? enemy.getProperty('turnsToAttack') || 1 : 1;
                    isReady = false;
                    walkComponent.jumpAttack({
                        targetX: Math.max(frontCritter.position.x + 32, 32),
                        other: frontCritter,
                        onHit: battleManager.onHit,
                        onFinished: function () {
                            isReady = true;
                        }
                    });
                }
                // about to attack
                if (battleManager.isBossBattle) {
                    if (component.turn === 1 && exclamation === null && frontEnemy === enemy) {
                        makeExclamation();
                    }
                    if ((component.turn !== 1 || enemy.getProperty('currentHp') <= 0 || !isReady) && exclamation) {
                        enemy.remove(exclamation);
                        exclamation = null;
                    }
                }
            }
        };
        var makeExclamation = function () {
            if (!isReady) {
                return;
            }
            exclamation = new Entity({
                z: Utils.Layers.CRITTERS,
                name: 'exclamation',
                position: new Vector2(16, -enemy.dimension.height),
                originRelative: new Vector2(0.5, 1),
                components: [
                    new Sprite({
                        imageName: 'worldmap/attackbubble'
                    })
                ]
            });
            exclamation.scale.x = -1;
            enemy.attach(exclamation);
        };

        hpBar.attach(heartIcon);

        return component;

    };
});