/**
 * Base component that listens to an event. (Note: does not care about pause state)
 * @moduleName EventListener
 * @snippet EventListener.snippet
EventListener({
    name: '${1:eventName}',
    onEvent: function (data) {
        $2
    }
})
 */
bento.define('components/eventlistener', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween
) {
    'use strict';
    /**
     * Describe your settings object parameters
     * @param {Object} settings
     * @param {String} settings.name - name of event (NOT the name of the component)
     * @param {Function} settings.onEvent - callback on event
     */
    return function (settings) {
        var viewport = Bento.getViewport();
        var eventName = settings.name || settings.eventName;
        var onEvent = settings.callback || settings.onEvent || function () {};
        var component = {
            name: 'eventListener',
            start: function (data) {
                if (!eventName) {
                    console.warn('ERROR: eventlistener needs a name!');
                }
                EventSystem.on(eventName, onEvent);
            },
            destroy: function (data) {
                EventSystem.off(eventName, onEvent);
            }
        };
        return component;
    };
});