/**
 * Joystick component
 */
bento.define('components/joystick', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/components/sprite',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    Sprite,
    EventSystem,
    Utils,
    Tween
) {
    'use strict';
    /**
     * Describe your settings object parameters
     * @param {Object} settings
     */
    return function (settings) {
        var viewport = Bento.getViewport();
        var pointerDown = 0;
        var oldPos = new Vector2(0, 0);
        var offset = new Vector2(0, 0);
        var moveCache = new Vector2(0, 0);
        var fastOffset = new Vector2(0, 0);
        var speed = 1.5;
        var radius = 16;
        var fastTimer = 0;
        var listener = {
            start: function () {
                // listen to mouse events
                EventSystem.on('pointerDown', pointerDown);
                EventSystem.on('pointerMove', pointerMove);
                EventSystem.on('pointerUp', pointerUp);
            },
            destroy: function () {
                EventSystem.off('pointerDown', pointerDown);
                EventSystem.off('pointerMove', pointerMove);
                EventSystem.off('pointerUp', pointerUp);
            }
        };
        var cursor = new Entity({
            name: 'cursor',
            position: new Vector2(0, 0),
            originRelative: new Vector2(0.5, 0.5),
            components: [
                new Sprite({
                    imageName: 'gui/arrow'
                }),
                listener
            ]
        });
        var pointerDown = function (e) {
            if (!isActivated) {
                return;
            }
            pointerDown = true;
            // initialize at feet
            cursor.position = new Vector2(0, 0);
            oldPos = e.position.clone();
            cursor.rotation = 0;
        };
        var isActivated = false;
        var pointerMove = function (e) {
            var length,
                difference = e.position.substract(oldPos);
            if (!isActivated) {
                return;
            }
            cursor.position.addTo(difference);
            length = cursor.position.magnitude();
            cursor.visible = pointerDown && true && !Bento.objects.isPaused();

            // limit
            if (length > radius) {
                cursor.position.multiplyWith(new Vector2(radius / length, radius / length));
            }
            fastOffset = cursor.position.clone();
            fastOffset.multiplyWith(new Vector2(radius / length, radius / length));
            oldPos = e.position.clone();
            cursor.rotation = cursor.position.angle();
        }
        var pointerUp = function (e) {
            pointerDown = false;
            cursor.visible = false;
        };

        cursor.extend({
            isPointerDown: function () {
                return pointerDown;
            },
            getRadius: function () {
                return radius;
            },
            activate: function (bool) {
                isActivated = bool;
            },
            release: function () {
                pointerUp();
            }
        });
        cursor.visible = false;


        return cursor;
    };
});