/**
 * Stretch to fill (iPhoneX) screen
 * @moduleName StretchToFill
 * @snippet StretchToFill.snippet
StretchToFill({
    retainRatio: ${1:true}
})
 */
bento.define('components/stretchtofill', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween
) {
    'use strict';
    return function (settings) {
        var viewport = Bento.getViewport();
        var retainRatio = settings.retainRatio || false;
        var entity;
        var scale = new Vector2(1, 1);
        var component = {
            name: 'stretchToFill',
            start: function (data) {
                if (Utils.isWideScreen) {
                    // stretch height
                    scale.y = Utils.screenSize.height / viewport.height;

                    if (retainRatio) {
                        scale.x = scale.y;
                    }
                }
            },
            attached: function (data) {
                entity = data.entity;
            },
            draw: function (data) {
                data.renderer.scale(scale.x, scale.y);
            }
        };
        return component;
    };
});