/**
 * Dev saver: use saveLocalStorage in the browser's console window to
 * backup your save file.
 * Drag the json file into the canvas to load the save file.
 */
bento.define('devsaver', [], function () {
    'use strict';
    return function () {
        window.saveLocalStorage = function () {
            // stringify localstorage
            console.log('saving savestate...');
            var jsonString = JSON.stringify(window.localStorage);
            var blob = new window.Blob([jsonString], {
                type: "application/json"
            });
            var url = window.URL.createObjectURL(blob);
            var anchor = document.createElement('a');
            anchor.download = "combocritters.json";
            anchor.href = url;
            anchor.click();
        };

        var input;
        var onFileRead = function (e) {
            // var json = toJSON(e.target.result, ',');
            // console.log(e.target.result);
            try {
                var json = JSON.parse(e.target.result);
                var key;
                for (key in json) {
                    if (!json.hasOwnProperty(key)) {
                        continue;
                    }
                    window.localStorage.setItem(key, json[key]);
                }
                console.log('Load complete');
            } catch (exception) {
                console.log('ERROR: loading save file failed');
            }
        };
        var handleFileSelect = function (evt) {
            console.log('Loading save file...');
            var files = evt.dataTransfer.files;
            var fileReader = new window.FileReader();
            var i, f;
            for (i = 0; f = files[i]; i++) {
                fileReader.readAsText(f);
                fileReader.onload = (function (file) {
                    return onFileRead;
                })(f);
            }
            evt.stopPropagation();
            evt.preventDefault();
        };
        var handleDragOver = function (evt) {
            evt.stopPropagation();
            evt.preventDefault();
            evt.dataTransfer.dropEffect = 'copy'; // Explicitly show this is a copy.
        };
        var dropZone = document.getElementById('canvas');
        dropZone.addEventListener('dragover', handleDragOver, false);
        dropZone.addEventListener('drop', handleFileSelect, false);

        console.log('Ready to read save file');
    };

});