bento.define('entities/battle/abilities/fireball', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'entities/effect'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    Utils,
    Tween,
    Effect
) {
    'use strict';
    return function (settings) {
        var viewport = Bento.getViewport();
        var screenMid = new Vector2(viewport.width / 2, viewport.height / 2);
        var sprite = new Sprite(Bento.assets.getJson('animations/effects/fireball'));
        var creature = settings.creature;
        /**
         * How much offset there should be from the creature's position. Creature has an originRelative of (0.5, 1)
         * @type {Vector2}
         */
        var offset = new Vector2(creature.spriteImage.width / 2, -creature.spriteImage.height / 2);
        var entity = new Entity({
            z: Utils.Layers.EFFECTS,
            name: 'fireball',
            position: creature.position.add(offset),
            originRelative: new Vector2(0.5, 0.5),
            family: ['abilities'],
            updateWhenPaused: Utils.PauseLevel.TUTORIAL_2,
            components: [
                sprite, {
                    start: function (data) {
                        new Tween({
                            from: 0.1,
                            to: 1,
                            'in': 10,
                            ease: 'easeOutQuad',
                            updateWhenPaused: 0,
                            'do': function (v, t) {
                                entity.scale.x = v;
                                entity.scale.y = v;
                            }
                        });
                    },
                    update: function (data) {
                        entity.position.x += 2;
                    }
                }
            ]
        });

        entity.extend({
            attack: settings.damage,
            removeAbility: function (enemy) {
                var impactEffect = new Effect({
                    position: enemy.position.clone(),
                    originRelative: new Vector2(0.5, 1),
                    sprite: new Sprite(Bento.assets.getJson('animations/effects/impact'))
                });

                Bento.objects.attach(impactEffect);

                Bento.objects.remove(entity);
            }
        });

        return entity;
    };
});