/**
 * Module description
 */
bento.define('entities/battle/attackbutton', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/gui/clickbutton',
    'bento/gui/text',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'gui/centeredlist',
    'gui/storydialog'
], function (
    Bento,
    Vector2,
    Rectangle,
    ClickButton,
    Text,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    Utils,
    Tween,
    CenteredList,
    StoryDialog
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            active: Boolean // is button active
            onClick: Function // click acllback
            coolDown: Number // turns until activated
        }*/
        var viewport = Bento.getViewport();
        var turns = 0;
        var coolDown = settings.coolDown || Infinity;
        var available = false;
        var manager = Bento.objects.get('battleManager');
        var abilityButton = new ClickButton({
            sfx: 'sfx_button',
            image: Bento.assets.getImage('battle/buttonattack'),
            frameCountX: 4,
            frameCountY: 1,
            animations: {
                up: {
                    frames: [0, 1],
                    speed: 0.05
                },
                down: {
                    frames: [2]
                },
                inactive: {
                    frames: [3]
                }
            },
            updateWhenPaused: settings.updateWhenPaused || 0,
            position: settings.position || new Vector2(-1, 34),
            onClick: function () {
                if (!available) {
                    return;
                }

                // reset turns
                turns = -1;
                bubbles.iterate(function (item) {
                    // clear bubbles
                    item.getComponent('effect', function (effect) {
                        item.remove(effect);
                    });
                });

                // do ability
                settings.onClick();

                checkAvailable();
            }
        });
        var checkAvailable = function () {
            var appear;
            var appearSprite;
            if (turns >= coolDown && !available) {
                // is available
                available = true;
                bubbles.visible = false;
                // animation
                // new Tween({
                //     from: 0,
                //     to: 1,
                //     'in': 30,
                //     ease: 'elastic',
                //     alpha: -5,
                //     beta: 2,
                //     'do': function (v, t) {
                //         abilityButton.scale.x = v;
                //         abilityButton.scale.y = v;
                //     },
                //     onComplete: function () {

                //     }
                // });
                appearSprite = new Sprite(Bento.assets.getJson('animations/effects/buttonappear'));
                appear = new Entity({
                    name: 'appearEffect',
                    position: abilityButton.position.add(new Vector2(0, -2)),
                    originRelative: new Vector2(0.5, 0.5),
                    components: [appearSprite]
                });
                abilityButton.parent.attach(appear);

                abilityButton.scale.x = 0;
                abilityButton.scale.y = 0;

                appearSprite.setAnimation('appear', function () {
                    abilityButton.scale.x = 1;
                    abilityButton.scale.y = 1;

                    tutorial();

                    appearSprite.setAnimation('disappear', function () {
                        abilityButton.parent.remove(appear);
                    });
                });
            } else if (turns < coolDown) {
                // not available
                available = false;
                bubbles.visible = true;
            }
            abilityButton.visible = available;

            if (Bento.saveState.load('specialAttackTutorial', false) === false &&
                Bento.saveState.load('firstBossBattle', false) === false) {
                // hide bubbles until boss battle
                bubbles.visible = false;
            }

            // cheat
            if (window.testMinigame === true) {
                abilityButton.visible = true;
                available = true;
                bubbles.visible = false;
                abilityButton.scale.x = 1;
                abilityButton.scale.y = 1;
            }
        };
        var bubbles = new CenteredList({
            position: abilityButton.position.clone(),
            spacing: 2
        });
        var getHandDiagonal = function (offsetY) {
            return new Entity({
                name: 'hand',
                position: new Vector2(0, 8),
                originRelative: new Vector2(0.5, 0),
                components: [new Sprite({
                    imageName: 'map/hand2'
                })]
            }).attach({
                offsetX: 0,
                offsetY: offsetY || 4,
                start: function (data) {
                    this.offsetX = data.entity.parent.dimension.width / 2;
                },
                update: function (data) {
                    var hand = data.entity;
                    hand.position.y = this.offsetY + 4 * Math.abs(Math.sin(hand.timer / 20));
                    hand.position.x = this.offsetX + 4 * Math.abs(Math.sin(hand.timer / 20));
                }
            });
        };
        var tutorial = function () {
            var dialog;
            var tutorialAttack;
            if (Bento.saveState.load('specialAttackTutorial', false) === true) {
                return;
            }

            abilityButton.attach(getHandDiagonal(-4));

            if (manager && manager.state === 'battle') {
                // tutorial...!!
                tutorialAttack = new Entity({
                    name: 'tutattack',
                    position: new Vector2(-3, 16),
                    originRelative: new Vector2(0.5, 0.5),
                    components: [
                        new Sprite({
                            imageName: 'gui/tutorialattack'
                        })
                    ]
                });
                dialog = new StoryDialog({
                    text: Utils.Localization.getText('attackTutorial'),
                    type: 'yes',
                    canClose: false,
                    onClose: function () {
                        Bento.saveState.save('specialAttackTutorial', true);
                        Utils.Analytics.event('Game', 'specialAttackTutorial');
                        Utils.Firebase.logEvent('specialAttackTutorial');
                    },
                    onNextPage: function () {
                        dialog.remove(tutorialAttack);
                    }
                });
                dialog.attach(tutorialAttack);
            }

        };

        Utils.repeat(coolDown, function () {
            var bubble = new Entity({
                name: 'bubble',
                originRelative: new Vector2(0.5, 0.5),
                components: [
                    new Sprite({
                        imageName: 'battle/cooldownback'
                    })
                ]
            });
            bubbles.addItem(bubble);
        });

        abilityButton.extend({
            addTurn: function () {
                var bubble;

                if (Bento.saveState.load('firstBossBattle', false) === false) {
                    // dont show special attack before first bossBattle
                    return;
                }

                if (turns <= coolDown) {
                    // attach bubble
                    bubble = bubbles.get(turns);
                    if (bubble) {
                        bubble.attach(new Entity({
                            name: 'effect',
                            originRelative: new Vector2(0.5, 0.5),
                            components: [
                                new Sprite(Bento.assets.getJson('animations/effects/cooldownbubble'))
                            ]
                        }));
                    }
                }

                turns += 1;

                checkAvailable();
            },
            getBubbles: function () {
                return bubbles;
            }
        });
        checkAvailable();

        abilityButton.name = 'attackButton';


        return abilityButton;
    };
});