/**
 * Module description
 */
bento.define('entities/battle/catchminigame', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    ClickButton,
    Counter,
    Text,
    EventSystem,
    Utils,
    Tween
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
            manager: Entity,
            frontEnemy: Entity,
            difficulty: Number,
            onComplete: Function
        }*/
        var viewport = Bento.getViewport();
        var groundY = settings.manager.getGroundY();
        var clickable = new Clickable({
            pointerDown: function (evt) {
                if (currentPointer) {
                    return;
                }
                currentPointer = evt;
            },
            pointerMove: function (evt) {
                if (currentPointer && currentPointer.id === evt.id) {
                    currentPointer = evt;
                }
            },
            pointerUp: function (evt) {
                if (currentPointer && currentPointer.id === evt.id) {
                    currentPointer = null;
                }
            }
        });
        var frontEnemy = settings.frontEnemy;
        var entity = new Entity({
            z: 0,
            name: 'catchMinigame',
            position: new Vector2(0, 0),
            originRelative: new Vector2(0, 0),
            updateWhenPaused: 0,
            family: [''],
            components: [
                
                clickable
            ]
        });
        var currentPointer = null;
        var beamSprite = new Sprite(Bento.assets.getJson('animations/catchbeam'));
        var beamRadius = 20;
        var beamAttraction = 0.05 * 2;
        var critterSpeed = new Vector2(0, 0);
        var timeToCatch = 60 * 10;
        var maxAttraction = -0.5;
        var maxSpeedX = 1;
        var difficulty = settings.difficulty / 100;
        var hasCaught = false;
        var followShip = {
            update: function () {
                var position = beam.position;
                var diffPosition = position.x - frontEnemy.position.x;
                var diffSize = Math.abs(diffPosition);
                var attract;
                var repel;
                var catchHeight = Math.min(1, (groundY - frontEnemy.position.y) / 24);
                position.x = spaceShip.position.x;

                timeToCatch -= 1;

                if (timeToCatch === 0) {
                    console.log('time over');
                }

                if (hasCaught) {
                    return;
                }

                // is over front enemy?
                if (diffSize < beamRadius) {
                    // frontEnemy.position.y -= 1;
                    critterSpeed.y = Utils.approach(critterSpeed.y, maxAttraction, beamAttraction);

                    repel = maxSpeedX * -Utils.sign(diffPosition) * Math.pow(diffSize / beamRadius, difficulty);
                    attract = 2 * maxSpeedX * Utils.sign(diffPosition) * diffSize / beamRadius;

                    critterSpeed.x = catchHeight * repel + (1 - catchHeight) * attract;

                    if (critterSpeed.x === 0) {
                        critterSpeed.x = 0.01;
                    }
                } else {
                    // drop enemy
                    critterSpeed.y += 0.5;
                }
                frontEnemy.position.x += critterSpeed.x;
                frontEnemy.position.y += critterSpeed.y;

                if (frontEnemy.position.y > groundY) {
                    frontEnemy.position.y = groundY;
                    critterSpeed.y = -critterSpeed.y * 0.5;
                    critterSpeed.x = critterSpeed.x * (0.5 + 0.6 * Math.abs(frontEnemy.position.x - viewport.width / 2) / (viewport.width / 2));
                    if (critterSpeed.y < maxAttraction) {
                        critterSpeed.y = maxAttraction;
                    }
                }
                if (critterSpeed.x < -maxSpeedX) {
                    critterSpeed.x = -maxSpeedX;
                }
                if (critterSpeed.x > maxSpeedX) {
                    critterSpeed.x = maxSpeedX;
                }

                // loop ?
                if (frontEnemy.position.x > viewport.width) {
                    frontEnemy.position.x -= viewport.width;
                    // frontEnemy.position.x = viewport.width / 2;
                }
                if (frontEnemy.position.x < 0) {
                    frontEnemy.position.x += viewport.width;
                    // frontEnemy.position.x = viewport.width / 2;
                }

                // caught?
                if (spaceShip.getBoundingBox().hasPosition(frontEnemy.position)) {
                    hasCaught = true;
                    frontEnemy.getComponent('walk', function (walk) {
                        frontEnemy.remove(walk);
                    });
                    new Tween({
                        from: 1,
                        to: 0,
                        'in': 30,
                        ease: 'linear',
                        'do': function (v, t) {
                            frontEnemy.scale.x = -v;
                            frontEnemy.scale.y = v;
                        }
                    });
                    new Tween({
                        from: frontEnemy.position.y,
                        to: frontEnemy.position.y - 16,
                        'in': 30,
                        ease: 'linear',
                        'do': function (v, t) {
                            frontEnemy.position.y = v;
                        },
                        onComplete: leave
                    });
                }
            }
        };
        var spaceShip = new Entity({
            z: Utils.Layers.SKY + 10,
            name: 'spaceShip',
            position: new Vector2(viewport.x + viewport.width / 2, viewport.y - 75),
            originRelative: new Vector2(0.5, 1),
            boundingBox: new Rectangle(-beamRadius, -4, beamRadius * 2, 8),
            components: [
                new Sprite({
                    imageName: 'worldmap/ship',
                })
            ]
        });
        var beam = new Entity({
            z: Utils.Layers.BACKGROUND,
            name: 'beam',
            position: new Vector2(viewport.width / 2, 0),
            originRelative: new Vector2(0.5, 0),
            components: [
                beamSprite,
                followShip
            ]
        });
        var followPointer = {
            update: function () {
                if (currentPointer && !hasCaught) {
                    spaceShip.position.x += (currentPointer.position.x - spaceShip.position.x) / 2;
                }
            }
        };
        var init = function () {
            var moveIn = function () {
                // move in
                new Tween({
                    from: viewport.y - 75,
                    to: viewport.y + 64,
                    'in': 60,
                    ease: 'squareroot',
                    'do': function (v, t) {
                        spaceShip.position.y = v;
                    },
                    onComplete: function () {
                        // show beam
                        showBeam();
                    }
                });
            };
            var showBeam = function () {
                beam.alpha = (0.7);
                beamSprite.setAnimation('start');
                beam.position.y = spaceShip.position.y - 4;
                Bento.objects.attach(beam);

                spaceShip.attach(followPointer);
            };
            moveIn();
            Bento.objects.attach(spaceShip);

        };
        var leave = function () {
            var moveOut = function () {
                new Tween({
                    from: spaceShip.position.y,
                    to: spaceShip.position.y - 80,
                    'in': 60,
                    ease: 'easeInBack',
                    'do': function (v, t) {
                        spaceShip.position.y = v;
                    },
                    onComplete: function () {
                        Bento.objects.remove(spaceShip);
                        settings.onComplete(hasCaught);
                    }
                });
            };
            beamSprite.setAnimation('end', function () {
                Bento.objects.remove(beam);
                moveOut();
            });
        };

        init();
        Bento.objects.attach(entity);
        return entity;
    };
});