/**
 * Popup asking to catch enemy
 */
bento.define('entities/battle/catchpopup', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'gui/coincounter',
    'gui/shop/shop',
    'modules/videoadtimer'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    Utils,
    Tween,
    ClickButton,
    Counter,
    Text,
    CoinCounter,
    Shop,
    VideoAdTimer
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity,
            chance: Number // chance in percentage
            onExit: Function // exit button pressed
        }*/
        var viewport = Bento.getViewport();
        var sprite = new Sprite({
            imageName: 'collect/catchwindow'
        });
        var entity = new Entity({
            z: Utils.Layers.GUI,
            name: 'catchPopup',
            position: settings.position || new Vector2(0, 0),
            originRelative: new Vector2(0.4, 0.5),
            updateWhenPaused: 0,
            components: [
                sprite
            ]
        });
        var text = new Text({
            position: new Vector2(8, -16),
            text: Utils.Localization.getText('chanceToCollect'),
            font: 'volter',
            fontSize: 9,
            fontSettings: Utils.FontSettings.get('font'),
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'bottom',
            maxWidth: 118,
            maxHeight: 18,
            linebreaks: false
        });
        var percentageText = new Counter({
            name: settings.name || 'counter',
            image: Bento.assets.getImage('gui/numberswhite'),
            frameWidth: 8,
            frameHeight: 8,
            spacing: new Vector2(-3, 0),
            position: new Vector2(56, -10),
            align: settings.align,
            animations: {
                '0': {
                    frames: [0]
                },
                '1': {
                    frames: [1]
                },
                '2': {
                    frames: [2]
                },
                '3': {
                    frames: [3]
                },
                '4': {
                    frames: [4]
                },
                '5': {
                    frames: [5]
                },
                '6': {
                    frames: [6]
                },
                '7': {
                    frames: [7]
                },
                '8': {
                    frames: [8]
                },
                '9': {
                    frames: [9]
                },
                '-': {
                    frames: [10]
                },
                'A': {
                    frames: [11]
                },
                'H': {
                    frames: [12]
                },
                'S': {
                    frames: [13]
                },
                '%': {
                    frames: [14]
                }
            }
        });
        var percent = new Entity({
            position: new Vector2(0, 0),
            originRelative: new Vector2(0, 0),
            family: [''],
            components: [new Sprite({
                imageName: 'gui/numberswhite',
                frameCountX: 5,
                frameCountY: 5,
                animations: {
                    'default': {
                        speed: 0,
                        frames: [14]
                    }
                }
            })]
        });
        var buttonCollect = new ClickButton({
            sfx: 'sfx_button',
            image: Bento.assets.getImage('collect/buttoncollectempty'),
            frameCountX: 2,
            frameCountY: 1,
            animations: {
                up: {
                    frames: [0]
                },
                down: {
                    frames: [1]
                }
            },
            position: new Vector2(8, 17),
            updateWhenPaused: 0,
            onClick: function () {
                var myCoins = Bento.saveState.load('myCoins', 0);
                // has enough coins
                if (myCoins < currentPrice) {
                    Bento.audio.playSound('sfx_error');
                    new Shop({});
                    // allow video ad when user doesnt have enough coins??
                    // VideoAdTimer.force();
                } else {
                    myCoins -= currentPrice;
                    Utils.trackCoins(-currentPrice, 'retry collect');
                    Bento.saveState.save('myCoins', myCoins);
                    EventSystem.fire('updateCoins');
                    settings.onCollect();
                }
            }
        });
        var symbol = new Entity({
            name: 'collectSymbol',
            position: new Vector2(0, 0),
            originRelative: new Vector2(0.5, 0.5),
            components: [
                new Sprite({
                    imageName: 'collect/collectsymbol'
                })
            ]
        });
        var buttonExit = new ClickButton({
            sfx: 'sfx_button',
            image: Bento.assets.getImage('collect/buttonexit'),
            frameCountX: 3,
            frameCountY: 1,
            animations: {
                up: {
                    frames: [0]
                },
                down: {
                    frames: [1]
                },
                inactive: {
                    frames: [2]
                }
            },
            position: new Vector2(-42, 16),
            updateWhenPaused: 0,
            onClick: function () {
                buttonExit.setActive(false);
                buttonCollect.setActive(false);
                entity.exit(settings.onExit);
            }
        });
        var barContainer = new Entity({
            name: 'percentage',
            position: new Vector2(-49, -10),
            originRelative: new Vector2(0, 0),
            components: []
        });
        var bars = [];
        var extraChance = false;
        var priceCounter = new CoinCounter({
            position: new Vector2(0, -4),
            align: 'center',
            value: '$0'
        });
        var currentPrice = 0;
        var addBar = function () {
            var bar = new Entity({
                name: 'percentage',
                position: new Vector2(0, 4),
                originRelative: new Vector2(0.5, 0.5),
                components: [new Sprite({
                    imageName: extraChance ? 'collect/chancebar2' : 'collect/chancebar'
                })]
            });
            bar.position.x = bars.length * (bar.dimension.width + 1) + 2;
            barContainer.attach(bar);
            bars.push(bar);
            new Tween({
                from: 0,
                to: 1,
                'in': 10,
                ease: 'elastic',
                alpha: -3,
                beta: 1,
                'do': function (v, t) {
                    bar.scale.x = v;
                    bar.scale.y = v;
                }
            });
        };
        var animate = function () {
            // animate
            entity.attach({
                count: 0,
                update: function () {
                    if (this.count < settings.chance) {
                        this.count += 1.5;
                        percentageText.setValue(Math.floor(this.count));
                        // bars
                        if (bars.length < Math.floor(this.count / 100 * 22)) {
                            Bento.audio.stopSound('sfx_battle_meter');
                            Bento.audio.playSound('sfx_battle_meter');
                            addBar();
                        }
                    } else {
                        this.count = settings.chance;
                        percentageText.setValue(Math.floor(this.count));

                    }
                }
            });
        };

        percentageText.attach(percent);

        buttonCollect.attach(symbol);

        entity.attach(text);
        entity.attach(percentageText);
        entity.attach(barContainer);
        entity.attach(buttonCollect);
        entity.attach(buttonExit);

        // disable exit button until tutorials
        if (Bento.saveState.load('sawTutorial2', false) === false) {
            buttonExit.setActive(false);
        }

        entity.extend({
            animate: animate,
            addChance: function (chance, price) {
                currentPrice = price;
                settings.chance += chance;
                settings.chance = Math.min(settings.chance, 100);
                extraChance = true;

                buttonCollect.setActive(true);
                buttonExit.setActive(true);

                buttonCollect.remove(symbol);

                // attach price
                priceCounter.setValue('$' + price);
                buttonCollect.attach(priceCounter);

                text.setText(Utils.Localization.getText('retry'));
            },
            exit: function (onExit) {
                buttonCollect.setActive(false);
                buttonExit.setActive(false);
                // tween out
                new Tween({
                    from: entity.position.x,
                    to: entity.position.x + viewport.width,
                    'in': 30,
                    ease: 'easeInExpo',
                    'do': function (v, t) {
                        entity.position.x = v;
                    },
                    onComplete: function () {
                        if (onExit) {
                            onExit();
                        }
                        // Bento.objects.remove(entity);
                    }
                });
            }
        });

        return entity;
    };
});