/**
 * Module description
 */
bento.define('entities/battle/coin', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'components/movable'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    ClickButton,
    Counter,
    Text,
    EventSystem,
    Utils,
    Tween,
    Movable
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
            speed: Vector2,
            manager: Entity,
            amount : Int, //the amount of coins that should be awarded to the player when this coin is collected
            animationKey : String, //the animaton key used for the coin's sprite
            scale : Number //the scale of the coin entity
        }*/
        var animationKey = (settings.animationKey !== null) ? settings.animationKey : 'animations/coinpickup';
        var viewport = Bento.getViewport();
        var sprite = new Sprite(Bento.assets.getJson(animationKey));
        var pointerDown = false;
        var coinCounters = Bento.objects.getByFamily('coinCounters');
        var gameOver = Bento.objects.getByFamily('gameOver');
        var delay = 0;
        var checkHover = function (evt) {
            if (pointerDown) {
                if (this.isHovering && gameOver.length === 0) {
                    collect();
                }
            }
        };
        var collect = function () {
            var coin;
            if (coinCounters.length) {
                coin = coinCounters[0].getComponent('coin');
                spawnTrail(coin.getWorldPosition(), function () {
                    var myCoins = Bento.saveState.load('myCoins', 0);
                    myCoins += settings.amount || 1;
                    // coins are tracked when the coin objects are spawned
                    Bento.saveState.save('myCoins', myCoins);
                    EventSystem.fire('updateCoins');
                    new Tween({
                        from: 1.5,
                        to: 1,
                        'in': 10,
                        ease: 'linear',
                        'do': function (v, t) {
                            coin.scale.x = v;
                            coin.scale.y = v;
                        }
                    });
                });
            }
            Bento.audio.stopSound('sfx_coin');
            Bento.audio.playSound('sfx_coin');
            Bento.objects.remove(entity);
        };
        var spawnTrail = function (targetPos, callback) {
            // var sprite = new Sprite({
            //     imageName: 'worldmap/coin'
            // });
            var sprite = new Sprite(Bento.assets.getJson(animationKey));
            var coin = new Entity({
                z: Utils.Layers.CRITTERS + 1,
                name: 'coin',
                position: entity.position.clone(),
                originRelative: new Vector2(0.5, 0.5),
                components: [sprite],
                scale: {
                    x: settings.scale || 1,
                    y: settings.scale || 1
                }
            });

            coin.attach({
                speed: new Vector2(
                    Utils.getRandomRange(-8, 8),
                    Utils.getRandomRange(-8, 8)
                ),
                targeting: 0,
                update: function () {
                    // move
                    coin.position.addTo(this.speed);

                    // damp speed
                    this.speed.scalarMultiplyWith(0.9);

                    // force to target pos
                    coin.position.x += (targetPos.x - coin.position.x) * this.targeting;
                    coin.position.y += (targetPos.y - coin.position.y) * this.targeting;

                    this.targeting += coin.timer / 1000;

                    if (coin.scale.x > 0) {
                        coin.scale.x -= 0.02;
                        coin.scale.y -= 0.02;
                    }

                    if (coin.position.distance(targetPos) < 1) {
                        Bento.objects.remove(coin);
                        callback();
                    }
                }
            });
            Bento.objects.attach(coin);
            return coin;
        };
        var clickable = new Clickable({
            pointerDown: function (evt) {
                if (gameOver.length === 0) {
                    pointerDown = true;
                }
                checkHover.apply(this, [evt]);
            },
            pointerMove: checkHover,
            pointerUp: function (evt) {
                pointerDown = false;
            }
        });
        var autoPickup = {
            isCollected: false,
            update: function () {
                if (entity.timer > 100 + delay && !this.isCollected) {
                    this.isCollected = true;
                    collect();
                }
            }
        };
        var movable = new Movable({
            gravity: 0.1,
            maxSpeed: new Vector2(4, 4),
            bounce: new Vector2(0.8, 0.5),
            friction: new Vector2(0.5, 0.8),
            onCollide: function (data) {}
        });
        var entity = new Entity({
            z: Utils.Layers.CRITTERS + 1,
            name: 'coin',
            position: settings.position || new Vector2(0, 0),
            originRelative: new Vector2(0.5, 1),
            boundingBox: new Rectangle(-8, -16, 16, 16),
            components: [
                sprite,
                movable,
                // clickable,
                autoPickup
            ],
            scale: {
                x: settings.scale || 1,
                y: settings.scale || 1
            }
        });

        entity.extend({
            setDelay: function (value) {
                delay = value;
            }
        });

        movable.speed = settings.speed;

        return entity;
    };
});