/**
 * Base critter module
 */
bento.define('entities/battle/critter', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'bento/packedimage'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    Utils,
    Tween,
    PackedImage
) {
    'use strict';
    return function (properties) {
        /*properties = {
            id: String // critter id
        }*/
        var viewport = Bento.getViewport();
        var id = properties.id;
        var critterData = Bento.assets.getJson('creatures/creatures')[id];
        var sprite;
        var entity;
        var image;

        if (properties.twitter) {
            //
            sprite = new Sprite({});
            image = new Image();
            image.src = properties.image;
            image.onload = function () {
                sprite.setup({
                    image: new PackedImage(image)
                });

                entity.scale.x = 0.25;
                entity.scale.y = entity.scale.x;
                entity.setOriginRelative(new Vector2(0.5, 1));
            }
        } else {
            sprite = new Sprite({
                imageName: 'creatures/' + id
            });
        }
        entity = new Entity({
            z: Utils.Layers.CRITTERS,
            name: 'critter-' + id,
            position: new Vector2(0, 0),
            originRelative: new Vector2(0.5, 1),
            family: ['critters'],
            components: [
                sprite
            ]
        });
        // 
        entity.extend({
            getProperty: function (name) {
                return properties[name];
            },
            setProperty: function (name, value) {
                properties[name] = value;
            },
            addToProperty: function (name, value) {
                properties[name] = parseFloat(properties[name]) + value;
            },
            getProperties: function () {
                return properties;
            }
        });
        return entity;
    };
});