/**
 * Game over popup: victory or defeat
 */
bento.define('entities/battle/gameover', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'gui/storydialog',
    'modules/videoadtimer'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    Utils,
    Tween,
    StoryDialog,
    VideoAdTimer
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            name: String // 'defeat' or 'victory'
            onClosed: Function // callback
        }*/
        var viewport = Bento.getViewport();
        var sprite = new Sprite(Bento.assets.getJson('animations/' + settings.name));
        var darkSprite = new Sprite({
            imageName: 'fade'
        });
        var darkEntity = new Entity({
            z: Utils.Layers.GUI,
            position: new Vector2(viewport.width / 2, viewport.height / 2),
            originRelative: new Vector2(0.5, 0.5),
            components: [darkSprite]
        });
        var clickable = new Clickable({
            pointerDown: function () {
                dismiss();
            }
        });
        var entity = new Entity({
            z: Utils.Layers.GUI + 1,
            name: 'gameOver',
            position: new Vector2(viewport.width / 2, viewport.height * 2 / 5),
            originRelative: new Vector2(0.5, 0.5),
            updateWhenPaused: 0,
            float: true,
            family: ['gameOver'],
            components: [
                sprite
            ]
        });
        var canDismiss = false;
        var hasDismissed = false;
        var dismiss = function () {
            if (hasDismissed || !canDismiss) {
                return;
            }
            hasDismissed = true;
            new Tween({
                from: entity.position.x,
                to: entity.position.x - viewport.width,
                'in': 30,
                ease: 'easeInBack',
                'do': function (v, t) {
                    entity.position.x = v;
                },
                onComplete: function () {
                    Bento.objects.remove(entity);
                    if (settings.onClosed) {
                        settings.onClosed();
                    }

                    if (settings.name === 'victory' && !Bento.audio.isPlayingMusic('bgm_catch')) {
                        Bento.audio.stopAllMusic();
                        Bento.audio.playMusic('bgm_catch');
                        Bento.audio.setVolume(1, 'bgm_catch');
                    }

                }
            });
            new Tween({
                from: darkEntity.alpha,
                to: 0,
                'in': 30,
                ease: 'linear',
                'do': function (v, t) {
                    darkEntity.alpha = (v);
                },
                onComplete: function () {
                    Bento.objects.remove(darkEntity);
                }
            });
        };
        if (settings.name === 'victory') {
            Bento.audio.playSound('sfx_battle_victory');
            Bento.audio.playSound('sfx_battle_cheer');
        } else {
            Bento.audio.playSound('sfx_battle_defeat');
            Bento.audio.playSound('sfx_battle_aw');
        }
        sprite.setAnimation('start', function () {
            canDismiss = true;
            Bento.audio.playSound('sfx_transition');
            sprite.setAnimation('loop');
        });

        Bento.audio.stopAllMusic();

        darkEntity.alpha = (0);
        Bento.objects.attach(darkEntity);
        new Tween({
            from: 0,
            to: 0.5,
            'in': 30,
            ease: 'linear',
            'do': function (v, t) {
                darkEntity.alpha = (v);
            }
        });

        // dismiss automatically
        Utils.timeout(100, function () {
            if (!settings.isTwitterBattle && settings.isBossBattle && Bento.saveState.load('bossLoseTutorial', false) === false) {
                new StoryDialog({
                    text: Utils.Localization.getText('bossLoseTutorial'),
                    onClose: function () {
                        Bento.saveState.save('bossLoseTutorial', true);
                        dismiss();
                    }
                });
            }
            // explain that you need stronger critters after losing a twitter battle for the first time
            else if (settings.isTwitterBattle && !Bento.saveState.load('alreadyLostFirstTwitterBattle', false)) {
                new StoryDialog({
                    text: Utils.Localization.getText('bossLoseTutorial'),
                    onClose: function () {
                        Bento.saveState.save('alreadyLostFirstTwitterBattle', true);
                        dismiss();
                    }
                });
            } else {
                dismiss();
            }
        });
        
        if (settings.isBossBattle) {
            // can watch ad after boss battle defeat/victory
            // VideoAdTimer.force();
        }

        Bento.objects.attach(entity);
        
        Bento.saveState.save('battles', Bento.saveState.load('battles', 0) + 1);
        
        return entity;
    };
});