/**
 * Grid holding cards with your critters
 */
bento.define('entities/battle/grid', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'bento/gui/clickbutton'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    Utils,
    Tween,
    ClickButton
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity,
            spacing: Number
            align: string
        }*/
        var viewport = Bento.getViewport();
        var position = settings.position || new Vector2(0, 0);

        var spacing = settings.spacing || 0;
        var alignment = settings.align || 'center';
        var wrap = settings.wrap || 3;
        // container
        var entity = new Entity({
            z: Utils.Layers.CRITTERS,
            name: 'grid',
            position: position.clone(),
            originRelative: new Vector2(0, 0),
            updateWhenPaused: 0,
            family: [''],
            components: [
            ]
        });
        var cards = [];
        var width = 0;
        var height = 0;
        var align = function () {
            var n = cards.length;

            if (alignment === 'left') {
                entity.position.x = position.x;
            } else if (alignment === 'right') {
                entity.position.x = position.x - width * (n - 1);
            } else {
                entity.position.x = position.x - width * (n - 1) / 2;
            }
        };

        entity.dimension.width = 0;
        entity.dimension.height = 0;

        entity.extend({
            addCard: function (card, ability) {
                var n;
                if (card === null) {
                    // can add nothing
                    cards.push(null);
                    align();
                    return;
                }
                width = card.dimension.width + spacing;
                height = card.dimension.height;

                n = cards.length;
                // entity.dimension.width += width;
                // entity.dimension.height += height;

                card.position = new Vector2(
                    (n % wrap) * width,
                    Math.floor(n / wrap) * height
                );
                entity.attach(card);
                cards.push(card);
                align();
            }
        });

        return entity;
    };
});