/**
 * Catch minigame
 */
bento.define('entities/battle/minigames/catch', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'entities/effect',
    'entities/battle/minigames/base'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    ClickButton,
    Counter,
    Text,
    EventSystem,
    Utils,
    Tween,
    Effect,
    Base
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
            rarity: Number, critter rarity
            onHit: Function,
            onComplete: Function(Number success, Number max) // minigame is completed
            critter: frontCritter,
            manager: entity,
            enemy: frontEnemy,
            attack: Function
        }*/
        var viewport = Bento.getViewport();
        var firstTime = !Bento.saveState.load('firstTimeCatch', false) || window.easyMinigames; // first time playing this you get infinite time
        var timeToComplete = firstTime ? Infinity : Math.max(240 - settings.rarity * 10, 120);
        var makeTurn = function (endOfTurn, text) {
            var timer = timeToComplete;
            var ended = false;
            var timerBehavior = {
                update: function () {
                    if (timer > 0) {
                        timer -= 1;
                    } else if (timer === 0) {
                        if (score >= sphereCount && !ended) {
                            ended = true;
                            endOfTurn(true);
                        } else {
                            // didn't complete swipe
                            Bento.audio.playSound('sfx_battle_fail');
                            endOfTurn(false);
                        }
                    }
                }
            };
            var score = 0;
            var sphereCount = 3;
            var hasTapped = false;
            var spheres = [];
            var sphereIndex = 0;
            var makeSphere = function () {
                var makeParticle = function (speed) {
                    var particleSprite = new Sprite({
                        imageName: 'minigames/catch/particle'
                    });
                    var fallBehavior = {
                        update: function () {
                            particle.position.x += speed.x;
                            particle.position.y += speed.y;
                            speed.y += 0.1;

                            particle.scale.x -= 1 / 15;
                            particle.scale.y = particle.scale.x;

                            if (particle.scale.x < 0) {
                                Bento.objects.remove(particle);
                            }
                        }
                    };
                    var particle = new Entity({
                        z: Utils.Layers.MINIGAME + 1,
                        name: 'particle',
                        position: sphere.getWorldPosition(),
                        originRelative: new Vector2(0.5, 0.5),
                        updateWhenPaused: Utils.PauseLevels.MINIGAME,
                        components: [
                            particleSprite,
                            fallBehavior
                        ]
                    });
                    Bento.objects.attach(particle);
                };
                var sphereSprite = new Sprite({
                    imageName: 'minigames/catch/ball'
                });
                var clickable = new Clickable({
                    pointerUp: function () {
                        hasTapped = false;
                    },
                    onClick: function () {
                        if (behavior.tapped || hasTapped) {
                            return;
                        }
                        Bento.audio.stopSound('sfx_battle_correct');
                        Bento.audio.playSound('sfx_battle_correct');
                        hasTapped = true;
                        score += 1;
                        behavior.tapped = true;
                        new Tween({
                            from: 1,
                            to: 0,
                            'in': 15,
                            ease: 'easeInBack',
                            ignoreGameSpeed: true,
                            updateWhenPaused: Utils.PauseLevels.MINIGAME,
                            'do': function (v, t) {
                                sphere.scale.x = v;
                                sphere.scale.y = v;
                            },
                            onComplete: function () {
                                // spawn particles
                                makeParticle(new Vector2(-1, -1));
                                makeParticle(new Vector2(-1, 0));
                                makeParticle(new Vector2(1, -1));
                                makeParticle(new Vector2(1, 0));

                                turn.remove(sphere);

                                var index = spheres.indexOf(sphere);
                                if (index >= 0) {
                                    spheres.splice(index, 1);
                                }

                                if (score >= sphereCount && !ended) {
                                    ended = true;
                                    endOfTurn(true);
                                }
                            }
                        });
                    }
                });
                var collidesWithOther = function (callback) {
                    var collided = false;

                    Utils.forEach(spheres, function (other, i, l, breakLoop) {
                        if (other !== sphere && sphere.position.isCloserThan(other.position, 32)) {
                            if (callback) {
                                callback(other);
                            }
                            collided = true;
                            breakLoop();
                            return;
                        }
                    });
                    return collided;
                };
                var behavior = {
                    tapped: false,
                    speed: new Vector2(0.5 + Utils.getRandomFloat(1), 0),
                    start: function () {
                        // randomize direction
                        this.speed.rotateRadian(Utils.getRandomFloat(Math.PI * 2));
                    },
                    update: function () {
                        if (this.tapped) {
                            return;
                        }
                        sphere.position.x += this.speed.x;
                        sphere.position.y += this.speed.y;
                        // edges
                        if (sphere.position.x > 72 && this.speed.x > 0) {
                            this.speed.x *= -1;
                            sphere.position.x = 72;
                        } else if (sphere.position.x < -72 && this.speed.x < 0) {
                            this.speed.x *= -1;
                            sphere.position.x = -72;
                        }
                        if (sphere.position.y > 64 && this.speed.y > 0) {
                            this.speed.y *= -1;
                            sphere.position.y = 64;
                        } else if (sphere.position.y < -64 && this.speed.y < 0) {
                            this.speed.y *= -1;
                            sphere.position.y = -64;
                        }

                        // bounce
                        collidesWithOther(function (other) {
                            sphere.position.x -= behavior.speed.x;
                            sphere.position.y -= behavior.speed.y;

                            sphere.reverse();
                            other.reverse();

                            behavior.speed.rotateRadian(Utils.getRandomRange(-Math.PI / 2, Math.PI / 2));
                        });
                    }
                };
                var sphere = new Entity({
                    name: 'sphere',
                    family: ['spheres'],
                    updateWhenPaused: Utils.PauseLevels.MINIGAME,
                    position: new Vector2(-64 + 64 * sphereIndex,
                        0
                    ),
                    originRelative: new Vector2(0.5, 0.5),
                    components: [
                        sphereSprite,
                        clickable,
                        behavior
                    ]
                });

                sphereIndex += 1;

                sphere.extend({
                    reverse: function () {
                        behavior.speed.x *= -1;
                        behavior.speed.y *= -1;
                    }
                });

                new Tween({
                    from: 1,
                    to: 0,
                    'in': timeToComplete,
                    ease: 'exponential',
                    ignoreGameSpeed: true,
                    updateWhenPaused: Utils.PauseLevels.MINIGAME,
                    beta: -20,
                    'do': function (v, t) {
                        sphere.alpha = (v);
                    }
                });

                // delay add
                new Tween({
                    'in': spheres.length * 30,
                    ease: 'linear',
                    ignoreGameSpeed: true,
                    updateWhenPaused: Utils.PauseLevels.MINIGAME,
                    onComplete: function () {
                        turn.attach(sphere);

                        spheres.push(sphere);
                        new Tween({
                            from: 0,
                            to: 1,
                            'in': 30,
                            ease: 'elastic',
                            alpha: -4,
                            beta: 1,
                            ignoreGameSpeed: true,
                            updateWhenPaused: Utils.PauseLevels.MINIGAME,
                            'do': function (v, t) {
                                sphere.scale.x = v;
                                sphere.scale.y = v;
                            }
                        });
                    }
                });

                return sphere;
            };
            var turn = new Entity({
                position: new Vector2(0, 32),
                originRelative: new Vector2(0.5, 0.5),
                updateWhenPaused: Utils.PauseLevels.MINIGAME,
                components: [
                    timerBehavior
                ]
            });

            // spawn spheres
            Utils.repeat(sphereCount, makeSphere);
            Bento.saveState.save('firstTimeCatch', true);

            text.setText(Utils.Localization.getText('minigameCatch'));

            return turn;
        };
        // var base = new Base(settings, 'Tap!', makeTurn);
        // var endOfTurn = base.endOfTurn;

        return makeTurn;
    };
});