/**
 * Order minigame
 */
bento.define('entities/battle/minigames/order', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'entities/effect',
    'entities/battle/minigames/base'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    ClickButton,
    Counter,
    Text,
    EventSystem,
    Utils,
    Tween,
    Effect,
    Base
) {
    'use strict';
    var PI = Math.PI;
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
            rarity: Number, critter rarity
            onHit: Function,
            onComplete: Function(Number success, Number max) // minigame is completed
            critter: frontCritter,
            manager: entity,
            enemy: frontEnemy,
            attack: Function
        }*/
        var viewport = Bento.getViewport();
        var firstTime = !Bento.saveState.load('firstTimeOrder', false) || window.easyMinigames; // first time playing this you get infinite time
        var timeToComplete = firstTime ? Infinity : Math.max(180 - settings.rarity * 5, 120);
        var makeTurn = function (endOfTurn, text) {
            var timer = timeToComplete;
            var ended = false;
            var timerBehavior = {
                start: function () {

                },
                update: function () {
                    if (timer > 0) {
                        timer -= 1;
                    } else if (timer === 0) {
                        // didn't complete swipe
                        Bento.audio.playSound('sfx_battle_fail');
                        endOfTurn(false);
                    }
                }
            };
            var balls = 0;
            var clicked = 0;
            var positions = [];
            var addPosition = function () {
                var i = positions.length;
                var pos = new Vector2(-32 + 64 * (i % 2), -32 + 64 * Math.floor(i / 2));

                if (Utils.getRandom(2)) {
                    positions.push(pos);
                } else {
                    positions.unshift(pos);
                }
            };
            var makeParticle = function (speed, sphere) {
                var particleSprite = new Sprite({
                    imageName: 'minigames/catch/particle'
                });
                var fallBehavior = {
                    update: function () {
                        particle.position.x += speed.x;
                        particle.position.y += speed.y;
                        speed.y += 0.1;

                        particle.scale.x -= 1 / 15;
                        particle.scale.y = particle.scale.x;

                        if (particle.scale.x < 0) {
                            Bento.objects.remove(particle);
                        }
                    }
                };
                var particle = new Entity({
                    z: Utils.Layers.MINIGAME + 1,
                    name: 'particle',
                    position: sphere.getWorldPosition(),
                    updateWhenPaused: Utils.PauseLevels.MINIGAME,
                    originRelative: new Vector2(0.5, 0.5),
                    components: [
                        particleSprite,
                        fallBehavior
                    ]
                });
                Bento.objects.attach(particle);
            };
            var addBall = function () {
                var ballSprite = new Sprite({
                    imageName: 'minigames/order/balls',
                    frameCountX: 2,
                    frameCountY: 2,
                    animations: {
                        default: {
                            frames: [balls]
                        }
                    }
                });
                var id = balls;
                var clickable = new Clickable({
                    onClick: function () {
                        if (clicked === id) {
                            clicked += 1;
                            Bento.audio.stopSound('sfx_battle_correct');
                            Bento.audio.playSound('sfx_battle_correct');
                            // spawn particles
                            makeParticle(new Vector2(-1, -1), ball);
                            makeParticle(new Vector2(-1, 0), ball);
                            makeParticle(new Vector2(1, -1), ball);
                            makeParticle(new Vector2(1, 0), ball);
                            entity.remove(ball);

                            if (clicked === balls) {
                                endOfTurn(true);
                            }
                        } else {
                            endOfTurn(false);
                        }
                    }
                });
                var ball = new Entity({
                    name: 'ball',
                    position: Utils.getRandom(2) ? positions.pop() : positions.shift(),
                    originRelative: new Vector2(0.5, 0.5),
                    updateWhenPaused: Utils.PauseLevels.MINIGAME,
                    components: [
                        ballSprite,
                        clickable
                    ]
                });
                entity.attach(ball);
                balls += 1;
            };
            var entity = new Entity({
                name: 'spinner',
                position: new Vector2(0, 0),
                updateWhenPaused: Utils.PauseLevels.MINIGAME,
                originRelative: new Vector2(0.5, 0.5),
                components: [
                    timerBehavior
                ]
            });

            Utils.repeat(4, addPosition);
            Utils.repeat(4, addBall);


            // spawn spinner
            Bento.saveState.save('firstTimeOrder', true);

            text.setText(Utils.Localization.getText('minigameOrder'));

            // fade
            new Tween({
                from: 1,
                to: 0,
                'in': timeToComplete,
                ease: 'exponential',
                ignoreGameSpeed: true,
                updateWhenPaused: Utils.PauseLevels.MINIGAME,
                beta: -20,
                'do': function (v, t) {
                    entity.alpha = (v);
                }
            });

            return entity;
        };
        // var base = new Base(settings, 'Tap!', makeTurn);
        // var endOfTurn = base.endOfTurn;

        return makeTurn;
    };
});