/**
 * Spin minigame
 */
bento.define('entities/battle/minigames/spin', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'entities/effect',
    'entities/battle/minigames/base'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    ClickButton,
    Counter,
    Text,
    EventSystem,
    Utils,
    Tween,
    Effect,
    Base
) {
    'use strict';
    var PI = Math.PI;
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
            rarity: Number, critter rarity
            onHit: Function,
            onComplete: Function(Number success, Number max) // minigame is completed
            critter: frontCritter,
            manager: entity,
            enemy: frontEnemy,
            attack: Function
        }*/
        var viewport = Bento.getViewport();
        var firstTime = !Bento.saveState.load('firstTimeSpin', false) || window.easyMinigames; // first time playing this you get infinite time
        var timeToComplete = firstTime ? Infinity : Math.max(180 - settings.rarity * 5, 120);
        var makeTurn = function (endOfTurn, text) {
            var timer = timeToComplete;
            var ended = false;
            var timerBehavior = {
                start: function () {
                    worldPosition = entity.getWorldPosition();
                    new Tween({
                        from: 1,
                        to: 0,
                        'in': timeToComplete,
                        ease: 'exponential',
                        ignoreGameSpeed: true,
                        updateWhenPaused: Utils.PauseLevels.MINIGAME,
                        beta: -20,
                        'do': function (v, t) {
                            entity.alpha = (v);
                        },
                        onComplete: function () {

                        }
                    });
                },
                update: function () {
                    if (timer > 0) {
                        timer -= 1;
                    } else if (timer === 0) {
                        // didn't complete swipe
                        Bento.audio.playSound('sfx_battle_fail');
                        endOfTurn(false);
                    }
                }
            };
            var pointerId = null;
            var oldPosition = new Vector2(0, 0);
            var oldAngle = 0;
            var worldPosition;
            var targetAngle = 0;
            var clickable = new Clickable({
                pointerDown: function (evt) {
                    if (pointerId === null) {
                        pointerId = evt.id;
                        oldAngle = worldPosition.angleBetween(evt.worldPosition);
                    }
                },
                pointerMove: function (evt) {
                    var diff;
                    var angle;
                    if (pointerId !== evt.id) {
                        return;
                    }
                    angle = worldPosition.angleBetween(evt.worldPosition);
                    diff = angle - oldAngle;

                    // angles changed sign?
                    if (Utils.sign(angle) !== Utils.sign(oldAngle)) {
                        if (!Utils.isBetween(-PI / 2, angle, PI / 2) && !Utils.isBetween(-PI / 2, oldAngle, PI / 2)) {
                            diff = (2 * PI - Math.abs(diff)) * Utils.sign(oldAngle);
                        }
                    }
                    // add to target
                    targetAngle += diff;
                    entity.rotation += diff;

                    oldAngle = angle;

                    // console.log((targetAngle / PI / 2).toFixed(1))
                    if (targetAngle < 0) {
                        targetAngle = 0;
                    }
                    // spin 5 times?
                    if (targetAngle > PI * 2 * 5) {
                        Bento.audio.stopSound('sfx_battle_correct');
                        Bento.audio.playSound('sfx_battle_correct');
                        endOfTurn(true);
                    }
                },
                pointerUp: function (evt) {
                    if (pointerId === evt.id) {
                        pointerId = null;
                    }
                }
            });
            var sprite = new Sprite({
                imageName: 'minigames/spin/spinner'
            });
            var entity = new Entity({
                name: 'spinner',
                position: new Vector2(0, 0),
                originRelative: new Vector2(0.5, 0.5),
                updateWhenPaused: Utils.PauseLevels.MINIGAME,
                components: [
                    sprite,
                    clickable,
                    timerBehavior
                ]
            });


            // spawn spinner
            Bento.saveState.save('firstTimeSpin', true);
            text.setText(Utils.Localization.getText('minigameSpin'));

            return entity;
        };
        // var base = new Base(settings, 'Spin 5x!', makeTurn);
        // var endOfTurn = base.endOfTurn;

        return makeTurn;
    };
});