/**
 * Swipe minigame
 */
bento.define('entities/battle/minigames/swipes', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'entities/effect',
    'entities/battle/minigames/base'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    ClickButton,
    Counter,
    Text,
    EventSystem,
    Utils,
    Tween,
    Effect,
    Base
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
            rarity: Number, critter rarity
            onHit: Function,
            onComplete: Function(Number success, Number max) // minigame is completed
            critter: frontCritter,
            manager: entity,
            enemy: frontEnemy,
            attack: Function
        }*/
        var viewport = Bento.getViewport();
        var firstTime = !Bento.saveState.load('firstTimeSwipes', false) || window.easyMinigames;
        // difficulty: 25ms is the hardest
        var timeToComplete = firstTime ? Infinity : Math.max(120 - settings.rarity * 10, 80);
        var SpeedComponent = function () {
            return {
                update: function (data) {
                    data.speed = 1;
                },
                attached: function (data) {
                    data.entity.moveComponentTo(this, 0);
                }
            };
        };
        var makeTurn = function (endOfTurn, text) {
            // timer behavior that ends the turn if player takes too long
            var timer = timeToComplete;
            var timerBehavior = {
                update: function () {
                    if (timer > 0) {
                        timer -= 1;
                    } else if (timer === 0) {
                        // didn't complete swipe
                        Bento.audio.playSound('sfx_battle_fail');
                        endOfTurn(false);
                    }
                }
            };
            // the clickable decides whether the swipe was correct
            var clickable = new Clickable({
                pointerDown: function () {},
                pointerUp: function (evt) {
                    // detect turns
                    // evt.diffPosition
                    if (evt.diffPosition.magnitude() < 5) {
                        return;
                    }
                    var diff = Math.PI / 8;
                    var target;
                    var targetMin;
                    var targetPlus;
                    var angle = evt.diffPosition.angle();
                    var angleMin = angle - 2 * Math.PI;
                    var anglePlus = angle + 2 * Math.PI;
                    var effect;

                    target = arrow.rotation;
                    targetMin = target - 2 * Math.PI;
                    targetPlus = target + 2 * Math.PI;
                    effect = new Effect({
                        z: Utils.Layers.MINIGAME + 1,
                        sprite: new Sprite(Bento.assets.getJson('animations/slash')),
                        updateWhenPaused: Utils.PauseLevels.MINIGAME,
                        originRelative: new Vector2(0.5, 0.5)
                    });
                    effect.rotation = angle + Math.PI / 4 + Math.PI;
                    effect.attach(new SpeedComponent());
                    // console.log(angle.toFixed(2) + '/' + (target - diff).toFixed(2) + ',' + (target + diff).toFixed(2))
                    // console.log(angleMin.toFixed(2) + '/' + (targetMin - diff).toFixed(2) + ',' + (targetMin + diff).toFixed(2))
                    // console.log(anglePlus.toFixed(2) + '/' + (targetPlus - diff).toFixed(2) + ',' + (targetPlus + diff).toFixed(2))
                    if (
                        Utils.isBetween(target - diff, angle, target + diff) ||
                        Utils.isBetween(targetMin - diff, angle, targetMin + diff) ||
                        Utils.isBetween(targetMin - diff, angleMin, targetMin + diff) ||
                        Utils.isBetween(targetPlus - diff, angle, targetPlus + diff) ||
                        Utils.isBetween(targetPlus - diff, anglePlus, targetPlus + diff)
                    ) {
                        Bento.audio.playSound('sfx_battle_correct');
                        endOfTurn(true);
                        Bento.saveState.save('firstTimeSwipes', true);
                    } else {
                        Bento.audio.playSound('sfx_battle_fail');
                        endOfTurn(false);
                    }
                }
            });
            var arrowSprite = new Sprite({
                imageName: 'gui/arrowswipe'
            });
            var arrow = new Entity({
                position: new Vector2(0, 0),
                originRelative: new Vector2(0.5, 0.5),
                updateWhenPaused: Utils.PauseLevels.MINIGAME,
                components: [
                    arrowSprite,
                    clickable,
                    timerBehavior
                ]
            });
            arrow.scale.x = 2;
            arrow.scale.y = 2;
            // set random angle
            // arrow.rotation = Utils.getRandomRangeFloat(-Math.PI, Math.PI);
            arrow.rotation = Math.PI * 2 / 8 * Utils.getRandom(8);

            new Tween({
                from: 1,
                to: 0,
                'in': timeToComplete,
                ease: 'exponential',
                ignoreGameSpeed: true,
                updateWhenPaused: Utils.PauseLevels.MINIGAME,
                beta: -20,
                'do': function (v, t) {
                    arrow.alpha = (v);
                },
                onComplete: function () {

                }
            });
            text.setText(Utils.Localization.getText('minigameSwipe'));
            
            return arrow;
        };
        // var base = new Base(settings, 'Swipe!', makeTurn);
        // var endOfTurn = base.endOfTurn;
        return makeTurn;
    };
});