/**
 * Trace minigame
 */
bento.define('entities/battle/minigames/trace', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'bento/packedimage',
    'entities/effect',
    'entities/battle/minigames/base'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    ClickButton,
    Counter,
    Text,
    EventSystem,
    Utils,
    Tween,
    PackedImage,
    Effect,
    Base
) {
    'use strict';
    var polylines;
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
            rarity: Number, critter rarity
            onHit: Function,
            onComplete: Function(Number success, Number max) // minigame is completed
            critter: frontCritter,
            manager: entity,
            enemy: frontEnemy,
            attack: Function
        }*/
        var viewport = Bento.getViewport();
        var firstTime = !Bento.saveState.load('firstTimeTrace', false) || window.easyMinigames;
        // difficulty: 25ms is the hardest
        var timeToComplete = firstTime ? Infinity : Math.max(240 - settings.rarity * 10, 120);
        var SpeedComponent = function () {
            return {
                update: function (data) {
                    data.speed = 1;
                },
                attached: function (data) {
                    data.entity.moveComponentTo(this, 0);
                }
            };
        };
        var makeTurn = function (endOfTurn, text) {
            // timer behavior that ends the turn if player takes too long
            var timer = timeToComplete;
            var timerBehavior = {
                start: function () {
                    Bento.objects.attach(pointer);
                    new Tween({
                        from: 1,
                        to: 0,
                        'in': timeToComplete,
                        ease: 'exponential',
                        ignoreGameSpeed: true,
                        updateWhenPaused: Utils.PauseLevels.MINIGAME,
                        beta: -20,
                        'do': function (v, t) {
                            entity.alpha = (v);
                        },
                        onComplete: function () {

                        }
                    });
                },
                destroy: function () {
                    // console.log('clean')
                    Bento.objects.remove(pointer);
                    Utils.forEach(triggers, function (trigger) {
                        Bento.objects.remove(trigger);
                    });
                },
                update: function () {
                    if (timer > 0) {
                        timer -= 1;
                    } else if (timer === 0) {
                        // didn't complete swipe
                        Bento.audio.playSound('sfx_battle_fail');
                        endOfTurn(false);
                    }
                }
            };
            var canvas = document.createElement('canvas');
            var context = canvas.getContext('2d');
            // using the clickable to draw on the canvas
            var pointerId = null;
            var oldPosition = new Vector2(0, 0);
            var triggers = [];
            var triggerId = 0;
            var touchedAll = false;
            var forward = false;
            var pointerBehavior = {
                touched: 0,
                update: function () {
                    pointer.collidesWith({
                        family: 'triggers',
                        firstOnly: false,
                        onCollide: function (other) {
                            var touched = pointerBehavior.touched;
                            // forward or backwards?
                            if (touched === 0) {
                                if (other.triggerId === 0) {
                                    forward = true;
                                } else if (other.triggerId === triggerId - 1) {
                                    forward = false;
                                }
                            }
                            if (forward && other.triggerId === touched) {
                                pointerBehavior.touched += 1;
                                touched = pointerBehavior.touched;
                                // console.log(pointerBehavior.touched + '/' + triggerId);
                                // Bento.objects.remove(other);
                                if (touched === triggerId) {
                                    touchedAll = true;
                                    // console.log('touched all');
                                    Bento.audio.playSound('sfx_battle_correct');
                                    endOfTurn(true);
                                }
                            } else if (!forward && other.triggerId - triggerId + 1 === -touched) {
                                pointerBehavior.touched += 1;
                                touched = pointerBehavior.touched;

                                if (touched === triggerId) {
                                    touchedAll = true;
                                    // console.log('touched all');
                                    Bento.audio.playSound('sfx_battle_correct');
                                    endOfTurn(true);
                                }
                            }
                        }
                    });
                }
            };
            var pointer = new Entity({
                z: Utils.Layers.MINIGAME,
                name: 'pointer',
                position: new Vector2(0, 0),
                originRelative: new Vector2(0.5, 0.5),
                boundingBox: new Rectangle(-8, -8, 16, 16),
                updateWhenPaused: Utils.PauseLevels.MINIGAME,
                components: [
                    // new Utils.Debug({}),
                    new Sprite({
                        // image: Bento.assets.getImage('battle/card')
                    }),
                    pointerBehavior
                ]
            });
            var clickable = new Clickable({
                pointerDown: function (evt) {
                    if (pointerId === null) {
                        pointerId = evt.id;
                        oldPosition = evt.position.clone();
                        context.clearRect(0, 0, canvas.width, canvas.height);
                        if (canvas.texture) {
                            canvas.texture = null;
                        }
                        setupLine();
                        context.strokeStyle = 'white';
                        context.lineWidth = 3;
                    }
                },
                pointerMove: function (evt) {
                    var diff, magnitude, steps, stepSize, i;
                    if (pointerId !== evt.id) {
                        return;
                    }
                    context.beginPath();
                    context.moveTo(oldPosition.x, oldPosition.y);
                    context.lineTo(evt.position.x, evt.position.y);
                    context.stroke();

                    // collision checking
                    diff = evt.position.subtract(oldPosition);
                    magnitude = diff.magnitude();
                    if (magnitude > 16) {
                        steps = Math.floor(magnitude / 8);
                        stepSize = diff.scalarMultiply(1 / magnitude);
                        for (i = 0; i < steps; ++i) {
                            pointer.position.add(stepSize);
                            pointerBehavior.update();
                        }
                    }

                    oldPosition = evt.position.clone();
                    pointer.position = evt.position.clone();
                },
                pointerUp: function (evt) {
                    if (pointerId === evt.id) {
                        pointerId = null;
                        pointerBehavior.touched = 0;
                    }

                }
            });
            var entity;
            var packedImage;
            var sprite;
            var polyline = polylines[Utils.getRandom(polylines.length)];
            var setupLine = function () {
                // pick one
                var spawn = triggers.length === 0;
                var corner = new Vector2(viewport.width / 2 - 80, viewport.height / 2 - 80);
                var addTrigger = function (position) {
                    var trigger = new Entity({
                        z: Utils.Layers.MINIGAME,
                        name: 'trigger',
                        family: ['triggers'],
                        position: position,
                        originRelative: new Vector2(0.5, 0.5),
                        updateWhenPaused: Utils.PauseLevels.MINIGAME,
                        boundingBox: new Rectangle(-16, -16, 32, 32),
                        components: [
                            // new Utils.Debug({}),
                            new Sprite({
                                // image: Bento.assets.getImage('battle/card')
                            })
                        ]
                    });
                    if (!spawn) {
                        return;
                    }
                    trigger.extend({
                        triggerId: triggerId
                    });
                    triggerId += 1;
                    triggers.push(trigger);
                    Bento.objects.attach(trigger);
                };
                var drawStart = function (pos1) {
                    context.beginPath();
                    context.arc(pos1.x, pos1.y, 10, 0, 2 * Math.PI);
                    context.fill();
                };
                // draw it
                context.strokeStyle = '#cccccc';
                context.fillStyle = '#cccccc';
                context.lineWidth = 8;
                Utils.forEach(polyline, function (line, i, j, breakLoop, next) {
                    var posMiddle;
                    var pos1 = new Vector2(corner.x + line.x, corner.y + line.y);
                    var pos2;
                    var diff;
                    if (i === 0) {
                        drawStart(pos1);
                        context.beginPath();
                        context.moveTo(pos1.x, pos1.y);
                        addTrigger(pos1);
                    }
                    if (next) {
                        pos2 = new Vector2(corner.x + next.x, corner.y + next.y);
                        context.lineTo(pos2.x, pos2.y);
                        // get middle
                        diff = pos2.subtract(pos1);
                        posMiddle = pos1.add(diff.scalarMultiply(0.5));
                        addTrigger(posMiddle);
                        addTrigger(pos2);
                    }
                });
                context.stroke();
            };

            // set up canvas
            canvas.width = viewport.width;
            canvas.height = viewport.height;

            // pick a polyline and draw it
            setupLine();

            packedImage = new PackedImage(canvas);
            sprite = new Sprite({
                image: packedImage
            });

            // set up entity
            entity = new Entity({
                z: Utils.Layers.MINIGAME,
                name: 'trace',
                position: new Vector2(-viewport.width / 2, -viewport.height / 2),
                originRelative: new Vector2(0, 0),
                updateWhenPaused: Utils.PauseLevels.MINIGAME,
                components: [
                    sprite,
                    clickable,
                    timerBehavior
                ]
            });
            Bento.saveState.save('firstTimeTrace', true);
            text.setText(Utils.Localization.getText('minigameTrace'));

            return entity;
        };
        // var base = new Base(settings, 'Trace!', makeTurn);
        // var endOfTurn = base.endOfTurn;
        var loadPolylines = function () {
            var tiled = Bento.assets.getJson('maps/traces');
            var objects = tiled.layers[0].objects;
            polylines = [];
            Utils.forEach(objects, function (object) {
                // add x and y to each point in the polyline
                if (!object.absolute) {
                    Utils.forEach(object.polyline, function (point) {
                        point.x += object.x;
                        point.y += object.y;
                    });
                    object.absolute = true;
                }
                // add to polylines
                polylines.push(object.polyline);
            });
        };

        // load polylines
        if (!polylines) {
            loadPolylines();
        }

        return makeTurn;
    };
});