bento.define('entities/camera', [
    'bento',
    'bento/math/vector2',
    'bento/entity',
    'utils'
], function (Bento, Vector2, Entity, Utils) {
    'use strict';
    return function () {
        var target = new Vector2(0, 0),
            ignoreEdge = false,
            viewport = Bento.getViewport(),
            shakePos = new Vector2(0, 0),
            shakeTimer = 0,
            shakeDuration = 0,
            shakeStrength = 0,
            shake = function () {
                if (shakeTimer > 0) {
                    shakeTimer -= 1;
                    if (~~(shakeTimer / 1) % 2) {
                        shakePos.x = shakeStrength * (Math.random() - 0.5) * shakeTimer / shakeDuration;
                        shakePos.y = shakeStrength * (Math.random() - 0.5) * shakeTimer / shakeDuration;
                    }
                } else {
                    shakePos.x = 0;
                    shakePos.y = 0;
                }
            },
            edge = null,
            camera = new Entity({
                z: Utils.Layers.CAMERA,
                name: 'camera',
                family: ['camera']
            }).extend({
                speed: new Vector2(1, 1),
                moveToTarget: true,
                centerTo: function (pos) {
                    var position = camera.position,
                        currentScreen = Bento.screens.getCurrentScreen(),
                        screenDimension;
                    if (currentScreen) {
                        screenDimension = currentScreen.dimension;
                    } else {
                        return;
                    }

                    if (edge) {
                        screenDimension = edge;
                        edge = null;
                    }

                    target.x = pos.x - viewport.width / 2;
                    target.y = pos.y - viewport.height / 2;

                    // cant place target outside screen dimension?
                    if (!ignoreEdge) {
                        target.x = Math.min(Math.max(target.x, screenDimension.x), screenDimension.getX2() - viewport.width);
                        target.y = Math.min(Math.max(target.y, screenDimension.y), screenDimension.getY2() - viewport.height);
                    } else {
                        ignoreEdge = false;
                    }
                },
                setSpeed: function (v) {
                    camera.speed = v;
                },
                forceTo: function (pos) {
                    var position = camera.position;
                    this.centerTo(pos);
                    position.x = target.x;
                    position.y = target.y;
                },
                shake: function (duration, strength) {
                    shakeTimer = duration;
                    shakeDuration = duration;
                    shakeStrength = strength;
                },
                ignoreEdge: function () {
                    // ignores edge once, sets itself to false afterwards
                    ignoreEdge = true;
                },
                setEdge: function (rectangle) {
                    var diff;
                    // sets new edge once, unsets itself afterwards
                    edge = rectangle;

                    // smaller than viewport?
                    if (edge.width < viewport.width) {
                        diff = viewport.width - edge.width;
                        edge.x -= diff / 2;
                        edge.width += diff;
                    }
                    if (edge.height < viewport.height) {
                        diff = viewport.height - edge.height;
                        edge.y -= diff / 2;
                        edge.height += diff;
                    }
                }
            }).attach({
                update: function (data) {
                    var position = camera.position,
                        currentScreen = Bento.screens.getCurrentScreen(),
                        screenDimension;
                    if (currentScreen) {
                        screenDimension = currentScreen.dimension;
                    } else {
                        return;
                    }
                    shake();
                    // move towards target
                    if (camera.moveToTarget) {
                        position.x += (target.x - position.x) * camera.speed.x;
                        position.y += (target.y - position.y) * camera.speed.y;
                    }
                    // set to scroll
                    viewport.x = position.x + shakePos.x;
                    viewport.y = position.y + shakePos.y;
                }
            });

        return camera;
    };
});