/**
 * Card with critter image inside
 */
bento.define('entities/card', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'bento/objectpool',
    'bento/packedimage',
    'bento/gui/counter',
    'gui/horizontalbar',
    'gui/hpbar',
    'components/card3dtouch'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    Utils,
    Tween,
    ObjectPool,
    PackedImage,
    Counter,
    HorizontalBar,
    HpBar,
    Card3DTouch
) {
    'use strict';
    var canvasPool = new ObjectPool({
        poolSize: 1,
        constructor: function () {
            var canvas = document.createElement('canvas');
            canvas.width = 42;
            canvas.height = 37;
            return canvas;
        },
        destructor: function (obj) {
            // clear canvas
            var context = obj.getContext('2d');
            context.clearRect(0, 0, obj.width, obj.height);
            // clear texture
            if (obj.texture) {
                obj.texture = null;
            }
            return obj;
        }
    });
    return function (properties, settings) {
        /*properties = {
            id: String // critter id
        }*/
        var viewport = Bento.getViewport();
        var sprite = new Sprite({
            imageName: 'battle/emptycard'
        });
        var entity = new Entity({
            name: 'card',
            position: new Vector2(0, 0),
            originRelative: new Vector2(0.5, 0.5),
            components: [
                sprite
            ]
        });
        var spriteCritter;
        var entityCritter;
        var hpBar = new HpBar(properties || {});
        var makeCounter = function (settings) {
            var counter = new Counter({
                name: settings.name || 'counter',
                image: Bento.assets.getImage(settings.imageName || 'gui/numberscard'),
                frameWidth: 8,
                frameHeight: 8,
                padding: settings.padding,
                spacing: settings.spacing || new Vector2(-3, 0),
                position: settings.position || new Vector2(0, 0),
                align: settings.align,
                animations: {
                    '0': {
                        frames: [0]
                    },
                    '1': {
                        frames: [1]
                    },
                    '2': {
                        frames: [2]
                    },
                    '3': {
                        frames: [3]
                    },
                    '4': {
                        frames: [4]
                    },
                    '5': {
                        frames: [5]
                    },
                    '6': {
                        frames: [6]
                    },
                    '7': {
                        frames: [7]
                    },
                    '8': {
                        frames: [8]
                    },
                    '9': {
                        frames: [9]
                    },
                    '-': {
                        frames: [10]
                    },
                    'A': {
                        frames: [11]
                    },
                    'H': {
                        frames: [12]
                    },
                    'S': {
                        frames: [13]
                    },
                    '?': {
                        frames: [16]
                    }
                }
            });
            counter.setValue(settings.value);
            entity.attach(counter);
        };
        var makeHiddenSprite = function () {
            var canvas = canvasPool.get();
            var context = canvas.getContext('2d');
            var critterImage = Bento.assets.getImage('selection/creatures/' + properties.id);
            var black = Bento.assets.getImage('selection/creatures/black');
            var question = Bento.assets.getImage('selection/creatures/questionmark');
            // draw critter
            context.drawImage(
                critterImage.image,
                critterImage.x,
                critterImage.y,
                critterImage.width,
                critterImage.height,
                0,
                0,
                critterImage.width,
                critterImage.height
            );
            // draw black overlay
            context.globalCompositeOperation = 'source-in';
            context.drawImage(
                black.image,
                black.x,
                black.y,
                black.width,
                black.height,
                0,
                0,
                black.width,
                black.height
            );
            // draw questionmark
            context.globalCompositeOperation = 'source-over';
            // context.drawImage(
            //     question.image,
            //     question.x,
            //     question.y,
            //     question.width,
            //     question.height,
            //     0,
            //     0,
            //     question.width,
            //     question.height
            // );
            return new Sprite({
                image: new PackedImage(canvas)
            });
        };
        var loadCritter = function () {
            var image;
            if (properties === null) {
                return;
            }
            // load critter
            if (properties.id) {
                if (settings.hidden) {
                    spriteCritter = makeHiddenSprite();
                    entityCritter = new Entity({
                        name: 'avatar',
                        position: new Vector2(-1, -5),
                        originRelative: new Vector2(0.5, 0.5),
                        components: [spriteCritter]
                    });
                    entity.attach(entityCritter);
                } else {
                    // this is for loading twitter cards
                    if (settings.image) {
                        // due to asynchronous nature, we set the sprite to an empty sprite
                        // and setup the sprite later
                        spriteCritter = new Sprite({});
                        image = new Image();
                        image.src = settings.image;
                        image.onload = function () {
                            spriteCritter.setup({
                                image: new PackedImage(image)
                            });

                            // should be 42 x 37
                            entityCritter.scale.x = 42 / image.width;
                            entityCritter.scale.y = 37 / image.height;
                            entityCritter.setOriginRelative(new Vector2(0.5, 0.5));
                        }
                    } else {
                        // load critter image from id
                        spriteCritter = new Sprite({
                            imageName: 'selection/creatures/' + properties.id
                        });
                    }
                    entityCritter = new Entity({
                        name: 'avatar',
                        position: new Vector2(-1, -5),
                        originRelative: new Vector2(0.5, 0.5),
                        components: [
                            spriteCritter
                        ]
                    });

                    entity.attach(entityCritter);
                }

                // hp
                makeCounter({
                    name: 'currentHp',
                    position: new Vector2(-24, 15),
                    align: 'left',
                    imageName: properties.hp > 999 ? 'gui/numberscardthin' : null,
                    spacing: properties.hp > 999 ? new Vector2(-4, 0) : null,
                    value: 'H' + Math.max(properties.currentHp, 0)
                });
                // attack
                makeCounter({
                    name: 'attack',
                    position: new Vector2(-2 + (properties.hp > 999 ? 2 : 0), 15),
                    imageName: properties.hp > 999 ? 'gui/numberscardthin' : null,
                    spacing: properties.hp > 999 ? new Vector2(-4, 0) : null,
                    align: 'left',
                    value: 'A' + properties.attack
                });

                // rank
            }

            entity.attach(hpBar);
            makeCounter({
                name: 'starCounter',
                imageName: 'gui/numberswhiteoutline',
                align: 'left',
                padding: 1,
                position: new Vector2(-22, -22 - 1),
                value: 'S' + (properties.rarity || 0)
            });
        };
        var addElement = function () {
            var container = new Entity({
                name: 'elementContainer',
                position: new Vector2(15, -18),
                originRelative: new Vector2(0.5, 0.5),
                components: [
                    new Sprite({
                        imageName: "elements/cardcorner"
                    })
                ]
            });
            var element;

            if (properties && properties.element) {
                element = new Entity({
                    name: 'element',
                    position: new Vector2(-1, 0),
                    originRelative: new Vector2(0.5, 0.5),
                    family: [''],
                    components: [
                        new Sprite({
                            imageName: 'elements/' + properties.element
                        })
                    ]
                });
                container.attach(element);

                // attach to card
                entity.attach(container);
            }
        };
        var touch;

        settings = settings || {};

        loadCritter();
        addElement();

        // hp checker
        // if (properties.currentHp > 999) {
        //     var attackCounter = entity.getComponent('attack');
        //     attackCounter.setValue('A?');
        //     // attackCounter.position.x += 8;
        // }

        // set hp
        entity.extend({
            getAvatar: function () {
                return entityCritter;
            },
            setHp: function (value) {
                hpBar.setValue(value);
                entity.getComponent('currentHp', function (counter) {
                    counter.setValue('H' + Math.max(0, properties.currentHp));
                });
            },
            addHp: function (value) {
                hpBar.addValue(value);
                entity.getComponent('currentHp', function (counter) {
                    counter.setValue('H' + Math.max(0, properties.currentHp));
                });
            },
            getId: function () {
                return properties.id;
            },
            getProperties: function () {
                return properties;
            },
            getProperty: function (key) {
                if (!properties) {
                    return undefined;
                }
                // TODO: a property might not exist in the clone
                // because it was added in an update
                // might need to check in the json also!
                return properties[key];
            },
            isHidden: function () {
                return settings.hidden;
            },
            on3DTouch: function (callback) {
                touch.on3DTouch = callback;
            }
        });
        
        touch = new Card3DTouch({});
        entity.attach(touch);

        return entity;
    };
});