/**
 * Grumpy Cat Updated Crosspromo with Road Warriors
 */
bento.define('entities/crosspromo', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/components/fill',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/utils',
    'bento/tween',
    'modules/crosspromo/inappstore',
    'bento/packedimage'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Fill,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    InAppStore,
    PackedImage
) {
    'use strict';
    var gameId = 'combocritters';
    var downloadComplete = false;
    var downloadedCrossPromoImage;
    var downloadCrossPromo = function () {
        // step 1: get all available games
        var getGames = function (onComplete) {
            var xhr = new window.XMLHttpRequest();
            var cleanUp = function (json) {
                // clean up received json
                // split it into the available variables
                var games = json.games;
                var waterFalls = json.waterFalls;
                var game;

                // get the waterfall for this game
                promoWaterfallList = waterFalls[gameId] || waterFalls['default'];

                // fill in the store links
                for (game in games) {
                    if (!games.hasOwnProperty(game)) {
                        continue;
                    }
                    // new data
                    storeLinks[game] = storeLinks[game] || {};
                    storeLinks[game].bundleId = games[game].bundleId;
                    storeLinks[game].urlScheme = games[game].urlScheme;
                    if (Utils.isNativeAndroid()) {
                        // on Android, the bundle id is used as urlScheme
                        storeLinks[game].urlScheme = storeLinks[game].bundleId;
                    }

                    // legacy stuff
                    storeLinks[game].iosId = games[game].iosId;
                    // construct links
                    storeLinks[game].ios = 'https://itunes.apple.com/app/apple-store/id' + games[game].iosId;
                    storeLinks[game].android = 'https://play.google.com/store/apps/details?id=' + games[game].bundleId;

                    // save each game in the array
                    promoApps.push(game);
                }

                onComplete(json);
            };

            xhr.open("GET", 'http://d3q8vwl9xlq0g.cloudfront.net/crosspromo/crosspromo.json');
            xhr.onreadystatechange = function () {
                var response;
                if (xhr.readyState === 4) {
                    if (xhr.status >= 200 && xhr.status < 300) {
                        try {
                            response = JSON.parse(xhr.responseText);
                            cleanUp(response);
                        } catch (e) {
                            // failed to parse: do nothing
                            console.error(e);
                        }
                    } else {
                        // wrong server response
                        console.error('Bad server response: ' + xhr.responseText);
                    }
                }
            };
            xhr.send();
        };
        // step 2: filter all games that were already downloaded: this requires the appAvailability plugin!
        var filterGames = function (onComplete) {
            var game;
            var i;
            var done = 0;
            // we clone the promoApps array because it may change during the loop, 
            // if checkAvailability is fast enough
            var availableGames = promoApps.concat([]);
            var checkAppAvailability = function (gameStr) {
                var urlScheme = storeLinks[gameStr].urlScheme;
                // use the plugin
                window.appAvailability.check(
                    urlScheme,
                    function () {
                        // Success callback
                        removeChoice(gameStr);
                        done += 1;
                        checkDone();
                    },
                    function () {
                        done += 1;
                        checkDone();
                    }
                );
            };
            var checkDone = function () {
                if (done === availableGames.length) {
                    onComplete();
                    // prevent onComplete from being called twice by redefining it as an empty function
                    onComplete = function () {};
                }
            };
            // filter self first
            removeChoice(gameId);

            if (!window.appAvailability) {
                // plugin does not exist
                onComplete();
                return;
            }

            // use the url schemes to check if a game is already downloaded
            for (i = 0; i < availableGames.length; ++i) {
                game = availableGames[i];
                if (storeLinks[game]) {
                    checkAppAvailability(game);
                }
            }
        };
        // step 3: decide on game
        var decideGame = function (onComplete) {
            var index;
            if (promoApps.length === 0) {
                // user already installed all apps!
                console.log('Crosspromo: no more apps to show.');

                // crosspromo ends here
                return;
            }

            // loop through the waterfall
            index = timesPromoShown % promoWaterfallList.length;
            crossPromoApp = promoWaterfallList[index];

            if (Utils.isNativeIos()) {
                // preload the InAppStore module
                preload();
            }

            // done
            onComplete();
        };
        // step 4: download the appropiate image
        var downloadImage = function (onComplete) {
            //get the current available languages and check to see if the current language is available
            var currentLanguage = Utils.Localization.getLanguage();
            var languageCode = availableLanguages[availableLanguages.indexOf(currentLanguage)] || 'en';
            var link = 'http://d3q8vwl9xlq0g.cloudfront.net/crosspromo/' + crossPromoApp + '/' + languageCode + '.png';
            var isCached = false;
            var cacheImage = function () {
                var canvas = document.createElement('canvas');
                var context = canvas.getContext('2d');

                canvas.width = downloadedCrossPromoImage.width;
                canvas.height = downloadedCrossPromoImage.height;

                // draw into canvas to get the base64 string
                try {
                    context.drawImage(
                        downloadedCrossPromoImage,
                        0, 0, canvas.width, canvas.height,
                        0, 0, canvas.width, canvas.height
                    );

                    // we use localStorage instead of Bento.saveState so the images won't be backed up in cloud
                    localStorage.setItem(link, canvas.toDataURL());
                } catch (e) {
                    // caching can fail on web because toDataURL is not allowed on "tainted" canvasses
                    console.log('Caching image failed');
                }

            };

            // get cache if possible
            var cache = localStorage.getItem(link);
            if (cache) {
                isCached = true;
                link = cache;
            }

            downloadedCrossPromoImage = new Image();
            downloadedCrossPromoImage.src = link;
            downloadedCrossPromoImage.onload = function () {
                downloadComplete = true;
                canShow = true;

                // cache image
                if (!isCached) {
                    cacheImage();
                }

                if (onComplete) {
                    onComplete();
                }
            };
        };

        canShow = false;
        getGames(function () {
            filterGames(function () {
                decideGame(function () {
                    downloadImage(function () {
                        console.log('Crosspromo download complete: ' + crossPromoApp);
                    });
                });
            });
        });
    };

    var timesPromoShown = 0;
    timesPromoShown = Bento.saveState.load('timesPromoShown', timesPromoShown);

    var promoApps = [];
    var promoWaterfallList = [];

    var referrer = {
        ios: '?pt=666586&ct=combocritters&mt=8',
        campaignToken: 'combocritters',
        providerToken: '666586',

        android: '&referrer=utm_source%3Dcombocritters%26utm_medium%3Dcrosspromo%26utm_campaign%3Dcrosspromo'
    };
    var storeLinks = {};
    var checkInstalled = function () {
        // has already installed?
        if (Utils.isNativeIos()) {
            if (Utils.canOpenUrl('lk01')) {
                removeChoice('nomcat');
            }
            if (Utils.canOpenUrl('lk02')) {
                removeChoice('skychasers');
            }
            if (Utils.canOpenUrl('lk03')) {
                removeChoice('combocritters');
            }
            if (Utils.canOpenUrl('lk04')) {
                removeChoice('grumpycat');
            }
            if (Utils.canOpenUrl('roadwarriors')) {
                removeChoice('roadwarriors');
            }
            if (Utils.canOpenUrl('lk05')) {
                removeChoice('beatstreet');
            }
            // not async
            preloadAll();
        } else if (Utils.isNativeAndroid() && window.appAvailability) {
            window.appAvailability.check(
                'com.heigames.nomcat', // URI Scheme
                function () { // Success callback
                    removeChoice('nomcat');
                }
            );
            window.appAvailability.check(
                'com.luckykat.skychasers', // URI Scheme
                function () { // Success callback
                    removeChoice('skychasers');
                }
            );
            window.appAvailability.check(
                'com.luckykat.combocritters', // URI Scheme
                function () { // Success callback
                    removeChoice('combocritters');
                }
            );
            window.appAvailability.check(
                'com.luckykat.grumpycat', // URI Scheme
                function () { // Success callback
                    removeChoice('grumpycat');
                }
            );
            window.appAvailability.check(
                'com.luckykat.roadwarriors', // URI Scheme
                function () { // Success callback
                    removeChoice('roadwarriors');
                }
            );
            window.appAvailability.check(
                'com.luckykat.beatstreet', // URI Scheme
                function () { // Success callback
                    removeChoice('beatstreet');
                }
            );
        } else {
            preloadAll();
        }
    };
    //
    var availableLanguages = ['en', 'ko', 'ja', 'zh', 'zt'];
    var width = Math.min(Bento.getViewport().width - 20, 160); // desired width of crosspromo image
    var isShowing = false;
    var canShow = false;

    var removeChoice = function (name) {
        var index = -1;
        console.log('Crosspromo: Already has ' + name);
        while (promoWaterfallList.indexOf(name) != -1) {
            index = promoWaterfallList.indexOf(name);
            if (index >= 0) {
                promoWaterfallList.splice(index, 1);
            } else {
                break;
            }
        }
        while (promoApps.indexOf(name) != -1) {
            index = promoApps.indexOf(name);
            if (index >= 0) {
                promoApps.splice(index, 1);
            } else {
                break;
            }
        }
    };

    var crossPromoApp = ''; // key string for storeLinks
    var hasLoadedStore = false;
    var failedInAppStore = false;
    var preload = function () {
        var isAtLeastIos9 = function () {
            var version;
            var os;
            if (!window.device || !window.device.version) {
                console.warn("LK_ERROR: Can't find window.device.version");
                return false;
            }
            version = window.device.version;
            os = parseInt(version.substring(0, version.indexOf('.')));

            return os > 8;
        };

        // newest version of in app store plugin (as of December 2016) crashes on iOS versions lower than 9
        if (isAtLeastIos9() === false) {
            console.log("LK_LOG: iOS version is lower than 9. Can't use InAppStore plugin.");
            failedInAppStore = true;
            return;
        }

        InAppStore.open(storeLinks[crossPromoApp].iosId, referrer.campaignToken, referrer.providerToken, function () {
            console.log('done loading');
            hasLoadedStore = true;
        }, function () {
            console.log('Could not load In App Store');
            failedInAppStore = true;
        });
    };
    var checked = 0;
    var total = promoApps.length + 1;
    var checkDone = function () {
        // wait until all apps are checked
        if (checked === total) {
            preloadAll();
        }
    };
    // done with all async checking
    var preloadAll = function () {
        // there are no choices left!
        if (promoWaterfallList.length === 0) {
            canShow = false;
            return;
        }

        // randomize crosspromo app if waterfall list is complete
        crossPromoApp = promoWaterfallList[timesPromoShown % promoWaterfallList.length];

        // load the crosspromo app asap
        preload();
    };

    downloadCrossPromo();
    // checkInstalled();

    var CrossPromo = function (settings) {
        /*settings = {
            z: Number
            pauseLevel: Number
        }*/
        var viewport = Bento.getViewport();

        var oldPauseLevel = Bento.objects.isPaused();
        var pauseLevel = settings.pauseLevel || 1;
        var z = settings.z || 1;

        var fill = new Fill({
            dimension: Utils.screenSize,
            color: [0, 0, 0, 0.5]
        });
        var buttonClose = new ClickButton({
            name: 'closeCrossPromo',
            imageName: 'crosspromo/buttonclose',
            frameCountX: 1,
            frameCountY: 2,
            position: new Vector2(viewport.width / 2 - width / 2 + 8, viewport.height / 2 - width / 2 - 10),
            updateWhenPaused: pauseLevel,
            onClick: function () {
                // this button does nothing actually
                // clicking outside the crosspromo will dismiss it
            }
        });
        
        //create the sprite for the crosspromo
        var sprite = new Sprite({});
        sprite.setup({
            image: downloadComplete ? new PackedImage(downloadedCrossPromoImage) : null
        });

        var isTweening = false;
        /**
         * Crosspromo image entity
         */
        var resizeCrossPromo = function () {
            var scale;
            scale = width / crossPromoContainer.dimension.width;
            crossPromoContainer.scale.x = scale;
            crossPromoContainer.scale.y = scale;
            scale = (width - 4) / crossPromoEntity.dimension.width / scale;
            crossPromoEntity.scale.x = scale;
            crossPromoEntity.scale.y = scale;
        };
        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {
                // resize crosspromo image
                resizeCrossPromo();

                // drop down
                isShowing = true;
                isTweening = true;

                new Tween({
                    from: crossPromoContainer.position.y,
                    to: viewport.height / 2,
                    'in': 60,
                    ease: 'elastic',
                    alpha: -8,
                    beta: 1,
                    updateWhenPaused: pauseLevel,
                    onUpdate: function (v, t) {
                        crossPromoContainer.position.y = v;
                    },
                    onComplete: function () {
                        isTweening = false;
                        entity.attach(buttonClose);
                    }
                });

                timesPromoShown++;
                Bento.saveState.save('timesPromoShown', timesPromoShown);

                if (Utils.Analytics) {
                    Utils.Analytics.trackView('crosspromo');
                    Utils.Analytics.event('Cross_Promo', 'viewed');
                }
                if (Utils.Firebase) {
                    Utils.Firebase.logEvent('crosspromo', {
                        type: 'shown'
                    });
                }
            }
        };
        var clickable = new Clickable({
            onClick: function (data) {
                if (isTweening) {
                    return;
                }
                // open crosspromo link
                if (Utils.isNativeAndroid()) {
                    window.Cocoon.App.openURL(storeLinks[crossPromoApp].android);
                } else if (Utils.isNativeIos()) {
                    if (failedInAppStore) {
                        window.Cocoon.App.openURL(storeLinks[crossPromoApp].ios);
                    } else {
                        InAppStore.show();

                        preload();
                    }
                }

                if (Utils.Analytics) {
                    Utils.Analytics.trackView('app_store');
                    Utils.Analytics.event('Cross_Promo_Static', 'clicked');
                }
                if (Utils.Firebase) {
                    Utils.Firebase.logEvent('crosspromo_static', {
                        value: 1,
                        type: 'clicked'
                    });
                }
            },
            onClickMiss: function (data) {
                // dismissed
                if (isTweening) {
                    return;
                }

                if (Utils.Analytics) {
                    Utils.Analytics.event('Cross_Promo_Static', 'ignored');
                }
                if (Utils.Firebase) {
                    Utils.Firebase.logEvent('crosspromo_static', {
                        type: 'ignored'
                    });
                }

                if (settings.sfx) {
                    Bento.audio.playSound(settings.sfx);
                }

                close();
            }
        });
        var crossPromoEntity = new Entity({
            name: 'crossPromoImage',
            position: new Vector2(0, 0),
            originRelative: new Vector2(0.5, 0.5),
            updateWhenPaused: settings.pauseLevel,
            components: [
                sprite,
                behavior,
                clickable
            ]
        });
        /**
         * Extra crosspromo container to show edge
         */
        var nineSlice = new Sprite({
            imageName: 'crosspromo/edge'
        });
        var crossPromoContainer = new Entity({
            name: 'crossPromoContainer',
            position: new Vector2(viewport.width / 2, -width),
            originRelative: new Vector2(0.5, 0.5),
            components: [
                nineSlice,
                crossPromoEntity
            ]
        });
        /**
         * Container
         */
        var entity = new Entity({
            z: z,
            name: 'crossPromo',
            position: new Vector2(0, 0),
            originRelative: new Vector2(0, 0),
            updateWhenPaused: settings.pauseLevel,
            components: [
                fill,
                crossPromoContainer
            ]
        });
        var close = function () {
            // tween out and remove
            entity.remove(buttonClose);
            isTweening = true;
            new Tween({
                from: crossPromoContainer.position.y,
                to: -width,
                'in': 30,
                ease: 'easeInBack',
                updateWhenPaused: pauseLevel,
                onUpdate: function (v, t) {
                    crossPromoContainer.position.y = v;
                },
                onComplete: function () {
                    Bento.objects.remove(entity);
                    // Bento.objects.pause(oldPauseLevel);
                    // not using pause for compatibility with old games
                    Bento.objects.resume();
                    isShowing = false;
                    // coming back from cross promo
                    Utils.Analytics.trackView('title');
                }
            });
        };

        // adds self
        if (canShow) {
            Bento.objects.pause(pauseLevel);
            Bento.objects.attach(entity);
        }

        return entity;
    };

    CrossPromo.isShowing = function () {
        return isShowing;
    };

    return CrossPromo;
});