/**
 * Class for small effect animations, like when enemies bump into each other.
 * Needs to manually be added to the object manager.
 *
 * Paramaters are passed as properties of the settings object.
 *
 * @param  {Sprite}  sprite         Required. Needs an animation called 'default'.
 * @param  {Boolean} removeWhenDone Optional, default: true. If true, removes the effect when the animation completes.
 * @param  {Vector2} position       Optional, default: middle of the screen.
 * @param  {Vector2} originRelative Optional, default: (0.5, 0.5).
 * @param  {String}  name           Optional, default: 'effect'
 * @param  {Function}onComplete     Optional
 * @return {Entity}
 */
bento.define('entities/effect', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    Utils,
    Tween
) {
    'use strict';
    return function (settings) {
        var viewport = Bento.getViewport(),
            screenMid = new Vector2(viewport.width / 2, viewport.height / 2),
            sprite = settings.sprite,
            removeWhenDone = Utils.isDefined(settings.removeWhenDone) ? settings.removeWhenDone : true,
            entity = new Entity({
                z: settings.z || Utils.Layers.EFFECTS,
                name: settings.name || 'effect',
                position: settings.position || screenMid.clone(),
                originRelative: settings.originRelative || new Vector2(0.5, 0.5),
                float: settings.float || false,
                updateWhenPaused: settings.updateWhenPaused || 0,
                family: ['effects'],
                components: [sprite],
                init: function () {
                    var self = this;
                    sprite.setAnimation('default', function () {
                        if (removeWhenDone)
                            Bento.objects.remove(self);
                        if (settings.onComplete) {
                            settings.onComplete();
                        }
                    });
                }
            });

        Bento.objects.attach(entity);

        return entity;
    };
});