/**
 * Planet object in map screen
 */
bento.define('entities/map/planet', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/gui/clickbutton',
    'bento/gui/text',
    'bento/canvas',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'entities/simplecard',
    'gui/centeredlist',
    'gui/fade',
    'gui/scrollinglist',
    'modules/playtimer'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    ClickButton,
    Text,
    Canvas,
    Entity,
    EventSystem,
    Utils,
    Tween,
    Card,
    CenteredList,
    Fade,
    ScrollingList,
    PlayTimer
) {
    'use strict';
    var PauseLevels = Utils.PauseLevels;
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var creatures = Bento.assets.getJson('creatures/creatures');
        var id = settings.tiled.tile.name;
        var buttonSettings = Bento.objects.get('menuButton');
        var showInfoPanel = function (planet) {
            var viewport = Bento.getViewport();
            var planetJson = Bento.assets.getJson('planets/' + planet.name);
            var darkSprite = new Sprite({
                imageName: 'fade'
            });
            var darkEntity = new Entity({
                z: 3,
                position: new Vector2(viewport.width / 2, viewport.height / 2),
                originRelative: new Vector2(0.5, 0.5),
                float: true,
                components: [darkSprite]
            });
            var canClose = false;
            var infoPanel = new Entity({
                z: 4,
                name: 'infoPanel',
                position: new Vector2(0, viewport.height / 2 + 32),
                originRelative: new Vector2(0.5, 1),
                updateWhenPaused: PauseLevels.INFO,
                float: true,
                components: [
                    new Sprite({
                        imageName: 'map/infopanel'
                    }),
                    new Clickable({
                        onClickMiss: function () {
                            canClose = true;
                        },
                        pointerUp: function (evt) {
                            if (!this.isHovering && canClose) {
                                buttonExit.doCallback();
                            } else {
                                canClose = false;
                            }
                        }
                    })
                ]
            });
            var planetIcon = new Entity({
                name: 'planet',
                position: new Vector2(26, -109),
                originRelative: new Vector2(0.5, 0.5),
                updateWhenPaused: PauseLevels.INFO,
                components: [
                    new Sprite({
                        imageName: planet.getComponent('sprite').animationSettings.imageName,
                        frameCountY: planet.getComponent('sprite').animationSettings.frameCountY
                    })
                ]
            });
            var enemyCritters = [];
            var planetInfo = new Entity({
                name: 'planetInfo',
                position: new Vector2(0, 0),
                originRelative: new Vector2(0.5, 0.5),
                components: [new Sprite({
                    imageName: 'map/planetinfo'
                })]
            });
            var planetText = new Text({
                position: new Vector2(-36, -116),
                text: Utils.Localization.getText('planetName-' + planet.name),
                font: 'volter',
                fontSize: 9,
                fontSettings: Utils.FontSettings.get('font'),
                fontColor: '#ffffff',
                align: 'center',
                textBaseline: 'bottom'
            });
            var buttonAttack = new ClickButton({
                sfx: 'sfx_transition',
                image: Bento.assets.getImage('map/buttonattack'),
                frameCountX: 1,
                frameCountY: 2,
                animations: {
                    up: {
                        frames: [0]
                    },
                    down: {
                        frames: [1]
                    }
                },
                position: new Vector2(-7, -18),
                updateWhenPaused: PauseLevels.INFO,
                onClick: function () {
                    // TEMP
                    Bento.pause(Utils.PauseLevels.ALWAYS);
                    // Bento.screens.show('screens/battle', {
                    //     planetName: planet.name,
                    //     enemyCritters: enemyCritters,
                    //     boss: planetJson.boss
                    // });
                    new Fade({
                        onFade: function (v, t) {
                            Bento.audio.setVolume(1 - v, 'bgm_title');
                        },
                        callback: function () {
                            Bento.resume();
                            Utils.worldMap.reset();
                            // Utils.worldMap.planetData = {
                            //     planetName: planet.name,
                            //     enemyCritters: enemyCritters,
                            //     boss: planetJson.boss,
                            //     reward: planetJson.reward,
                            //     rewardImage: planetJson.rewardImage,
                            //     unlocks: planetJson.unlocks,
                            //     bgm: planetJson.bgm || 'bgm_map1',
                            //     sfx: planetJson.sfx || 'sfx_beam_end_map1'
                            // };
                            Utils.worldMap.planetData = Utils.clone(planetJson);
                            Utils.worldMap.planetData.planetName = planet.name;
                            Utils.worldMap.planetData.enemyCritters = enemyCritters;

                            Bento.screens.show(
                                'screens/worldmap',
                                Utils.worldMap.planetData,
                                function () {
                                    // set time of world entry and save current planet to track playtime per planet
                                    Bento.saveState.save('prevTimeSample', PlayTimer.playTimer);
                                    Bento.saveState.save('currentPlanet', planet.name);
                                }
                            );

                        }
                    });
                }
            });
            var buttonExit = new ClickButton({
                z: 3,
                image: Bento.assets.getImage('map/buttonexit'),
                frameCountX: 2,
                frameCountY: 2,
                animations: {
                    up: {
                        frames: [0]
                    },
                    down: {
                        frames: [1]
                    },
                    inactive: {
                        frames: [2]
                    }
                },
                position: new Vector2(12, 10),
                float: true,
                updateWhenPaused: PauseLevels.INFO,
                onClick: function () {
                    Bento.remove(infoPanel);
                    Bento.objects.resume();
                    buttonSettings.visible = true;
                }
            }).attach(
                // back button pressed
                new Utils.BackButton(function (data) {
                    data.entity.doCallback();
                })
            );
            var stars = new CenteredList({
                position: new Vector2(-36, -108),
                spacing: 1
            });
            var horizontalRule = new Entity({
                name: 'stripe',
                position: new Vector2(-36, -100),
                originRelative: new Vector2(0.5, 0.5),
                family: [''],
                components: [new Sprite({
                    imageName: 'map/stripe'
                })]
            });
            var addStar = function () {
                stars.addItem(new Entity({
                    name: 'star',
                    originRelative: new Vector2(0.5, 0.5),
                    family: [''],
                    components: [new Sprite({
                        imageName: 'map/star'
                    })]
                }));
            };
            var createEnemyList = function () {
                var i;
                var card;
                var enemyList = planetJson.enemies || [];
                var canvas = new Canvas({
                    name: 'enemyArea',
                    position: new Vector2(-6, -57),
                    originRelative: new Vector2(0.5, 0.5),
                    width: 131,
                    height: 48
                });
                var scrollList = new ScrollingList({
                    updateWhenPaused: PauseLevels.INFO,
                    position: new Vector2(-43, 0),
                    direction: 'x',
                    spacing: 6,
                    // maxOffset: 86, // <- note: looks buggy when only 1 item
                    // snap: true
                });
                var addCard = function (critter) {
                    card = new Card(critter, {
                        hidden: !Utils.hasInCollection(critter.id)
                    });
                    // remove hp bar
                    card.getComponent('hpBar', function (component) {
                        card.remove(component);
                    });
                    scrollList.addItem(card);
                };

                canvas.attach(scrollList);
                canvas.attach({
                    start: function () {
                        // the grab area for scrolling
                        var canvasWorldPos;
                        var area;
                        var distance;
                        canvasWorldPos = canvas.getWorldPosition(); // wrong ..?
                        area = new Rectangle(canvasWorldPos.x + 28, canvasWorldPos.y - 48 / 2, 131, 48);
                        scrollList.setArea(area);

                        distance = -scrollList.getTotalSize();
                        // Slide through the enemy list to show that it's scrollable
                        new Tween({
                            from: distance,
                            to: 0,
                            'in': 45,
                            ease: 'easeInOutSine',
                            updateWhenPaused: PauseLevels.INFO,
                            'do': function (v, t) {
                                scrollList.forceTo(v);
                            }
                        });
                    },
                    update: function () {
                        scrollList.position.x = Math.round(scrollList.position.x);
                    }
                });

                // fill enemy critters
                for (i = 0; i < enemyList.length; ++i) {
                    enemyCritters.push(Utils.clone(creatures.creatures[enemyList[i]]));
                }

                // load cards
                for (i = 0; i < enemyCritters.length; ++i) {
                    addCard(enemyCritters[i]);
                }
                // load boss card
                addCard(Utils.clone(creatures.creatures[planetJson.boss.id]));

                return canvas;
            };
            // star rating
            Utils.repeat(planetJson.stars || 0, addStar);

            // overlay
            darkEntity.alpha = (0.75);

            planetIcon.attach(planetInfo);
            infoPanel.attach(planetIcon);
            infoPanel.attach(planetText);
            infoPanel.attach(buttonAttack);
            infoPanel.attach(stars);
            infoPanel.attach(horizontalRule);
            infoPanel.attach(createEnemyList());
            infoPanel.attach({
                start: function () {
                    Bento.attach(darkEntity);
                    Bento.attach(buttonExit);
                },
                destroy: function () {
                    Bento.remove(darkEntity);
                    Bento.remove(buttonExit);
                }
            });
            Bento.attach(infoPanel);

            // move in
            new Tween({
                from: viewport.width * 1.5 + 8,
                to: viewport.width / 2 + 8,
                'in': 30,
                ease: 'easeOutExpo',
                updateWhenPaused: PauseLevels.INFO,
                'do': function (v, t) {
                    infoPanel.position.x = v;
                },
                onComplete: function () {

                }
            });

            buttonSettings.visible = false;

        };
        var addClickableBehavior = function () {
            var clickable = new Clickable({
                onClick: function () {
                    var entity = this.parent;

                    if (entity.getComponent('sprite').currentAnimation.name === 'locked') {
                        // quick fix
                        return;
                    }

                    // pause lower layer
                    Bento.objects.pause(PauseLevels.INFO);

                    // move ship here!
                    Bento.objects.get('ship', function (ship) {
                        var time = 60;
                        // already at the planet?
                        if (entity.collidesWith(ship)) {
                            time = 1;
                        }
                        new Tween({
                            from: ship.position,
                            to: entity.position,
                            'in': time,
                            ease: 'exponential',
                            alpha: -5,
                            updateWhenPaused: PauseLevels.INFO,
                            'do': function (v, t) {
                                ship.position = v;
                            },
                            onComplete: function () {
                                // show planet info
                                showInfoPanel(entity);
                            }
                        });
                    });
                    Bento.audio.stopSound('sfx_button');
                    Bento.audio.playSound('sfx_button');
                }
            });
            return clickable;
        };
        var sprite = new Sprite({
            imageName: 'map/planets/' + id,
            frameCountY: 2,
            animations: {
                default: {
                    frames: [0]
                },
                locked: {
                    frames: [1]
                }
            }
        });
        var entity = new Entity({
            z: 1,
            name: id,
            position: settings.position || new Vector2(0, 0),
            originRelative: new Vector2(0.5, 0.5),
            updateWhenPaused: 0,
            family: [''],
            components: [
                sprite,
                addClickableBehavior()
            ]
        });
        var unlockedPlanets = Bento.saveState.load('unlockedPlanetNames', ['starti']);
        var logNames = function () {
            var names = {
                strings: {}
            };
            Utils.forEach(creatures.creatures, function (critter, i, l, breakLoop) {
                names.strings[critter.id] = critter.name;
            });
            console.log(JSON.stringify(names, null, 4));
        };
        // unlocked?
        if (unlockedPlanets.indexOf(id) < 0) {
            sprite.setAnimation('locked');
        }

        return entity;
    };
});