/**
 * Paralax background layer
 */
bento.define('entities/paralax', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    Utils,
    Tween
) {
    'use strict';
    return function (settings) {
        var viewport = Bento.getViewport(),
            offsetX = 0, // offset spawns paralax at the same spot consistently
            offsetY = settings.offsetY || 0, // offset spawns paralax at the same spot consistently
            dimension,
            imageName = settings.imageName,
            scrollFactor = settings.scrollFactor || 0,
            z = settings.z || 0,
            entity = new Entity({
                z: z,
                name: 'paralax',
                position: new Vector2(0, 0),
                originRelative: new Vector2(0, 0),
                updateWhenPaused: settings.updateWhenPaused,
                components: [
                    new Sprite({
                        image: Bento.assets.getImage(imageName)
                    })
                ],
                family: ['paralax']
            }),
            left = new Entity({
                z: 0,
                name: '',
                position: new Vector2(0, 0),
                originRelative: new Vector2(0, 0),
                components: [
                    new Sprite({
                        image: Bento.assets.getImage(imageName)
                    })
                ]
            }),
            right = new Entity({
                z: 0,
                name: '',
                position: new Vector2(0, 0),
                originRelative: new Vector2(0, 0),
                components: [
                    new Sprite({
                        image: Bento.assets.getImage(imageName)
                    })
                ]
            });

        dimension = entity.dimension;

        // paralax behavior
        entity.attach({
            update: function (data) {
                var position = entity.position;
                // x direction: looping paralax layer
                var offset = viewport.x * scrollFactor + offsetX;
                while (offset < 0) {
                    offset += dimension.width;
                }
                position.x = viewport.width + viewport.x - (offset % dimension.width);

                // linear interpolate between edges
                // position.x = ((levelDimension.width - dimension.width) / (levelDimension.width - viewport.width)) * viewport.x;
                // position.y = ((levelDimension.height - dimension.height) / (levelDimension.height - viewport.height)) * (viewport.y - levelDimension.y) + levelDimension.y;
                position.y = viewport.y + offsetY;
            }
        });

        // attach the same image left and right
        left.position.x = -dimension.width;
        entity.attach(left);
        right.position.x = dimension.width;
        entity.attach(right);

        // public
        entity.extend({
            getLeft: function () {
                return left;
            },
            getRight: function () {
                return right;
            }
        });

        return entity;
    };
});