/**
 * Module description
 */
bento.define('entities/simplecard', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'bento/canvas',
    'bento/transform',
    'entities/card',
    'components/card3dtouch'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    ClickButton,
    Counter,
    Text,
    EventSystem,
    Utils,
    Tween,
    Canvas,
    Transform,
    Card,
    Card3DTouch
) {
    'use strict';
    return function (properties, settings) {
        var viewport = Bento.getViewport();
        var entity = new Entity({
            name: 'simpleCard',
            originRelative: new Vector2(0.5, 0.5),
            components: []
        });
        var card = new Card(properties, settings);
        var container;
        var touch;
        var refresh = function () {
            // once the card is drawn, we don't need to have the card attached anymore
            var remover = {
                name: 'remover',
                update: function () {
                    if (container.timer > 2) {
                        container.remove(card);
                        container.remove(remover);
                    }
                }
            };

            // remove old container
            if (container) {
                entity.remove(container);
            }
            container = new Canvas({
                name: 'container',
                width: 45,
                height: 48,
                drawOnce: true,
                originRelative: new Vector2(0.5, 0.5),
                components: [
                    card,
                    // remover
                ]
            });

            entity.attach(container);
            entity.dimension = card.dimension.clone();
            entity.boundingBox = new Rectangle(-22, -23, 42, 35);

            entity.draw(Bento.getGameData());

            container.remove(card);
        };
        entity.extend({
            getAvatar: card.getAvatar,
            setHp: card.setHp,
            addHp: card.addHp,
            getId: card.getId,
            getProperties: card.getProperties,
            getProperty: card.getProperty,
            refresh: refresh,
            isHidden: card.isHidden,
            on3DTouch: function (callback) {
                touch.on3DTouch = callback;
            }
        });
        
        Transform.subPixel = false;
        refresh();
        Transform.subPixel = true;
        
        touch = new Card3DTouch({});
        entity.attach(touch);
        
        return entity;
    };
});