/**
 * 3D object
 */
bento.define('entities/three', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'bento/packedimage'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    Utils,
    Tween,
    PackedImage
) {
    'use strict';
    return function (settings) {
        /*settings = {
			// describe your settings object parameters
			position: Vector2 // positions the entity
		}*/
        var viewport = Bento.getViewport();
        var sprite;
        var entity;
        var threeRenderer;

        var canvas = document.createElement('canvas');
        var scene = new THREE.Scene();
        var camera = new THREE.PerspectiveCamera(60, viewport.width / viewport.height, 0.1, 1000);
        var ThreeRenderer = Utils.isCocoonJs() ? THREE.CanvasRenderer : THREE.WebGLRenderer

        // var renderer = new ThreeRenderer({
        var renderer = new THREE.CanvasRenderer({
            canvas: canvas,
            // context: canvas.getContext('webgl'),
            alpha: true
        });
        renderer.autoClear = false; // prevents holes
        var meshes = [];
        var makeMesh = function (position) {
            var geometry = new THREE.PlaneGeometry(64, 64, 1, 1);
            // var geometry = new THREE.CylinderGeometry( 5, 5, 10, 20 );
            var texture = new THREE.Texture(Bento.assets.getImage('title/test').image);
            // var texture = new THREE.TextureLoader().load( "assets/game/images/title/test.png" );
            texture.needsUpdate = true;
            // texture.generateMipmaps = false;
            var material = new THREE.MeshBasicMaterial({
                color: 0xffffff,
                map: texture
            });

            var mesh = new THREE.Mesh(geometry, material);
            mesh.position.x = position.x;
            mesh.position.y = position.y;
            mesh.rotation.z = Math.PI;
            meshes.push(mesh);
            scene.add(mesh);
            return mesh;
        };
        renderer.setSize(viewport.width, viewport.height);

        makeMesh(new Vector2(0, 64 * 0));
        makeMesh(new Vector2(0, 64 * 1));
        makeMesh(new Vector2(0, 64 * 2));
        makeMesh(new Vector2(0, 64 * 3));
        makeMesh(new Vector2(64, 64 * 0));
        makeMesh(new Vector2(64, 64 * 1));
        makeMesh(new Vector2(64, 64 * 2));
        makeMesh(new Vector2(64, 64 * 3));
        makeMesh(new Vector2(-64, 64 * 0));
        makeMesh(new Vector2(-64, 64 * 1));
        makeMesh(new Vector2(-64, 64 * 2));
        makeMesh(new Vector2(-64, 64 * 3));


        camera.position.z = 50;
        camera.rotation.z = -Math.PI;
        camera.rotation.x = -Math.PI * 1 / 4;
        // camera.quaternion.setFromAxisAngle(new THREE.Vector3(1, 0, 0), Math.PI / 4);
        // shape.rotation.x = -Math.PI / 4;
        // shape.rotation.z = -Math.PI / 2;


        // bento
        threeRenderer = {
            name: '3DRenderer',
            update: function () {
                camera.position.y += 0.05;
                // camera.rotation.z += 0.01;
            },
            draw: function (data) {
                // shape.rotation.z += 0.01;
                // shape.rotation.x += 0.01;
                // shape.rotation.y += 0.01;
                // texture.offset.y += 0.001;
                // camera.rotation.x += 0.01;
                renderer.render(scene, camera);
                if (entity.timer === 10) {
                    // console.log(canvas.toDataURL())
                    // console.log(renderer.domElement.toDataURL("image/png"));
                }

                // var img = new Image();
                // img.src = canvas.toDataURL();
                // img.onload = function () {
                //     sprite.setup({
                //         image: new PackedImage(img)
                //     });
                // }
            }
        };
        sprite = new Sprite({
            image: new PackedImage(canvas)
        });
        entity = new Entity({
            z: Utils.Layers.MODE7,
            name: 'three',
            position: new Vector2(viewport.width / 2, viewport.height / 2),
            originRelative: new Vector2(0.5, 0.5),
            updateWhenPaused: 0,
            components: [
                threeRenderer,
                sprite
            ]
        });

        return entity;
    };
});