/**
 * Very simple hashgrid to be used for ground tiles
 */
bento.define('entities/tilecolliders', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/utils',
    'bento/tween'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    Utils,
    Tween
) {
    'use strict';
    var floor = Math.floor;
    return function () {
        var grid = {},
            cellSize = 16,
            invCellSize = 1 / cellSize,
            entity = new Entity({
                z: 0,
                name: 'tileColliders',
                family: ['tileColliders']
            }),
            posToKey = function (x, y) {
                return floor(x * invCellSize).toString() + ':' + floor(y * invCellSize);
            };

        // public
        entity.extend({
            insert: function (entity) {
                var dimension = entity.boundingBox,
                    x = dimension.x,
                    y = dimension.y,
                    key = posToKey(x, y);
                grid[key] = entity;
            },
            remove: function (entity) {
                // clear the entity there
                var dimension = entity.boundingBox,
                    x = dimension.x,
                    y = dimension.y,
                    key = posToKey(x, y);
                delete grid[key];
            },
            checkCell: function (position) {
                var key = posToKey(position.x, position.y);
                return grid[key];
            },
            collidesWith: function (entity, offset, callback) {
                var box = entity.getBoundingBox(),
                    minX,
                    maxX,
                    minY,
                    maxY,
                    x,
                    y,
                    key,
                    gridEntry,
                    hasCollided = false;

                box.x += offset.x;
                box.y += offset.y;

                minX = floor(box.x * invCellSize);
                maxX = floor(box.getX2() * invCellSize);
                minY = floor(box.y * invCellSize);
                maxY = floor(box.getY2() * invCellSize);

                // check all grids
                for (x = minX; x <= maxX; ++x) {
                    for (y = minY; y <= maxY; ++y) {
                        key = x.toString() + ':' + y;
                        gridEntry = grid[key];
                        if (gridEntry) {
                            if (callback) {
                                callback(gridEntry);
                            }
                            hasCollided = true;
                        }
                    }
                }
                return hasCollided;
            }
        });

        return entity;
    };
});