/**
 * Boss on world map
 */
bento.define('entities/worldmap/boss', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/gui/clickbutton',
    'bento/gui/text',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'components/sort',
    'components/depth',
    'components/movable'
], function (
    Bento,
    Vector2,
    Rectangle,
    ClickButton,
    Text,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    Utils,
    Tween,
    Sort,
    Depth,
    Movable
) {
    'use strict';
    var PI = Math.PI;
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
            planetData: Object //data coming from screens/map
        }*/
        var viewport = Bento.getViewport();
        var spriteShadow = new Sprite({
            imageName: 'worldmap/shadow'
        });
        var shadow = new Entity({
            z: Utils.Layers.CRITTERS + 1,
            name: 'shadow',
            position: new Vector2(0, 0),
            originRelative: new Vector2(0.5, 1),
            components: [
                spriteShadow
            ]
        });
        var sprite = new Sprite({
            imageName: 'worldmap/' + Utils.worldMap.planetData.planetName + '/boss',
            frameCountX: 4,
            frameCountY: 2,
            animations: {
                'up': {
                    speed: 0.05,
                    frames: [0, 4]
                },
                'down': {
                    speed: 0.05,
                    frames: [1, 5]
                },
                'left': {
                    speed: 0.05,
                    frames: [2, 6]
                },
                'right': {
                    speed: 0.05,
                    frames: [3, 7]
                }
            }
        });
        var maxSpeed = 1;
        var movable = new Movable({
            maxSpeed: new Vector2(maxSpeed, maxSpeed)
        });
        var player;
        var bossDepth = new Depth({});
        var hideBehavior = {
            update: function () {
                if (entity.name === 'inactiveBoss') {
                    boss.visible = false;
                    return;
                }
            }
        };
        var boss = new Entity({
            z: Utils.Layers.CRITTERS + 3,
            name: 'boss',
            position: new Vector2(0, 0),
            originRelative: new Vector2(0.5, 1),
            family: [''],
            components: [
                sprite,
                shadow,
                bossDepth,
                new Sort({}),
                hideBehavior
            ]
        });
        var addBoss = {
            start: function () {
                player = Bento.objects.get('player');
                Bento.objects.attach(boss);
            },
            destroy: function () {
                Bento.objects.remove(boss);
            },
            update: function () {
                // boss.position = entity.position.clone();
                bossDepth.position.x = entity.position.x;
                bossDepth.position.y = entity.position.y;
            }
        };
        var movingBehavior = {
            direction: 'down',
            timer: 300,
            state: 'stop',
            speed: 0.5,
            randomize: function () {
                this.timer = 60 + Utils.getRandom(60);
                this.state = Utils.pickRandom('stop', 'walk');
                this.direction = Utils.pickRandom('up', 'down', 'left', 'right');
            },
            update: function () {
                if (entity.name === 'inactiveBoss') {
                    return;
                }
                if (this.timer <= 0) {
                    this.randomize();
                    return;
                } else {
                    this.timer -= 1;
                }

                if (!viewport.hasPosition(boss.position)) {
                    movable.speed.x = Utils.approach(movable.speed.x, 0, 0.05);
                    movable.speed.y = Utils.approach(movable.speed.y, 0, 0.05);
                    return;
                }

                // dont move until player can
                if (!player.isActivated()) {
                    this.timer = 300;
                    return;
                }

                if (this.state === 'walk') {
                    if (this.direction === 'up') {
                        movable.speed.x = Utils.approach(movable.speed.x, 0, 0.05);
                        movable.speed.y = Utils.approach(movable.speed.y, -this.speed, 0.05);
                    }
                    if (this.direction === 'down') {
                        movable.speed.x = Utils.approach(movable.speed.x, 0, 0.05);
                        movable.speed.y = Utils.approach(movable.speed.y, this.speed, 0.05);
                    }
                    if (this.direction === 'left') {
                        movable.speed.x = Utils.approach(movable.speed.x, -this.speed, 0.05);
                        movable.speed.y = Utils.approach(movable.speed.y, 0, 0.05);
                    }
                    if (this.direction === 'right') {
                        movable.speed.x = Utils.approach(movable.speed.x, this.speed, 0.05);
                        movable.speed.y = Utils.approach(movable.speed.y, 0, 0.05);
                    }
                    sprite.setAnimation(this.direction);
                }
                if (this.state === 'stop') {
                    movable.speed.x = Utils.approach(movable.speed.x, 0, 0.05);
                    movable.speed.y = Utils.approach(movable.speed.y, 0, 0.05);
                }
                // moved too far from original position

            }
        };
        var planetName = Utils.worldMap.planetData.planetName;
        var triggerBehavior = {
            start: function () {
                var hash;
                // was already touched before
                hash = entity.getHash();
                // dont show boss until tutorial
                // if (Bento.saveState.load('sawTutorial2', false) === false) {
                //     Bento.objects.remove(entity);
                //     return;
                // }

                if (Utils.hasDefeatedBoss(Utils.worldMap.planetData.boss.id) && Utils.worldMap.collected[hash] === true) {
                    Bento.objects.remove(entity);
                }
                
                // collided with player on start
                entity.collidesWith({
                    family: 'player',
                    onCollide: function (player) {
                        // what do? remove the boss?
                        // Bento.objects.remove(entity);

                        // or just move it away a little?
                        if (player.position.x > entity.position.x) {
                            entity.position.x -= 16;
                        } else {
                            entity.position.x += 16;
                        }
                        if (player.position.y > entity.position.y) {
                            entity.position.y -= 32;
                        } else {
                            entity.position.y += 32;
                        }
                    }
                });
            }
        };
        var entity = new Entity({
            z: Utils.Layers.CRITTERS + 3,
            name: 'boss',
            position: settings.position || new Vector2(0, 0),
            boundingBox: new Rectangle(-8, -8, 16, 8),
            family: ['triggers'],
            components: [
                
                // boss,
                movable,
                addBoss,
                movingBehavior,
                triggerBehavior
            ]
        });

        entity.extend({
            getHash: function () {
                return settings.tiled.object.id;
            },
            getEnemies: function () {
                var boss = Bento.assets.getJson('creatures/creatures').creatures[Utils.worldMap.planetData.boss.id];
                return [Utils.clone(boss)];
            },
            boss: boss
        });
        if (Bento.saveState.load(planetName + '-showBoss', false) === false) {
            entity.name = 'inactiveBoss';
            boss.visible = false;
        }

        sprite.setAnimation('down');

        return entity;
    };
});