/**
 * Moving trigger on world map
 */
bento.define('entities/worldmap/movingtrigger', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/gui/clickbutton',
    'bento/gui/text',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'components/chaser',
    'components/sort',
    'components/depth',
    'components/movable',
    'entities/effect'
], function (
    Bento,
    Vector2,
    Rectangle,
    ClickButton,
    Text,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    Utils,
    Tween,
    Chaser,
    Sort,
    Depth,
    Movable,
    Effect
) {
    'use strict';
    var PI = Math.PI;
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var spriteShadow = new Sprite({
            imageName: 'worldmap/shadow'
        });
        var stopped = settings.tiled.object.stopped || false;
        var position = null;
        var followTrigger = {
            name: 'followTrigger',
            start: function (data) {
                shadow.visible = false;
            },
            update: function (data) {
                shadow.position.x = trigger.position.x;
                shadow.position.y = trigger.position.y;
                shadow.scale.x = trigger.scale.x;
                shadow.scale.y = trigger.scale.y;
                
                // simple workaround for flashing shadows in (0, 0) in the world
                if (shadow.timer > 1) {
                    shadow.visible = trigger.visible;
                }
            }
        };
        var shadow = new Entity({
            z: Utils.Layers.CRITTERS + 1,
            name: 'shadow',
            position: new Vector2(0, 0),
            originRelative: new Vector2(0.5, 1),
            components: [
                spriteShadow,
                followTrigger
            ]
        });
        var player;
        var planetName = Utils.worldMap.planetData.planetName;
        var sprite = new Sprite({
            imageName: 'worldmap/' + planetName + '/enemy',
            frameCountX: 4,
            frameCountY: 2,
            animations: {
                'up': {
                    speed: 0.05,
                    frames: [0, 4]
                },
                'down': {
                    speed: 0.05,
                    frames: [1, 5]
                },
                'left': {
                    speed: 0.05,
                    frames: [2, 6]
                },
                'right': {
                    speed: 0.05,
                    frames: [3, 7]
                }
            }
        });
        var maxSpeed = 0.75;
        var movable = new Movable({
            maxSpeed: new Vector2(maxSpeed, maxSpeed)
        });
        var depth = new Depth({});
        var trigger = new Entity({
            z: Utils.Layers.CRITTERS + 3,
            name: 'trigger',
            position: new Vector2(0, 0),
            originRelative: new Vector2(0.5, 1),
            family: [''],
            components: [
                sprite,
                depth,
                new Sort({})
            ]
        });
        var addTrigger = {
            start: function () {
                player = Bento.objects.get('player');
                Bento.objects.attach(trigger);
                Bento.objects.attach(shadow);
            },
            destroy: function () {
                var effect = new Effect({
                    sprite: new Sprite(Bento.assets.getJson('animations/poof')),
                    position: entity.position.clone(),
                    originRelative: new Vector2(0.5, 0.75)
                });
                effect.attach(new Depth({}));

                Bento.objects.remove(trigger);
                Bento.objects.remove(shadow);
            },
            update: function () {
                // trigger.position = entity.position.clone();
                depth.position.x = entity.position.x;
                depth.position.y = entity.position.y;
            }
        };
        var setAnimationToSpeed = function () {
            // set animation direction to speed direction
            var angle;

            if (movable.speed.x === 0 && movable.speed.y === 0) {
                return;
            }

            angle = movable.speed.angle();

            // angle should be between -PI to PI
            while (angle < -Math.PI) {
                angle += Math.PI;
            }
            while (angle > Math.PI) {
                angle -= Math.PI;
            }

            //
            lookAtAngle(angle);
        };
        var lookAtAngle = function (angle) {
            if (Utils.isBetween(-Math.PI / 4, angle, Math.PI / 4)) {
                sprite.setAnimation('right');
            } else if (Utils.isBetween(-Math.PI * 3 / 4, angle, -Math.PI / 4)) {
                sprite.setAnimation('up');
            } else if (Utils.isBetween(Math.PI / 4, angle, Math.PI * 3 / 4)) {
                sprite.setAnimation('down');
            } else {
                sprite.setAnimation('left');
            }
        };
        var escapeFromPlayer = function () {
            var diffVector = entity.position.subtract(player.position);
            var magnitude = diffVector.magnitude();
            var overshoot = magnitude / (maxSpeed);

            // diffVector.rotateRadian(Math.PI);

            movable.speed.x = diffVector.x / overshoot;
            movable.speed.y = diffVector.y / overshoot;

        };
        var jump = function () {
            var self = this;
            var jumpHeight = 8;
            var time = 20;
            var diffVector = player.position.subtract(entity.position);
            var angle = diffVector.angle();
            movable.speed.x = 0;
            movable.speed.y = 0;

            this.state = 'jumping';

            lookAtAngle(angle);

            new Tween({
                from: 0,
                to: jumpHeight,
                'in': time / 2,
                ease: 'squareroot',
                'do': function (v, t) {
                    entity.setZ(v);
                },
                onComplete: function () {
                    new Tween({
                        from: jumpHeight,
                        to: 0,
                        'in': time / 2,
                        ease: 'quadratic',
                        'do': function (v, t) {
                            entity.setZ(v);
                        },
                        onComplete: function () {
                            var team = Utils.getTeam();
                            var teamRank = 0;
                            var teamCount = 0;
                            Utils.forEach(team, function (critter) {
                                if (!critter) {
                                    return;
                                }
                                teamRank += critter.rarity;
                                teamCount += 1;
                            });
                            teamRank /= teamCount;

                            // choose between chasing and escaping
                            if (averageRank + 1 >= teamRank) {
                                self.state = 'chasing';
                                entity.attach(new Chaser({
                                    target: Bento.objects.get('player'),
                                    speed: maxSpeed,
                                }));
                            } else {
                                self.state = 'escaping';
                            }
                        }
                    });
                }
            });
        };
        var movingBehavior = {
            direction: 'down',
            timer: 0,
            state: 'stop',
            speed: 0.5,
            randomize: function () {
                this.timer = 60 + Utils.getRandom(60);
                this.state = Utils.pickRandom('stop', 'walk');
                this.direction = Utils.pickRandom('up', 'down', 'left', 'right');
            },
            update: function () {
                var vp = viewport.clone();
                vp.grow(20);
                if (!vp.hasPosition(trigger.position)) {
                    movable.speed.x = Utils.approach(movable.speed.x, 0, 0.05);
                    movable.speed.y = Utils.approach(movable.speed.y, 0, 0.05);
                    return;
                }

                setAnimationToSpeed();

                if (this.state === 'chasing' || this.state === 'jumping') {
                    // can't go back to any other state
                    return;
                }
                
                if (stopped) {
                    // critter wont move
                    return;
                }
                
                // dont move until player can
                if (!player.isActivated()) {
                    this.timer = 120;
                    return;
                }

                // player gets close
                if (
                    this.state !== 'chasing' &&
                    this.state !== 'escaping' &&
                    this.state !== 'jumping' &&
                    player &&
                    player.position.isCloserThan(entity.position, 64)
                ) {
                    jump.apply(this);
                    return;
                }
                // test with chasing
                if (false) {
                    this.state = 'chasing';
                    entity.attach(new Chaser({
                        target: Bento.objects.get('player'),
                        speed: maxSpeed,
                    }));
                }

                // test with escaping
                if (this.state === 'escaping') {
                    escapeFromPlayer();
                    return;
                }
                if (this.timer <= 0) {
                    this.randomize();
                    return;
                } else {
                    this.timer -= 1;
                }

                if (this.state === 'walk') {
                    if (this.direction === 'up') {
                        movable.speed.x = Utils.approach(movable.speed.x, 0, 0.05);
                        movable.speed.y = Utils.approach(movable.speed.y, -this.speed, 0.05);
                    }
                    if (this.direction === 'down') {
                        movable.speed.x = Utils.approach(movable.speed.x, 0, 0.05);
                        movable.speed.y = Utils.approach(movable.speed.y, this.speed, 0.05);
                    }
                    if (this.direction === 'left') {
                        movable.speed.x = Utils.approach(movable.speed.x, -this.speed, 0.05);
                        movable.speed.y = Utils.approach(movable.speed.y, 0, 0.05);
                    }
                    if (this.direction === 'right') {
                        movable.speed.x = Utils.approach(movable.speed.x, this.speed, 0.05);
                        movable.speed.y = Utils.approach(movable.speed.y, 0, 0.05);
                    }
                    // sprite.setAnimation(this.direction);
                }
                if (this.state === 'stop') {
                    movable.speed.x = Utils.approach(movable.speed.x, 0, 0.05);
                    movable.speed.y = Utils.approach(movable.speed.y, 0, 0.05);
                }
                // moved too far from original position

            }
        };
        var triggerBehavior = {
            start: function () {
                var hash;
                // remember original position
                position = entity.position.clone();

                // was already touched before
                hash = entity.getHash();

                if (Utils.worldMap.collected[hash] === true) {
                    Bento.objects.remove(entity);
                }
                // collided with player on start
                entity.collidesWith({
                    family: 'player',
                    onCollide: function (player) {
                        // remove
                        Bento.objects.remove(entity);
                    }
                });
                
            }
        };
        var entity = new Entity({
            z: Utils.Layers.CRITTERS + 3,
            name: 'trigger',
            position: settings.position || new Vector2(0, 0),
            boundingBox: new Rectangle(-8, -8, 16, 8),
            family: ['triggers'],
            components: [
                
                movable,
                addTrigger,
                movingBehavior,
                triggerBehavior
            ]
        });
        var enemies = [];
        var averageRank = 0;
        var setEnemies = function () {
            var i;
            var enemyCritters = Utils.worldMap.planetData.enemyCritters || [];
            var enemyCount = 1 + Utils.getRandom(3);
            var randomEnemies = [];
            var randomEnemy;

            // push in array
            for (i = 0; i < enemyCritters.length; ++i) {
                Utils.repeat(enemyCritters[i].chance, function () {
                    randomEnemies.push(enemyCritters[i]);
                });
            }
            
            // tutorial not played yet?
            if (Bento.saveState.load('sawTutorial2', false) === false) {
                if (Bento.saveState.load('battles', 0) % 2 === 0) {
                    randomEnemies = [Bento.assets.getJson('creatures/creatures').creatures[Utils.tutorialEnemy]];
                } else {
                    randomEnemies = [Bento.assets.getJson('creatures/creatures').creatures[Utils.tutorialEnemy2]];
                }
                enemyCount = 1;
            }

            // pick at random
            for (i = 0; i < enemyCount; ++i) {
                randomEnemy = randomEnemies[Utils.getRandom(randomEnemies.length)];
                enemies.push(Utils.clone(randomEnemy));

                averageRank += randomEnemy.rarity;
            }

            averageRank /= enemyCount;

        };

        // generate a random set of enemies
        setEnemies();

        entity.extend({
            setZ: function (height) {
                trigger.transform.y = -height;
                shadow.alpha = (Utils.clamp(0, 1 - height / 64, 1));
            },
            getHash: function () {
                return settings.tiled.object.id + ',' + position.x + ',' + position.y;
            },
            getEnemies: function () {
                // flee tutorial
                if (planetName === 'veggi' && Bento.saveState.load('sawFleeTutorial', false) === false) {
                    return [
                        Utils.clone(Bento.assets.getJson('creatures/creatures').creatures['bananaman']),
                        Utils.clone(Bento.assets.getJson('creatures/creatures').creatures['broccotree'])
                    ];
                }
                
                return enemies;
            }
        });

        sprite.setAnimation('down');

        return entity;
    };
});