/**
 * Module description
 */
bento.define('entities/worldmap/token', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'components/depth',
    'components/sort'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    ClickButton,
    Counter,
    Text,
    EventSystem,
    Utils,
    Tween,
    Depth,
    Sort
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var tiledId = settings.tiled.object.id;
        var planetName = Utils.worldMap.planetData.planetName;
        var sprite = new Sprite({
            imageName: 'worldmap/token',
            frameCountX: 3,
            frameCountY: 3,
            animations: {
                "default": {
                    speed: 0.15,
                    frames: [0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 2, 3, 4, 5, 6, 7]
                }
            }
        });
        var tokenCollection = Utils.tokens;
        var depth = new Depth({});
        var sort = new Sort({});
        var behavior = {
            start: function () {
                // entity.position.y += 16 * 3;
                if (Bento.saveState.load(planetName + '-token-' + tiledId, false) === true) {
                    if (tokenCollection.has(tiledId)) {
                        // Bento.objects.remove(entity);
                    } else {
                        entity.alpha = 0.25;
                    }
                } else {
                    // collided with player on start
                }
            },
            update: function () {
                entity.collidesWith({
                    family: 'player',
                    onCollide: function () {
                        var tokenTotal = Bento.saveState.load(planetName + '-tokenTotal', 0);

                        if (Bento.saveState.load(planetName + '-token-' + tiledId, false) === false) {
                            Bento.objects.remove(entity);
                            Bento.audio.playSound('sfx_coin');

                            // save total
                            Bento.saveState.save(planetName + '-tokenTotal', tokenTotal + 1);
                            // save individual
                            Bento.saveState.save(planetName + '-token-' + tiledId, true);
                            // update text
                            Bento.objects.get('tokenCounter', function (tokenCounter) {
                                tokenCounter.updateValue();
                            });
                        }
                        // tokenCollection.add(tiledId, true);

                    }
                });
            }
        };
        var spriteShadow = new Sprite({
            imageName: 'worldmap/shadow'
        });
        var shadow = new Entity({
            z: Utils.Layers.CRITTERS + 1,
            name: 'shadow',
            position: new Vector2(0, 0),
            originRelative: new Vector2(0.5, 0.5),
            components: [
                spriteShadow,
            ]
        });
        var entity = new Entity({
            z: 1,
            name: 'token',
            position: settings.position || new Vector2(0, 0),
            originRelative: new Vector2(0.5, 1),
            updateWhenPaused: 0,
            boundingBox: new Rectangle(-8, -8, 16, 8),
            components: [
                sprite,
                shadow,
                sort,
                behavior,
                depth
            ]
        });
        // entity.origin.y -= 16 * 4;
        return entity;
    };
});