/**
 * Module description
 */
bento.define('entities/worldmap/trigger', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'components/depth',
    'components/sort'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    ClickButton,
    Counter,
    Text,
    EventSystem,
    Utils,
    Tween,
    Depth,
    Sort
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var sprite = new Sprite(Bento.assets.getJson('animations/worldmap/marker'));
        var depth = new Depth({});
        var position = null;
        var behavior = {
            start: function () {
                var hash;
                // remember original position
                position = depth.getOriginalPosition() || entity.position.clone();

                // was already touched before
                hash = entity.getHash();
                if (Utils.worldMap.collected[hash] === true) {
                    Bento.objects.remove(entity);
                } else {
                    // collided with player on start
                    entity.collidesWith({
                        family: 'player',
                        onCollide: function () {
                            Bento.objects.remove(entity);
                        }
                    });
                }
            },
            update: function () {}
        };
        var entity = new Entity({
            z: 1,
            name: 'trigger',
            position: settings.position || new Vector2(0, 0),
            originRelative: new Vector2(0.5, 1),
            boundingBox: new Rectangle(-8, -8, 16, 8),
            family: ['triggers'],
            components: [
                sprite,
                depth,
                behavior,
                // new Utils.Debug({}),
                new Sort({})
            ]
        });

        entity.extend({
            getHash: function () {
                return settings.tiled.object.id + ',' + position.x + ',' + position.y;
            }
        });

        // entity.visible = false;
        return entity;
    };
});