// global variables
window.VERSION = '1.3.5';
window.VERSION = "1.3.9";
window.BUILD = 1030610;

// undefine require
if (window.require) {
    window.require = undefined;
}
if (window.define) {
    window.define = undefined;
}

bento.require.config({
    baseUrl: 'js'
});

window.startGame = function () {
    bento.require([
        'bento',
        'bento/math/vector2',
        'bento/math/rectangle',
        'bento/components/animation',
        'bento/components/sprite',
        'bento/eventsystem',
        'components/eventlistener',
        'bento/tween',
        'autoresize',
        'utils',
        'dummy',
        'modules/betatest',
        'modules/savestate',
        'devsaver',
        'init',
        'bento/entity',
        'modules/cocoonmigration',
        'modules/nativestorage'
    ], function (
        Bento,
        Vector2,
        Rectangle,
        Animation,
        Sprite,
        EventSystem,
        EventListener,
        Tween,
        AutoResize,
        Utils,
        Dummy,
        BetaTest,
        SaveState,
        DevSaver,
        Init,
        Entity,
        CocoonMigration,
        NativeStorage
    ) {
        var canvasDimension = AutoResize(new Rectangle(0, 0, 160, 284), 256, 284);
        var originalCanvasScale = null;

        if (!window.Cocoon && !Utils.isMobile()) {
            // on desktop
            canvasDimension = new Rectangle(0, 0, 160, 284);
        }

        var pixelSize = Utils.isAndroid() ? 2 : 4;
        var wideScreenSize = new Rectangle(0, 0, 0, 0);
        var onResize = function () {
            var viewport = Bento.getViewport();
            var canvas = document.getElementById('canvas');
            var context;
            var screenRatio = 1;
            var wentOverMinimum = false;
            if (!canvas) {
                return;
            }
            context = canvas.getContext('2d');

            canvasDimension = new AutoResize(
                new Rectangle(0, 0, 160, 284), // base size
                256, // minimum height
                284 // maximum height
            );
            // max/min width
            if (canvasDimension.width > 284) {
                canvasDimension.width = 284;
            }
            if (Math.ceil(canvasDimension.width) < 160) {
                canvasDimension.width = 160;
                wentOverMinimum = true;
            }
            canvas.width = canvasDimension.width * pixelSize;
            canvas.height = canvasDimension.height * pixelSize;
            viewport.width = canvasDimension.width;
            viewport.height = canvasDimension.height;
            wideScreenSize = viewport.clone();

            // pad extra pixels
            if (wentOverMinimum) {
                screenRatio = window.innerWidth / window.innerHeight;
                // append to height
                canvasDimension.height = Math.round(canvasDimension.width / screenRatio);
                canvas.height = canvasDimension.height * pixelSize;
                viewport.height = canvasDimension.height;
                wideScreenSize.height = canvasDimension.height;
                Utils.isWideScreen = true;
            }
            Utils.screenSize = canvasDimension.clone();

            // fit to height
            canvas.style.height = window.innerHeight + 'px';
            canvas.style.width = (viewport.width / viewport.height * window.innerHeight) + 'px';


            // Bento input updates its viewport before this event, so call this manually
            if (Bento.input) {
                Bento.input.updateCanvas();
            }

            // cut off the extra pixels so the viewport becomes 2:1
            var maxRatio = (284 / 160);
            if (viewport.height > viewport.width * maxRatio) {
                var cutOff = viewport.height - viewport.width * maxRatio;
                // cut off bottom
                viewport.height -= cutOff;
                // offset view
                viewOffset.position.x = 0;
                viewOffset.position.y = cutOff / 2;
                Bento.input.setOffset(new Vector2(0, -cutOff / 2));
                Utils.screenSize.y = -cutOff / 2;
            }
            // prevent the canvas being blurry after resizing
            if (context.imageSmoothingEnabled) {
                context.imageSmoothingEnabled = false;
            }
            if (context.webkitImageSmoothingEnabled) {
                context.webkitImageSmoothingEnabled = false;
            }
            if (context.mozImageSmoothingEnabled) {
                context.mozImageSmoothingEnabled = false;
            }
            if (context.msImageSmoothingEnabled) {
                context.msImageSmoothingEnabled = false;
            }
        };
        var viewOffset = new Entity({
            z: 1000000,
            name: 'viewOffset',
            global: true,
            float: true,
            updateWhenPaused: 100000,
            components: [
                new EventListener({
                    name: 'preDrawLoop',
                    onEvent: function (data) {
                        data.renderer.save();
                        data.renderer.translate(viewOffset.position.x, viewOffset.position.y);
                    }
                }),
                new EventListener({
                    name: 'postDrawLoop',
                    onEvent: function (data) {
                        var height = viewOffset.position.y;
                        var viewport = Bento.getViewport();
                        // data.renderer.fillRect([0, 0, 0, 1], 0, -height, viewport.width, height);
                        // data.renderer.fillRect([0, 0, 0, 1], 0, viewport.height, viewport.width, height);
                        data.renderer.restore();
                    }
                }),
                new Object({
                    name: 'behavior',
                    update: function (data) {
                        // workaround: sometimes canvasscale changes after going to home screen
                        if (!originalCanvasScale && data && data.canvasScale) {
                            originalCanvasScale = new Vector2(0, 0);
                            originalCanvasScale.x = data.canvasScale.x;
                            originalCanvasScale.y = data.canvasScale.y;
                            return;
                        } else if (originalCanvasScale && data && data.canvasScale && originalCanvasScale.x !== data.canvasScale.x || originalCanvasScale.y !== data.canvasScale.y) {
                            data.canvasScale.x = originalCanvasScale.x;
                            data.canvasScale.y = originalCanvasScale.y;
                        }
                    }
                })
            ]
        });

        // save state setup
        if (Utils.isFB()) {
            Bento.saveState = SaveState;
        }
        Bento.saveState.setId('combocritters/');
        Bento.saveState.saveKeys = true;

        // the animation warning is starting to annoy me :)
        Sprite.suppressWarnings = true;
        Animation.suppressWarnings = true;
        EventSystem.suppressWarnings = true;

        Bento.setup({
            name: 'ComboCritters',
            canvasId: 'canvas',
            canvasDimension: canvasDimension,
            renderer: 'canvas2d',
            // renderer: 'pixi',
            pixelSize: pixelSize,
            subPixel: true,
            preventContextMenu: true,
            reload: {
                simple: 'mouseDown-right',
                assets: 'buttonDown-1',
                jump: 'buttonDown-2'
            },
            manualResize: true,
            dev: false
        }, function () {
            // resize canvas/viewport dynamically
            // this is done after Bento setup, or the canvas will start blurry
            console.log(Utils);
            if (!Utils.isCocoonJs()){
                window.addEventListener('resize', onResize, false);
                window.addEventListener('orientationchange', onResize, false);
            }
            onResize();
            Bento.objects.attach(viewOffset);

            if (Utils.isFB()) {
                window.FBInstant.player.getDataAsync(['save']).then(function (data) {
                    SaveState.setData(data.save || {});
                    Init();
                });
            } else {
                CocoonMigration.migrateToNativeStorage({
                    doOnce: true, // will rename ls.cdf afterwards
                    onKey: function (key, valueAsString) {
                        // note: key will include the id if present, for example "beatstreet/coins"
                        // return true to ignore key for migration
                        return false;
                    },
                    onComplete: function (error, keysMigrated) {
                        // completion is asynchronous!
                        // note: make sure migrate from Cocoon first BEFORE you call NativeStorage.init() (--> so call NativeStorage.init here)
                        NativeStorage.init({
                            migrateFromLocalStorage: true,
                            onComplete: function () {
                                Init();
                            },
                        });
                    }
                });
            }
        });
    });
};

window.patchGame = function () {
    var hasStarted = false;

    // window.patcher('http://localhost:8000/SkyChasers/misc/patch','v0.6.1', function (res) { // <- for testing
    window.patcher('http://d3q8vwl9xlq0g.cloudfront.net/combocritters/data', 'v' + window.VERSION, function (res) {
        if (res) {
            console.log('Ended with', res);
        }
        if (!hasStarted) {
            window.startGame();
            hasStarted = true;
        }
    });
};

document.addEventListener('deviceready', function () {
    // native app version: undefine FBInstant if it exists
    if (window.FBInstant) {
        window.FBInstant = undefined;
    }
    // proceed to patcher
    window.patchGame();
}, false);

