/**
 * Asks for a rating once
 */
bento.define('gui/askrating', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'gui/dialog'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    ClickButton,
    Counter,
    Text,
    EventSystem,
    Utils,
    Tween,
    Dialog
) {
    'use strict';
    return function (settings) {
        var entity;
        var oldPauseLevel = Bento.objects.isPaused();
        
        if (Bento.saveState.load('hasRatedV' + window.VERSION, false)) {
            return null;
        }
        if (Utils.isFB() || Utils.WEB) {
            return null;
        }
        
        Bento.objects.pause(Utils.PauseLevels.ALWAYS);
        entity = new Dialog({
            name: 'askRating',
            canClose: true,
            type: 'yesno',
            pauseLevel: Utils.PauseLevels.ALWAYS,
            text: Utils.Localization.getText('askLike').replace('{TITLE}', Utils.Localization.getText('title')),
            imageYes: 'gui/dialog/buttonthumbsup',
            imageNo: 'gui/dialog/buttonthumbsdown',
            onYes: function () {
                // redirect to app store
                var link = Utils.getStoreLink();
                if (window.Cocoon) {
                    window.Cocoon.App.openURL(link);
                }
                Bento.saveState.save('hasRatedV' + window.VERSION, true);
            },
            onNo: function () {
                // redirect to support email
                var link = 'mailto:support@lucky-kat.com';
                if (window.Cocoon) {
                    window.Cocoon.App.openURL(link);
                }
                Bento.saveState.save('hasRatedV' + window.VERSION, true);
            },
            onClose: function () {
                Bento.saveState.save('askedRatingV' + window.VERSION, true);
                
                // resume
                Bento.objects.pause(oldPauseLevel);
            }
        });

        return entity;
    };
});