/**
 *Pause menu in battle
 */
bento.define('gui/battle/menu', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'gui/coincounter',
    'gui/dialog',
    'gui/togglemusic',
    'gui/togglesound'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    ClickButton,
    Counter,
    Text,
    EventSystem,
    Utils,
    Tween,
    CoinCounter,
    Dialog,
    ToggleMusic,
    ToggleSound
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var entity = new ClickButton({
            z: Utils.Layers.GUI,
            sfx: 'sfx_button',
            name: 'battlePauseButton',
            image: Bento.assets.getImage('gui/buttonflee'),
            frameCountX: 2,
            frameCountY: 2,
            animations: {
                up: {
                    frames: [0]
                },
                down: {
                    frames: [1]
                },
                inactive: {
                    frames: [0]
                }
            },
            position: new Vector2(16, 12),
            float: true,
            onClick: function () {
                // attach menu
                // Bento.objects.pause(Utils.PauseLevels.MENU);
                // Bento.objects.attach(menu);

                // if (settings.onClick) {
                //     settings.onClick();
                // }

                // note: simply replaced the menu button with the flee button
                // so the code in the flee button can be reused
                var pauseLevel = Bento.objects.isPaused();
                Bento.objects.pause(Utils.PauseLevels.ALWAYS);
                new Dialog({
                    name: 'flee',
                    text: Utils.Localization.getText('flee'),
                    type: 'yesno',
                    pauseLevel: Utils.PauseLevels.ALWAYS,
                    onNo: function () {
                        // nothing happens
                    },
                    onYes: function () {
                        flee();
                    },
                    onClose: function () {
                        // resume
                        Bento.objects.pause(pauseLevel);
                    }
                });
            }
        }).attach(
            // back button pressed
            new Utils.BackButton(function (data) {
                data.entity.doCallback();
            })
        );
        var menu = new Entity({
            z: Utils.Layers.GUI + 1,
            name: 'menu',
            position: new Vector2(viewport.width / 2, viewport.height / 2),
            updateWhenPaused: Utils.PauseLevels.MENU,
            float: true,
            components: [

            ]
        });
        var overlay = new Entity({
            name: 'overlay',
            position: new Vector2(0, 0),
            originRelative: new Vector2(0.5, 0.5),
            components: [
                new Sprite({
                    imageName: 'fade'
                })
            ]
        });
        var buttonResume = new ClickButton({
            z: Utils.Layers.GUI,
            sfx: 'sfx_button',
            image: Bento.assets.getImage('worldmap/buttonexitmenu'),
            frameCountX: 2,
            frameCountY: 1,
            animations: {
                up: {
                    frames: [0]
                },
                down: {
                    frames: [1]
                },
                inactive: {
                    frames: [0]
                }
            },
            position: new Vector2(-viewport.width / 2 + 12, -viewport.height / 2 + 10),
            float: true,
            updateWhenPaused: Utils.PauseLevels.MENU,
            onClick: function () {
                // detach menu
                Bento.objects.resume();
                Bento.objects.remove(menu);
            }
        });
        var toggleMusic = new ToggleMusic({
            updateWhenPaused: Utils.PauseLevels.MENU,
            position: new Vector2(-viewport.width / 2 + 12, -viewport.height / 2 + 40)
        });
        var toggleSound = new ToggleSound({
            updateWhenPaused: Utils.PauseLevels.MENU,
            position: new Vector2(-viewport.width / 2 + 12, -viewport.height / 2 + 40 + 24)
        });
        var flee = function () {
            var myCoins = Bento.saveState.load('myCoins', 0);
            var amountOfFlees; // hehe, fleas

            if (myCoins >= cost) {
                myCoins -= cost;
                Utils.trackCoins(-cost, 'flee');
                Bento.saveState.save('myCoins', myCoins);
                EventSystem.fire('updateCoins');
            } else {
                Bento.audio.playSound('sfx_error');
                return;
            }

            if (!settings.isTwitterBattle) {
                Utils.saveTeam();
                Bento.screens.show('screens/worldmap', {
                    battled: true
                });
            } else {
                Bento.screens.show('screens/twitter');
            }
            Bento.objects.resume();

            amountOfFlees = Bento.saveState.load('amountOfFlees', 0);
            // Achievement: Flee from 5 battles
            if (++amountOfFlees >= 5)
                Utils.postAchievement('CgkI--zjoo8UEAIQCg', function () {});
            Bento.saveState.save('amountOfFlees', amountOfFlees);
        };
        var buttonFlee = new ClickButton({
            sfx: 'sfx_button',
            image: Bento.assets.getImage('gui/buttonflee'),
            frameCountX: 2,
            frameCountY: 2,
            animations: {
                up: {
                    frames: [0]
                },
                down: {
                    frames: [1]
                },
                inactive: {
                    frames: [2]
                }
            },
            position: new Vector2(0, 0),
            updateWhenPaused: Utils.PauseLevels.MENU,
            onClick: flee
        });
        var cost = 0; //settings.fleeCost || 0;
        var costCounter = new CoinCounter({
            position: new Vector2(0, -20),
            align: 'center',
            value: '$'
        });

        if (cost > 0) {
            costCounter.setValue('$' + cost);
            buttonFlee.attach(costCounter);
        }

        overlay.alpha = 0.75;
        menu.attach(overlay);
        menu.attach(buttonResume);
        menu.attach(buttonFlee);
        menu.attach(toggleMusic);
        menu.attach(toggleSound);

        return entity;
    };
});