/**
 * Horizontally centered list of entities
 */
bento.define('gui/centeredlist', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    Utils,
    Tween
) {
    'use strict';
    return function (settings) {
        var viewport = Bento.getViewport(),
            items = [],
            spacing = settings.spacing || 8,
            entity = new Entity({
                z: settings.z,
                name: 'centeredList',
                position: settings.position || new Vector2(0, 0),
                originRelative: new Vector2(0, 0),
                components: []
            }),
            width = 0,
            reposition = function () {
                var i = 0;
                var totalWidth = (width + spacing) * (items.length - 1);
                var item;
                for (i = 0; i < items.length; ++i) {
                    item = items[i];
                    item.position = new Vector2(-totalWidth / 2 + (width + spacing) * i, 0);
                }
            },
            addItem = function (item) {
                width = item.dimension.width;
                items.push(item);
                entity.attach(item);
                reposition();
                return item;
            };

        entity.extend({
            iterate: function (callback) {
                var i, l;
                var stop = false;
                var breakLoop = function () {
                    stop = true;
                };
                for (i = 0, l = items.length; i < l; ++i) {
                    callback(items[i], i, l, breakLoop);
                    
                    if (stop) {
                        return;
                    }
                }
            },
            length: function () {
                return items.length;
            },
            get: function (index) {
                return items[index];
            },
            getOffset: function () {
                return new Vector2((items.length - 1) * spacing / 2, 0);
            },
            addItem: addItem
        });

        return entity;
    };
});