/**
 * Module that handles watching video for coins and giving the reward
 */
bento.define('gui/coinsforad', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'ads/heyzap',
    'gui/dialog',
    'modules/videoadtimer',
    'modules/forcedadtimer',
    'gui/rewardpopup'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    ClickButton,
    Counter,
    Text,
    EventSystem,
    Utils,
    Tween,
    HeyZap,
    Dialog,
    VideoAdTimer,
    ForcedAdTimer,
    RewardPopup
) {
    'use strict';

    var oldPauseLevel = 0;
    var rewardAmount = Bento.assets.getJson('creatures/rewardamount');
    var module = {
        getReward: function (success, onSuccess) {
            var unlockedRarity = Bento.saveState.load('unlockedRarity', 3);
            var reward = rewardAmount[unlockedRarity] + Utils.getRandom(rewardAmount[unlockedRarity]);
            var multiplier = 10;
            var extra = 0;

            if (!success) {
                Bento.objects.pause(oldPauseLevel);
                return;
            }

            // popup with reward
            new RewardPopup({
                trackCoins: false,
                reward: reward,
                pauseLevel: Bento.objects.isPaused() + 1,
                onClose: function () {
                    Bento.objects.pause(oldPauseLevel);

                    if (onSuccess) {
                        onSuccess();
                    }
                    // reset video timer
                    VideoAdTimer.reset();
                    ForcedAdTimer.reset();
                }
            });
        },
        watchAd: function (onSuccess) {
            var totalAdsWatched = Bento.saveState.load('totalAdsWatched', 0);
            var coinAdsWatched = Bento.saveState.load('coinAdsWatched', 0);
            var forcedAdsWatched = Bento.saveState.load('forcedAdsWatched', 0);
            var rerollAdsWatched = Bento.saveState.load('rerollAdsWatched', 0);

            oldPauseLevel = Bento.objects.isPaused();
            Bento.objects.pause(oldPauseLevel + 1);
            if (Utils.EveryPlay.isRecording()) {
                Utils.EveryPlay.pause();
            }

            // don't show ads on DGG
            if (Utils.DGG_DEMO) {
                window.alert('You are now watching an ad.');
                module.getReward(true, onSuccess);
                return;
            }

            // watch ad
            if (HeyZap.canShowRewarded()) {
                Bento.audio.muteMusic(true);

                ++totalAdsWatched;
                ++coinAdsWatched;

                // user started watching ad
                Utils.Analytics.event('Ads', 'startCoinAd_' + coinAdsWatched);
                Utils.Analytics.event('Ads', 'startTotalAd_' + totalAdsWatched);

                HeyZap.showRewarded(function () {}, function (hasCompleted) {
                    var coins = Math.floor(Bento.saveState.load('myCoins', 0) / 100) * 100;
                    var framesSinceReady = Bento.saveState.load('framesSinceReady', null);
                    // framesSinceReady is updated every 600 frames, starting from 1
                    framesSinceReady = framesSinceReady ? Math.floor((framesSinceReady - 1) / 60) : null;

                    // user finished watching ad
                    Utils.Analytics.event('Ads', 'finishCoinAd_' + coinAdsWatched);
                    Utils.Analytics.event('Ads', 'finishTotalAd_' + totalAdsWatched);

                    Utils.Analytics.event('Ads', 'forCoins_' + coinAdsWatched);
                    Utils.Analytics.event('Ads', 'total_' + totalAdsWatched);

                    if (window.tenjin && Utils.isNativeIos()) {
                        window.tenjin.sendEvent('rewardedAd', function () {}, function () {});
                    }

                    Utils.Firebase.logEvent('watched_coin_ad', {
                        value: 1,
                        forced_ads_watched: forcedAdsWatched,
                        coin_ads_watched: coinAdsWatched,
                        total_ads_watched: totalAdsWatched,
                        reroll_ads_watched: rerollAdsWatched
                    });
                    Utils.Firebase.logEvent('watched_ad', {
                        value: 1,
                        ad_type: 'coin',
                        forced_ads_watched: forcedAdsWatched,
                        coin_ads_watched: coinAdsWatched,
                        total_ads_watched: totalAdsWatched,
                        reroll_ads_watched: rerollAdsWatched
                    });

                    // how many coins did user have when deciding to watch ad for coins
                    Utils.Analytics.event('Ads', 'coinsBefrCoinAd_' + coins);
                    // how much time has passed since ad for coins was ready to be watched
                    if (framesSinceReady) {
                        Utils.Analytics.event('Ads', 'secsAftCnAdRdy_' + framesSinceReady);
                    }

                    Bento.saveState.save('totalAdsWatched', totalAdsWatched);
                    Bento.saveState.save('coinAdsWatched', coinAdsWatched);

                    Bento.audio.muteMusic(Bento.saveState.load('muteMusic', false), true);
                    bento.require(['audia'], function (Audia) {
                        var audioContext = Audia.audioContext;
                        if (audioContext && audioContext.state === 'suspended') {
                            audioContext.resume();
                        }
                    });
                    // module.getReward(true, onSuccess);
                    module.getReward(hasCompleted, onSuccess);
                    Utils.Analytics.event('UI', 'adForCoins');
                    Utils.GooglePlay.submitEvent('CgkI--zjoo8UEAIQAw', 1);
                    if (Utils.EveryPlay.isPaused()) {
                        Utils.EveryPlay.resume();
                    }
                });
            } else {
                if (Utils.isCocoonJS()) {
                    // alert('Sorry, no ad available');
                    new Dialog({
                        name: 'noAd',
                        text: Utils.Localization.getText("noAd"),
                        type: 'yes',
                        onClose: function () {
                            module.getReward(false, onSuccess);
                        }
                    });
                } else if ((Utils.isFB() && Utils.softlaunch)) {
                    // give the user gift without watching ad during softlaunch
                    module.getReward(true, onSuccess);
                } else if (Bento.isDev()) {
                    // simulate watching
                    window.alert('You are now watching an ad.');
                    module.getReward(true, onSuccess);
                } else {
                    new Dialog({
                        name: 'noAd',
                        text: Utils.Localization.getText("noAd"),
                        type: 'yes',
                        onClose: function () {
                            module.getReward(false, onSuccess);
                        }
                    });
                }
            }
        }
    };

    return module;
});