/**
 * Combiner overlay (warning: lots of duplicate code with teamedit)
 */
bento.define('gui/combiner/combiner', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'bento/screen',
    'bento/gui/clickbutton',
    'bento/gui/text',
    'components/debug',
    'components/draggable',
    'components/droptarget',
    'gui/centeredlist',
    'gui/coincounter',
    'gui/storydialog',
    'gui/scrollinglist',
    'gui/team/page',
    'gui/team/slot',
    'entities/simplecard',
    'gui/combiner/result',
    'gui/shop/shop',
    'modules/videoadtimer'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    Utils,
    Tween,
    Screen,
    ClickButton,
    Text,
    Debug,
    Draggable,
    DropTarget,
    CenteredList,
    CoinCounter,
    StoryDialog,
    ScrollingList,
    Page,
    Slot,
    Card,
    Result,
    Shop,
    VideoAdTimer
) {
    'use strict';
    var combinations;
    var creatures;
    return function (settings) {
        var viewport = Bento.getViewport();
        var oldPauseLevel;
        var container = new Entity({
            z: Utils.Layers.TEAMEDIT,
            name: 'combineContainer',
            position: new Vector2(0, 0),
            float: true,
            updateWhenPaused: Utils.PauseLevels.TEAMEDIT,
            components: [

            ]
        });
        var background = new Entity({
            z: 0,
            name: 'background',
            position: new Vector2(viewport.width / 2, viewport.height / 2),
            originRelative: new Vector2(0.5, 0.5),
            components: [new Sprite({
                imageName: 'team/background'
            })]
        });
        var combiner = new Entity({
            z: 1,
            name: 'combiner',
            position: new Vector2(viewport.width / 2, viewport.height / 5 + 1 + 4),
            originRelative: new Vector2(0.5, 0.5),
            components: [new Sprite({
                imageName: 'combiner/combiner'
            })]
        });
        var combinerOverlay = new Entity({
            z: 10,
            name: 'combinerOverlay',
            position: combiner.position.add(new Vector2(17, -25)),
            originRelative: new Vector2(0.5, 0.5),
            components: [new Sprite({
                imageName: 'combiner/combiner_overlay'
            })]
        });
        var combinerCounter = new CoinCounter({
            imageName: 'gui/numberswhite',
            position: new Vector2(-44, -19),
            align: 'center',
            value: '*' + Bento.saveState.load('unlockedRarity', 3)
        });
        var scrollArea = new Rectangle(viewport.x, viewport.y + viewport.height / 3, viewport.width, viewport.height * 2 / 3);
        var returnArea = new Rectangle(0, viewport.height / 3, viewport.width, viewport.height * 2 / 3);

        var buttonExit = new ClickButton({
            z: 3,
            sfx: 'sfx_button',
            image: Bento.assets.getImage('combiner/buttonexit'),
            frameCountX: 2,
            frameCountY: 1,
            animations: {
                up: {
                    frames: [0]
                },
                down: {
                    frames: [1]
                }
            },
            position: new Vector2(12, 10),
            updateWhenPaused: Utils.PauseLevels.TEAMEDIT,
            onClick: function () {
                Bento.objects.pause(oldPauseLevel);
                Bento.objects.remove(container);

                // save critter lists!
                save();

                if (settings.onClose) {
                    settings.onClose();
                }
                EventSystem.fire('closedCombiner');
            }
        }).attach(
            // back button pressed
            new Utils.BackButton(function (data) {
                data.entity.doCallback();
            })
        );
        var coinCost = new CoinCounter({
            position: new Vector2(-28, 29 - 5),
            align: 'center',
            value: '$0'
        }).attach({
            start: function () {
                // coinCost.scale.x = 1.5;
                // coinCost.scale.y = 1.5;
            },
            update: function () {
                if (buttonCombine.isActive()) {
                    coinCost.visible = true;
                } else {
                    coinCost.visible = false;
                }
            }
        });

        var buttonCombine = new ClickButton({
            sfx: 'sfx_button',
            image: Bento.assets.getImage('combiner/buttoncombine'),
            frameCountX: 2,
            frameCountY: 2,
            animations: {
                up: {
                    speed: 0.025,
                    frames: [0, 3]
                },
                down: {
                    frames: [1]
                },
                inactive: {
                    frames: [2]
                }
            },
            position: new Vector2(-28, 29),
            updateWhenPaused: Utils.PauseLevels.TEAMEDIT,
            onClick: function () {
                // enough coins?
                var myCoins = Bento.saveState.load('myCoins', 0);
                var totalCombinations = Bento.saveState.load('totalCombinations', 0);
                var newCritter = !Utils.hasInCollection(predictCard.getId());

                // show '!' new icon on Critterdex
                if (newCritter) {
                    Bento.objects.get('menuButton', function (obj) {
                        obj.showNewIcon();
                    });
                }

                // remove any hand
                buttonCombine.getComponent('hand', function (hand) {
                    buttonCombine.remove(hand);
                });

                // is unlocked?
                if (predictionProp && predictionProp.rarity > Bento.saveState.load('unlockedRarity', 3)) {
                    new StoryDialog({
                        text: Utils.Localization.getText('lockedRarity'),
                        type: 'yes',
                        canClose: false,
                        onClose: function () {}
                    });
                    return;
                }

                if (myCoins < comboCost) {
                    // not enough coins!
                    Bento.audio.playSound('sfx_error');
                    new Shop({});
                    // allow video ad when user doesnt have enough coins??
                    // VideoAdTimer.force();
                    return;
                }

                // subtract cost
                myCoins -= comboCost;
                Utils.trackCoins(-comboCost, 'combine');
                Bento.saveState.save('myCoins', myCoins);
                EventSystem.fire('updateCoins');
                // track the amount of combinations
                ++totalCombinations;
                Bento.saveState.save('totalCombinations', totalCombinations);
                Utils.Analytics.event('Critters', 'amntOfCmbntns_' + totalCombinations);
                Utils.Firebase.logEvent('combinations', {
                    value: 1,
                    total_combinations: totalCombinations,
                    critter1: slot1.getItem().getId(),
                    critter2: slot2.getItem().getId(),
                    result: predictCard.getId(),
                    health1: Math.max(0, slot1.getItem().getProperty('currentHp') / slot1.getItem().getProperty('hp')),
                    health2: Math.max(0, slot2.getItem().getProperty('currentHp') / slot2.getItem().getProperty('hp')),
                    cost: comboCost
                });

                new Result({
                    critter1: slot1.getItem().getId(),
                    critter2: slot2.getItem().getId(),
                    result: predictCard.getId(),
                    resultName: Utils.Localization.getName(predictCard.getProperty('id')),
                    newCritter: newCritter,
                    onEnd: function () {
                        //??
                    }
                });
                makeResult();
            }
        });
        var buttonLeft = new ClickButton({
            sfx: 'sfx_button',
            z: 3,
            image: Bento.assets.getImage('team/buttonarrowleft'),
            frameCountX: 1,
            frameCountY: 1,
            animations: {
                up: {
                    frames: [0]
                },
                down: {
                    frames: [0]
                },
                inactive: {
                    frames: [0]
                }
            },
            updateWhenPaused: Utils.PauseLevels.TEAMEDIT,
            position: new Vector2(6, viewport.height / 3 * 2 + 8),
            onClick: function () {
                if (Draggable.isDragging()) {
                    return;
                }
                scrollList.snapTo(scrollList.getSelection() - 1);
            }
        });
        var buttonRight = new ClickButton({
            sfx: 'sfx_button',
            z: 3,
            image: Bento.assets.getImage('team/buttonarrowright'),
            frameCountX: 1,
            frameCountY: 1,
            animations: {
                up: {
                    frames: [0]
                },
                down: {
                    frames: [0]
                },
                inactive: {
                    frames: [0]
                }
            },
            position: new Vector2(viewport.width - 6, viewport.height / 3 * 2 + 8),
            updateWhenPaused: Utils.PauseLevels.TEAMEDIT,
            onClick: function () {
                if (Draggable.isDragging()) {
                    return;
                }
                scrollList.snapTo(scrollList.getSelection() + 1);
            }
        });
        var scrollList = new ScrollingList({
            z: 2,
            updateWhenPaused: Utils.PauseLevels.TEAMEDIT,
            position: new Vector2(viewport.width / 2 - 2, viewport.height / 2 + 16),
            direction: 'x',
            spacing: viewport.width,
            snap: true,
            snapMinSlideSpeed: 0.5,
            snapSpeed: 0.2,
            damping: 0.95,
            area: scrollArea,
            onSnap: function (item, index) {
                if (item) {
                    item.activate();
                }
                pagesDots.iterate(function (item, i) {
                    if (i === index) {
                        item.alpha = 1;
                    } else {
                        item.alpha = 0.5;
                    }
                });
            },
            onLoseFocus: function (item) {
                if (item) {
                    item.deactivate();
                }
            },
            onScrollStart: function () {
                // shouldnt scroll when dragging
                if (Draggable.isDragging()) {
                    scrollList.cancel();
                }
            }
        });
        var pagesDots = new CenteredList({
            position: new Vector2(viewport.width / 2, viewport.height - 4),
            spacing: 2
        });
        var slot1;
        var slot2;
        var playerCritters;
        var teamCritters;
        var predictCard;
        var team = Utils.getTeam();
        var myCritters = Utils.getCritters();
        var changed = false;
        var combineCount = Bento.saveState.load('combineCount', 0);
        var comboCost;
        var wrong = new Entity({
            name: 'wrong',
            position: new Vector2(0, 0),
            originRelative: new Vector2(0.5, 0.5),
            components: [
                new Sprite({
                    image: Bento.assets.getImage('combiner/tierlock')
                })
            ]
        });
        var showArrows = function (card) {
            var arrowRight = new Entity({
                name: 'arrowRight',
                position: new Vector2(-34, 20),
                originRelative: new Vector2(0.5, 0.5),
                components: [new Sprite({
                    imageName: 'gui/arrowright'
                })]
            });
            var textHp = new Text({
                position: new Vector2(0, 10),
                text: Utils.Localization.getText('health'),
                font: 'volter',
                fontSize: 9,
                fontSettings: Utils.FontSettings.get('font'),
                fontColor: '#ffffff',
                align: 'center',
                textBaseline: 'top'
            });
            var arrowLeft = new Entity({
                name: 'arrowLeft',
                position: new Vector2(33, 20),
                originRelative: new Vector2(0.5, 0.5),
                components: [new Sprite({
                    imageName: 'gui/arrowleft'
                })]
            });
            var textAttack = new Text({
                position: new Vector2(0, 10),
                text: Utils.Localization.getText('attack'),
                font: 'volter',
                fontSize: 9,
                fontSettings: Utils.FontSettings.get('font'),
                fontColor: '#ffffff',
                align: 'center',
                textBaseline: 'top'
            });
            arrowRight.attach(textHp);
            card.attach(arrowRight);
            arrowLeft.attach(textAttack);
            card.attach(arrowLeft);
        };
        var popoutPredictCard = function () {
            var clone = new Card(predictionProp, {
                hidden: true
            });
            clone.position = predictCard.getWorldPosition();
            clone.z = Utils.Layers.FADE - 1;
            Bento.objects.attach(clone);
            // hide old one
            predictCard.visible = false;

            // move towards middle
            new Tween({
                from: clone.position,
                to: viewport.getCenter().add(new Vector2(0, -32)),
                'in': 30,
                ease: 'cuberoot',
                updateWhenPaused: Utils.PauseLevels.ALWAYS,
                'do': function (v, t) {
                    clone.position = v;
                },
                onComplete: function () {
                    showArrows(clone);
                }
            });

            // resize
            new Tween({
                from: 1,
                to: 1.2,
                'in': 30,
                ease: 'linear',
                updateWhenPaused: Utils.PauseLevels.ALWAYS,
                'do': function (v, t) {
                    clone.scale.x = v;
                    clone.scale.y = v;
                }
            });

            return clone;
        };
        var moveBackPredictCard = function (clone) {
            clone.getComponent('arrowLeft', function (arrow) {
                clone.remove(arrow);
            });
            clone.getComponent('arrowRight', function (arrow) {
                clone.remove(arrow);
            });
            // move back
            new Tween({
                from: clone.position,
                to: predictCard.getWorldPosition(),
                'in': 30,
                ease: 'cuberoot',
                updateWhenPaused: Utils.PauseLevels.ALWAYS,
                'do': function (v, t) {
                    clone.position = v;
                },
                onComplete: function () {
                    Bento.objects.remove(clone);
                    if (predictCard) {
                        predictCard.visible = true;
                    }
                }
            });

            // resize
            new Tween({
                from: clone.scale.x,
                to: 1,
                'in': 30,
                ease: 'linear',
                updateWhenPaused: Utils.PauseLevels.ALWAYS,
                'do': function (v, t) {
                    clone.scale.x = v;
                    clone.scale.y = v;
                }
            });
        };
        var predictionProp;
        var predictCardClone;
        var predictSlot = new Entity({
            z: 2,
            name: 'predictSlot',
            position: combiner.position.add(new Vector2(52, -13)),
            originRelative: new Vector2(0.5, 0.5),
            components: [],
            updateWhenPaused: Utils.PauseLevels.TEAMEDIT,
        }).attach({
            name: 'predictBehavior',
            prediction: null,
            update: function () {
                var slot1Card = slot1.getItem();
                var slot2Card = slot2.getItem();
                var dialog;
                var pauseLevel = Bento.objects.isPaused();
                var combi1 = null;

                if (changed) {
                    // update when slots changed
                    changed = false;
                    // wrong.visible = false;
                    if (predictCard) {
                        predictSlot.remove(predictCard);
                        predictCard.remove(wrong);
                    }
                    if (wrong.parent) {
                        wrong.parent.remove(wrong);
                    }
                    buttonCombine.setActive(false);
                    predictCard = null;
                    predictionProp = null;
                }

                if (slot1Card && slot2Card) {
                    if (!predictCard) {
                        // check if this combination exist
                        combi1 = combinations[slot1Card.getId()];
                        if (combi1) {
                            this.prediction = combi1[slot2Card.getId()];
                        } else {
                            this.prediction = null;
                        }
                        if (this.prediction) {
                            predictionProp = creatures.creatures[this.prediction];
                            predictCard = new Card(predictionProp, {
                                hidden: !Utils.hasInCollection(this.prediction)
                            });
                            predictSlot.attach(predictCard);
                            buttonCombine.setActive(true);
                            predictCard.attach(wrong);


                            // can combine into this tier?
                            if (predictionProp.rarity > Bento.saveState.load('unlockedRarity', 3)) {
                                // buttonCombine.setActive(false);
                                wrong.visible = true;

                                // show a tutorial if needed
                                if (Bento.saveState.load('rarityTutorial', false) === false) {
                                    dialog = new StoryDialog({
                                        text: Utils.Localization.getText('lockedRarity'),
                                        type: 'yes',
                                        canClose: false,
                                        onClose: function () {
                                            // restore pauselevel
                                            Bento.saveState.save('rarityTutorial', true);
                                        }
                                    });
                                }
                            } else {
                                // buttonCombine.setActive(true);
                                wrong.visible = false;
                            }

                            if (Bento.saveState.load('combineCount', 0) === 0) {
                                // combine tutorial
                                new StoryDialog({
                                    text: [
                                        Utils.Localization.getText('firstCombine')[0],
                                        Utils.Localization.getText('firstCombine')[1]
                                        .replace("{HEALTH}", predictionProp.hp)
                                        .replace("{ATTACK}", predictionProp.attack)
                                    ],
                                    hideCat: true,
                                    onNextPage: function (page) {
                                        if (page === 1) {
                                            // popout predict card
                                            predictCardClone = popoutPredictCard();
                                        }
                                    },
                                    onClose: function () {
                                        // place hand on combine button
                                        buttonCombine.attach(getHand());

                                        moveBackPredictCard(predictCardClone);
                                    }
                                });
                            }

                            // show cost
                            comboCost = Bento.assets.getJson('creatures/combinecost')[predictionProp.rarity] || 100;
                            if (Bento.saveState.load('sawTutorial2', false) === false) {
                                // free combinations for tutorial
                                comboCost = 0;
                            }
                            new Tween({
                                from: 0,
                                to: comboCost,
                                'in': 20,
                                ease: 'linear',
                                updateWhenPaused: Utils.PauseLevels.TEAMEDIT,
                                'do': function (v, t) {
                                    coinCost.setValue('$' + Math.floor(v));
                                }
                            });
                        }
                    }
                } else {
                    if (predictCard) {
                        predictSlot.remove(predictCard);
                        buttonCombine.setActive(false);
                        predictCard = null;
                        this.prediction = null;
                        coinCost.setValue('$0');
                    }
                }

                // predictcard should glow
                if (predictCard && this.prediction) {
                    predictCard.alpha = 0.9;
                    predictCard.getComponent('avatar', function (avatar) {
                        avatar.alpha = 0.5 + 0.5 * Math.sin(predictCard.timer / 20);
                    });
                }
            }
        });
        var makeDraggableCard = function (critterProp) {
            var card;
            var addDraggable = function (card) {
                var draggable = new Draggable({
                    dragInit: function () {
                        scrollList.cancel();
                    },
                    dragStart: function (data) {
                        var parent = card.parent;
                        data.entity.scale.x = 0.9;
                        data.entity.scale.y = 0.9;

                        // check parent
                        if (parent.name === 'page') {
                            // move card to top
                            parent.moveComponentTo(card, parent.components.length);
                            // move scrollist to top
                            container.moveComponentTo(scrollList, container.components.length);
                            // overlay
                            container.moveComponentTo(combinerOverlay, container.components.length);
                            container.moveComponentTo(buttonExit, container.components.length);
                        } else if (parent.name === 'slot') {
                            // parent is slot
                            // move slots to top
                            if (parent === slot1) {
                                container.moveComponentTo(slot1, container.components.length);
                            } else if (parent === slot2) {
                                container.moveComponentTo(slot2, container.components.length);
                            }
                            // overlay
                            container.moveComponentTo(combinerOverlay, container.components.length);
                            container.moveComponentTo(buttonExit, container.components.length);
                        }
                    },
                    dragEnd: function (data) {
                        var distance = data.startPosition.subtract(data.endPosition).magnitude();
                        data.entity.scale.x = 1;
                        data.entity.scale.y = 1;

                        // hardly moved: wiggle
                        if (distance < 1) {
                            new Tween({
                                from: 0,
                                to: Math.PI / 64,
                                'in': 30,
                                ease: 'sin',
                                alpha: 2,
                                updateWhenPaused: Utils.PauseLevels.TEAMEDIT,
                                'do': function (v, t) {
                                    data.entity.rotation = v;
                                }
                            });
                        }
                        if (!data.droppedSlot && !data.droppedReturn) {
                            // tween back
                            draggable.deactivate();
                            data.entity.position = data.endPosition.clone();
                            new Tween({
                                from: data.endPosition,
                                to: data.startPosition,
                                'in': 20,
                                ease: 'elastic',
                                alpha: -8,
                                beta: 1,
                                updateWhenPaused: Utils.PauseLevels.TEAMEDIT,
                                'do': function (v, t) {
                                    data.entity.position = v;
                                },
                                onComplete: function () {
                                    draggable.activate();
                                }
                            });
                        }
                    }
                });
                card.boundingBox = new Rectangle(-22, -23, 42, 35);
                // card.boundingBox.grow(-8);
                card.updateWhenPaused = Utils.PauseLevels.TEAMEDIT;
                // add draggable component
                card.attach(draggable);
                card.attach({
                    name: 'checkDrag',
                    update: function () {
                        if (scrollList.isScrolling()) {
                            draggable.canDrag = false;
                        } else {
                            draggable.canDrag = true;
                        }
                    }
                });
            };
            card = new Card(critterProp);
            addDraggable(card);
            return card;
        };
        var makeCritterList = function (team, critters) {
            var i = 0;
            var card;
            var currentPage;

            var addPage = function () {
                var page = new Page({
                    maxItems: 9,
                    spacing: new Vector2(2, 2)
                });
                var addDot = function () {
                    var dotSprite = new Sprite({
                        imageName: 'gui/dot'
                    });
                    var dot = new Entity({
                        originRelative: new Vector2(0.5, 0.5),
                        components: [dotSprite]
                    });
                    if (i !== 0) {
                        dot.alpha = (0.5);
                    }
                    pagesDots.addItem(dot);
                };
                // add to list
                scrollList.addItem(page);
                addDot();

                // set current
                if (currentPage && pages.length > 1) {
                    currentPage.deactivate();
                }

                currentPage = page;

                return page;
            };
            var pages = [];
            var returnCardArea = new Entity({
                name: 'returnCardArea',
                boundingBox: returnArea,
                updateWhenPaused: Utils.PauseLevels.TEAMEDIT,
                components: [

                    new DropTarget({
                        onDrop: function (data) {
                            var parent = data.entity.parent;
                            var incomingItem = data.entity;
                            var page;
                            var worldPosition;
                            if (parent.name === 'slot') {
                                // put monster back in current page
                                page = pages[scrollList.getSelection()];
                                if (!page.isFull()) {
                                    // incomingItem.position = data.endPosition; // copy end position (because draggables snap back)
                                    // worldPosition = incomingItem.getWorldPosition();
                                    parent.removeItem(incomingItem);
                                    // incomingItem.position = page.getLocalPosition(worldPosition); // TODO: doesnt take into account the scroll??
                                    page.addItem(incomingItem);
                                }
                                // add 'droppedReturn' property
                                data.droppedReturn = true;
                            }
                        }
                    })
                ]
            });
            container.attach(returnCardArea);
            // go through the lists and add items into pages
            for (i = 0; i < team.length; ++i) {
                if (!currentPage || currentPage.isFull()) {
                    pages.push(addPage());
                }
                if (team[i] === null) {
                    continue;
                }
                // skip own critter during tutorial
                if (Bento.saveState.load('sawTutorial2', false) === false) {
                    if (team[i] && (team[i].id !== Utils.tutorialEnemy && team[i].id !== Utils.tutorialEnemy2)) {
                        continue;
                    }
                }

                // make card
                card = makeDraggableCard(team[i]);

                currentPage.addItem(card);
            }
            for (i = 0; i < critters.length; ++i) {
                if (!currentPage || currentPage.isFull()) {
                    pages.push(addPage());
                }
                // skip own critter during tutorial
                if (Bento.saveState.load('sawTutorial2', false) === false) {
                    if (critters[i].id !== Utils.tutorialEnemy && critters[i].id !== Utils.tutorialEnemy2) {
                        continue;
                    }
                }

                // make card
                card = makeDraggableCard(critters[i]);

                currentPage.addItem(card);
            }
            if (pages.length <= 1) {
                pagesDots.visible = false;
            }
        };
        var makeSlot = function (settings) {
            var slot = new Slot({
                onDrop: function (data) {
                    var incomingItem = data.entity;
                    var oldItem;
                    var page;
                    var otherSlot;
                    var item = slot.getItem();
                    // incomingItem was dropped in here
                    // does it originate from the scrollist?
                    if (incomingItem.parent.name === 'page') {
                        page = incomingItem.parent;

                        changed = true;

                        if (item) {
                            // already has item? remove that and put it in the page, swapping it
                            page.swapItems(incomingItem, slot.removeItem(), slot.getWorldPosition());
                        } else {
                            // only put item out of page
                            page.removeItem(incomingItem);
                        }
                        // add to slot
                        incomingItem.position.x = 0;
                        incomingItem.position.y = 0;
                        slot.setItem(incomingItem);
                    } else if (incomingItem.parent.name === 'slot') {
                        otherSlot = incomingItem.parent;
                        // incomingItem was dragged from other slot
                        otherSlot.removeItem();
                        // swap items
                        if (item) {
                            oldItem = item;
                            slot.removeItem();
                            otherSlot.setItem(oldItem);
                        }

                        // add to slot
                        incomingItem.position.x = 0;
                        incomingItem.position.y = 0;
                        slot.setItem(incomingItem);
                    }
                    // add 'droppedSlot' property
                    data.droppedSlot = true;
                }
            });
            slot.z = settings.z;
            slot.position = settings.position;
            return slot;
        };
        var save = function () {
            Utils.saveCritters();
            Utils.saveTeam();
        };
        var makeResult = function () {
            var resultCard = makeDraggableCard(Utils.clone(predictCard.getProperties()));
            var teamPosition = -1;
            var discoveredByCombining = 0;
            var searchTeam = function () {
                var i = 0;
                var c1 = slot1.getItem().getProperties();
                var c2 = slot2.getItem().getProperties();

                for (i = 0; i < team.length; ++i) {
                    if (team[i] === c1 || team[i] === c2) {
                        teamPosition = i;
                        team[i] = null;
                    }
                }
                // put result in team
                if (teamPosition >= 0) {
                    team[teamPosition] = resultCard.getProperties();
                } else {
                    // put result in inventory
                    myCritters.push(resultCard.getProperties());
                }
                // remove cards from inventory
                Utils.removeCritter(c1);
                Utils.removeCritter(c2);
            };
            if (!Utils.hasInCollection(resultCard.getProperty('id'))) {
                discoveredByCombining = Bento.saveState.load('discoveredByCombining', 0);
                // Achievement: Discover 3 new critters by combining
                if (++discoveredByCombining >= 3)
                    Utils.postAchievement('CgkI--zjoo8UEAIQBQ', function () {});
                Bento.saveState.save('discoveredByCombining', discoveredByCombining);
            }
            // add to collection
            Utils.addToCollection(resultCard.getProperty('id'));

            // if one of the slots was in your team, the result should go in
            searchTeam();

            // remove slot1 and 2 cards
            slot1.removeItem();
            slot2.removeItem();

            resultCard.position = predictSlot.position.clone();

            // find page that can add a card
            scrollList.iterate(function (page, i, l, breakLoop) {
                if (!page.isFull()) {
                    resultCard.position.x = 32;
                    resultCard.position.y = -100;
                    page.addItem(resultCard);
                    scrollList.snapTo(i);
                    breakLoop();
                }
                // failed to add card? note: its not possible right now because
                // you always have to drag cards to the slot
            });

            // save at end of combining
            save();
            combineCount += 1;
            Bento.saveState.save('combineCount', combineCount);
        };
        var getHand = function () {
            return new Entity({
                name: 'hand',
                position: new Vector2(0, 8),
                originRelative: new Vector2(0.5, 0),
                updateWhenPaused: Utils.PauseLevels.CAUGHT,
                components: [new Sprite({
                    imageName: 'map/hand'
                })]
            }).attach({
                update: function (data) {
                    var hand = data.entity;
                    hand.position.y = 4 + 4 * Math.abs(Math.sin(hand.timer / 20));
                }
            });
        };
        slot1 = makeSlot({
            z: 2,
            position: combiner.position.add(new Vector2(-53, -12))
        });
        slot2 = makeSlot({
            z: 2,
            position: combiner.position.add(new Vector2(0, -12))
        });

        // combiner.attach(buttonExit);
        combiner.attach(buttonCombine);
        combiner.attach(coinCost);
        // combinerOverlay.attach(combinerCounter);
        buttonCombine.setActive(false);


        container.attach(background);
        container.attach(combiner);
        container.attach(slot1);
        container.attach(slot2);
        container.attach(predictSlot);
        container.attach(combinerOverlay);
        container.attach(scrollList);
        container.attach(pagesDots);
        container.attach(buttonExit);

        // tutorial
        if (Bento.saveState.load('sawTutorial2', false) === false) {
            buttonExit.attach(getHand());
            container.attach({
                update: function () {
                    if (combineCount > 0) {
                        buttonExit.visible = true;
                        buttonExit.setActive(true);
                    } else {
                        buttonExit.visible = false;
                        buttonExit.setActive(false);
                    }
                }
            });
        }

        makeCritterList(team, myCritters);

        if (scrollList.getItems().length > 1) {
            container.attach(buttonLeft);
            container.attach(buttonRight);
        }
        // load jsons
        if (!combinations) {
            combinations = Bento.assets.getJson('creatures/combinations');
        }
        if (!creatures) {
            creatures = Bento.assets.getJson('creatures/creatures');
        }

        oldPauseLevel = Bento.objects.isPaused();
        Bento.objects.pause(Utils.PauseLevels.TEAMEDIT);
        Utils.Analytics.trackView('combiner');

        return container;
    };
});