bento.define('gui/combiner/confetti', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/utils',
    'bento/tween'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    Utils,
    Tween
) {
    'use strict';
    return function (settings) {
        var viewport = Bento.getViewport();
        var pauseLevel = settings ? settings.pauseLevel || Utils.PauseLevels.COMBINERESULT : Utils.PauseLevels.COMBINERESULT;
        var entity = new Entity({
            z: settings ? (settings.z || 0) : 0,
            name: 'confetti',
            position: settings.position || new Vector2(0, 0),
            originRelative: new Vector2(0, 0),
            float: true,
            updateWhenPaused: pauseLevel,
            init: function () {
                var self = this;
                new Tween({
                    from: 0,
                    to: 1,
                    'in': 10,
                    ease: 'linear',
                    updateWhenPaused: pauseLevel,
                    'do': function (v, t) {
                        self.alpha = v;
                    },
                    onComplete: function () {
                        Utils.timeout(180, function () {
                            new Tween({
                                from: 1,
                                to: 0,
                                'in': 30,
                                ease: 'linear',
                                updateWhenPaused: pauseLevel,
                                'do': function (v, t) {
                                    self.alpha = v;
                                },
                                onComplete: function () {
                                    Bento.objects.remove(self);
                                }
                            });
                        }, pauseLevel);
                    }
                });
            }
        });
        var spawnConfetti = function () {
            var confetti = new Entity({
                position: new Vector2(Utils.getRandom(viewport.width), Utils.getRandom(viewport.height)),
                originRelative: new Vector2(0.5, 0.5),
                components: [new Sprite({
                    image: Bento.assets.getImage('gui/confetti'),
                    frameCountX: 3,
                    animations: {
                        'default': {
                            speed: 0,
                            frames: [Utils.getRandom(3)]
                        }
                    }
                })],
                init: function () {
                    this.timer = Utils.getRandom(100);
                }
            }).attach({
                speedX: 0,
                rotationSpeed: Utils.getRandom(5),
                update: function () {
                    var dimension = confetti.dimension,
                        position = confetti.position;
                    if (position.x < -dimension.width) {
                        position.x = viewport.width + dimension.width;
                    } else if (position.x > viewport.width + dimension.width) {
                        position.x = -dimension.width;
                    }
                    if (position.y < -dimension.height) {
                        position.y = viewport.height + dimension.height;
                    } else if (position.y > viewport.height + dimension.height) {
                        position.y = -dimension.height;
                    }

                    this.speedX += (Utils.getRandom(10) - Utils.getRandom(10)) / 10;
                    this.speedX = Math.min(Math.max(this.speedX, -2), 2);

                    position.x += this.speedX;
                    position.y += 1;

                    // flip
                    confetti.scale.x = Math.sin(confetti.timer / (this.rotationSpeed + 5));
                }
            });

            entity.attach(confetti);
        };

        Utils.repeat(25, spawnConfetti);

        // Bento.audio.stopSound('sfx_battle_cheer');
        // Bento.audio.playSound('sfx_battle_cheer');
        Bento.audio.stopSound('sfx_applause');
        Bento.audio.playSound('sfx_applause');

        return entity;
    };
});