/**
 * Overlay for combiner result
 */
bento.define('gui/combiner/result', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'bento/canvas',
    'entities/effect',
    'bento/gui/text',
    'bento/gui/clickbutton',
    'modules/share/combine',
    'gui/combiner/confetti',
    'gui/newcritter'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    Utils,
    Tween,
    Canvas,
    Effect,
    Text,
    ClickButton,
    ShareCombine,
    Confetti,
    NewCritter
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            critter1: Number // left critter id
            critter2: Number // right critter id
            result: Number // result critter id
            resultName: String // result critter name
            onEnd: Function
        }*/
        var viewport = Bento.getViewport();
        var oldPauseLevel;
        var sprite = new Sprite({
            imageName: 'team/background'
        });
        var entity = new Entity({
            z: Utils.Layers.COMBINERESULT,
            name: 'combineResult',
            position: new Vector2(viewport.width / 2, viewport.height / 2),
            originRelative: new Vector2(0.5, 0.5),
            updateWhenPaused: Utils.PauseLevels.COMBINERESULT,
            float: true,
            components: [
                sprite
            ]
        });
        // press
        var pressLeft = new Entity({
            name: 'pressLeft',
            position: new Vector2(-viewport.width / 2 + 16, 0),
            originRelative: new Vector2(0, 0.5),
            components: [new Sprite({
                imageName: 'combiner/press'
            })]
        });
        var pressRight = new Entity({
            name: 'pressRight',
            position: new Vector2(viewport.width / 2 - 16, 0),
            originRelative: new Vector2(0, 0.5),
            components: [new Sprite({
                imageName: 'combiner/press'
            })]
        });
        // entities
        var critterLeft = new Entity({
            name: 'critterLeft',
            position: new Vector2(-32, 36),
            originRelative: new Vector2(0.5, 1),
            components: [new Sprite({
                imageName: 'creatures/' + settings.critter1
            })]
        });
        var critterRight = new Entity({
            name: 'critterLeft',
            position: new Vector2(32, 36),
            originRelative: new Vector2(0.5, 1),
            components: [new Sprite({
                imageName: 'creatures/' + settings.critter2
            })]
        });
        var critterResult = new Entity({
            name: 'critterResult',
            position: new Vector2(0, 36),
            originRelative: new Vector2(0.5, 1),
            components: [new Sprite({
                imageName: 'creatures/' + settings.result
            })]
        });
        var critterResultOverlay = new Entity({
            name: 'critterOverlay',
            position: new Vector2(0, 0),
            originRelative: new Vector2(0.5, 1),
            components: [new Sprite({
                imageName: 'creatures/' + settings.result
            })]
        });
        var critterOverlay = new Canvas({
            // drawOnce: true,
            position: new Vector2(0, 0),
            width: critterResult.dimension.width,
            height: critterResult.dimension.height,
            originRelative: new Vector2(0.5, 1)
        });
        var doAnimation = function () {
            var effect;
            Bento.audio.playSound('sfx_combine_start');
            new Tween({
                from: -viewport.width / 2 + 16,
                to: 0,
                'in': 80,
                ease: 'easeInBack',
                updateWhenPaused: Utils.PauseLevels.COMBINERESULT,
                'do': function (v, t) {
                    pressLeft.position.x = v;
                    pressRight.position.x = -v;

                    if (pressLeft.collidesWith(critterLeft)) {
                        if (critterLeft.position.x < 0) {
                            critterLeft.position.x = pressLeft.position.x + critterLeft.dimension.width / 2;
                        } else {
                            critterLeft.position.x = 0;
                            critterLeft.scale.x = Math.abs(pressLeft.position.x / (critterLeft.dimension.width / 2));
                        }
                    }
                    if (pressRight.collidesWith(critterRight)) {
                        if (critterRight.position.x > 0) {
                            critterRight.position.x = pressRight.position.x - critterRight.dimension.width / 2;
                        } else {
                            critterRight.position.x = 0;
                            critterRight.scale.x = -Math.abs(pressRight.position.x / (critterRight.dimension.width / 2));
                        }
                    }

                    if (!effect && t > 80 - (1 / 0.2 * 6)) { // at v=0.2, 7th frame is reached on 35th frame
                        effect = new Effect({
                            z: Utils.Layers.COMBINERESULT,
                            updateWhenPaused: Utils.PauseLevels.COMBINERESULT,
                            sprite: new Sprite({
                                imageName: 'combiner/combineeffect',
                                frameCountX: 4,
                                frameCountY: 3,
                                animations: {
                                    'default': {
                                        speed: 0.2,
                                        frames: [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11]
                                    }
                                }
                            }),
                            float: true
                        });
                        effect.scale.x = 1.1;
                        effect.scale.y = 1.1;
                    }
                },
                onComplete: function () {
                    Bento.audio.playSound('sfx_combine_bump');
                    Utils.timeout(20, function () {
                        // 'new!' animation from Nom Cat
                        var newText = new NewCritter({
                            position: new Vector2(10, 36),
                        });
                        Bento.audio.playSound('sfx_combine');
                        entity.remove(critterLeft);
                        entity.remove(critterRight);
                        // create result
                        entity.attach(critterResult);
                        // move behind
                        entity.getComponent('pressLeft', function (component, index) {
                            entity.moveComponentTo(critterResult, index);
                        });

                        // fade out white
                        new Tween({
                            from: 1,
                            to: 0,
                            'in': 60,
                            ease: 'linear',
                            updateWhenPaused: Utils.PauseLevels.COMBINERESULT,
                            'do': function (v, t) {
                                critterResultOverlay.alpha = v;
                            },
                            onComplete: function () {
                                // show text
                                entity.attach(new Text({
                                    position: new Vector2(0, 48),
                                    text: settings.resultName,
                                    font: 'volter',
                                    fontSize: 9,
                                    fontSettings: Utils.FontSettings.get('font'),
                                    fontColor: '#ffffff',
                                    align: 'center',
                                    textBaseline: 'to'
                                }));

                                entity.attach(buttonPlay);
                                if (!Utils.WEB) {
                                    // do not show share button on web
                                    entity.attach(buttonShare);
                                } else {
                                    buttonPlay.position.x = 0;
                                }

                                entity.attach(new Confetti({
                                    position: new Vector2(-viewport.width / 2, -viewport.height / 2)
                                }));

                                if (settings.newCritter) {
                                    entity.attach(newText);
                                }
                            }
                        });

                        // move back
                        new Tween({
                            from: 0,
                            to: -viewport.width / 2 + 16,
                            'in': 60,
                            ease: 'linear',
                            updateWhenPaused: Utils.PauseLevels.COMBINERESULT,
                            'do': function (v, t) {
                                pressLeft.position.x = v;
                                pressRight.position.x = -v;
                            },
                            onComplete: function () {

                            }
                        });
                    }, Utils.PauseLevels.COMBINERESULT);

                }
            });
        };
        var end = function () {
            settings.onEnd();
            Bento.objects.pause(oldPauseLevel);
            // fade out overlay
            new Tween({
                from: 1,
                to: 0,
                'in': 10,
                ease: 'linear',
                updateWhenPaused: Utils.PauseLevels.COMBINERESULT,
                'do': function (v, t) {
                    entity.alpha = (v);
                },
                onComplete: function () {
                    Bento.objects.remove(entity);
                }
            });
        };
        var buttonPlay = new ClickButton({
            name: 'buttonPlay',
            sfx: 'sfx_button',
            image: Bento.assets.getImage('map/buttonattack'),
            frameCountX: 1,
            frameCountY: 2,
            animations: {
                up: {
                    frames: [0]
                },
                down: {
                    frames: [1]
                },
                inactive: {
                    frames: [0]
                }
            },
            position: new Vector2(32, 96),
            updateWhenPaused: Utils.PauseLevels.COMBINERESULT,
            onClick: function () {
                end();
                buttonPlay.setActive(false);
                buttonShare.setActive(false);
            }
        });
        var buttonShare = new ClickButton({
            name: 'buttonShare',
            sfx: 'sfx_button',
            image: Bento.assets.getImage('gui/buttonshare'),
            frameCountX: 2,
            frameCountY: 1,
            animations: {
                up: {
                    frames: [0]
                },
                down: {
                    frames: [1]
                },
                inactive: {
                    frames: [0]
                }
            },
            position: new Vector2(-32, 96),
            updateWhenPaused: Utils.PauseLevels.COMBINERESULT,
            onClick: function () {
                var shareScore = Utils.Localization.getText('shareText3'),
                    link = Utils.getStoreLink();
                var shareImage;
                var creatures = Bento.assets.getJson('creatures/creatures').creatures;

                if (window.Cocoon && window.Cocoon.Share) {
                    shareScore = shareScore.replace('[LINK]', link);
                    shareScore = shareScore.replace('{NAME1}', creatures[settings.critter1].name);
                    shareScore = shareScore.replace('{NAME2}', creatures[settings.critter2].name);
                    shareScore = shareScore.replace('{NAME3}', creatures[settings.result].name);

                    shareImage = ShareCombine({
                        critter1: settings.critter1,
                        critter2: settings.critter2,
                        critter3: settings.result
                    });
                    Cocoon.Share.share({
                        message: shareScore,
                        image: shareImage.getBase64()
                    }, function () {
                        // success
                        // Standard Firebase event for finding the most viral content in your app
                        Utils.Firebase.logEvent(Utils.Firebase.Events.SHARE, {
                            content_type: 'combine result',
                            item_id: settings.result
                        });
                    });
                } else if (Utils.isFB()) {
                    shareScore = shareScore.replace('[LINK]', link);
                    shareScore = shareScore.replace('{NAME1}', creatures[settings.critter1].name);
                    shareScore = shareScore.replace('{NAME2}', creatures[settings.critter2].name);
                    shareScore = shareScore.replace('{NAME3}', creatures[settings.result].name);

                    shareImage = ShareCombine({
                        critter1: settings.critter1,
                        critter2: settings.critter2,
                        critter3: settings.result
                    });
                    window.FBInstant.shareAsync({
                        intent: 'SHARE', // INVITE, REQUEST, CHALLENGE, SHARE
                        image: shareImage.getBase64(),
                        text: shareScore,
                        // data: {}
                    }).then(function (data) {
                        // ???
                    });
                }
            }
        });

        // draw white critter overlay
        critterOverlay.attach(critterResultOverlay);
        critterOverlay.attach({
            draw: function () {
                var context = critterOverlay.getContext();
                var dimension = critterResult.dimension;
                // draw white overlay inside
                context.globalCompositeOperation = 'source-in';
                context.fillStyle = 'white';
                context.fillRect(-dimension.width / 2, -dimension.height, dimension.width, dimension.height);
            }
        });
        critterResult.attach(critterOverlay);

        pressLeft.scale.x = -1;
        critterRight.scale.x = -1;

        critterLeft.dimension.width = Math.min(critterLeft.dimension.width, viewport.width / 2 - 16);
        critterRight.dimension.width = Math.min(critterRight.dimension.width, viewport.width / 2 - 16);

        entity.attach(pressLeft);
        entity.attach(pressRight);
        entity.attach(critterLeft);
        entity.attach(critterRight);

        oldPauseLevel = Bento.objects.isPaused();
        Bento.objects.pause(Utils.PauseLevels.COMBINERESULT);
        Bento.objects.attach(entity);

        Utils.timeout(30, doAnimation, Utils.PauseLevels.COMBINERESULT);

        if (Utils.WEB) {
            // no sharing

        }

        return entity;
    };
});