/**
 * Copy of combiner result, with a few minor changes for the splash
 */
bento.define('gui/combiner/splash', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'bento/canvas',
    'entities/effect',
    'bento/gui/text',
    'bento/gui/clickbutton',
    'components/stretchtofill'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    Utils,
    Tween,
    Canvas,
    Effect,
    Text,
    ClickButton,
    StretchToFill
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            critter1: Number // left critter id
            critter2: Number // right critter id
            result: Number // result critter id
            resultName: Strinh // result critter name
            onEnd: Function
        }*/
        var viewport = Bento.getViewport();
        var oldPauseLevel;
        var sprite = new Sprite({
            imageName: 'combiner/background'
        });
        var entity = new Entity({
            z: Utils.Layers.COMBINERESULT,
            name: 'combineResult',
            position: new Vector2(viewport.width / 2, viewport.height / 2),
            originRelative: new Vector2(0.5, 0.5),
            updateWhenPaused: Utils.PauseLevels.COMBINERESULT,
            float: true,
            components: [sprite]
        });
        // press
        var pressLeft = new Entity({
            name: 'pressLeft',
            position: new Vector2(-viewport.width / 2 + 16, 0),
            originRelative: new Vector2(0, 0.5),
            components: [new Sprite({
                imageName: 'combiner/press'
            })]
        });
        var pressRight = new Entity({
            name: 'pressRight',
            position: new Vector2(viewport.width / 2 - 16, 0),
            originRelative: new Vector2(0, 0.5),
            components: [new Sprite({
                imageName: 'combiner/press'
            })]
        });
        // entities
        var critterLeft = new Entity({
            name: 'critterLeft',
            position: new Vector2(-32, 36),
            originRelative: new Vector2(0.5, 1),
            components: [new Sprite({
                imageName: settings.critter1
            })]
        });
        var critterRight = new Entity({
            name: 'critterLeft',
            position: new Vector2(32, 36),
            originRelative: new Vector2(0.5, 1),
            components: [new Sprite({
                imageName: settings.critter2
            })]
        });
        var critterResult = new Entity({
            name: 'critterResult',
            position: new Vector2(0, 0),
            originRelative: new Vector2(0.5, 0.5),
            components: [new Sprite({
                imageName: settings.result
            })]
        });
        var critterResultOverlay = new Entity({
            name: 'critterOverlay',
            position: new Vector2(0, 0),
            originRelative: new Vector2(0.5, 0.5),
            components: [new Sprite({
                imageName: settings.result
            })]
        });
        var critterOverlay = new Canvas({
            // drawOnce: true,
            position: new Vector2(0, 0),
            width: critterResult.dimension.width,
            height: critterResult.dimension.height,
            originRelative: new Vector2(0.5, 0.5)
        });
        var doAnimation = function () {
            var effect;
            Bento.audio.playSound('sfx_combine_start');
            new Tween({
                from: -viewport.width / 2 + 16,
                to: 0,
                'in': 80,
                ease: 'easeInBack',
                updateWhenPaused: Utils.PauseLevels.COMBINERESULT,
                'do': function (v, t) {
                    pressLeft.position.x = v;
                    pressRight.position.x = -v;

                    if (pressLeft.collidesWith(critterLeft)) {
                        if (critterLeft.position.x < 0) {
                            critterLeft.position.x = pressLeft.position.x + critterLeft.dimension.width / 2;
                        } else {
                            critterLeft.position.x = 0;
                            critterLeft.scale.x = Math.abs(pressLeft.position.x / (critterLeft.dimension.width / 2));
                        }
                    }
                    if (pressRight.collidesWith(critterRight)) {
                        if (critterRight.position.x > 0) {
                            critterRight.position.x = pressRight.position.x - critterRight.dimension.width / 2;
                        } else {
                            critterRight.position.x = 0;
                            critterRight.scale.x = -Math.abs(pressRight.position.x / (critterRight.dimension.width / 2));
                        }
                    }

                    if (!effect && t > 80 - (1 / 0.2 * 6)) { // at v=0.2, 7th frame is reached on 35th frame
                        effect = new Effect({
                            z: Utils.Layers.COMBINERESULT,
                            updateWhenPaused: Utils.PauseLevels.COMBINERESULT,
                            sprite: new Sprite({
                                imageName: 'combiner/combineeffect',
                                frameCountX: 4,
                                frameCountY: 3,
                                animations: {
                                    'default': {
                                        speed: 0.2,
                                        frames: [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11]
                                    }
                                }
                            }),
                            float: true
                        });
                        effect.scale.x = 1.1;
                        effect.scale.y = 1.1;
                    }
                },
                onComplete: function () {
                    Bento.audio.playSound('sfx_combine_bump');
                    Utils.timeout(20, function () {
                        Bento.audio.playSound('sfx_combine');
                        entity.remove(critterLeft);
                        entity.remove(critterRight);
                        // create result
                        entity.attach(critterResult);
                        // move behind
                        entity.getComponent('pressLeft', function (component, index) {
                            entity.moveComponentTo(critterResult, index);
                        });

                        // small
                        critterResult.scale.x = 0.5;
                        critterResult.scale.y = 0.5;
                        critterResultOverlay.scale.x = 0.5;
                        critterResultOverlay.scale.y = 0.5;


                        // fade out white
                        new Tween({
                            from: 1,
                            to: 0,
                            'in': 60,
                            ease: 'linear',
                            updateWhenPaused: Utils.PauseLevels.COMBINERESULT,
                            'do': function (v, t) {
                                critterResultOverlay.alpha = v;
                            },
                            onComplete: function () {
                                // play meow
                                Bento.audio.playSound('sfx_meow');
                                new Tween({
                                    from: critterResult.scale.x,
                                    to: 1,
                                    'in': 30,
                                    ease: 'elastic',
                                    alpha: -8,
                                    beta: 3,
                                    updateWhenPaused: Utils.PauseLevels.COMBINERESULT,
                                    'do': function (v, t) {
                                        critterResult.scale.x = v;
                                        critterResult.scale.y = v;
                                    },
                                    onComplete: function () {
                                        Utils.timeout(30, settings.onComplete, Utils.PauseLevels.COMBINERESULT);
                                    }
                                });
                            }
                        });

                        // move back
                        new Tween({
                            from: 0,
                            to: -viewport.width / 2 + 16,
                            'in': 60,
                            ease: 'linear',
                            updateWhenPaused: Utils.PauseLevels.COMBINERESULT,
                            'do': function (v, t) {
                                pressLeft.position.x = v;
                                pressRight.position.x = -v;
                            },
                            onComplete: function () {

                            }
                        });
                    }, Utils.PauseLevels.COMBINERESULT);

                }
            });
        };
        var end = function () {
            settings.onEnd();
            Bento.objects.pause(oldPauseLevel);
            // fade out overlay
            new Tween({
                from: 1,
                to: 0,
                'in': 10,
                ease: 'linear',
                updateWhenPaused: Utils.PauseLevels.COMBINERESULT,
                'do': function (v, t) {
                    entity.alpha = (v);
                },
                onComplete: function () {
                    Bento.objects.remove(entity);
                }
            });
        };
        var buttonPlay = new ClickButton({
            name: 'buttonPlay',
            sfx: 'sfx_button',
            image: Bento.assets.getImage('map/buttonattack'),
            frameCountX: 1,
            frameCountY: 2,
            animations: {
                up: {
                    frames: [0]
                },
                down: {
                    frames: [1]
                },
                inactive: {
                    frames: [0]
                }
            },
            position: new Vector2(32, 96),
            updateWhenPaused: Utils.PauseLevels.COMBINERESULT,
            onClick: function () {
                end();
                buttonPlay.setActive(false);
                buttonShare.setActive(false);
            }
        });
        var buttonShare = new ClickButton({
            name: 'buttonShare',
            sfx: 'sfx_button',
            image: Bento.assets.getImage('gui/buttonshare'),
            frameCountX: 2,
            frameCountY: 1,
            animations: {
                up: {
                    frames: [0]
                },
                down: {
                    frames: [1]
                },
                inactive: {
                    frames: [0]
                }
            },
            position: new Vector2(-32, 96),
            updateWhenPaused: Utils.PauseLevels.COMBINERESULT,
            onClick: function () {
                var shareScore = Utils.Localization.getText('shareText3'),
                    link = Utils.getStoreLink();

                if (window.Cocoon && window.Cocoon.Share) {
                    shareScore = shareScore.replace('[LINK]', link);

                    // TODO: what to share?
                }
            }
        });

        // draw white critter overlay
        // critterOverlay.attach(critterResultOverlay);
        // critterOverlay.attach({
        //     draw: function () {
        //         var context = critterOverlay.getContext();
        //         var dimension = critterResult.dimension;
        //         // draw white overlay inside
        //         context.globalCompositeOperation = 'source-in';
        //         context.fillStyle = 'white';
        //         context.fillRect(-dimension.width / 2, -dimension.height, dimension.width, dimension.height);
        //     }
        // });
        // critterResult.attach(critterOverlay);

        pressLeft.scale.x = -1;
        critterRight.scale.x = -1;

        critterLeft.dimension.width = Math.min(critterLeft.dimension.width, viewport.width / 2 - 16);
        critterRight.dimension.width = Math.min(critterRight.dimension.width, viewport.width / 2 - 16);

        entity.attach(pressLeft);
        entity.attach(pressRight);
        entity.attach(critterLeft);
        entity.attach(critterRight);

        oldPauseLevel = Bento.objects.isPaused();
        Bento.objects.pause(Utils.PauseLevels.COMBINERESULT);
        Bento.objects.attach(entity);

        Utils.timeout(30, doAnimation, Utils.PauseLevels.COMBINERESULT);

        return entity;
    };
});