/**
 * Credits overlay
 */
bento.define('gui/credits/credits', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'bento/screen',
    'bento/gui/clickbutton',
    'bento/gui/text',
    'components/debug',
    'components/draggable',
    'components/droptarget',
    'gui/centeredlist',
    'gui/coincounter',
    'gui/storydialog',
    'gui/scrollinglist',
    'gui/team/page',
    'gui/team/slot',
    'entities/card'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    Utils,
    Tween,
    Screen,
    ClickButton,
    Text,
    Debug,
    Draggable,
    DropTarget,
    CenteredList,
    CoinCounter,
    StoryDialog,
    ScrollingList,
    Page,
    Slot,
    Card
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var oldPauseLevel = Bento.objects.isPaused();
        var container = new Entity({
            z: Utils.Layers.TEAMEDIT,
            name: 'creditsContainer',
            position: new Vector2(0, 0),
            float: true,
            updateWhenPaused: Utils.PauseLevels.TEAMEDIT,
            components: [

            ]
        });
        var background = new Entity({
            z: 0,
            name: 'background',
            position: new Vector2(viewport.width / 2, viewport.height / 2),
            originRelative: new Vector2(0.5, 0.5),
            components: [new Sprite({
                imageName: 'combiner/background'
            })]
        });
        var buttonExit = new ClickButton({
            z: 3,
            sfx: 'sfx_button',
            image: Bento.assets.getImage('combiner/buttonexit'),
            frameCountX: 2,
            frameCountY: 1,
            animations: {
                up: {
                    frames: [0]
                },
                down: {
                    frames: [1]
                }
            },
            position: new Vector2(12, 10),
            updateWhenPaused: Utils.PauseLevels.TEAMEDIT,
            onClick: function () {
                Bento.objects.pause(oldPauseLevel);
                Bento.objects.remove(container);

                if (settings.onClose) {
                    settings.onClose();
                }
            }
        }).attach(
            // back button pressed
            new Utils.BackButton(function (data) {
                data.entity.doCallback();
            })
        );
        var scroller = new Entity({
            name: 'scroller',
            components: [{
                update: function () {
                    scroller.position.y -= 0.2;

                    if (scroller.position.y < -viewport.height * 1.5) {
                        scroller.position.y += viewport.height * (1.5 + 1);
                    }
                }
            }]
        });
        var luckyKat = new Entity({
            name: 'luckyKat',
            position: new Vector2(viewport.width / 2, viewport.height / 2),
            originRelative: new Vector2(0.5, 0.5),
            components: [new Sprite({
                imageName: 'gui/luckykat80'
            })]
        });
        var text = new Text({
            name: 'credits',
            position: new Vector2(viewport.width / 2, luckyKat.position.y + 50),
            text: 'Combo Critters\n\nBy Lucky Kat Studios\n\nProducer\nHerdjie Zhou\n\nProgramming\nHernan Zhou\nNikola Pilipovic\n\nArt\nDiane de Wilde\nJasper Boerstra\nRichard Lems\n\nMusic\nMax Coburn\n\nTesting\nBas Weijenberg',
            font: 'volter',
            fontSize: 9,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'top'
        });

        container.attach(background);
        container.attach(buttonExit);
        scroller.attach(luckyKat);
        scroller.attach(text);
        container.attach(scroller);


        Bento.objects.pause(Utils.PauseLevels.TEAMEDIT);

        Utils.Analytics.trackView('credits');

        return container;
    };
});