/**
 * Popup when 3D touched
 */
bento.define('gui/critterdetails', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'entities/battle/critter'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    ClickButton,
    Counter,
    Text,
    EventSystem,
    Utils,
    Tween,
    Critter
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var oldPauseLevel = Bento.objects.isPaused();
        var overlay = new Entity({
            name: 'overlay',
            position: new Vector2(0, 0),
            originRelative: new Vector2(0.5, 0.5),
            components: [new Sprite({
                imageName: 'fade'
            })],
            init: function () {
                this.alpha = 0.75;
            }
        });
        var extraHeight = 0;
        // exit button
        var exitButton = new ClickButton({
            sfx: 'sfx_button',
            image: Bento.assets.getImage('team/buttonexit'),
            frameCountX: 2,
            frameCountY: 1,
            animations: {
                up: {
                    frames: [0]
                },
                down: {
                    frames: [1]
                },
                inactive: {
                    frames: [0]
                }
            },
            position: new Vector2(-viewport.width / 2 + 12, -viewport.height / 2 + 10),
            updateWhenPaused: Utils.PauseLevels.DETAILS,
            onClick: function () {
                Bento.objects.remove(entity);

                if (settings.onClose) {
                    settings.onClose();
                }
            }
        }).attach(
            // back button pressed     
            new Utils.BackButton(function (data) {
                data.entity.doCallback();
            })
        );
        var cleaner = {
            destroy: function () {
                Bento.objects.pause(oldPauseLevel);
            }
        };
        var critter = new Critter({
            id: settings.critter.id,
            position: new Vector2(0, 0)
        });
        var text = new Text({
            position: new Vector2(0, 24),
            text: Utils.Localization.getName(settings.critter.id),
            font: 'volter',
            fontSize: 9,
            fontSettings: Utils.FontSettings.get('font'),
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'top',
            ySpacing: 2
        });
        var detailsWindow = new Entity({
            name: 'window',
            position: new Vector2(2, 0),
            originRelative: new Vector2(0.5, 0.5),
            updateWhenPaused: Utils.PauseLevels.DETAILS,
            components: [
                new Sprite({
                    imageName: 'gui/critterwindow'
                }),
                critter,
                text,
                new Clickable({
                    onClickMiss: function (evt) {
                        if (evt.localPosition.y > 0 && evt.localPosition.y < extraHeight) {
                            // this is about the position where the buttons are attached
                            return;
                        }
                        exitButton.doCallback();
                    }
                })
            ]
        });
        // main container
        var entity = new Entity({
            z: Utils.Layers.DETAILS,
            name: 'shop',
            position: new Vector2(viewport.width / 2, viewport.height / 2),
            updateWhenPaused: Utils.PauseLevels.DETAILS,
            float: true,
            components: [
                overlay,
                detailsWindow,
                exitButton,
                cleaner
            ]
        });
        
        // critter details
        critter.position.y = 16;
        text.setText(
            Utils.Localization.getName(settings.critter.id) + '\n' +
            Utils.Localization.getText('health') + ' ' + settings.critter.currentHp + '/' + settings.critter.hp + '   ' +
            Utils.Localization.getText('attack') + ' ' + settings.critter.attack
        );


        // pause the game
        Bento.objects.pause(Utils.PauseLevels.DETAILS);

        // move in
        new Tween({
            from: detailsWindow.position.x + viewport.width,
            to: detailsWindow.position.x,
            'in': 60,
            ease: 'elastic',
            alpha: -10,
            beta: 1,
            updateWhenPaused: Utils.PauseLevels.DETAILS,
            'do': function (v, t) {
                detailsWindow.position.x = v;
            }
        });

        Bento.objects.attach(entity);
        
        entity.extend({
            close: function () {
                exitButton.doCallback();
            },
            attachToWindow: function (component) {
                detailsWindow.attach(component);
                extraHeight = 91;
            }
        });

        return entity;
    };
});