/**
 * Daily gift overlay
 */
bento.define('gui/dailygift', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'entities/card',
    'entities/effect',
    'gui/combiner/confetti',
    'gui/rewardpopup',
    'gui/storydialog',
    'gui/newcritter',
    'gui/dialog',
    'ads/heyzap',
    'modules/rerollad',
    'modules/fbnotification'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    ClickButton,
    Counter,
    Text,
    EventSystem,
    Utils,
    Tween,
    Card,
    Effect,
    Confetti,
    RewardPopup,
    StoryDialog,
    NewCritter,
    Dialog,
    HeyZap,
    RerollAd,
    FBNotification
) {
    'use strict';
    var DailyGift = function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var oldPauseLevel = Bento.objects.isPaused();
        var unlockedRarity = Bento.saveState.load('unlockedRarity', 3);
        var rewardAmount = Bento.assets.getJson('creatures/rewardamount');
        var overlay = new Entity({
            name: 'overlay',
            position: new Vector2(0, 0),
            originRelative: new Vector2(0.5, 0.5),
            alpha: 0,
            components: [new Sprite({
                imageName: 'fade'
            })]
        });
        var text = new Text({
            position: new Vector2(0, -24),
            text: Utils.Localization.getText('chooseGift'),
            font: 'volter',
            fontSize: 9,
            fontSettings: Utils.FontSettings.get('font'),
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle',
            maxWidth: 140,
            ySpacing: 2
        });
        var detailsWindow = new Entity({
            name: 'window',
            position: new Vector2(2, -32),
            originRelative: new Vector2(0.5, 0.5),
            updateWhenPaused: Utils.PauseLevels.DAILYGIFT,
            components: [
                new Sprite({
                    imageName: 'gui/critterwindow'
                }),
                text
            ]
        });
        var entity = new Entity({
            z: Utils.Layers.DAILYGIFT,
            name: 'dailyGiftContainer',
            position: new Vector2(viewport.width / 2, viewport.height / 2),
            float: true,
            updateWhenPaused: Utils.PauseLevels.DAILYGIFT,
            components: [
                overlay,
                detailsWindow
            ]
        });
        var hasClicked = false;
        var rerolled = false;
        var types = ['coins', 'coins', 'card', 'card'];
        var gifts = [];
        var makeGift = function (delay) {
            var unpackedGift;
            var giftType = Utils.getRandom(2) ? types.shift() : types.pop();
            var reward = rewardAmount[unlockedRarity] + Utils.getRandom(rewardAmount[unlockedRarity]);
            var critterProps;
            var gift = new Entity({
                name: 'gift',
                position: new Vector2(-48 + 48 * gifts.length, 0),
                originRelative: new Vector2(0.5, 0.5),
                updateWhenPaused: Utils.PauseLevels.DAILYGIFT,
                components: [
                    new Sprite({
                        imageName: 'gift/gift'
                    }),
                    new Clickable({
                        onClick: function () {
                            if (hasClicked) {
                                // already clicked on a gift
                                return;
                            }
                            hasClicked = true;

                            Bento.audio.stopSound('sfx_button');
                            Bento.audio.playSound('sfx_button');

                            // set others to transparent
                            Utils.forEach(gifts, function (item, i, l, breakLoop) {
                                // remove hand if the gift has one pointing to it
                                item.getComponent('hand', function (obj) {
                                    item.remove(obj);
                                });

                                if (item === gift) {
                                    // self
                                    return;
                                }
                                item.alpha = 0.5;

                                // remove other gifts
                                // new Tween({
                                //     from: 1,
                                //     to: 0,
                                //     'in': 60,
                                //     ease: 'linear',
                                //     updateWhenPaused: Utils.PauseLevels.DAILYGIFT,
                                //     'do': function (v, t) {
                                //         item.alpha = v;
                                //     }
                                // });
                            });

                            // start animation
                            makeSmall();
                        }
                    })
                ]
            });
            var makeSmall = function () {
                new Tween({
                    from: 1,
                    to: 0.75,
                    'in': 40,
                    ease: 'exponential',
                    alpha: -3,
                    updateWhenPaused: Utils.PauseLevels.DAILYGIFT,
                    'do': function (v, t) {
                        gift.scale.x = v;
                        gift.scale.y = v;
                    },
                    onComplete: explode
                });
            };
            var shake = function () {
                new Tween({
                    'in': 30,
                    ease: 'linear',
                    updateWhenPaused: Utils.PauseLevels.DAILYGIFT,
                    'do': function (v, t) {
                        gift.scale.x = 0.75 + 0.05 * (Math.floor(t / 2) % 2);
                        gift.scale.y = 0.75 + 0.05 * (Math.floor(t / 2) % 2);
                    },
                    onComplete: explode
                });
            };
            var explode = function () {
                Bento.audio.playSound('sfx_getcoins');
                new Tween({
                    from: 0.5,
                    to: 1.2,
                    'in': 5,
                    ease: 'squareroot',
                    updateWhenPaused: Utils.PauseLevels.DAILYGIFT,
                    'do': function (v, t) {
                        gift.scale.x = v;
                        gift.scale.y = v;
                    },
                    onComplete: function () {
                        new Effect({
                            z: Utils.Layers.DAILYGIFT,
                            updateWhenPaused: Utils.PauseLevels.DAILYGIFT,
                            sprite: new Sprite(Bento.assets.getJson('animations/effects/pickup')),
                            position: gift.getWorldPosition(),
                            onComplete: function () {
                                Bento.objects.attach(new Confetti({
                                    z: Utils.Layers.DAILYGIFT,
                                    pauseLevel: Utils.PauseLevels.DAILYGIFT
                                }));

                                // zoom in unpacked gift
                                zoomInGift();
                            }
                        });
                        detailsWindow.remove(gift);
                        // remove from gifts array so it forEach doesn't mess with it
                        gifts.splice(gifts.indexOf(gift), 1);
                        detailsWindow.attach(unpackedGift);
                    }
                });
            };
            var zoomInGift = function () {
                new Tween({
                    from: giftType === 'card' ? 0.5 : 1,
                    to: giftType === 'card' ? 1 : 1.5,
                    'in': 60,
                    ease: 'elastic',
                    alpha: -5,
                    beta: 4,
                    updateWhenPaused: Utils.PauseLevels.DAILYGIFT,
                    'do': function (v, t) {
                        unpackedGift.scale.x = v;
                        unpackedGift.scale.y = v;
                    },
                    onComplete: giftType === "coins" ? rewardCoins : rewardCard
                });
            };
            var rewardCard = function () {
                var critters = Utils.getCritters();
                var newCritter = !Utils.hasInCollection(critterProps.id);
                critters.unshift(critterProps);
                Utils.addToCollection(critterProps.id);
                Utils.saveCritters();
                hasGifted();

                if (newCritter) {
                    // show 'new' animation if player didn't have critter yet
                    unpackedGift.attach(
                        new NewCritter({
                            position: new Vector2(4, -23),
                            small: true
                        })
                    );
                    // show '!' new icon on Critterdex
                    Bento.objects.get('menuButton', function (obj) {
                        obj.showNewIcon();
                    });
                }

                new StoryDialog({
                    z: Utils.Layers.DAILYGIFT + 1,
                    text: Utils.Localization.getText('newCritter') /* + '\n' + Utils.Localization.getName(critterProps.id)*/ ,
                    hideCat: true,
                    onClose: function () {
                        if (
                            HeyZap.canShowRewarded() &&
                            !rerolled &&
                            !Bento.saveState.load('highSpender', false) // disable reroll ads for high spenders
                        ) {
                            askReroll();
                        } else {
                            revealGifts();
                        }
                    }
                });
            };
            var rewardCoins = function () {
                // popup with reward
                hasGifted();
                new RewardPopup({
                    z: Utils.Layers.DAILYGIFT + 1,
                    trackCoins: true,
                    reward: reward,
                    pauseLevel: Utils.PauseLevels.DAILYGIFT,
                    pauseLevelCoins: Utils.PauseLevels.DAILYGIFT,
                    onClose: function () {
                        if (
                            HeyZap.canShowRewarded() &&
                            !rerolled &&
                            !Bento.saveState.load('highSpender', false) // disable reroll ads for high spenders
                        ) {
                            askReroll();
                        } else {
                            revealGifts();
                        }
                    }
                });
            };
            var askReroll = function () {
                var done = true;
                new Dialog({
                    name: 'askReroll',
                    z: Utils.Layers.DAILYGIFT + 1,
                    type: 'yes',
                    canClose: true,
                    text: Utils.Localization.getText('askReroll'),
                    position: new Vector2(viewport.x + viewport.width / 2, viewport.y + viewport.height * 3 / 4),
                    buttonY: 13,
                    pauseLevel: Utils.PauseLevels.DAILYGIFT + 2,
                    spriteYes: new Sprite({
                        image: Bento.assets.getImage('gui/buttonadreroll'),
                        frameCountX: 2,
                        frameCountY: 2,
                        animations: {
                            down: {
                                frames: [1]
                            },
                            up: {
                                speed: 0.025,
                                frames: [0, 2]
                            }
                        }
                    }),
                    onYes: function () {
                        Utils.Firebase.logEvent('reroll_accepted', {
                            value: 1
                        });
                        // allow second gift
                        done = false;
                        RerollAd.watchAd(
                            // on succes
                            function () {
                                Utils.forEach(gifts, function (item, i, l, breakLoop) {
                                    item.alpha = 1;

                                    // add hands to unopened gifts when rerolling
                                    var hand = new Entity({
                                        name: 'hand',
                                        position: new Vector2(0, 0),
                                        originRelative: new Vector2(0.5, 0),
                                        updateWhenPaused: Bento.objects.isPaused(),
                                        components: [new Sprite({
                                            imageName: 'map/hand'
                                        })]
                                    }).attach({
                                        start: function () {
                                            hand.position.y = item.dimension.height / 2 + 4 * Math.abs(Math.sin(hand.timer / 20));
                                        },
                                        update: function () {
                                            hand.position.y = item.dimension.height / 2 + 4 * Math.abs(Math.sin(hand.timer / 20));
                                        }
                                    });
                                    item.attach(hand);
                                });
                                hasClicked = false;
                                rerolled = true;
                            },
                            // on fail
                            function () {
                                done = true;
                                revealGifts();
                            }
                        );
                    },
                    onClose: function () {
                        if (done) {
                            Utils.Firebase.logEvent('reroll_denied', {
                                value: 1
                            });
                            revealGifts();
                        }
                    }
                });
            };
            var revealGifts = function () {
                // reveal other rewards
                Utils.forEach(gifts, function (item, i, l, breakLoop) {
                    item.reveal();
                });

                new Tween({
                    from: 0,
                    to: 1,
                    // wait 90 frames before moveOut() so that the gifts have time to reveal themselves
                    'in': 90,
                    ease: 'linear',
                    updateWhenPaused: Bento.objects.isPaused(),
                    onComplete: function () {
                        moveOut();
                    }
                });
            };

            gift.extend({
                reveal: function () {
                    // spin gift and show reward
                    unpackedGift.alpha = 0.5;
                    unpackedGift.scale.x = 0;
                    unpackedGift.scale.y = 0.5;

                    // spin out
                    new Tween({
                        from: 1,
                        to: 0,
                        'in': 20,
                        ease: 'easeInQuad',
                        updateWhenPaused: Utils.PauseLevels.DAILYGIFT,
                        'do': function (v, t) {
                            gift.scale.x = v;
                        },
                        onComplete: function () {
                            detailsWindow.remove(gift);
                            // remove from gifts array so it forEach doesn't mess with it
                            gifts.splice(gifts.indexOf(gift), 1);
                            detailsWindow.attach(unpackedGift);
                            new Tween({
                                from: 0,
                                to: 0.5,
                                'in': 20,
                                ease: 'easeOutQuad',
                                updateWhenPaused: Utils.PauseLevels.DAILYGIFT,
                                'do': function (v, t) {
                                    unpackedGift.scale.x = v;
                                }
                            });
                        }
                    });
                }
            });

            gifts.push(gift);

            Utils.timeout(delay, function () {
                detailsWindow.attach(gift);
                new Tween({
                    from: 0,
                    to: 1,
                    'in': 40,
                    ease: 'easeOutBounce',
                    updateWhenPaused: Utils.PauseLevels.DAILYGIFT,
                    'do': function (v, t) {
                        gift.position.y = v * 20;
                        if (hasClicked) {
                            return;
                        }
                        gift.alpha = v;

                    },
                    onComplete: function () {

                    }
                });
            }, Utils.PauseLevels.DAILYGIFT);

            // spawn the real gift
            if (giftType === 'coins') {
                unpackedGift = new Entity({
                    name: 'unpackedGift',
                    position: gift.position.clone(),
                    originRelative: new Vector2(0.5, 0.5),
                    components: [new Sprite({
                        imageName: 'gift/coins'
                    })]
                });
                unpackedGift.position.y += 16;
            } else {
                critterProps = getRandomCritter();
                unpackedGift = new Card(critterProps);
                unpackedGift.position = gift.position.clone();
                unpackedGift.position.y += 16;
                unpackedGift.scale.x = 0.5;
                unpackedGift.scale.y = 0.5;
            }
        };
        var moveOut = function () {
            new Tween({
                to: detailsWindow.position.x + viewport.width + 32,
                from: detailsWindow.position.x,
                'in': 60,
                ease: 'elastic',
                alpha: -10,
                beta: 1,
                updateWhenPaused: Utils.PauseLevels.DAILYGIFT,
                'do': function (v, t) {
                    detailsWindow.position.x = v;
                },
                onComplete: function () {
                    Bento.objects.pause(oldPauseLevel);
                    Bento.objects.remove(entity);

                    // Schedule notifications after daily gift screen is removed
                    // If notifications have not been initialized yet
                    // the player will now be asked to receive notifications
                    Utils.Notification.sendNotification(Utils.getNotifications().dailyGift, settings.onComplete);
                }
            });
            // fade out
            new Tween({
                to: 0,
                from: 0.75,
                'in': 30,
                ease: 'linear',
                updateWhenPaused: Utils.PauseLevels.DAILYGIFT,
                'do': function (v, t) {
                    overlay.alpha = v;
                }
            });
        };
        var getRandomCritter = function () {
            // pick a random critter
            var randomIndex = Utils.getRandom(desiredCritters.length);
            // remove from pool
            var randomCritter = desiredCritters.splice(randomIndex, 1);
            // return it
            return Utils.clone(allCritters[randomCritter]);
        };
        var hasGifted = function () {
            // remember the time when gift was given
            Bento.saveState.save('lastGiftTime', Date.now());

            // facebook: prepare notification 24 hours ahead
            if (Utils.isFB()) {
                // cancel old first
                FBNotification.remove(Bento.saveState.load('notification24', '{}'), function (error, msg) {
                    if (error) {
                        console.error(msg);
                    }
                });
                // schedule a new one in 24 hours
                FBNotification.schedule(24, {
                    message: Utils.Localization.getText('dailyGift'),
                    title: Utils.Localization.getText('title'),
                    buttonText: 'Play',
                    imageUrl: 'https://s3.amazonaws.com/lucky-kat/combocritters/messenger.jpg'
                }, function (error, messageDataStr) {
                    if (error) {
                        console.error(messageDataStr);
                        return;
                    }
                    // save messageData which can be used to delete the notification
                    Bento.saveState.save('notification24', messageDataStr);
                });
            }
        };
        var allJson = Bento.assets.getAssets().json;
        var bosses = [];
        // pick a random critter based on unlocked rarity
        var allCritters = Bento.assets.getJson('creatures/creatures').creatures;
        var desiredCritters = [];

        // collect the critters with desired rarity
        Utils.forEach(allCritters, function (critter, i, l, breakLoop) {
            if (critter.rarity === unlockedRarity - 2) {
                desiredCritters.push(critter.id);
            }
        });

        // find boss ids
        Utils.forEach(allJson, function (json, key, l, breakLoop) {
            if (key.indexOf('planets/') >= 0 && json.boss) {
                bosses.push(json.boss.id);
            }
        });

        // remove bosses
        Utils.forEach(bosses, function (boss, i, l, breakLoop) {
            var index = desiredCritters.indexOf(boss);
            if (index >= 0) {
                desiredCritters.splice(index, 1);
            }
        });

        // can receive gift at all?
        if (!DailyGift.canGive()) {
            return null;
        }

        // already running gift
        if (Bento.objects.get('dailyGiftContainer')) {
            return null;
        }


        // pause the game
        Bento.objects.pause(Utils.PauseLevels.DAILYGIFT);

        // move in
        new Tween({
            from: detailsWindow.position.x + viewport.width,
            to: detailsWindow.position.x,
            'in': 60,
            ease: 'elastic',
            alpha: -10,
            beta: 1,
            updateWhenPaused: Utils.PauseLevels.DAILYGIFT,
            'do': function (v, t) {
                detailsWindow.position.x = v;
            },
            onComplete: function () {
                // make presents
                Utils.repeat(3, function (i) {
                    makeGift(i * 10);
                });
                Bento.audio.playSound('sfx_transition');
            }
        });

        // fade in
        new Tween({
            from: 0,
            to: 0.75,
            'in': 30,
            ease: 'linear',
            updateWhenPaused: Utils.PauseLevels.DAILYGIFT,
            'do': function (v, t) {
                overlay.alpha = v;
            }
        });

        Bento.objects.attach(entity);

        // reset iOS badge number when getting a new daily gift
        Utils.Notification.resetBadgeNumber();

        return entity;
    };

    // static function
    DailyGift.canGive = function () {
        // has 24 hours passed since last gift?
        var lastTime = Bento.saveState.load('lastGiftTime', 0);
        var diff = Date.now() - lastTime;
        var diffHours = diff / 1000 / 60 / 60;

        // hasn't seen boss yet
        if (Bento.saveState.load('firstBossBattle', false) === false) {
            return false;
        }

        if (diffHours >= 24) {
            return true;
        } else {
            return false;
        }
    };

    return DailyGift;
});