/**
 * Animated damage counter
 */
bento.define('gui/damagecounter', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'bento/gui/counter'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    Utils,
    Tween,
    Counter
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var entity = new Counter({
            z: Utils.Layers.EFFECTS + 10,
            name: 'damageCounter',
            image: Bento.assets.getImage('gui/numberstiny'),
            frameWidth: 12,
            frameHeight: 14,
            spacing: new Vector2(-4, 0),
            position: settings.position || new Vector2(0, 0),
            align: 'center'
        });
        var lifeTime = 0;
        var opacity;
        var playAnimation = function () {
            var delay = 0;
            entity.loopDigits(function (digit) {
                var posY = digit.position.y;
                digit.visible = false;
                Utils.timeout(delay, function () {
                    digit.visible = true;
                    new Tween({
                        from: posY,
                        to: posY - 16,
                        'in': 5,
                        ease: 'squareroot',
                        'do': function (v, t) {
                            digit.position.y = v;
                        },
                        onComplete: function () {
                            new Tween({
                                from: posY - 16,
                                to: posY,
                                'in': 30,
                                ease: 'easeOutBounce',
                                'do': function (v, t) {
                                    digit.position.y = v;
                                },
                                onComplete: function () {

                                }
                            });

                        }
                    });

                });
                delay += 5;
            });
        };

        // animation
        lifeTime = entity.getDigits() * 35;
        entity.setValue(settings.value);
        playAnimation();

        // remove self
        Utils.timeout(lifeTime, function () {
            new Tween({
                from: 1,
                to: 0,
                'in': 20,
                ease: 'linear',
                'do': function (v, t) {
                    entity.alpha = v;
                },
                onComplete: function () {
                    Bento.objects.remove(entity);
                }
            });
        });

        Bento.objects.attach(entity);
        return entity;
    };
});