/**
 * Module description
 */
bento.define('gui/dashedline', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'bento/canvas'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    Utils,
    Tween,
    Canvas
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var polyline = settings.tiled.object.polyline;
        var addDash = function (position, angle) {
            var dash = new Entity({
                name: 'dash',
                position: position,
                originRelative: new Vector2(0.5, 0.5),
                components: [
                    new Sprite({
                        image: Bento.assets.getImage('map/dash')
                    })
                ]
            });
            // entity.attach(dash);
            dash.rotation = angle;
            return dash;
        };
        var dashLength = 6; // image is 4 pixels wide, so there will be 2 px spacing
        var entity = new Entity({
            z: 0,
            name: 'dashedLine',
            components: []
        });

        // loop through polyline
        Utils.forEach(polyline, function (line, j, l, breakLoop, next) {
            if (!next) {
                breakLoop();
                return;
            }
            var vector1 = new Vector2(line.x, line.y);
            var vector2 = new Vector2(next.x, next.y);
            var diff = vector2.subtract(vector1);
            var magnitude = diff.magnitude();
            var angle = diff.angle();
            var dashes = Math.floor(magnitude / dashLength);
            var i;
            var dash;
            var container = new Canvas({
                position: vector1.clone(),
                width: Math.abs(diff.x),
                height: Math.abs(diff.y),
                drawOnce: true
            });
            for (i = 0; i < dashes; ++i) {
                dash = addDash(
                    diff.scalarMultiply(i * dashLength / magnitude),
                    angle
                );
                container.attach(dash);
            }
            
            entity.attach(container);
        });
        return entity;
    };
});