/**
 * Combo dex
 */
bento.define('gui/dex/dex', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'bento/screen',
    'bento/gui/clickbutton',
    'bento/gui/text',
    'components/debug',
    'components/draggable',
    'components/droptarget',
    'gui/centeredlist',
    'gui/coincounter',
    'gui/storydialog',
    'gui/scrollinglist',
    'gui/team/page',
    'gui/team/slot',
    'entities/simplecard',
    'gui/newcritter'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    Utils,
    Tween,
    Screen,
    ClickButton,
    Text,
    Debug,
    Draggable,
    DropTarget,
    CenteredList,
    CoinCounter,
    StoryDialog,
    ScrollingList,
    Page,
    Slot,
    Card,
    NewCritter
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var oldPauseLevel = Bento.objects.isPaused();
        var container = new Entity({
            z: Utils.Layers.TEAMEDIT,
            name: 'dexContainer',
            position: new Vector2(0, 0),
            float: true,
            updateWhenPaused: Utils.PauseLevels.TEAMEDIT,
            components: [

            ]
        });
        var background = new Entity({
            z: 0,
            name: 'background',
            position: new Vector2(viewport.width / 2, viewport.height / 2),
            originRelative: new Vector2(0.5, 0.5),
            components: [new Sprite({
                imageName: 'combiner/background'
            })]
        });
        var buttonExit = new ClickButton({
            z: 3,
            sfx: 'sfx_button',
            image: Bento.assets.getImage('combiner/buttonexit'),
            frameCountX: 2,
            frameCountY: 1,
            animations: {
                up: {
                    frames: [0]
                },
                down: {
                    frames: [1]
                }
            },
            position: new Vector2(12, 10),
            updateWhenPaused: Utils.PauseLevels.TEAMEDIT,
            onClick: function () {
                Bento.objects.pause(oldPauseLevel);
                Bento.objects.remove(container);

                if (settings.onClose) {
                    settings.onClose();
                }
            }
        }).attach(
            // back button pressed
            new Utils.BackButton(function (data) {
                data.entity.doCallback();
            })
        );
        // var scrollList = new ScrollingList({
        //     z: 2,
        //     updateWhenPaused: Utils.PauseLevels.TEAMEDIT,
        //     position: new Vector2(viewport.width / 2, viewport.height / 2),
        //     direction: 'y',
        //     spacing: 4,
        //     snap: true,
        //     snapMinSlideSpeed: 0.5,
        //     snapSpeed: 0.2,
        //     damping: 0.95,
        //     onSnap: function (item, index) {},
        //     onLoseFocus: function (item) {}
        // });
        var scrollList = new ScrollingList({
            z: 2,
            updateWhenPaused: Utils.PauseLevels.TEAMEDIT,
            position: new Vector2(viewport.width / 2 - 2, viewport.height / 2 + 16),
            direction: 'x',
            spacing: viewport.width,
            snap: true,
            snapMinSlideSpeed: 0.5,
            snapSpeed: 0.2,
            damping: 0.95,
            // area: scrollArea,
            onSnap: function (item, index) {
                if (item) {
                    item.activate();
                }
                pagesDots.iterate(function (item, i) {
                    if (i === index) {
                        item.alpha = (1);
                    } else {
                        item.alpha = (0.5);
                    }
                });
            },
            onLoseFocus: function (item) {
                if (item) {
                    item.deactivate();
                }
            }
        });
        var buttonLeft = new ClickButton({
            sfx: 'sfx_button',
            z: 3,
            image: Bento.assets.getImage('team/buttonarrowleft'),
            frameCountX: 1,
            frameCountY: 1,
            animations: {
                up: {
                    frames: [0]
                },
                down: {
                    frames: [0]
                },
                inactive: {
                    frames: [0]
                }
            },
            updateWhenPaused: Utils.PauseLevels.TEAMEDIT,
            position: new Vector2(6, viewport.height / 3 * 2 + 8),
            onClick: function () {
                scrollList.snapTo(scrollList.getSelection() - 1);
            }
        });
        var buttonRight = new ClickButton({
            sfx: 'sfx_button',
            z: 3,
            image: Bento.assets.getImage('team/buttonarrowright'),
            frameCountX: 1,
            frameCountY: 1,
            animations: {
                up: {
                    frames: [0]
                },
                down: {
                    frames: [0]
                },
                inactive: {
                    frames: [0]
                }
            },
            position: new Vector2(viewport.width - 6, viewport.height / 3 * 2 + 8),
            updateWhenPaused: Utils.PauseLevels.TEAMEDIT,
            onClick: function () {
                scrollList.snapTo(scrollList.getSelection() + 1);
            }
        });
        var pagesDots = new CenteredList({
            position: new Vector2(viewport.width / 2, viewport.height - 4),
            spacing: 2
        });
        var infoAvatar = new Entity({
            name: 'infoAvatar',
            position: new Vector2(-58, -1),
            originRelative: new Vector2(0.5, 0.5),
            components: []
        });
        var infoName = new Text({
            position: new Vector2(18, -16),
            text: '',
            font: 'volter',
            fontSize: 9,
            fontSettings: Utils.FontSettings.get('font'),
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'bottom',
            maxWidth: 96,
            linebreaks: false,
            updateWhenPaused: Utils.PauseLevels.TEAMEDIT
        });
        var infoDescription = new Text({
            position: new Vector2(20, -9),
            text: 'Click on a card for info.',
            font: 'volter',
            fontSize: 9,
            fontSettings: Utils.FontSettings.get('font'),
            fontColor: '#ffffff',
            align: 'center',
            maxWidth: 96,
            maxHeight: 32,
            textBaseline: 'top',
            updateWhenPaused: Utils.PauseLevels.TEAMEDIT
        });
        var collected = {
            collected: 0,
            total: 0,
            tier1: 0,
            tier2: 0,
            tier3: 0,
            tier4: 0,
            tier5: 0,
            tier6: 0,
            tier7: 0,
            tier8: 0,
            tier9: 0,
            tier10: 0,
            tier11: 0
        };
        var infoText1 = new Text({
            position: new Vector2(0, -32),
            text: 'Critters 0/100',
            font: 'volter',
            fontSize: 9,
            fontSettings: Utils.FontSettings.get('font'),
            fontColor: '#ffffff',
            align: 'center',
            maxWidth: 146,
            maxHeight: 64,
            textBaseline: 'top',
            updateWhenPaused: Utils.PauseLevels.TEAMEDIT
        });
        var infoText2 = new Text({
            position: new Vector2(-76, -18),
            text: '★1: 10/10\n★2: 10/10\n★3: 10/10\n★4: 10/10\n★5: 10/10',
            font: 'volter',
            fontSize: 9,
            fontSettings: Utils.FontSettings.get('font'),
            fontColor: '#ffffff',
            align: 'left',
            maxWidth: 78,
            maxHeight: 64,
            textBaseline: 'top',
            updateWhenPaused: Utils.PauseLevels.TEAMEDIT
        });
        var infoText3 = new Text({
            position: new Vector2(0, -18),
            text: '★1: 10/10\n★2: 10/10\n★3: 10/10\n★4: 10/10\n★5: 10/10',
            font: 'volter',
            fontSize: 9,
            fontSettings: Utils.FontSettings.get('font'),
            fontColor: '#ffffff',
            align: 'left',
            maxWidth: 78,
            maxHeight: 64,
            textBaseline: 'top',
            updateWhenPaused: Utils.PauseLevels.TEAMEDIT
        });
        var infoSeperator = new Entity({
            name: 'seperator',
            position: new Vector2(18, -12),
            originRelative: new Vector2(0.5, 0.5),
            updateWhenPaused: Utils.PauseLevels.TEAMEDIT,
            components: [new Sprite({
                imageName: 'map/stripe'
            })]
        });
        var info = new Entity({
            name: 'infoPanelCritter',
            position: new Vector2(viewport.width / 2 + 5, viewport.height / 2 - 80),
            originRelative: new Vector2(0.5, 0.5),
            components: [
                new Sprite({
                    imageName: 'team/infopanel'
                }),
                infoName,
                // infoSeperator,
                // infoDescription,
                infoText1,
                // infoText2,
                // infoText3,
                infoAvatar
            ]
        });
        // array of critters that have been caught and seen in critterdex
        var seen = Bento.saveState.load('seenInCritterDex', []);
        var makeCritterList = function (critters) {
            var i = 0;
            var currentPage;

            var addPage = function () {
                var page = new Page({
                    maxItems: 9,
                    spacing: new Vector2(2, 2)
                });
                var addDot = function () {
                    var dotSprite = new Sprite({
                        imageName: 'gui/dot'
                    });
                    var dot = new Entity({
                        originRelative: new Vector2(0.5, 0.5),
                        components: [dotSprite]
                    });
                    if (i !== 0) {
                        dot.alpha = (0.5);
                    }
                    pagesDots.addItem(dot);
                };
                // add to list
                scrollList.addItem(page);
                addDot();

                // set current
                if (currentPage && pages.length > 1) {
                    currentPage.deactivate();
                }

                currentPage = page;

                return page;
            };
            var pages = [];

            // go through the lists and add items into pages
            Utils.forEach(critters, function (critter) {
                var card;
                var rarity = critter.rarity;
                var hasInCollection = Utils.hasInCollection(critter.id);
                if (!currentPage || currentPage.isFull()) {
                    pages.push(addPage());
                }

                // make card
                card = new Card(critter, {
                    hidden: !hasInCollection
                });

                // add 'new' animation if a critter was caught but not yet seen
                if (hasInCollection && seen.indexOf(critter.id) < 0) {
                    seen.push(critter.id);
                    card.attach(
                        new NewCritter({
                            position: new Vector2(5, -23),
                            small: true
                        })
                    );
                }

                card.attach(new Clickable({
                    onClick: function () {
                        // setInfoPanel(card, hasInCollection);
                    }
                }));
                // remove hp bar
                card.getComponent('hpBar', function (component) {
                    card.remove(component);
                });
                card.updateWhenPaused = Utils.PauseLevels.TEAMEDIT;
                currentPage.addItem(card);

                // count
                collected.total += 1;
                collected['tier' + rarity] = collected['tier' + rarity] || 0;
                collected['tier' + rarity + 'Total'] = collected['tier' + rarity + 'Total'] || 0;
                collected['tier' + rarity + 'Total'] += 1;
                if (hasInCollection) {
                    collected.collected += 1;
                    collected['tier' + rarity] += 1;
                }

                if (totalTiers < rarity) {
                    totalTiers = rarity;
                }
            });
            // save which critters have already been seen
            Bento.saveState.save('seenInCritterDex', seen);

            if (pages.length <= 1) {
                pagesDots.visible = false;
            }
        };
        var setInfoPanel = function (card, hasInCollection) {
            var ability;
            if (infoAvatar) {
                // remove old
                info.remove(infoAvatar);
                infoAvatar = null;
            }
            infoName.setText(card.getProperty('name'));

            if (!hasInCollection) {
                infoDescription.setText('???');
                return;
            }
            infoAvatar = new Entity({
                name: 'infoAvatar',
                position: new Vector2(-54, 2),
                originRelative: new Vector2(0.5, 0.5),
                components: [new Sprite({
                    imageName: 'selection/creatures/' + card.getId()
                })]
            });
            info.attach(infoAvatar);

            // TODO get language
            infoDescription.setText(Bento.assets.getJson('creatures/descriptions/' + 'en')[card.getId()] || '?');
        };
        var makeCritters = function () {
            var creatures = Bento.assets.getJson('creatures/creatures').creatures;
            var critters = [];

            // collect and sort
            Utils.forEach(creatures, function (critter) {
                critters.push(Utils.clone(critter));
            });
            // sort by *
            // secondary sort: health?
            critters.sort(function (a, b) {
                return ((a.rarity || 0) * 10000 + (a.hp || 0)) - ((b.rarity || 0) * 10000 + (b.hp || 0));
            });

            makeCritterList(critters);
            // Utils.forEach(critters, function (critter) {
            //     var properties = Utils.clone(critter);
            //     var hasInCollection = Utils.hasInCollection(properties.id);
            //     var card = new Card(properties, {
            //         hidden: !hasInCollection
            //     });
            //     var text = new Text({
            //         position: new Vector2(24, 0),
            //         text: properties.name,
            //         font: 'Volter',
            //         fontSize: 9,fontSettings: Utils.FontSettings.get('font'),
            //         fontColor: hasInCollection ? '#ffffff' : '#aaaaaa',
            //         align: 'left',
            //         textBaseline: 'middle'
            //     });

            //     card.position.x -= 32;
            //     card.attach(text);

            //     scrollList.addItem(card);
            // });
        };
        var counters = 0;
        var totalTiers = 0;
        var showCollected = function () {
            Utils.repeat(11, addStar);
        };
        var addStar = function () {
            var tier = counters + 1;
            var coinCounter = new CoinCounter({
                imageName: 'gui/numbersdex',
                position: new Vector2(-74 + Math.floor(counters / (totalTiers / 3)) * 46, -16 + Math.floor(counters % (totalTiers / 3)) * 10),
                value: "*" + tier + ":" + collected['tier' + tier] + "/" + collected['tier' + tier + 'Total']
            });
            info.attach(coinCounter);
            counters += 1;
        };

        makeCritters();
        // show collected
        infoText1.setText(Utils.Localization.getText('critters') + ' ' + collected.collected + '/' + collected.total);
        infoText2.setText(
            '★1: ' + collected['tier1'] + '/' + collected['tier1Total'] + '\n' +
            '★2: ' + collected['tier2'] + '/' + collected['tier2Total'] + '\n' +
            '★3: ' + collected['tier3'] + '/' + collected['tier3Total'] + '\n' +
            '★4: ' + collected['tier4'] + '/' + collected['tier4Total'] + '\n'
        );
        infoText3.setText(
            '★5: ' + collected['tier5'] + '/' + collected['tier5Total'] + '\n' +
            '★6: ' + collected['tier6'] + '/' + collected['tier6Total'] + '\n' +
            '★7: ' + collected['tier7'] + '/' + collected['tier7Total'] + '\n' +
            '★8: ' + collected['tier8'] + '/' + collected['tier8Total'] + '\n'
        );
        showCollected();

        container.attach(background);
        container.attach(info);
        container.attach(buttonExit);
        container.attach(scrollList);

        container.attach(buttonLeft);
        container.attach(buttonRight);

        Bento.objects.pause(Utils.PauseLevels.TEAMEDIT);

        Utils.Analytics.trackView('critterDex');

        return container;
    };
});