/**
 * Paramaters are passed as properties of the settings object.
 * @param  {Boolean}  fadeIn         Optional. True if fade is meant to disappear
 * @param  {Function} callback       Optional. If provided, will be called at end of curtain movement
 */
bento.define('gui/fade', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'components/stretchtofill'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    Utils,
    Tween,
    StretchToFill
) {
    'use strict';
    return function (settings) {
        var viewport = Bento.getViewport(),
            screenMid = new Vector2(viewport.width / 2, viewport.height / 2),
            sprite = new Sprite({
                imageName: 'fade'
            }),
            end,
            start,
            fadeIn = settings.fadeIn,
            callback = settings.callback || null,
            type = settings.type || 'fade',
            entity = new Entity({
                z: Utils.Layers.FADE,
                name: 'fade',
                position: screenMid.clone(),
                originRelative: new Vector2(0.5, 0.5),
                components: [
                    new StretchToFill({
                        retainRatio: false
                    }),
                    sprite
                ],
                updateWhenPaused: Utils.PauseLevels.ALWAYS,
                float: true
            }),
            setFade = function (v, t) {
                if (type === 'drop') {
                    entity.setOriginRelative(new Vector2(0.5, 1));
                    entity.position.y = Utils.screenSize.y + v * Utils.screenSize.height;
                } else {
                    entity.alpha = (v);
                }
                if (settings.onFade) {
                    settings.onFade(v, t);
                }
            };

        Bento.objects.add(entity);


        if (!fadeIn) {
            start = 0;
            end = 1;
        } else {
            start = 1;
            end = 0;
        }
        // init
        setFade(start);

        new Tween({
            from: start,
            to: end,
            'in': 40,
            ease: 'easeOutCubic',
            updateWhenPaused: Utils.PauseLevels.ALWAYS,
            'do': setFade,
            onComplete: function () {
                if (callback)
                    callback();
                Bento.objects.remove(entity);
            }
        });

        return entity;
    };
});