/**
 * Same one as in Sky Chasers. Maybe some small differences in positional placement or whatever.
 */
bento.define('gui/horizontalbar', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    Utils,
    Tween
) {
    'use strict';
    return function (settings) {
        var viewport = Bento.getViewport(),
            maxValue = settings.maxValue,
            middleWidth = 1,
            totalWidth = settings.totalWidth,
            value = settings.value,
            spriteLeft = new Sprite({
                imageName: settings.leftImageName
            }),
            spriteMiddle = new Sprite({
                imageName: settings.middleImageName
            }),
            spriteRight = new Sprite({
                imageName: settings.rightImageName
            }),
            entity = new Entity({
                z: settings.z || 0,
                name: settings.name || 'horizontalBar',
                position: settings.position || new Vector2(0, 0.5),
                components: [],
                updateWhenPaused: settings.updateWhenPaused
            }),
            entityLeft = new Entity({
                name: 'left',
                position: new Vector2(0, 0),
                originRelative: new Vector2(0, 0.5),
                components: [spriteLeft]
            }),
            entityMiddle = new Entity({
                name: 'middle',
                position: new Vector2(0, 0),
                originRelative: new Vector2(0, 0.5),
                components: [spriteMiddle]
            }),
            entityRight = new Entity({
                name: 'right',
                position: new Vector2(0, 0),
                originRelative: new Vector2(0, 0.5),
                components: [spriteRight]
            }),
            opacities = [
                entityLeft.getComponent('opacity'),
                entityMiddle.getComponent('opacity'),
                entityRight.getComponent('opacity')
            ],
            setMiddlePos = function () {
                entityMiddle.position.x = entityLeft.dimension.width;
            },
            update = function () {
                // set up positions and scale
                entityMiddle.scale.x = ((value / maxValue) * totalWidth) / middleWidth;
                entityRight.position.x = ((value / maxValue) * totalWidth);

                if (value <= 0.000001) {
                    entity.visible = false;
                } else {
                    entity.visible = true;
                }
            };

        // setup scenegraph
        entity.attach(entityLeft);
        entity.attach(entityMiddle);
        entity.attach(entityRight);
        middleWidth = entityMiddle.dimension.width;

        update();

        // public
        entity.extend({
            setTotalWidth: function (px) {
                totalWidth = px;
                update();
                setMiddlePos();
            },
            setMaxValue: function (val) {
                maxValue = val;
                update();
                setMiddlePos();
            },
            setValue: function (val) {
                value = val;
                value = Math.min(value, maxValue);
                value = Math.max(value, 0);
                update();
            },
            getValue: function () {
                return value;
            },
            addValue: function (val) {
                value += val;
                value = Math.min(value, maxValue);
                value = Math.max(value, 0);
                update();
            },
            /**
             * Change the left, middle, and right sprites.
             * @param {Array} sprites - array with the imageNames of the sprites
             */
            setSprites: function (sprites) {
                entityLeft.getComponent('sprite').setup({
                    imageName: sprites[0]
                });
                entityMiddle.getComponent('sprite').setup({
                    imageName: sprites[1]
                });
                entityRight.getComponent('sprite').setup({
                    imageName: sprites[2]
                });
            },
            /**
             * Sets the opacity of entity left, middle, and right.
             * @param {Number} values - array of 3 numbers between 0 and 1 for opacity
             */
            setOpacity: function (values) {
                for (var i = 0, len = opacities.length; i < len; i++)
                    opacities[i].setOpacity(values[i]);
            }
        });

        if (totalWidth)
            entity.setTotalWidth(totalWidth);
        if (maxValue)
            entity.setMaxValue(maxValue);

        return entity;
    };
});