/**
 * CoinCounter with a clickable +coins attached
 * Listens to updateCoins event
 */
bento.define('gui/hudcoincounter', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'gui/coincounter',
    'gui/shop/shop'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    ClickButton,
    Counter,
    Text,
    EventSystem,
    Utils,
    Tween,
    CoinCounter,
    Shop
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var coinCounter = new CoinCounter({
            z: Utils.Layers.SHOP + 1,
            name: 'myCoins',
            position: settings.position || new Vector2(viewport.width - 2, 2),
            float: true,
            align: 'right',
            family: ['coinCounters'],
            value: Bento.saveState.load('myCoins', 0),
            updateWhenPaused: Utils.PauseLevels.SHOP
        });
        var coinUpdater = {
            name: 'coinUpdater',
            onCoinUpdate: function () {
                coinCounter.setValue(Bento.saveState.load('myCoins', 0));
                coinBehavior.update();
            },
            start: function (data) {
                EventSystem.on('updateCoins', this.onCoinUpdate);
            },
            destroy: function (data) {
                EventSystem.off('updateCoins', this.onCoinUpdate);
            }
        };
        var coinBehavior = {
            update: function (data) {
                var digits = coinCounter.getDigits();
                coin.position.x = -digits * 5 - 3;
            }
        };
        var plus = new Entity({
            name: 'plus',
            position: new Vector2(-2, -3),
            originRelative: new Vector2(0.5, 0.5),
            components: [
                new Sprite({
                    imageName: 'shop/plus'
                })
            ]
        });
        var coin = new Entity({
            name: 'coin',
            originRelative: new Vector2(1, 0.5),
            position: new Vector2(-1, 5),
            boundingBox: new Rectangle(-16, -16, 64, 32),
            updateWhenPaused: Utils.PauseLevels.SHOP - 1,
            components: [
                new Sprite({
                    imageName: 'shop/coinplus'
                }),
                plus,
                coinBehavior,
                new Clickable({
                    onClick: function () {
                        // show shop
                        var hasShop = Bento.objects.get('shop');
                        if (!hasShop) {
                            new Shop({});
                        }
                    }
                })
            ]
        });
        // check if player has liked a social media page but has not yet received their reward
        var checkIfReceivedSocialReward = function () {
            // liked facebook but hasn't received reward
            if (Bento.saveState.load('likedFacebook', false) && !Bento.saveState.load('receivedFacebookReward', false)) {
                new Shop({
                    socialReward: 'Facebook'
                });
            }
            // liked twitter but hasn't received reward
            else if (Bento.saveState.load('likedTwitter', false) && !Bento.saveState.load('receivedTwitterReward', false)) {
                new Shop({
                    socialReward: 'Twitter'
                });
            }
        };

        coinCounter.attach(coin);
        coinCounter.attach(coinUpdater);

        // allow resizing of boundingbox
        coinCounter.extend({
            setBoundingBox: function (rect) {
                coin.boundingBox = rect;
            }
        });

        // wait 45 frames before checking for social reward as there seems
        // to be some sort weird pause bug if it is done right away
        // and the daily gift check comes after 30 frames
        Utils.timeout(45, checkIfReceivedSocialReward, Utils.PauseLevels.DAILYGIFT - 1);

        return coinCounter;
    };
});