/**
 * Pause overlay
 */
bento.define('gui/pause', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'bento/gui/clickbutton',
    'bento/gui/togglebutton'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    Utils,
    Tween,
    ClickButton,
    ToggleButton
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var sprite = new Sprite({
            imageName: 'fade'
        });
        var overlay = new Entity({
            name: 'overlay',
            position: new Vector2(0, 0),
            originRelative: new Vector2(0.5, 0.5),
            components: [sprite]
        });
        var entity = new Entity({
            z: Utils.Layers.PAUSE,
            name: 'pause',
            position: new Vector2(viewport.width / 2, viewport.height / 2),
            originRelative: new Vector2(0.5, 0.5),
            updateWhenPaused: Utils.PauseLevels.PAUSE,
            float: true,
            components: [
                
            ]
        });
        var buttonReset = ClickButton({
            sfx: 'sfx_button',
            z: Utils.Layers.PAUSE,
            image: Bento.assets.getImage('gui/buttonreset'),
            frameWidth: 128,
            frameHeight: 48,
            updateWhenPaused: Utils.PauseLevels.PAUSE,
            position: new Vector2(-viewport.width / 2 + 48, viewport.height / 2 - 24),
            onClick: function () {
                var res,
                    removeSaveFile = function () {
                        Utils.reset();
                        Bento.resume();
                        Bento.screens.show('screens/title');
                    };
                if (Utils.isCocoonJS() && Cocoon) {
                    Cocoon.Dialog.confirm({
                        title: "Warning",
                        message: "You are about to delete your savefile! Are you sure?"
                    }, function (accepted) {
                        if (accepted) {
                            removeSaveFile();
                        }
                    });
                } else {
                    res = confirm("You are about to delete your savefile! Are you sure?");
                    if (res) {
                        removeSaveFile();
                    }
                }
            }
        });
        var toggleMusic = new ToggleButton({
            z: Utils.Layers.PAUSE,
            toggled: Bento.saveState.load('muteMusic', false),
            image: Bento.assets.getImage('gui/buttonmusic'),
            position: new Vector2(viewport.width / 2 - 24, viewport.height / 2 - 24 - 48),
            frameCountX: 2,
            frameCountY: 1,
            updateWhenPaused: Utils.PauseLevels.PAUSE,
            sfx: 'sfx_button',
            onToggle: function () {
                Bento.audio.muteMusic(this.isToggled(), true);
                Bento.saveState.save('muteMusic', this.isToggled());
            }
        });
        var toggleSound = new ToggleButton({
            z: Utils.Layers.PAUSE,
            toggled: Bento.saveState.load('muteSound', false),
            image: Bento.assets.getImage('gui/buttonsound'),
            position: new Vector2(viewport.width / 2 - 24, viewport.height / 2 - 24),
            frameCountX: 2,
            frameCountY: 1,
            updateWhenPaused: Utils.PauseLevels.PAUSE,
            sfx: 'sfx_button',
            onToggle: function () {
                Bento.audio.muteSound(this.isToggled());
                Bento.saveState.save('muteSound', this.isToggled());
            }
        });
        var buttonResume = new ClickButton({
            sfx: 'sfx_button',
            updateWhenPaused: Utils.PauseLevels.PAUSE,
            image: Bento.assets.getImage('gui/buttonresume'),
            frameCountX: 1,
            frameCountY: 2,
            animations: {
                up: {
                    frames: [0]
                },
                down: {
                    frames: [1]
                }
            },
            position: new Vector2(0, 0),
            onClick: function () {
                Bento.objects.resume();
                Bento.objects.remove(entity);
            }
        });

        overlay.alpha = (0.75);
        entity.attach(overlay);
        entity.attach(buttonReset);
        entity.attach(buttonResume);
        entity.attach(toggleMusic);
        entity.attach(toggleSound);

        Bento.objects.attach(entity);
        Bento.objects.pause(Utils.PauseLevels.PAUSE);

        return entity;
    };
});