/**
 * Progress bar entity. Consists of 2 sprites, one is always drawn in the background and one is drawn partially from left to right depending on the current progress
 */
bento.define('gui/progressbar', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/utils',
    'bento/tween'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var assetPath = settings.assetPath || '';
        var easeFactor = settings.easeFactor || 0.25;

        var progress = 0;
        var targetProgress = 0;

        var backgroundSprite = new Sprite({
            imageName: settings.backgroundImageName || (assetPath + 'progressbar')
        });

        var barFillImage = Bento.assets.getImage(settings.fillImageName || (assetPath + 'barfill'));
        var imgWidth = barFillImage.width;
        var imgHeight = barFillImage.height;

        var barFillEntity = new Entity({
            z: 0,
            name: 'barFill',
            position: new Vector2(0, 0),
            originRelative: new Vector2(0.5, 0.5),
            components: [{
                draw: function (data) {
                    //firefox doesn't allow source regions smaller than 1*1 pixel to be drawn....
                    if (progress === 0) {
                        return;
                    }
                    data.renderer.translate(-Math.round(imgWidth / 2), -Math.round(imgHeight / 2));
                    data.renderer.drawImage(barFillImage, 0, 0, Math.ceil(imgWidth * progress), imgHeight, 0, 0, Math.ceil(imgWidth * progress), imgHeight);
                    data.renderer.translate(Math.round(imgWidth / 2), Math.round(imgHeight / 2));
                }
            }]
        });


        var entity = new Entity({
            z: settings.z || 0,
            name: 'progressBar',
            position: settings.position || new Vector2(0, 0),
            originRelative: new Vector2(0.5, 0.5),
            updateWhenPaused: 0,
            components: [
                backgroundSprite,
                barFillEntity, {
                    update: function (data) {
                        progress += (targetProgress - progress) * easeFactor;
                        if (Math.abs(targetProgress - progress) < 0.01) {
                            progress = targetProgress;
                        }
                    }
                }
            ]
        }).extend({
            setProgress: function (_progress) {
                targetProgress = Utils.clamp(0, _progress, 1);
            },
            setProgressPercent: function (percent) {
                entity.setProgress(percent / 100);
            },
            getProgress: function () {
                return targetProgress;
            },
            getProgressPercent: function () {
                return Math.floor(targetProgress * 100);
            },
            addProgress: function (_progress) {
                entity.setProgress(targetProgress + _progress);
            },
            addProgressPercent: function (percent) {
                entity.addProgress(percent / 100);
            },
        });
        return entity;
    };
});