/**
 * Reward popup with coins and storydialog
 */
bento.define('gui/rewardpopup', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'gui/storydialog'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    ClickButton,
    Counter,
    Text,
    EventSystem,
    Utils,
    Tween,
    StoryDialog
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
            reward: Number,
            pauseLevel: Number,
            trackCoins: Bool, // should track coins? default = true
            onClose: Function,
            giveImmediately: Bool // should reward be given before dialog is closed? default = false
        }*/
        var viewport = Bento.getViewport();
        var reward = settings.reward || 0;
        var trackCoins = Utils.isDefined(settings) ? settings.trackCoins : true;
        var delay = 0;
        var entity;
        var pauseLevel = settings.pauseLevel || Utils.PauseLevels.STORY;
        var pauseLevelCoins = settings.pauseLevelCoins;
        var onClose = settings.onClose || function () {};
        var giveImmediately = settings.giveImmediately || false;
        var spawnCoin = function () {
            var coinSprite = new Sprite(Bento.assets.getJson('animations/coinpickup'));
            var behavior = {
                lifeTime: Utils.getRandomRange(80, 100),
                speed: new Vector2(
                    Utils.getRandomRangeFloat(-2, 2),
                    Utils.getRandomRangeFloat(-6, -3)
                ),
                update: function () {
                    coin.position.x += this.speed.x;
                    coin.position.y += this.speed.y;

                    this.speed.y += 0.1;
                    if (coin.timer > this.lifeTime) {
                        coin.visible = coin.timer % 2;
                    }
                    if (coin.timer > this.lifeTime + 20) {
                        Bento.objects.remove(coin);
                    }
                }
            };
            var coin = new Entity({
                z: Utils.Layers.FADE - 1,
                name: 'coin',
                position: new Vector2(viewport.width / 2, viewport.height / 2).add(new Vector2(0, 16)),
                originRelative: new Vector2(0.5, 0.5),
                float: true,
                updateWhenPaused: pauseLevelCoins || Utils.PauseLevels.ALWAYS,
                components: [
                    coinSprite,
                    behavior
                ]
            });
            Utils.timeout(delay, function () {
                Bento.objects.attach(coin);
                Bento.audio.stopSound('sfx_coin');
                Bento.audio.playSound('sfx_coin');
            }, pauseLevel);
            delay += 5;
            return coin;
        };
        var giveReward = function () {
            Bento.objects.get('myCoins', function (myCoins) {
                var coins = Bento.saveState.load('myCoins', 0);
                coins += reward;
                if (trackCoins) {
                    Utils.trackCoins(reward);
                }
                if (settings.doNotReward) {
                    return;
                }
                Bento.saveState.save('myCoins', coins);
                EventSystem.fire('updateCoins');
            });
        };

        // cheer sound
        Utils.timeout(90, function () {
            Bento.audio.playSound('sfx_cheer');
        }, pauseLevel);

        Utils.timeout(30, function () {
            Bento.audio.playSound('sfx_combine');
            // flying coins
            Utils.repeat(10, spawnCoin);
        }, pauseLevel);

        entity = new StoryDialog({
            z: settings.z,
            text: Utils.Localization.getText('reward').replace("{REWARD}", reward),
            pauseLevel: pauseLevel,
            onClose: function () {
                if (!giveImmediately) {
                    giveReward();
                }

                onClose();
            }
        });

        if (giveImmediately) {
            giveReward();
        }

        return entity;
    };
});