/**
 * Module description
 */
bento.define('gui/shop/buttonrestore', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'gui/shop/iapoverlay',
    'gui/shop/shop',
    'store'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    ClickButton,
    Counter,
    Text,
    EventSystem,
    Utils,
    Tween,
    IapOverlay,
    Shop,
    Store
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var iapOverlay = new IapOverlay({});
        var cleaned = false;
        var entity = new ClickButton({
            sfx: 'sfx_button',
            z: Utils.Layers.PAUSE,
            name: 'buttonCredits',
            image: Bento.assets.getImage('gui/buttonrestore'),
            frameCountX: 1,
            frameCountY: 2,
            updateWhenPaused: Utils.PauseLevels.MENU,
            position: settings.position || new Vector2(-viewport.width / 2 + 30, viewport.height / 2 - 26),
            onClick: function () {
                var success = Store.restore();

                cleaned = false;

                // show overlay
                Bento.objects.attach(iapOverlay);
                iapOverlay.setText(Utils.Localization.getText('restore'));

                // wait for callback
                EventSystem.on('restore_success', onPurchaseSuccess);
                EventSystem.on('restore_error', onPurchaseError);
                EventSystem.on('purchase_error', onPurchaseError);

                if (!success) {
                    if (!Utils.isCocoonJS()) {
                        onPurchaseError('Only available in the App version!');
                    } else {
                        onPurchaseError(Utils.Localization.getText('errorstore'));
                    }
                }

                // timeout
                Utils.timeout(60 * 5, function () {
                    if (!cleaned) {
                        // todo: add localization for timeout
                        iapOverlay.setText("Timed out!");
                        iapOverlay.exit(60, function () {});
                        cleanUp();
                    }
                }, Utils.PauseLevels.ALWAYS);
            }
        });
        var onPurchaseSuccess = function (purchaseInfo) {
            iapOverlay.setText(Utils.Localization.getText('purchased'));
            iapOverlay.exit(60, function () {});
            cleanUp();
        };

        var onPurchaseError = function (err) {
            console.log('purchase failed!', JSON.stringify(err));
            Utils.Analytics.event('UI', 'restoreError', undefined, JSON.stringify(err));
            iapOverlay.setText(Utils.Localization.getText('error') + '\n' + JSON.stringify(err));
            iapOverlay.exit(120);
            cleanUp();
        };
        var cleanUp = function () {
            EventSystem.off('restore_success', onPurchaseSuccess);
            EventSystem.off('restore_error', onPurchaseError);
            EventSystem.off('purchase_error', onPurchaseError);
            cleaned = true;
        };
        return entity;
    };
});