/**
 * Overlay that waits for IAP callback
 */
bento.define('gui/shop/iapoverlay', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'store'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    ClickButton,
    Counter,
    Text,
    EventSystem,
    Utils,
    Tween,
    Store
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            timeOutTime: Number // frames until timeout, defaults to 600 (10 seconds)
            onTimeOut: Function // what should happen if the IAP timed out
        }*/
        var viewport = Bento.getViewport();
        // timeout
        var timeOutTime = settings.timeOutTime || 600;
        var onTimeOut = settings.onTimeOut;
        // dark overlay
        var overlay = new Entity({
            name: 'overlay',
            position: new Vector2(0, 0),
            originRelative: new Vector2(0.5, 0.5),
            components: [new Sprite({
                imageName: 'fade'
            })],
            init: function () {
                this.alpha = 0.75;
            }
        });
        var text = new Text({
            position: new Vector2(0, 0),
            text: '...',
            fontSettings: Utils.FontSettings.get('font'),
            fontColor: '#ffffff',
            align: 'center',
            maxWidth: viewport.width,
            textBaseline: 'middle'
        });
        var pauseBehavior = {
            oldPauseLevel: 0,
            start: function () {
                console.log('attached')
                this.oldPauseLevel = Bento.objects.isPaused();
                Bento.objects.pause(Utils.PauseLevels.SHOPRESPONSE);
            },
            destroy: function () {
                console.log('removed')
                Bento.objects.pause(this.oldPauseLevel);
            }
        };
        var timeOutBehavior = {
            hasTimedOut: false,
            update: function () {
                if (this.hasTimedOut === false && entity.timer > timeOutTime) {
                    this.hasTimedOut = true;
                    if (onTimeOut) {
                        onTimeOut();
                    }
                }
            }
        };
        var entity = new Entity({
            z: Utils.Layers.SYSTEM,
            name: 'iapOverlay',
            position: new Vector2(viewport.width / 2, viewport.height / 2),
            updateWhenPaused: Utils.PauseLevels.SHOPRESPONSE,
            float: true,
            components: [
                overlay,
                text,
                pauseBehavior,
                timeOutBehavior
            ]
        });

        // public
        entity.extend({
            /**
             * Close overlay (with a delay)
             */
            exit: function (delay, callback) {
                Utils.timeout(delay || 0, function () {
                    Bento.objects.remove(entity);
                    if (callback) {
                        callback();
                    }
                }, Utils.PauseLevels.SHOPRESPONSE);
            },
            /**
             * Set overlay text
             */
            setText: function (str) {
                text.setText(str);
            }
        });

        return entity;
    };
});