/**
 * Shop overlay
 */
bento.define('gui/shop/shop', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'store',
    'gui/shop/iapoverlay',
    'gui/coinsforad',
    'modules/videoadtimer',
    'gui/rewardpopup',
    'modules/playtimer',
    'gui/dialog'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    ClickButton,
    Counter,
    Text,
    EventSystem,
    Utils,
    Tween,
    Store,
    IapOverlay,
    CoinsForAd,
    VideoAdTimer,
    RewardPopup,
    PlayTimer,
    Dialog
) {
    'use strict';
    // global purchase event
    var purchaseCoins = function (productId) {
        var myCoins = Bento.saveState.load('myCoins', 0);

        myCoins += Utils.rewardsIAP[productId];

        Bento.saveState.save('myCoins', myCoins);
        EventSystem.fire('updateCoins');
        Store.consume(productId, function () {
            console.log(productId + ' consumed.');
        });

        // update counters?

    };
    EventSystem.on('purchase_success', function (productInfo) {
        var productId = productInfo.productId;
        var product = productInfo.product;
        var transactionId = productInfo.transactionId;
        var trackPurchase = true;
        var lastReceipt = productInfo.lastReceipt;

        // removed ads?
        if (productId === 'removeads') {
            // already bought this: don't send an event
            if (Bento.saveState.load('removeAds', false)) {
                trackPurchase = false;
            }
            // set global flag
            Bento.saveState.save('removeAds', true);
        } else {
            // consume coinpack
            purchaseCoins(productId);
        }

        if (product && trackPurchase) {
            Utils.Analytics.trackPurchase(
                productInfo.transactionId, // id
                product.title, // title
                productInfo.productId, // sku
                'IAP', // category
                product.price, // price
                1, // quantity
                product.currency || '' // currencycode
            );

            // Tenjin
            if (window.tenjin && Utils.isNativeIos()) {
                // iOS only for now (android crashes)
                window.tenjin.iosTransaction(
                    productInfo.productId, // sku 
                    product.currency || '', // currencycode
                    1, // quantity
                    product.price, // price
                    productInfo.transactionId, // transaction id
                    productInfo.lastReceipt // receipt in base64
                );
                // window.tenjin.transaction(
                //     productInfo.productId, // sku
                //     product.currency || '', // currencycode
                //     1, // quantity
                //     product.price, // price
                //     function () {
                //         console.log('Tenjin transaction complete.');
                //     },
                //     function () {
                //         console.log('Tenjin transaction failure.');
                //     }
                // );
            }
        }

    });
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var screenMid = new Vector2(viewport.width / 2, viewport.height / 2);
        var minutesSinceLastVisit = Math.floor((PlayTimer.playTimer - Bento.saveState.load('timeOfLastShopVisit', 0)) / 60 / 60);
        var oldPauseLevel = Bento.objects.isPaused();
        var overlay = new Entity({
            name: 'overlay',
            position: new Vector2(0, 0),
            originRelative: new Vector2(0.5, 0.5),
            components: [new Sprite({
                imageName: 'fade'
            })],
            init: function () {
                this.alpha = 0.75;
            }
        });
        var products = Store.getProducts();
        var iapOverlay = new IapOverlay({});
        var spendProbability = Bento.saveState.load('playerStats', {
            spendProbability: -1 // -1 is the default value when there's not enough data
        }).spendProbability;
        var highSpender = Bento.saveState.load('highSpender', false);
        var noNoAds = highSpender ? 'nonoads' : '';
        var yPositions = {
            'adButton': 43 * 0 + 29,
            'buttonFacebook': 43 * 1 + 29,
            'buttonTwitter': 43 * 1 + 29,
            'button0': 43 * 1 + 29,
            'button1': 43 * 2 + 29,
            'button2': 43 * 3 + 29,
            'button3': 43 * 4 + 29,
            'button4': 43 * 5 + 29,
            'button5': 43 * 6 + 29,
            'button6': 43 * 7 + 29,
            'shop': -viewport.height / 2 + 25
        };
        var getPrice = function (id) {
            var i;
            if (products) {
                // search
                for (i = 0; i < products.length; ++i) {
                    if (products[i].productId === id) {
                        return products[i].localizedPrice;
                    }
                }
            }
            // if products can't be loaded, load price from save state
            else if (Bento.saveState.load(id + 'Price', null)) {
                return Bento.saveState.load(id + 'Price', null);
            }
            return '$0.99';
        };
        var watchAd = function () {
            CoinsForAd.watchAd(function () {
                // clicked on 'ad for coins' from the shop
                Utils.Analytics.event('Ads', 'fromShop');
                Utils.Firebase.logEvent('clicked_ad_from_shop', {
                    value: 1
                });
                // reset ad button
                buttonAd.setActive(false);
                buttonAd.attach(timeLeft);
            });
        };
        var purchase = function (id) {
            var success;

            // show overlay
            Bento.objects.attach(iapOverlay);
            iapOverlay.setText('Purchasing...');

            // wait for callback
            EventSystem.on('purchase_success', onPurchaseSuccess);
            EventSystem.on('purchase_error', onPurchaseError);

            // purchase!
            success = Store.purchase(id);

            if (!success) {
                if (!Utils.isCocoonJS()) {
                    onPurchaseError('Only available in the App version!');
                } else {
                    onPurchaseError(Utils.Localization.getText('errorstore'));
                }
            }

        };
        var onPurchaseSuccess = function (purchaseInfo) {
            var productId = purchaseInfo.productId;
            var reward = Utils.rewardsIAP[productId];
            var iapAmount = Bento.saveState.load('iapAmount', 0);

            // increment the amount of IAP's purchased
            iapAmount++;
            Bento.saveState.save('iapAmount', iapAmount);

            // which IAP was bought
            Utils.Analytics.event('IAP', 'bought_' + productId);
            // amount of IAP's purchased along with the name of the IAP being purchased
            Utils.Analytics.event('IAP', 'iapAmount_' + iapAmount, undefined, productId);
            // when buying the 'no ads' IAP, send the amount of ads watched
            if (productId === 'removeads') {
                Utils.Analytics.event('IAP', 'noAdsWhnFrcdAds_' + Bento.saveState.load('forcedAdsWatched', 0));
                Utils.Analytics.event('IAP', 'noAdsWhnTtalAds_' + Bento.saveState.load('totalAdsWatched', 0));
                Utils.Analytics.event('IAP', 'noAdsWhnCoinAds_' + Bento.saveState.load('coinAdsWatched', 0));
                disableNoAdsButton();
            }
            Utils.Firebase.logEvent('iap_' + productId, {
                value: 1,
                high_spender: highSpender ? 1 : 0,
                spend_probability: spendProbability,
                iaps_so_far: iapAmount,
                forced_ads_watched: Bento.saveState.load('forcedAdsWatched', 0),
                coin_ads_watched: Bento.saveState.load('coinAdsWatched', 0),
                total_ads_watched: Bento.saveState.load('totalAdsWatched', 0)
            });
            if (highSpender) {
                Utils.Firebase.logEvent('high_spender_purchase', {
                    product: productId,
                    price: getPrice(productId),
                    iaps_so_far: iapAmount,
                    spend_probability: spendProbability
                });
            }

            // old event, remove?
            Utils.Analytics.event('UI', 'bought_' + productId);

            iapOverlay.setText(Utils.Localization.getText('purchased'));
            iapOverlay.exit(60, function () {
                if (productId !== 'removeads') {
                    new RewardPopup({
                        trackCoins: false,
                        updateWhenPaused: Utils.PauseLevels.SHOP + 1,
                        reward: reward,
                        doNotReward: true // already rewarded in purchaseCoins function
                    });
                }
            });
            cleanUp();
        };
        var onPurchaseError = function (err) {
            console.log('purchase failed!', JSON.stringify(err));
            Utils.Analytics.event('UI', 'purchaseError', undefined, JSON.stringify(err));
            iapOverlay.setText(Utils.Localization.getText('error') + '\n' + JSON.stringify(err));
            iapOverlay.exit(120);
            cleanUp();
        };
        var cleanUp = function () {
            EventSystem.off('purchase_success', onPurchaseSuccess);
            EventSystem.off('purchase_error', onPurchaseError);
        };
        var hideNoAdsButton = function () {
            var buttons = ['button1', 'button2', 'button3', 'button4', 'button5', 'button6'];
            var i = 0;

            shop.getComponent('button0', function (obj) {
                shop.remove(obj);
            });

            // move coinpack buttons up one space to close the gap left by removing the 'noAds' button
            for (i = 0; i < buttons.length; ++i) {
                shop.getComponent(buttons[i], function (obj) {
                    obj.position.y -= 43;
                });
            }
        };
        var showSocialButton = function (button) {
            var buttons = ['button0', 'button1', 'button2', 'button3', 'button4', 'button5', 'button6'];
            var i = 0;

            shopSprite.setup({
                imageName: 'shop/shopsocialmedialong' + noNoAds
            });

            for (i = 0; i < buttons.length; ++i) {
                shop.getComponent(buttons[i], function (obj) {
                    obj.position.y = yPositions[obj.name] + 43;
                });
            }

            shop.attach(button);
        };
        var hideSocialButton = function () {
            // twitter is checked first because if facebook is checked first
            // the twitter button can be added and removed right away

            // after pressing twitter button shop should revert to old style
            shop.getComponent('buttonTwitter', function (obj) {
                var buttons = ['button0', 'button1', 'button2', 'button3', 'button4', 'button5', 'button6'];
                var i = 0;

                shopSprite.setup({
                    imageName: 'shop/shoplong' + noNoAds
                });

                for (i = 0; i < buttons.length; ++i) {
                    shop.getComponent(buttons[i], function (obj) {
                        obj.position.y = yPositions[obj.name];
                        // for high spenders there is no 'noAds' button and we have to move everything up one more space
                        if (highSpender) {
                            obj.position.y -= 43;
                        }
                    });
                }

                shop.remove(obj);
                shop.position.y = yPositions['shop']; // revert to original y position
            });

            // after pressing facebook button, twitter button should be next in line to be shown
            shop.getComponent('buttonFacebook', function (obj) {
                shop.remove(obj);
                shop.attach(buttonTwitter);
            });
        };
        /**
         * Show a loading dialog while waiting for player to come back from social media
         * @param  {String} socialNetwork - string of the social network the reward is for. First letter is capitalized
         *                                    - possibilities:
         *                                        - 'Facebook'
         *                                        - 'Twitter' (default)
         */
        var waitForSocialMediaLike = function (socialNetwork) {
            var oldPauseLevel = Bento.objects.isPaused();
            socialNetwork = socialNetwork || 'Twitter';

            // set pause level higher so that the IAP buttons don't work
            Bento.objects.pause(oldPauseLevel + 1);

            new Dialog({
                name: 'waitFor' + socialNetwork + 'Like',
                type: 'yes',
                pauseLevel: Bento.objects.isPaused() + 1,
                text: Utils.Localization.getText('loading'),
                onClose: function () {
                    Bento.objects.pause(oldPauseLevel);
                    receiveSocialReward(socialNetwork);
                }
            });
        };
        /**
         * Show a reward popup for clicking on the social media buttons
         * @param  {String} socialNetwork - string of the social network the reward is for. First letter is capitalized
         *                                    - possibilities:
         *                                        - 'Facebook'
         *                                        - 'Twitter' (default)
         */
        var receiveSocialReward = function (socialNetwork) {
            var rewardAmount = Bento.assets.getJson('creatures/rewardamount');
            var unlockedRarity = Bento.saveState.load('unlockedRarity', 3);
            var reward = rewardAmount[unlockedRarity] + Utils.getRandom(rewardAmount[unlockedRarity]);
            var oldPauseLevel = Bento.objects.isPaused();
            socialNetwork = socialNetwork || 'Twitter';

            // set pause level higher so that the IAP buttons don't work
            Bento.objects.pause(oldPauseLevel + 1);

            new Dialog({
                name: 'receive' + socialNetwork + 'Reward',
                type: 'yes',
                pauseLevel: oldPauseLevel + 1,
                text: Utils.Localization.getText('purchased'),
                onClose: function () {
                    new RewardPopup({
                        trackCoins: false,
                        reward: reward,
                        pauseLevel: Bento.objects.isPaused() + 1,
                        onClose: function () {
                            Bento.saveState.save('received' + socialNetwork + 'Reward', true);
                            Bento.objects.pause(oldPauseLevel);
                        }
                    });
                }
            });

            hideSocialButton();
        };
        // buttons
        var adAsGift = (Utils.isFB() && Utils.softlaunch) || Utils.NoAdsVersion; // facebook softlaunch
        var timeLeft = new Text({
            position: new Vector2(0, 0),
            text: "00:00:00",
            fontSettings: Utils.FontSettings.get('font'),
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle'
        });
        var buttonAd = new ClickButton({
            name: 'adButton',
            sfx: 'sfx_button',
            image: Bento.assets.getImage(adAsGift ? 'shop/giftbutton' : 'shop/adbutton'),
            frameCountX: 2,
            frameCountY: 2,
            animations: {
                up: {
                    frames: [0]
                },
                down: {
                    frames: [1]
                },
                inactive: {
                    frames: [2]
                }
            },
            position: new Vector2(29, yPositions['adButton']),
            updateWhenPaused: Utils.PauseLevels.SHOP,
            onClick: function () {
                this.setActive(false);
                watchAd();
            }
        });
        var button0 = new ClickButton({
            name: 'button0',
            sfx: 'sfx_button',
            image: Bento.assets.getImage('shop/button'),
            frameCountX: 3,
            frameCountY: 1,
            animations: {
                up: {
                    frames: [0]
                },
                down: {
                    frames: [1]
                },
                inactive: {
                    frames: [2]
                }
            },
            position: new Vector2(29, yPositions['button0']),
            updateWhenPaused: Utils.PauseLevels.SHOP,
            onClick: function () {
                purchase('removeads');
            }
        }).attach(new Text({
            position: new Vector2(0, 0),
            text: getPrice('removeads'),
            fontSettings: Utils.FontSettings.get('font'),
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle'
        }));
        var button1 = new ClickButton({
            name: 'button1',
            sfx: 'sfx_button',
            image: Bento.assets.getImage('shop/button'),
            frameCountX: 3,
            frameCountY: 1,
            animations: {
                up: {
                    frames: [0]
                },
                down: {
                    frames: [1]
                },
                inactive: {
                    frames: [2]
                }
            },
            position: new Vector2(29, yPositions['button1']),
            updateWhenPaused: Utils.PauseLevels.SHOP,
            onClick: function () {
                purchase('coinpack1');
            }
        }).attach(new Text({
            position: new Vector2(0, 0),
            text: getPrice('coinpack1'),
            fontSettings: Utils.FontSettings.get('font'),
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle'
        }));
        var button2 = new ClickButton({
            name: 'button2',
            sfx: 'sfx_button',
            image: Bento.assets.getImage('shop/button'),
            frameCountX: 3,
            frameCountY: 1,
            animations: {
                up: {
                    frames: [0]
                },
                down: {
                    frames: [1]
                },
                inactive: {
                    frames: [2]
                }
            },
            position: new Vector2(29, yPositions['button2']),
            updateWhenPaused: Utils.PauseLevels.SHOP,
            onClick: function () {
                purchase('coinpack2');
            }
        }).attach(new Text({
            position: new Vector2(0, 0),
            text: getPrice('coinpack2'),
            fontSettings: Utils.FontSettings.get('font'),
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle'
        }));
        var button3 = new ClickButton({
            name: 'button3',
            sfx: 'sfx_button',
            image: Bento.assets.getImage('shop/button'),
            frameCountX: 3,
            frameCountY: 1,
            animations: {
                up: {
                    frames: [0]
                },
                down: {
                    frames: [1]
                },
                inactive: {
                    frames: [2]
                }
            },
            position: new Vector2(29, yPositions['button3']),
            updateWhenPaused: Utils.PauseLevels.SHOP,
            onClick: function () {
                purchase('coinpack3');
            }
        }).attach(new Text({
            name: 'text',
            position: new Vector2(0, 0),
            text: getPrice('coinpack3'),
            fontSettings: Utils.FontSettings.get('font'),
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle'
        }));
        var button4 = new ClickButton({
            name: 'button4',
            sfx: 'sfx_button',
            image: Bento.assets.getImage('shop/button'),
            frameCountX: 3,
            frameCountY: 1,
            animations: {
                up: {
                    frames: [0]
                },
                down: {
                    frames: [1]
                },
                inactive: {
                    frames: [2]
                }
            },
            position: new Vector2(29, yPositions['button4']),
            updateWhenPaused: Utils.PauseLevels.SHOP,
            onClick: function () {
                purchase('coinpack4new');
            }
        }).attach(new Text({
            name: 'text',
            position: new Vector2(0, 0),
            text: getPrice('coinpack4new'),
            fontSettings: Utils.FontSettings.get('font'),
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle'
        }));
        var button5 = new ClickButton({
            name: 'button5',
            sfx: 'sfx_button',
            image: Bento.assets.getImage('shop/button'),
            frameCountX: 3,
            frameCountY: 1,
            animations: {
                up: {
                    frames: [0]
                },
                down: {
                    frames: [1]
                },
                inactive: {
                    frames: [2]
                }
            },
            position: new Vector2(29, yPositions['button5']),
            updateWhenPaused: Utils.PauseLevels.SHOP,
            onClick: function () {
                purchase('coinpack5');
            }
        }).attach(new Text({
            name: 'text',
            position: new Vector2(0, 0),
            text: getPrice('coinpack5'),
            fontSettings: Utils.FontSettings.get('font'),
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle'
        }));
        var button6 = new ClickButton({
            name: 'button6',
            sfx: 'sfx_button',
            image: Bento.assets.getImage('shop/button'),
            frameCountX: 3,
            frameCountY: 1,
            animations: {
                up: {
                    frames: [0]
                },
                down: {
                    frames: [1]
                },
                inactive: {
                    frames: [2]
                }
            },
            position: new Vector2(29, yPositions['button6']),
            updateWhenPaused: Utils.PauseLevels.SHOP,
            onClick: function () {
                purchase('coinpack6');
            }
        }).attach(new Text({
            name: 'text',
            position: new Vector2(0, 0),
            text: getPrice('coinpack6'),
            fontSettings: Utils.FontSettings.get('font'),
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle'
        }));
        var buttonFacebook = new ClickButton({
            name: 'buttonFacebook',
            sfx: 'sfx_button',
            image: Bento.assets.getImage('shop/facebookbutton'),
            frameCountX: 2,
            frameCountY: 1,
            animations: {
                up: {
                    frames: [0]
                },
                down: {
                    frames: [1]
                }
            },
            position: new Vector2(29, yPositions['buttonFacebook']),
            updateWhenPaused: Utils.PauseLevels.SHOP,
            onClick: function () {
                this.setActive(false);

                // go to Lucky Kat facebook page
                if (Utils.isNativeIos()) {
                    Utils.Cocoon.openUrl('fb://profile/466268083523876');
                } else {
                    Utils.Cocoon.openUrl('https://www.facebook.com/luckykatstudios');
                }

                Utils.Firebase.logEvent('clicked_facebook');

                Bento.saveState.save('likedFacebook', true);

                waitForSocialMediaLike('Facebook');
            }
        });
        var buttonTwitter = new ClickButton({
            name: 'buttonTwitter',
            sfx: 'sfx_button',
            image: Bento.assets.getImage('shop/twitterbutton'),
            frameCountX: 2,
            frameCountY: 1,
            animations: {
                up: {
                    frames: [0]
                },
                down: {
                    frames: [1]
                }
            },
            position: new Vector2(29, yPositions['buttonTwitter']),
            updateWhenPaused: Utils.PauseLevels.SHOP,
            onClick: function () {
                this.setActive(false);

                // go to Lucky Kat Twitter page
                if (Utils.isNativeIos()) {
                    Utils.Cocoon.openUrl('twitter://user?screen_name=LuckyKatStudios');
                } else {
                    Utils.Cocoon.openUrl('https://twitter.com/LuckyKatStudios');
                }

                Utils.Firebase.logEvent('clicked_twitter');

                Bento.saveState.save('likedTwitter', true);

                waitForSocialMediaLike('Twitter');
            }
        });
        var scrollable = {
            name: 'scrollable',
            start: function (data) {
                EventSystem.on('pointerDown', scrollable.pointerDown);
                EventSystem.on('pointerMove', scrollable.pointerMove);
                EventSystem.on('pointerUp', scrollable.pointerUp);
            },
            destroy: function (data) {
                EventSystem.off('pointerDown', scrollable.pointerDown);
                EventSystem.off('pointerMove', scrollable.pointerMove);
                EventSystem.off('pointerUp', scrollable.pointerUp);
            },
            shopY: yPositions['shop'],
            startY: 0,
            touchY: 0,
            maxOffsetY: 43 * 3,
            pointerDown: function (evt) {
                if (scrollable.touchY !== null) {
                    return;
                }
                scrollable.touchY = evt.position.y;
                scrollable.shopY = shop.position.y;

                // add an offset of three times the height of the spacing
                // between buttons for the three new coin packages
                scrollable.maxOffsetY = 43 * 3;
                // increase offset by one space if the social media buttons are still on screen
                if (!Bento.saveState.load('likedTwitter', false)) {
                    scrollable.maxOffsetY += 43;
                }
                // decrease offset by one space for high spenders because they don't have the 'noads' button
                if (highSpender) {
                    scrollable.maxOffsetY -= 43;
                }
            },
            pointerMove: function (evt) {
                shop.position.y = Utils.clamp(yPositions['shop'] - scrollable.maxOffsetY, scrollable.shopY - scrollable.touchY + evt.position.y, yPositions['shop']);
            },
            pointerUp: function (evt) {
                scrollable.touchY = null;
            }
        };

        // shop image, which is the container for buttons
        var shopSprite = new Sprite({
            imageName: (Utils.isFB() || Utils.WEB) ? 'shop/shopfb' : ('shop/shoplong' + noNoAds)
        });
        var shop = new Entity({
            name: 'shopContainer',
            position: new Vector2(5, yPositions['shop']),
            originRelative: new Vector2(0.5, 0),
            components: [
                shopSprite,
                buttonAd,
                button0,
                button1,
                button2,
                button3,
                button4,
                button5,
                button6,
                scrollable
            ]
        });
        // exit button
        var exitButton = new ClickButton({
            sfx: 'sfx_button',
            image: Bento.assets.getImage('team/buttonexit'),
            frameCountX: 2,
            frameCountY: 1,
            animations: {
                up: {
                    frames: [0]
                },
                down: {
                    frames: [1]
                },
                inactive: {
                    frames: [0]
                }
            },
            position: new Vector2(-viewport.width / 2 + 12, -viewport.height / 2 + 10),
            updateWhenPaused: Utils.PauseLevels.SHOP,
            onClick: function () {
                Bento.objects.remove(entity);
                Bento.saveState.save('timeOfLastShopVisit', PlayTimer.playTimer);

                if (settings.onClose) {
                    settings.onClose();
                }
            }
        }).attach(
            // back button pressed
            new Utils.BackButton(function (data) {
                data.entity.doCallback();
            })
        );
        // checker for ads
        var adChecker = {
            update: function () {
                // update time every second
                if (!buttonAd.isActive() && entity.timer % 60) {
                    timeLeft.setText(VideoAdTimer.getTimeString());

                    // canplay?
                    if (VideoAdTimer.canPlay()) {
                        buttonAd.setActive(true);
                        buttonAd.remove(timeLeft);
                    }
                }
            }
        };
        // the unpause should happen on the destroy event,
        // because the shop can be closed with a screen end
        var cleaner = {
            destroy: function () {
                Bento.objects.pause(oldPauseLevel);
            }
        };
        var disableNoAdsButton = function () {
            button0.setActive(false);
            button0.getComponent('text', function (text) {
                text.setText('-', {
                    fontColor: '#aaa'
                });
            });
        };

        // main container
        var entity = new Entity({
            z: Utils.Layers.SHOP,
            name: 'shop',
            position: new Vector2(viewport.width / 2, viewport.height / 2),
            updateWhenPaused: Utils.PauseLevels.SHOP,
            float: true,
            components: [
                overlay,
                shop,
                exitButton,
                adChecker,
                cleaner
            ]
        });

        // pause the game
        Bento.objects.pause(Utils.PauseLevels.SHOP);

        // move in shop
        new Tween({
            from: shop.position.x + viewport.width,
            to: shop.position.x,
            'in': 60,
            ease: 'elastic',
            alpha: -10,
            beta: 1,
            updateWhenPaused: Utils.PauseLevels.SHOP,
            'do': function (v, t) {
                shop.position.x = v;
            }
        });

        Bento.objects.attach(entity);

        // can watch ads?
        if (VideoAdTimer.canPlay()) {
            buttonAd.setActive(true);
        } else {
            buttonAd.setActive(false);
            buttonAd.attach(timeLeft);
        }

        // already purchased no ads
        if (Bento.saveState.load('removeAds', false) === true) {
            disableNoAdsButton();
        }

        if (!Utils.isFB() && !Utils.WEB) {
            // update Feb 4 2020: Facebook doesn't like this button, patched out
            if (false && !Bento.saveState.load('likedFacebook', false)) {
                showSocialButton(buttonFacebook);
            } else if (!Bento.saveState.load('likedTwitter', false)) {
                showSocialButton(buttonTwitter);
            }
            if (highSpender) {
                hideNoAdsButton();
            }
            // did game crash when liking our social media pages?
            if (settings.socialReward) {
                receiveSocialReward(settings.socialReward);
            }

        }

        if (Utils.isFB() || Utils.WEB) {
            shop.remove(button0);
            shop.remove(button1);
            shop.remove(button2);
            shop.remove(button3);
            shop.remove(button4);
            shop.remove(button5);
            shop.remove(button6);
            shop.remove(scrollable);
        }

        Utils.Analytics.event('Shop', 'shopVisitAt_' + minutesSinceLastVisit);

        Utils.Firebase.logEvent(Utils.Firebase.Events.VIEW_ITEM_LIST, {
            item_category: 'shop'
        });

        return entity;
    };
});