/**
 * A dialog window with text and buttons
 * Automatically adds itself to game
 */
bento.define('gui/storydialog', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'bento/gui/clickbutton',
    'bento/gui/text'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    Utils,
    Tween,
    ClickButton,
    Text
) {
    'use strict';
    var endsWith = function (string, searchString) {
        var subjectString = string;
        var position = subjectString.length;
        position -= searchString.length;
        var lastIndex = subjectString.indexOf(searchString, position);
        return lastIndex !== -1 && lastIndex === position;
    };

    return function (settings) {
        /*settings = {
            name: name,
            canClose: Boolean, // default false
            type: String, // 'yesno' or 'yes'
            text: String or array, // string to display in window
            attachments: { Number: Entity }, // items to attach each page
            imageWindow: String, // imageName for dialog window image
            imageYes: String, // imageName for yes image
            imageNo: String, // imageName for no image
            onYes: Function, // callback for pressing yes
            onNo: Function, // callback for pressing no
            onClose: Function // callback when closed, after pressing yes, no or close
            onNextPage: Function //
            onComplete: {Number: Function} // callback when text has ended
            pauseLevel: Number
        }*/
        var viewport = Bento.getViewport(),
            isMoving = true,
            z = settings.z || Utils.Layers.DIALOG,
            type = settings.type || '',
            hideCat = settings.hideCat || false,
            attachments = settings.attachments || {},
            lastAttachment = null,
            canClose = settings.canClose || false,
            onComplete = settings.onComplete || {},
            imageYes = settings.imageYes || 'gui/dialog/buttonyes',
            imageNo = settings.imageNo || 'gui/dialog/buttonno',
            oldPauseLevel = Bento.objects.isPaused(),
            pauseLevel = settings.pauseLevel || Utils.PauseLevels.STORY,
            cat = new Entity({
                name: 'cat',
                position: new Vector2(32 * 5, -32),
                originRelative: new Vector2(0.5, 1),
                components: [
                    new Sprite({
                        image: Bento.assets.getImage('gui/dialog/cat')
                    })
                ]
            }),
            sprite = new Sprite({
                imageName: settings.imageWindow || 'gui/dialog/window'
            }),
            page = 0,
            strings = Utils.clone(settings.text) || [],
            fade = new Entity({
                z: z - 2,
                name: 'fade',
                position: new Vector2(viewport.width / 2, viewport.height / 2),
                originRelative: new Vector2(0.5, 0.5),
                float: true,
                components: [
                    new Sprite({
                        imageName: 'fade'
                    })
                ]
            }),
            floatArrow = {
                originalY: 0,
                attached: function (data) {
                    this.originalY = data.entity.position.y;
                },
                update: function (data) {
                    arrow.position.y = this.originalY - 3 * Math.abs(Math.sin(arrow.timer / 20));
                }
            },
            arrow = new Entity({
                name: 'arrow',
                position: new Vector2(60, 30),
                originRelative: new Vector2(0.5, 1),
                updateWhenPaused: pauseLevel,
                components: [
                    new Sprite({
                        image: Bento.assets.getImage('gui/dialog/arrow')
                    }),
                    floatArrow
                ]
            }),
            buttonClose = new ClickButton({
                sfx: 'sfx_button',
                image: Bento.assets.getImage('gui/dialog/buttonexit'),
                frameCountX: 2,
                frameCountY: 1,
                updateWhenPaused: pauseLevel,
                position: new Vector2(100, -48),
                onClick: function () {
                    if (!isMoving) {
                        Utils.Analytics.event('Dialog', 'dialog-' + settings.name + '-close');
                        Utils.Firebase.logEvent('storydialog_close', {
                            dialog_name: settings.name,
                            close_type: 'close'
                        });
                        moveOut();
                    }
                }
            }),
            addFade = {
                start: function () {
                    fade.alpha = (0);
                    Bento.objects.attach(fade);
                    new Tween({
                        from: 0,
                        to: 0.5,
                        'in': 30,
                        ease: 'linear',
                        updateWhenPaused: pauseLevel,
                        'do': function (v, t) {
                            fade.alpha = (v);
                        }
                    });
                },
                destroy: function () {
                    new Tween({
                        from: 0.5,
                        to: 0,
                        'in': 10,
                        ease: 'linear',
                        updateWhenPaused: pauseLevel,
                        'do': function (v, t) {
                            fade.alpha = (v);
                        },
                        onComplete: function () {
                            Bento.objects.remove(fade);
                        }
                    });
                }
            },
            entity = new Entity({
                z: z - 1,
                name: 'dialog',
                position: settings.position || new Vector2(viewport.width / 2, viewport.height * 3 / 4),
                originRelative: new Vector2(0.5, 0.5),
                updateWhenPaused: pauseLevel,
                float: true,
                components: [
                    cat,
                    sprite,
                    addFade
                ]
            }),
            text = new Text({
                z: 0,
                updateWhenPaused: pauseLevel,
                position: new Vector2(-2 - 145 / 2, -30),
                text: '', //settings.text || 'Dialog text',
                font: 'volter',
                fontSize: 9,
                fontSettings: Utils.FontSettings.get('font'),
                fontColor: '#ffffff',
                align: 'left',
                textBaseline: 'top',
                ySpacing: 1,
                maxWidth: 145,
                // maxHeight: 60,
                // fontSettings: Utils.FontSettings.get('font')
            }),
            internalText = new Text({
                z: 0,
                updateWhenPaused: pauseLevel,
                position: new Vector2(-2 - 145 / 2, -30),
                text: '', //settings.text || 'Dialog text',
                font: 'volter',
                fontSize: 9,
                fontSettings: Utils.FontSettings.get('font'),
                fontColor: '#ffffff',
                align: 'left',
                textBaseline: 'top',
                ySpacing: 1,
                maxWidth: 145,
                // maxHeight: 60,
                // fontSettings: Utils.FontSettings.get('font')
            }),
            setupYesButton = function () {
                var yesButton = new ClickButton({
                    sfx: 'sfx_button',
                    image: Bento.assets.getImage(imageYes),
                    frameCountX: 1,
                    frameCountY: 3,
                    animations: {
                        up: {
                            frames: [0]
                        },
                        down: {
                            frames: [1]
                        },
                        inactive: {
                            frames: [2]
                        }
                    },
                    position: new Vector2(-5, 49),
                    updateWhenPaused: pauseLevel,
                    onClick: function () {
                        if (isMoving) {
                            return;
                        }
                        if (strings.length) {
                            // show next text
                            // text.setText(strings.shift());
                            setText();
                            page += 1;
                            if (settings.onNextPage) {
                                settings.onNextPage(page);
                            }
                        } else {
                            moveOut();
                        }
                    }
                }).attach(
                    // back button pressed
                    new Utils.BackButton(function (data) {
                        data.entity.doCallback();
                    })
                );
                entity.attach(yesButton);
            },
            setupOther = function () {
                var yesButton = new Clickable({
                    pointerDown: function () {
                        if (isMoving) {
                            return;
                        }
                        if (!arrow.visible) {
                            return;
                        }
                        if (strings.length) {
                            // show next text
                            page += 1;
                            setText();
                            if (settings.onNextPage) {
                                settings.onNextPage(page);
                            }
                            // remove any attachments
                            if (lastAttachment) {
                                entity.remove(lastAttachment);
                                lastAttachment = null;
                            }
                        } else {
                            moveOut();
                        }
                    }
                });
                entity.attach(yesButton);
            },
            moveIn = function () {
                new Tween({
                    from: viewport.width * 3 / 2,
                    to: viewport.width / 2 + 5,
                    'in': 30,
                    ease: 'exponential',
                    alpha: -6,
                    updateWhenPaused: pauseLevel,
                    'do': function (v, t) {
                        entity.position.x = v;
                    },
                    onComplete: function () {
                        isMoving = false;

                        // move in cat
                        new Tween({
                            from: cat.position.x,
                            to: 32,
                            'in': 30,
                            ease: 'exponential',
                            alpha: -6,
                            updateWhenPaused: pauseLevel,
                            'do': function (v, t) {
                                cat.position.x = v;
                            }
                        });
                    }
                });
            },
            moveOut = function () {
                isMoving = true;
                new Tween({
                    from: entity.position.x,
                    to: viewport.width * 3 / 2 + 10,
                    'in': 30,
                    ease: 'exponential',
                    alpha: -6,
                    updateWhenPaused: pauseLevel,
                    'do': function (v, t) {
                        entity.position.x = v;
                    },
                    onComplete: function () {
                        isMoving = false;
                        Bento.objects.remove(entity);
                        Bento.objects.pause(oldPauseLevel);
                        if (settings.onClose) {
                            settings.onClose();
                        }
                    }
                });
            },
            formattedText = '',
            commands = {},
            setText = function () {
                var internalStrings;

                // reset
                formattedText = '';
                shown = 0;
                arrow.visible = false;

                // get new text
                internalText.setText(strings.shift());
                internalStrings = internalText.getStrings();

                // set up single strings
                Utils.forEach(internalStrings, function (str) {
                    var text = str.string;
                    formattedText += text + '\n';
                });

                // Bento.audio.stopSound('sfx_robot_cat');
                // Bento.audio.playSound('sfx_robot_cat');
            },
            shown = 0,
            letterByLetterComponent = {
                update: function (data) {
                    var str;
                    if (isMoving) {
                        return;
                    }
                    // show letter one by one
                    if (data.entity.timer % 1 === 0 && shown < formattedText.length) {
                        shown += 1;
                        str = formattedText.substring(0, shown);

                        if (data.entity.timer % 4 === 0)
                            Bento.audio.playSound('sfx_cat' + (1 + Utils.getRandom(3)));

                        // cocoon bug: if the text ends with a space, the text gets jumpy
                        // quick fix: ignore the last character
                        if (endsWith(str, ' ')) {
                            // trim() removes whitespace at the start and end of strings
                            // https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/String/Trim
                            str = str.trim();
                        }
                        text.setText(str);

                        if (shown >= formattedText.length && arrow.visible === false) {
                            arrow.visible = true;

                            // event?
                            if (onComplete[page]) {
                                onComplete[page]();
                            }
                            // attach an item?
                            if (attachments[page]) {
                                lastAttachment = attachments[page];
                                entity.attach(lastAttachment);

                                // tween?
                                new Tween({
                                    from: 0,
                                    to: 1,
                                    'in': 30,
                                    ease: 'elastic',
                                    alpha: -5,
                                    beta: 2,
                                    updateWhenPaused: pauseLevel,
                                    'do': function (v, t) {
                                        if (lastAttachment) {
                                            lastAttachment.scale.x = v;
                                            lastAttachment.scale.y = v;
                                        }
                                    }
                                });
                            }
                        }
                    }

                    // for debugging
                    if (Bento.input.isKeyDown('esc')) {
                        moveOut();
                    }
                }
            };

        if (hideCat) {
            cat.visible = false;
        }

        // automatic pause
        Bento.objects.pause(pauseLevel);

        if (!Utils.isArray(strings)) {
            strings = [strings];
        }

        // text.setText(strings.shift());
        setText();

        entity.attach(text);
        entity.attach(letterByLetterComponent);
        entity.attach(arrow);
        arrow.visible = false;
        Bento.objects.attach(entity);

        // setup buttons
        setupOther();

        if (canClose) {
            entity.attach(buttonClose);
        }

        // move in
        moveIn();

        return entity;
    };
});