/**
 * Simple container for a card
 */
bento.define('gui/team/slot', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'components/droptarget'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    Utils,
    Tween,
    DropTarget
) {
    'use strict';
    return function (settings) {
        var item = null; // slot item
        var slot = new Entity({
            name: 'slot',
            originRelative: new Vector2(0.5, 0.5),
            updateWhenPaused: Utils.PauseLevels.TEAMEDIT,
            components: [
                new Sprite({
                    imageName: 'gui/emptyslot'
                }),
                new DropTarget({
                    onDrop: settings.onDrop
                })
            ]
        });
        slot.extend({
            getItem: function () {
                return item;
            },
            hasItem: function () {
                return item !== null;
            },
            setItem: function (incoming) {
                if (incoming.parent) {
                    console.log('Warning: already attached');
                    return;
                }
                slot.attach(incoming);
                item = incoming;
            },
            removeItem: function () {
                var oldItem = item;
                slot.remove(item);
                item = null;
                return oldItem;
            }
        });

        return slot;
    };
});