/**
 * Team edit overlay
 */
bento.define('gui/team/teamedit', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'bento/gui/clickbutton',
    'components/draggable',
    'components/droptarget',
    'bento/gui/text',
    'gui/centeredlist',
    'gui/scrollinglist',
    'gui/team/page',
    'gui/team/slot',
    'entities/simplecard'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    Utils,
    Tween,
    ClickButton,
    Draggable,
    DropTarget,
    Text,
    CenteredList,
    ScrollingList,
    Page,
    Slot,
    Card
) {
    'use strict';
    return function (settings) {
        /*settings = {
            onClose: Function
        }*/
        var viewport = Bento.getViewport();
        var sprite = new Sprite({
            imageName: 'team/background'
        });
        var oldPauseLevel = Bento.isPaused();
        var corner = new Vector2(-viewport.width / 2, -viewport.height / 2);
        var scrollArea = new Rectangle(viewport.x, viewport.y + viewport.height / 2 - 48, viewport.width, viewport.height / 2 + 48);
        var returnArea = new Rectangle(-viewport.width / 2, -48, viewport.width, viewport.height / 2 + 48);
        var entity = new Entity({
            z: Utils.Layers.TEAMEDIT,
            name: 'teamEditOverlay',
            position: new Vector2(viewport.width / 2, viewport.height / 2),
            originRelative: new Vector2(0.5, 0.5),
            updateWhenPaused: Utils.PauseLevels.TEAMEDIT,
            float: true,
            components: [
                sprite
            ]
        });
        var exitButton = new ClickButton({
            sfx: 'sfx_button',
            image: Bento.assets.getImage('team/buttonexit'),
            frameCountX: 2,
            frameCountY: 1,
            animations: {
                up: {
                    frames: [0]
                },
                down: {
                    frames: [1]
                }
            },
            position: corner.add(new Vector2(12, 10)),
            updateWhenPaused: Utils.PauseLevels.TEAMEDIT,
            onClick: function () {
                var crittersCount = 0;
                slots.iterate(function (slot) {
                    if (slot.getItem()) {
                        crittersCount += 1;
                    }
                });
                // can't leave if there are no critters in team
                if (crittersCount === 0) {
                    Bento.audio.stopSound('sfx_button');
                    Bento.audio.playSound('sfx_error');
                    return;
                }
                Bento.objects.pause(oldPauseLevel);
                Bento.objects.remove(entity);

                // save critter lists!
                save();

                if (settings.onClose) {
                    settings.onClose();
                }
                
                EventSystem.fire('closedTeamEdit');
            }
        }).attach(
            // back button pressed     
            new Utils.BackButton(function (data) {
                data.entity.doCallback();
            })
        );
        var border = new Entity({
            name: 'border',
            position: new Vector2(0, -80),
            originRelative: new Vector2(0.5, 0.5),
            components: [
                new Sprite({
                    imageName: 'battle/gridsmall'
                })
            ]
        });
        var slots = new CenteredList({
            position: new Vector2(-1, -80),
            spacing: 6
        });
        var critterName = new Text({
            position: new Vector2(0, -34),
            text: '',
            font: 'volter',
            fontSize: 9,
            fontSettings: Utils.FontSettings.get('font'),
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'bottom',
            maxWidth: 96,
            linebreaks: false,
            updateWhenPaused: Utils.PauseLevels.TEAMEDIT
        });
        var infoAvatar = new Entity({
            name: 'infoAvatar',
            position: new Vector2(-58, -1),
            originRelative: new Vector2(0.5, 0.5),
            components: [new Sprite({
                imageName: 'gui/dragtutorial'
            })]
        });
        var infoName = new Text({
            position: new Vector2(18, -16),
            text: '',
            font: 'volter',
            fontSize: 9,
            fontSettings: Utils.FontSettings.get('font'),
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'bottom',
            maxWidth: 96,
            linebreaks: false,
            updateWhenPaused: Utils.PauseLevels.TEAMEDIT
        });
        var infoDescription = new Text({
            position: new Vector2(20, -9),
            text: 'Drag cards to the upper slots',
            font: 'volter',
            fontSize: 9,
            fontSettings: Utils.FontSettings.get('font'),
            fontColor: '#ffffff',
            align: 'center',
            maxWidth: 96,
            maxHeight: 32,
            textBaseline: 'top',
            updateWhenPaused: Utils.PauseLevels.TEAMEDIT
        });
        var infoSeperator = new Entity({
            name: 'seperator',
            position: new Vector2(18, -12),
            originRelative: new Vector2(0.5, 0.5),
            updateWhenPaused: Utils.PauseLevels.TEAMEDIT,
            components: [new Sprite({
                imageName: 'map/stripe'
            })]
        });
        var info = new Entity({
            name: 'infoPanelCritter',
            position: new Vector2(5, -31),
            originRelative: new Vector2(0.5, 0.5),
            components: [
                new Sprite({
                    imageName: 'team/infopanel'
                }),
                infoName,
                infoSeperator,
                infoDescription,
                infoAvatar
            ]
        });
        var buttons = new CenteredList({
            position: new Vector2(0, 10),
            spacing: 8
        });
        var pagesDots = new CenteredList({
            position: new Vector2(0, viewport.height / 2 - 6),
            spacing: 2
        });
        var buttonSortStar = new ClickButton({
            sfx: 'sfx_button',
            image: Bento.assets.getImage('team/buttonstarup'),
            frameCountX: 2,
            frameCountY: 1,
            animations: {
                up: {
                    frames: [0]
                },
                down: {
                    frames: [1]
                }
            },
            updateWhenPaused: Utils.PauseLevels.TEAMEDIT,
            onClick: function () {

            }
        });
        var buttonSortHealth = new ClickButton({
            sfx: 'sfx_button',
            image: Bento.assets.getImage('team/buttonhealthup'),
            frameCountX: 2,
            frameCountY: 1,
            animations: {
                up: {
                    frames: [0]
                },
                down: {
                    frames: [1]
                }
            },
            updateWhenPaused: Utils.PauseLevels.TEAMEDIT,
            onClick: function () {

            }
        });
        var buttonSortAttack = new ClickButton({
            sfx: 'sfx_button',
            image: Bento.assets.getImage('team/buttonattackup'),
            frameCountX: 1,
            frameCountY: 1,
            animations: {
                up: {
                    frames: [0]
                },
                down: {
                    frames: [0]
                },
                inactive: {
                    frames: [0]
                }
            },
            updateWhenPaused: Utils.PauseLevels.TEAMEDIT,
            onClick: function () {

            }
        });
        var buttonLeft = new ClickButton({
            sfx: 'sfx_button',
            image: Bento.assets.getImage('team/buttonarrowleft'),
            frameCountX: 1,
            frameCountY: 1,
            animations: {
                up: {
                    frames: [0]
                },
                down: {
                    frames: [0]
                }
            },
            updateWhenPaused: Utils.PauseLevels.TEAMEDIT,
            position: new Vector2(-viewport.width / 2 + 6, 48),
            onClick: function () {
                if (Draggable.isDragging()) {
                    return;
                }
                scrollList.snapTo(scrollList.getSelection() - 1);
                critterName.setText('');
            }
        });
        var buttonRight = new ClickButton({
            sfx: 'sfx_button',
            image: Bento.assets.getImage('team/buttonarrowright'),
            frameCountX: 1,
            frameCountY: 1,
            animations: {
                up: {
                    frames: [0]
                },
                down: {
                    frames: [0]
                },
                inactive: {
                    frames: [0]
                }
            },
            position: new Vector2(viewport.width / 2 - 6, 48),
            updateWhenPaused: Utils.PauseLevels.TEAMEDIT,
            onClick: function () {
                if (Draggable.isDragging()) {
                    return;
                }
                scrollList.snapTo(scrollList.getSelection() + 1);
                critterName.setText('');
            }
        });
        var scrollList = new ScrollingList({
            updateWhenPaused: Utils.PauseLevels.TEAMEDIT,
            position: new Vector2(-2, 14),
            direction: 'x',
            spacing: viewport.width,
            snap: true,
            snapMinSlideSpeed: 0.5,
            snapSpeed: 0.2,
            damping: 0.95,
            area: scrollArea,
            onSnap: function (item, index) {
                if (item) {
                    item.activate();
                }
                pagesDots.iterate(function (item, i) {
                    if (i === index) {
                        item.alpha = (1);
                    } else {
                        item.alpha = (0.5);
                    }
                });
            },
            onLoseFocus: function (item) {
                if (item) {
                    item.deactivate();
                }
            },
            onScrollStart: function () {
                // shouldnt scroll when dragging
                if (Draggable.isDragging()) {
                    scrollList.cancel();
                }
            }
        });
        var makeTeam = function (team) {
            var i;
            var card;
            var makeSlot = function (index) {
                var slot = new Slot({
                    onDrop: function (data) {
                        var incomingItem = data.entity;
                        var oldItem;
                        var page;
                        var otherSlot;
                        var item = slot.getItem();
                        // incomingItem was dropped in here
                        // does it originate from the scrollist?
                        if (incomingItem.parent.name === 'page') {
                            page = incomingItem.parent;

                            if (item) {
                                // already has item? remove that and put it in the page, swapping it
                                page.swapItems(incomingItem, slot.removeItem(), slot.getWorldPosition());
                            } else {
                                // only put item out of page
                                page.removeItem(incomingItem);
                            }
                            // add to slot
                            incomingItem.position.x = 0;
                            incomingItem.position.y = 0;
                            slot.setItem(incomingItem);
                        } else if (incomingItem.parent.name === 'slot') {
                            otherSlot = incomingItem.parent;
                            // incomingItem was dragged from other slot
                            otherSlot.removeItem();
                            // swap items
                            if (item) {
                                oldItem = item;
                                slot.removeItem();
                                otherSlot.setItem(oldItem);
                            }

                            // add to slot
                            incomingItem.position.x = 0;
                            incomingItem.position.y = 0;
                            slot.setItem(incomingItem);
                        }

                        // add 'droppedSlot' property
                        data.droppedSlot = true;
                    }
                });
                return slot;
            };

            slots.addItem(makeSlot(0));
            slots.addItem(makeSlot(1));
            slots.addItem(makeSlot(2));

            // attach team critters
            for (i = 0; i < slots.length(); ++i) {
                if (team[i]) {
                    card = makeDraggableCard(team[i]);
                    slots.get(i).setItem(card);
                }
            }
            return slots;
        };
        var makeDraggableCard = function (critterProp) {
            var card;
            var addDraggable = function (card) {
                var draggable = new Draggable({
                    dragInit: function () {
                        scrollList.cancel();
                    },
                    dragStart: function (data) {
                        var parent = card.parent;
                        data.entity.scale.x = 0.9;
                        data.entity.scale.y = 0.9;

                        // show info
                        setInfoPanel(card);

                        // check parent
                        if (parent.name === 'page') {
                            // parent is page
                            // move card to top
                            parent.moveComponentTo(card, parent.components.length);
                            // move scrollist to top
                            entity.moveComponentTo(scrollList, entity.components.length);

                        } else if (parent.name === 'slot') {
                            // parent is slot
                            // move slot to top
                            slots.moveComponentTo(parent, slots.components.length);
                            // move slots to top
                            entity.moveComponentTo(slots, entity.components.length);
                        }
                        setButtonsToTop();
                    },
                    dragEnd: function (data) {
                        var distance = data.startPosition.subtract(data.endPosition).magnitude();
                        data.entity.scale.x = 1;
                        data.entity.scale.y = 1;
                        // hardly moved: wiggle
                        if (distance < 1) {
                            new Tween({
                                from: 0,
                                to: Math.PI / 64,
                                'in': 30,
                                ease: 'sin',
                                alpha: 2,
                                updateWhenPaused: Utils.PauseLevels.TEAMEDIT,
                                'do': function (v, t) {
                                    data.entity.rotation = v;
                                }
                            });
                        }
                        if (!data.droppedSlot && !data.droppedReturn) {
                            // tween back
                            data.entity.position = data.endPosition.clone();
                            draggable.deactivate();
                            new Tween({
                                from: data.endPosition,
                                to: data.startPosition,
                                'in': 20,
                                ease: 'elastic',
                                alpha: -8,
                                beta: 1,
                                updateWhenPaused: Utils.PauseLevels.TEAMEDIT,
                                'do': function (v, t) {
                                    data.entity.position = v;
                                },
                                onComplete: function () {
                                    draggable.activate();
                                }
                            });
                        }
                    }
                });
                card.boundingBox = new Rectangle(-22, -23, 42, 35);
                // card.boundingBox.grow(-8);
                card.updateWhenPaused = Utils.PauseLevels.TEAMEDIT;
                // add draggable component
                card.attach(draggable);
                card.attach({
                    name: 'checkDrag',
                    update: function () {
                        if (scrollList.isScrolling()) {
                            draggable.canDrag = false;
                        } else {
                            draggable.canDrag = true;
                        }
                    }
                });
            };
            //
            card = new Card(critterProp);
            addDraggable(card);
            return card;
        };
        var makeCritterList = function (critters) {
            var i = 0;
            var card;
            var currentPage;
            var addPage = function () {
                var page = new Page({
                    maxItems: 9,
                    spacing: new Vector2(2, 2)
                });
                var addDot = function () {
                    var dotSprite = new Sprite({
                        imageName: 'gui/dot'
                    });
                    var dot = new Entity({
                        originRelative: new Vector2(0.5, 0.5),
                        components: [dotSprite]
                    });
                    if (i !== 0) {
                        dot.alpha = (0.5);
                    }
                    pagesDots.addItem(dot);
                };
                // add to list
                scrollList.addItem(page);
                addDot();

                // set current
                if (currentPage && pages.length > 1) {
                    currentPage.deactivate();
                }

                currentPage = page;

                return page;
            };
            var pages = [];
            var returnCardArea = new Entity({
                name: 'returnCardArea',
                boundingBox: returnArea,
                updateWhenPaused: Utils.PauseLevels.TEAMEDIT,
                components: [

                    // new Utils.Debug({}),
                    new DropTarget({
                        onDrop: function (data) {
                            var parent = data.entity.parent;
                            var incomingItem = data.entity;
                            var page;
                            var worldPosition;
                            var lastItem;
                            var moveItemToNextPage = function (pageIndex) {
                                var current = pages[pageIndex];
                                var next = pages[pageIndex + 1];
                                var lastItem = current.getLastItem();

                                // does next page exist at all?
                                if (!next) {
                                    next = addPage();
                                    pages.push(next);
                                }
                                // should have room in the next
                                if (next.isFull()) {
                                    moveItemToNextPage(pageIndex + 1);
                                }
                                // should be ok now..?
                                current.removeItem(lastItem);
                                next.addItem(lastItem, true);
                            };

                            // drop item
                            if (parent.name === 'slot') {
                                // put monster back in current page
                                page = pages[scrollList.getSelection()];

                                if (!page) {
                                    page = addPage();
                                    pages.push(page);
                                }

                                if (!page.isFull()) {
                                    parent.removeItem(incomingItem);
                                    page.addItem(incomingItem);
                                } else {
                                    console.log('page is full');
                                    // move last item into next page
                                    moveItemToNextPage(scrollList.getSelection());

                                    // should be ok now
                                    parent.removeItem(incomingItem);
                                    page.addItem(incomingItem);
                                }
                                // add 'droppedReturn' property
                                data.droppedReturn = true;
                            }

                        }
                    })
                ]
            });

            entity.attach(returnCardArea);

            // go through the list and add items into pages
            for (i = 0; i < critters.length; ++i) {
                if (!currentPage || currentPage.isFull()) {
                    pages.push(addPage());
                }
                // make card
                card = makeDraggableCard(critters[i]);

                currentPage.addItem(card);
            }
            if (pages.length <= 1) {
                pagesDots.visible = false;
            }
            // deactivate last page
            // if (pages.length) {
            //     pages[0].activate();
            // }
        };
        var setButtonsToTop = function () {
            entity.moveComponentTo(exitButton, entity.components.length);
            entity.moveComponentTo(buttonLeft, entity.components.length);
            entity.moveComponentTo(buttonRight, entity.components.length);
        };
        var save = function () {
            var i;
            var item;
            var slot;
            var playerCritters = Utils.getCritters();
            var teamCritters = Utils.getTeam();
            for (i = 0; i < slots.length(); ++i) {
                slot = slots.get(i);
                item = slot.getItem();
                if (item) {
                    teamCritters[i] = item.getProperties();
                } else {
                    teamCritters[i] = null;
                }
            }
            // reinsert all critters
            while (playerCritters.length) {
                playerCritters.pop();
            }
            scrollList.iterate(function (page) {
                if (!page) {
                    return;
                }
                page.iterate(function (item) {
                    playerCritters.push(item.getProperties());
                });
            });
            Utils.saveTeam();
            Utils.saveCritters();
        };
        var setInfoPanel = function (card) {
            var ability;
            if (infoAvatar) {
                // remove old
                info.remove(infoAvatar);
                infoAvatar = null;
            }
            infoAvatar = new Entity({
                name: 'infoAvatar',
                position: new Vector2(-54, 2),
                originRelative: new Vector2(0.5, 0.5),
                components: [new Sprite({
                    imageName: 'selection/creatures/' + card.getId()
                })]
            });
            info.attach(infoAvatar);

            infoName.setText(card.getProperty('name'));
            critterName.setText(Utils.Localization.getName(card.getProperty('id')));
            // critterName.position.x = critterName.getLocalPosition(card.getWorldPosition()).x + 16;
            // TODO get language
            infoDescription.setText(Bento.assets.getJson('creatures/descriptions/' + 'en')[card.getId()] || '?');
        };
        var getHand = function () {
            return new Entity({
                name: 'hand',
                position: new Vector2(0, 8),
                originRelative: new Vector2(0.5, 0),
                updateWhenPaused: Utils.PauseLevels.CAUGHT,
                components: [new Sprite({
                    imageName: 'map/hand'
                })]
            }).attach({
                update: function (data) {
                    var hand = data.entity;
                    hand.position.y = 4 + 4 * Math.abs(Math.sin(hand.timer / 20));
                }
            });
        };
        var exitChecker = {
            update: function () {
                var total = slots.get(0).hasItem() + slots.get(1).hasItem() + slots.get(2).hasItem();
                if (total > 0) {
                    exitButton.visible = true;
                    exitButton.setActive(true);
                } else {
                    exitButton.visible = false;
                    exitButton.setActive(false);
                }
            }
        };
        Bento.objects.pause(Utils.PauseLevels.TEAMEDIT);

        // scenegraph
        // entity.attach(info);
        entity.attach(border);
        entity.attach(critterName);
        entity.attach(slots);
        entity.attach(exitButton);

        // tutorial
        if (Bento.saveState.load('sawTutorial', false) === false) {
            exitButton.attach(getHand());
            entity.attach({
                update: function () {
                    var total = slots.get(0).hasItem() + slots.get(1).hasItem() + slots.get(2).hasItem();
                    if (total >= 2) {
                        exitButton.visible = true;
                        exitButton.setActive(true);
                    } else {
                        exitButton.visible = false;
                        exitButton.setActive(false);
                    }
                }
            });
        } else {
            // 
            entity.attach(exitChecker);
        }

        // buttons
        // TEMP
        // buttons.addItem(buttonSortStar);
        // buttons.addItem(buttonSortHealth);
        // buttons.addItem(buttonSortAttack);
        entity.attach(buttons);

        // scroll list
        entity.attach(scrollList);
        entity.attach(pagesDots);

        makeTeam(Utils.getTeam());
        makeCritterList(Utils.getCritters());

        if (scrollList.getItems().length > 1) {
            entity.attach(buttonLeft);
            entity.attach(buttonRight);
        }
        Utils.Analytics.trackView('teamEdit');

        return entity;
    };
});