/**
 * Caught twitter critter overlay
 */
bento.define('gui/twitter/caught', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'bento/packedimage',
    'modules/share/catch'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    ClickButton,
    Counter,
    Text,
    EventSystem,
    Utils,
    Tween,
    PackedImage,
    ShareCatch
) {
    'use strict';
    return function (settings, onClose) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var background = new Entity({
            name: 'background',
            originRelative: new Vector2(0.5, 0.5),
            components: [new Sprite({
                imageName: 'team/background'
            })]
        });
        var caught = settings.caught || settings.opponent;
        var critter = caught.card.getProperties();
        var text = new Text({
            position: new Vector2(0, -viewport.height / 4),
            text: Utils.Localization.getText(settings.caught ? 'caught' : 'twitterDefeat').replace('{NAME}', critter.name),
            font: 'volter',
            fontSize: 9,
            fontSettings: Utils.FontSettings.get('font'),
            fontColor: '#ffffff',
            align: 'center',
            maxWidth: viewport.width,
            linebreaks: true
        });
        var buttonPlay = new ClickButton({
            name: 'buttonPlay',
            sfx: 'sfx_button',
            image: Bento.assets.getImage('map/buttonattack'),
            frameCountX: 1,
            frameCountY: 2,
            animations: {
                up: {
                    frames: [0]
                },
                down: {
                    frames: [1]
                },
                inactive: {
                    frames: [0]
                }
            },
            position: new Vector2(32, viewport.height / 4 + 8),
            updateWhenPaused: Utils.PauseLevels.TWITTER,
            onClick: function () {
                Bento.objects.resume();
                Bento.objects.remove(entity);

                if (onClose) {
                    onClose();
                }
            }
        });
        var hasClicked = false;
        var buttonShare = new ClickButton({
            name: 'buttonShare',
            sfx: 'sfx_button',
            image: Bento.assets.getImage('gui/buttonshare'),
            frameCountX: 2,
            frameCountY: 1,
            animations: {
                up: {
                    frames: [0]
                },
                down: {
                    frames: [1]
                },
                inactive: {
                    frames: [0]
                }
            },
            position: new Vector2(-32, viewport.height / 4 + 8),
            updateWhenPaused: Utils.PauseLevels.TWITTER,
            onClick: function () {
                // on android: no point in choosing, everyplay is disabled
                // QUICKFIX: everyplay popup
                if (true || Utils.isNativeAndroid()) {
                    buttonShareImage.doCallback();
                    return;
                }

                if (!hasClicked) {
                    hasClicked = true;
                    // show camere and replay
                    buttonShare.attach(buttonShareImage);
                    buttonShare.attach(buttonShareVideo);

                    new Tween({
                        from: 0,
                        to: 1,
                        'in': 20,
                        ease: 'elastic',
                        alpha: -8,
                        beta: 2,
                        updateWhenPaused: Utils.PauseLevels.TWITTER,
                        'do': function (v, t) {
                            buttonShareImage.scale.x = v;
                            buttonShareImage.scale.y = v;
                        }
                    });

                    buttonShareVideo.scale.x = 0;
                    buttonShareVideo.scale.y = 0;
                    Utils.timeout(5, function () {
                        new Tween({
                            from: 0,
                            to: 1,
                            'in': 20,
                            ease: 'elastic',
                            alpha: -8,
                            beta: 2,
                            updateWhenPaused: Utils.PauseLevels.TWITTER,
                            'do': function (v, t) {
                                buttonShareVideo.scale.x = v;
                                buttonShareVideo.scale.y = v;
                            }
                        });
                    }, Utils.PauseLevels.TWITTER);
                } else {
                    hasClicked = false;
                    buttonShare.remove(buttonShareImage);
                    buttonShare.remove(buttonShareVideo);
                }
            }
        });
        var buttonShareImage = new ClickButton({
            name: 'buttonShareImage',
            sfx: 'sfx_button',
            image: Bento.assets.getImage('gui/popupcamera'),
            frameCountX: 2,
            frameCountY: 1,
            position: new Vector2(-16, -28),
            updateWhenPaused: Utils.PauseLevels.TWITTER,
            onClick: function () {
                var shareScore = Utils.Localization.getText(settings.caught ? 'twitterShare' : 'twitterShareDefeat'),
                    link = Utils.getStoreLink();

                if (window.Cocoon && window.Cocoon.Share) {
                    shareScore = shareScore.replace('[LINK]', link).replace('{NAME}', critter.handle || critter.name);

                    Cocoon.Share.share({
                        message: shareScore,
                        image: ShareCatch({
                            critterImage: critterImage
                        }).getBase64()
                    }, function () {
                        // success
                        // Standard Firebase event for finding the most viral content in your app
                        Utils.Firebase.logEvent(Utils.Firebase.Events.SHARE, {
                            content_type: 'twitter share image',
                            item_id: critter.handle || critter.name
                        });
                    });
                }
            }
        });
        var buttonShareVideo = new ClickButton({
            name: 'buttonShareVideo',
            sfx: 'sfx_button',
            image: Bento.assets.getImage('gui/popupreplay'),
            frameCountX: 2,
            frameCountY: 1,
            position: new Vector2(16, -28),
            updateWhenPaused: Utils.PauseLevels.TWITTER,
            onClick: function () {
                var shareScore = Utils.Localization.getText(settings.caught ? 'twitterShare' : 'twitterShareDefeat'),
                    link = Utils.getStoreLink();

                shareScore = shareScore.replace('[LINK]', link).replace('{NAME}', critter.handle || critter.name);

                Utils.EveryPlay.setData({
                    text: shareScore
                });
                Bento.audio.stopAllMusic();
                Utils.EveryPlay.replay();
            }
        });
        var entity = new Entity({
            z: Utils.Layers.TEAMEDIT + 100,
            name: 'twitterCaught',
            position: new Vector2(viewport.width / 2, viewport.height / 2),
            updateWhenPaused: Utils.PauseLevels.TWITTER,
            components: [
                background,
                text,
                buttonPlay,
                buttonShare,
                caught
            ]
        });
        var critterImage = new Image();

        critterImage.src = critter.image;
        critterImage.onload = function () {
            critterImage = new PackedImage(this);
        };

        caught.setOriginRelative(new Vector2(0.5, 0.5));
        caught.position.x = 0;
        caught.position.y = 0;
        caught.rotation = 0;
        caught.getComponent('walk', function (component) {
            caught.remove(component);
        });
        caught.getComponent('enemyBehavior', function (component) {
            caught.remove(component);
        });
        caught.getComponent('exclamation', function (component) {
            caught.remove(component);
        });


        if (settings.caught) {
            Bento.audio.playSound('sfx_new_critter');
        }
        new Tween({
            from: 0,
            to: 1,
            'in': 60,
            ease: 'elastic',
            alpha: -6,
            beta: 3,
            updateWhenPaused: Utils.PauseLevels.TWITTER,
            'do': function (v, t) {
                caught.scale.x = v;
                caught.scale.y = v;
            },
            onComplete: function () {
                Bento.audio.playMusic('bgm_title');
                Bento.audio.setVolume(1, 'bgm_title');
            }
        });

        Bento.objects.pause(Utils.PauseLevels.TWITTER);

        Bento.objects.attach(entity);

        Bento.audio.stopAllMusic();

        return entity;
    };
});