/**
 * List of caught twitter critters
 */
bento.define('gui/twitter/twitter', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'bento/screen',
    'bento/gui/clickbutton',
    'bento/gui/text',
    'components/debug',
    'components/draggable',
    'components/droptarget',
    'gui/centeredlist',
    'gui/coincounter',
    'gui/storydialog',
    'gui/scrollinglist',
    'gui/team/page',
    'gui/team/slot',
    'entities/card',
    'bento/packedimage',
    'modules/share/catch'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    Utils,
    Tween,
    Screen,
    ClickButton,
    Text,
    Debug,
    Draggable,
    DropTarget,
    CenteredList,
    CoinCounter,
    StoryDialog,
    ScrollingList,
    Page,
    Slot,
    Card,
    PackedImage,
    ShareCatch
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var mode = Utils.Localization.isChinese() ? 'weibo' : 'twitter';
        var oldPauseLevel = Bento.objects.isPaused();
        var container = new Entity({
            z: Utils.Layers.TEAMEDIT,
            name: 'dexContainer',
            position: new Vector2(0, 0),
            float: true,
            updateWhenPaused: Utils.PauseLevels.TEAMEDIT,
            components: [
                new Sprite({})
            ]
        });
        var background = new Entity({
            z: 0,
            name: 'background',
            position: new Vector2(viewport.width / 2, viewport.height / 2),
            originRelative: new Vector2(0.5, 0.5),
            components: [new Sprite({
                imageName: 'team/background'
            })]
        });
        var buttonExit = new ClickButton({
            z: 3,
            sfx: 'sfx_button',
            image: Bento.assets.getImage('combiner/buttonexit'),
            frameCountX: 2,
            frameCountY: 1,
            animations: {
                up: {
                    frames: [0]
                },
                down: {
                    frames: [1]
                }
            },
            position: new Vector2(12, 10),
            updateWhenPaused: Utils.PauseLevels.TEAMEDIT,
            onClick: function () {
                Bento.objects.remove(container);

                if (settings.onClose) {
                    settings.onClose();
                }
            }
        }).attach(
            // back button pressed
            new Utils.BackButton(function (data) {
                data.entity.doCallback();
            })
        );
        var scrollList = new ScrollingList({
            z: 2,
            updateWhenPaused: Utils.PauseLevels.TEAMEDIT,
            position: new Vector2(viewport.width / 2, viewport.height / 2),
            direction: 'y',
            spacing: 4,
            snap: true,
            snapMinSlideSpeed: 0.5,
            snapSpeed: 0.2,
            damping: 0.95,
            onSnap: function (item, index) {},
            onLoseFocus: function (item) {}
        });
        var makeCritters = function () {
            var critters = Utils.getTwitterCritters();

            Utils.forEach(critters, function (critter) {
                var properties = Utils.clone(critter);
                var card = new Card(properties, {
                    image: critter.image
                });
                var text = new Text({
                    position: new Vector2(24, -16),
                    text: properties.name,
                    font: 'Volter',
                    fontSize: 9,
                    fontColor: '#ffffff',
                    align: 'left',
                    textBaseline: 'middle'
                });
                var buttonShare = new ClickButton({
                    name: 'buttonShare',
                    sfx: 'sfx_button',
                    image: Bento.assets.getImage('gui/buttonshare'),
                    frameCountX: 2,
                    frameCountY: 1,
                    animations: {
                        up: {
                            frames: [0]
                        },
                        down: {
                            frames: [1]
                        },
                        inactive: {
                            frames: [0]
                        }
                    },
                    position: new Vector2(64, 14),
                    onClick: function () {
                        var shareScore = Utils.Localization.getText('twitterShare'),
                            link = Utils.getStoreLink();
                        var critterImage = new Image();

                        if (window.Cocoon && window.Cocoon.Share) {
                            critterImage.src = critter.image;
                            critterImage.onload = function () {
                                critterImage = new PackedImage(this);
                                shareScore = shareScore.replace('[LINK]', link).replace('{NAME}', critter.handle || critter.name);

                                Cocoon.Share.share({
                                    message: shareScore,
                                    image: ShareCatch({
                                        critterImage: critterImage
                                    }).getBase64()
                                }, function () {
                                    // success
                                    // Standard Firebase event for finding the most viral content in your app
                                    Utils.Firebase.logEvent(Utils.Firebase.Events.SHARE, {
                                        content_type: 'twitter share',
                                        item_id: critter.handle || critter.name
                                    });
                                });
                            };
                        }
                    }
                });

                card.position.x -= 32;
                card.attach(text);
                card.attach(buttonShare);

                scrollList.addItem(card);
            });

            if (critters.length === 0) {
                Bento.objects.attach(explanation);
            }
        };
        var buttonTwitter = new ClickButton({
            name: 'buttonTwitter',
            sfx: 'sfx_button',
            image: Bento.assets.getImage(mode === 'twitter' ? 'gui/buttonsearch' : 'gui/buttonsearchweibo'),
            frameCountX: 2,
            frameCountY: 1,
            position: new Vector2(0, 0),
            updateWhenPaused: 0,
            onClick: function () {
                if (settings.onClick) {
                    settings.onClick();
                }
            }
        });
        var explanation = new Text({
            z: Utils.Layers.TEAMEDIT,
            position: new Vector2(viewport.width / 2, viewport.height / 2 + 64),
            text: Utils.Localization.getText('twitterExplanation'),
            font: 'volter',
            fontSize: 9,
            fontSettings: Utils.FontSettings.get('font'),
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'bottom',
            maxWidth: viewport.width
        });

        makeCritters();
        scrollList.addItem(buttonTwitter);

        container.attach(buttonExit);
        container.attach(scrollList);

        scrollList.snapTo(scrollList.getTotalSize());

        return container;
    };
});