/**
 * Module description
 */
bento.define('gui/worldmap/menu', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'bento/packedimage',
    'entities/card',
    'gui/dex/dex',
    'gui/coincounter',
    'gui/dialog',
    'gui/team/teamedit',
    'gui/combiner/combiner',
    'gui/centeredlist',
    'gui/togglemusic',
    'gui/togglesound',
    'modules/photo',
    'modules/stackblur'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    ClickButton,
    Counter,
    Text,
    EventSystem,
    Utils,
    Tween,
    PackedImage,
    Card,
    Dex,
    CoinCounter,
    Dialog,
    TeamEdit,
    Combiner,
    CenteredList,
    ToggleMusic,
    ToggleSound,
    Photo,
    StackBlur
) {
    'use strict';
    // suspend
    if (navigator.isCocoonJS) {
        document.addEventListener('pause', function () {
            // find pause button
            var button = Bento.objects.get('menuButton');
            if (Bento.objects.isPaused() < Utils.PauseLevels.MENU && button) {
                button.doCallback();
            }
        });
    }

    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var blurredSprite;
        var buttonStartY = -40;
        var buttonDistance = 40;
        var blurredBackground;
        var blur = function () {
            var canvas;
            var photo = new Photo({
                scale: 1
            });
            Bento.objects.get('mode7', function (mode7) {
                var renderer = mode7.getRenderer();
                renderer.setSize(photo.getCanvas().width, photo.getCanvas().height, false);
            });
            photo.snap();
            Bento.objects.get('mode7', function (mode7) {
                var renderer = mode7.getRenderer();
                renderer.setSize(Bento.getCanvas().width, Bento.getCanvas().height, false);
            });
            canvas = photo.getCanvas();

            // blur snapshot
            StackBlur.canvasRGBA(canvas, 0, 0, canvas.width, canvas.height, 10);

            blurredSprite = new Sprite({
                image: new PackedImage(canvas)
            });
            blurredBackground = new Entity({
                name: 'blur',
                position: new Vector2(0, 0),
                originRelative: new Vector2(0.5, 0.5),
                components: [
                    blurredSprite
                ]
            });
            // fit to screen
            blurredBackground.scale.x = viewport.width / blurredBackground.dimension.width;
            blurredBackground.scale.y = viewport.height / blurredBackground.dimension.height;

            menu.attach(blurredBackground);
        };
        var entity = new ClickButton({
            z: Utils.Layers.GUI + 1,
            sfx: 'sfx_button',
            name: 'menuButton',
            image: Bento.assets.getImage('worldmap/buttonmenu'),
            frameCountX: 2,
            frameCountY: 1,
            animations: {
                up: {
                    frames: [0]
                },
                down: {
                    frames: [1]
                },
                inactive: {
                    frames: [0]
                }
            },
            position: new Vector2(12, 10),
            float: true,
            onClick: function () {
                // hide critters
                Bento.objects.get('teamPanel', function (teamPanel) {
                    // teamPanel.visible = false;
                });

                // blur();
                createTeam();

                // attach menu
                Bento.objects.pause(Utils.PauseLevels.MENU);
                Bento.objects.attach(menu);

                if (settings.onClick) {
                    settings.onClick();
                }
            }
        }).attach(
            // back button pressed
            new Utils.BackButton(function (data) {
                data.entity.doCallback();
            })
        );
        var menu = new Entity({
            z: Utils.Layers.GUI + 1,
            name: 'menu',
            position: new Vector2(viewport.width / 2, viewport.height / 2),
            updateWhenPaused: Utils.PauseLevels.MENU,
            float: true,
            components: [

            ]
        });
        var overlay = new Entity({
            name: 'overlay',
            position: new Vector2(0, 0),
            originRelative: new Vector2(0.5, 0.5),
            components: [
                new Sprite({
                    imageName: 'fade'
                })
            ]
        });
        var buttonCombine = new ClickButton({
            name: 'buttonCombine',
            sfx: 'sfx_button',
            image: Bento.assets.getImage('map/buttoncombine'),
            frameCountX: 2,
            frameCountY: 2,
            animations: {
                up: {
                    frames: [0]
                },
                down: {
                    frames: [1]
                },
                inactive: {
                    frames: [2]
                }
            },
            position: new Vector2(0, buttonStartY),
            updateWhenPaused: Utils.PauseLevels.MENU,
            onClick: function () {
                // go to team select screen
                var combiner = new Combiner({
                    onClose: function () {
                        setVisible(true);

                        // reload team
                        settings.makeTeamCritters();
                        createTeam();
                    }
                });
                Bento.objects.attach(combiner);
                // make everything else invisible
                setVisible(false);
            }
        });
        var buttonEditTeam = new ClickButton({
            name: 'buttonEditTeam',
            sfx: 'sfx_button',
            image: Bento.assets.getImage('map/buttonteamedit'),
            frameCountX: 2,
            frameCountY: 2,
            animations: {
                up: {
                    frames: [0]
                },
                down: {
                    frames: [1]
                },
                inactive: {
                    frames: [2]
                }
            },
            position: new Vector2(0, buttonStartY + buttonDistance),
            updateWhenPaused: Utils.PauseLevels.MENU,
            onClick: function () {
                // go to team select screen
                var teamEdit = new TeamEdit({
                    onClose: function () {
                        setVisible(true);

                        // reload team
                        settings.makeTeamCritters();
                        createTeam();
                    }
                });
                Bento.objects.attach(teamEdit);
                // make everything else invisible
                setVisible(false);
            }
        });
        var buttonPlanet = new ClickButton({
            sfx: 'sfx_transition',
            name: 'buttonPlanet',
            image: Bento.assets.getImage('map/buttonmap'),
            frameCountX: 2,
            frameCountY: 2,
            animations: {
                up: {
                    frames: [0]
                },
                down: {
                    frames: [1]
                },
                inactive: {
                    frames: [2]
                }
            },
            position: new Vector2(0, buttonStartY - buttonDistance),
            float: false,
            updateWhenPaused: Utils.PauseLevels.MENU,
            onClick: function () {
                Bento.objects.resume();
                // TODO: transition to map
                Bento.screens.show('screens/map');
            }
        });
        var buttonDex = new ClickButton({
            sfx: 'sfx_transition',
            name: 'buttonDex',
            image: Bento.assets.getImage('map/buttondex'),
            frameCountX: 2,
            frameCountY: 2,
            animations: {
                up: {
                    frames: [0]
                },
                down: {
                    frames: [1]
                },
                inactive: {
                    frames: [2]
                }
            },
            position: new Vector2(0, buttonStartY + buttonDistance * 2),
            float: false,
            updateWhenPaused: Utils.PauseLevels.MENU,
            onClick: function () {
                // go to combodex
                var dex = new Dex({
                    onClose: function () {
                        setVisible(true);
                    }
                });
                Bento.objects.attach(dex);
                // make everything else invisible
                setVisible(false);
                settings.newCritter = false;
                buttonDex.remove(newCritters);
            }
        });
        var buttonResume = new ClickButton({
            name: 'buttonExit',
            z: Utils.Layers.GUI,
            sfx: 'sfx_button',
            image: Bento.assets.getImage('map/buttonexit'),
            frameCountX: 2,
            frameCountY: 2,
            float: true,
            animations: {
                up: {
                    frames: [0]
                },
                down: {
                    frames: [1]
                },
                inactive: {
                    frames: [2]
                }
            },
            position: new Vector2(-viewport.width / 2 + 12, -viewport.height / 2 + 10),
            updateWhenPaused: Utils.PauseLevels.MENU,
            onClick: function () {
                // detach menu
                Bento.objects.resume();
                Bento.objects.remove(menu);

                // unhide and update critters
                settings.makeTeamCritters();
                Bento.objects.get('teamPanel', function (teamPanel) {
                    teamPanel.visible = true;
                });

                if (settings.onClose) {
                    settings.onClose();
                }
                EventSystem.fire('closedMenu');
            }
        }).attach(
            // back button pressed
            new Utils.BackButton(function (data) {
                var oldPauseLevel = Bento.objects.isPaused();

                Bento.objects.pause(Utils.PauseLevels.EXIT);
                new Dialog({
                    name: 'exitGame',
                    text: Utils.Localization.getText('exit'),
                    pauseLevel: Utils.PauseLevels.EXIT,
                    type: 'yesno',
                    onYes: function () {
                        navigator.app.exitApp();
                    },
                    onNo: function () {},
                    onClose: function () {
                        Bento.objects.pause(oldPauseLevel);
                    }
                });
            })
        );
        var toggleMusic = new ToggleMusic({
            updateWhenPaused: Utils.PauseLevels.MENU,
            position: new Vector2(-viewport.width / 2 + 12, -viewport.height / 2 + 40)
        });
        var toggleSound = new ToggleSound({
            updateWhenPaused: Utils.PauseLevels.MENU,
            position: new Vector2(-viewport.width / 2 + 12, -viewport.height / 2 + 40 + 24)
        });
        var setVisible = function (value) {
            Bento.objects.get('mode7', function (entity) {
                entity.visible = value;
            });
            Bento.objects.get('player', function (entity) {
                entity.visible = value;
            });
            Bento.objects.getByFamily('triggers', function (array) {
                Utils.forEach(array, function (item, i, l, breakLoop) {
                    item.visible = value;
                });
            });
        };
        var teamList;
        var teamPanel = new Entity({
            name: 'teamPanel',
            position: new Vector2(0, viewport.height / 2 - 64),
            components: [

            ]
        });
        // '!' notification for new critters in critterDex
        var newCritters = new Entity({
            name: 'newCritters',
            position: new Vector2(21, -14),
            originRelative: new Vector2(0, 0),
            updateWhenPaused: Utils.PauseLevels.MENU,
            components: [
                new Sprite({
                    imageName: 'gui/newcritterdex',
                    frameCountX: 9,
                    frameCountY: 1,
                    animations: {
                        'default': {
                            speed: 0.5,
                            frames: [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 2, 3, 4, 5, 6, 7, 8]
                        }
                    }
                })
            ]
        });
        var createTeam = function () {
            var i = 0;
            var makeTeamCritters = function () {
                var teamCritters = Utils.getTeam();
                var slot;
                var card;

                if (teamList) {
                    // remove old
                    teamPanel.remove(teamList);
                }
                teamList = new CenteredList({
                    position: new Vector2(0, 32),
                    spacing: 2
                });

                Utils.forEach(teamCritters, function (properties, i, l, breakLoop) {
                    var card;
                    var coinPrice;
                    var buttonHeal = new ClickButton({
                        sfx: 'sfx_button',
                        image: Bento.assets.getImage('worldmap/buttonheal'),
                        frameCountX: 2,
                        frameCountY: 2,
                        position: new Vector2(-1, -35),
                        animations: {
                            up: {
                                frames: [0]
                            },
                            down: {
                                frames: [1]
                            },
                            inactive: {
                                frames: [2]
                            }
                        },
                        updateWhenPaused: Utils.PauseLevels.MENU,
                        onClick: function () {
                            // enough cash?
                            var myCoins = Bento.saveState.load('myCoins', 0);
                            var healedFrom0 = Bento.saveState.load('healedFrom0', 0);
                            if (myCoins < coinPrice) {
                                // not enough
                                Bento.audio.playSound('sfx_error');
                                return;
                            } else {
                                myCoins -= coinPrice;
                                Utils.trackCoins(-coinPrice, 'heal');
                                Bento.saveState.save('myCoins', myCoins);
                                EventSystem.fire('updateCoins');
                            }

                            // Achievement: Heal 10 critters from 0 health
                            if (properties.currentHp <= 0) {
                                if (++healedFrom0 >= 10) {
                                    Utils.postAchievement('CgkI--zjoo8UEAIQDQ', function () {});
                                }
                                Bento.saveState.save('healedFrom0', healedFrom0);
                            }

                            // TODO: ask confirmation to heal?
                            // card.remove(buttonHeal);
                            buttonHeal.setActive(false);
                            buttonHeal.visible = false;
                            price.visible = false;
                            properties.currentHp = properties.hp;
                            card.setHp(properties.currentHp);
                            Utils.saveTeam();
                            Bento.audio.playSound('sfx_heal');
                        }
                    });
                    var price = new CoinCounter({
                        position: new Vector2(0, -9 - 9),
                        align: 'center',
                        value: '$'
                    });

                    buttonHeal.attach(price);

                    if (properties) {
                        coinPrice = properties.hp;
                        price.setValue('$' + coinPrice);
                        card = new Card(properties);

                        // attach a clickbutton if critter needs healing
                        if (properties.currentHp < properties.hp) {
                            buttonHeal.setActive(true);
                            buttonHeal.visible = true;
                        } else {
                            buttonHeal.setActive(false);
                            buttonHeal.visible = false;
                            price.visible = false;
                        }
                    } else {
                        card = new Entity({
                            name: 'empty',
                            originRelative: new Vector2(0.5, 0.5),
                            components: [new Sprite({
                                imageName: 'gui/emptyslot'
                            })]
                        });
                        card.dimension.width = 45;
                        buttonHeal.setActive(false);
                        buttonHeal.visible = false;
                        price.visible = false;
                    }
                    card.attach(buttonHeal);
                    teamList.addItem(card);
                });
                // add self to teamPanel
                // teamPanel.attach(teamList);
            };
            var team = Utils.getTeam();

            makeTeamCritters();

            // perform a save here(?)
            // Utils.saveTeam();
            // Utils.saveCritters();

            return teamList;
        };
        var addText = function (string, button) {
            var text = new Text({
                position: new Vector2(0, 11),
                text: string,
                font: 'volter',
                fontSize: 9,
                fontSettings: Utils.FontSettings.get('font'),
                fontColor: '#ffffff',
                align: 'center',
                textBaseline: 'top'
            });
            button.attach(text);
        };
        // setup menu
        // blur();
        overlay.alpha = 0.5;

        addText(Utils.Localization.getText('menuCombine'), buttonCombine);
        addText(Utils.Localization.getText('menuTeamEdit'), buttonEditTeam);
        addText(Utils.Localization.getText('menuLevelSelect'), buttonPlanet);
        addText(Utils.Localization.getText('menuCritterdex'), buttonDex);

        menu.attach(overlay);
        menu.attach(buttonDex);
        menu.attach(buttonEditTeam);
        menu.attach(buttonCombine);
        menu.attach(toggleMusic);
        menu.attach(toggleSound);
        menu.attach(buttonPlanet);
        menu.attach(teamPanel);
        menu.attach(buttonResume);

        // dont combine until tutorial
        if (Bento.saveState.load('sawTutorial2', false) === false) {
            buttonCombine.setActive(false);
            buttonCombine.getComponent('text', function (text) {
                text.alpha = 0.5;
            });
            // don't show 'new' next to critterdex until after the combine tutorial
            settings.newCritter = false;
        }
        if (Bento.saveState.load('sawTutorial', false) === false) {
            buttonEditTeam.setActive(false);
            buttonEditTeam.getComponent('text', function (text) {
                text.alpha = 0.5;
            });
            // don't show 'new' next to critterdex until after the combine tutorial
            settings.newCritter = false;
        }

        if (settings.newCritter) {
            buttonDex.attach(newCritters);
        }

        entity.extend({
            removeComponent: function (name) {
                menu.getComponent(name, function (entity) {
                    menu.remove(entity);
                });
            },
            attachComponent: function (other) {
                menu.attach(other);
            },
            getComponentMenu: function (name, callback) {
                return menu.getComponent(name, callback);
            },
            showNewIcon: function () {
                if (newCritters.parent) {
                    newCritters.parent.remove(newCritters);
                }
                buttonDex.attach(newCritters);
            }
        });

        return entity;
    };
});