/**
 * Spawns the token counter and enemy counter
 */
bento.define('gui/worldmap/objectives', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    ClickButton,
    Counter,
    Text,
    EventSystem,
    Utils,
    Tween
) {
    'use strict';
    return function (settings) {
        var viewport = Bento.getViewport();
        var teamPanel = settings.teamPanel;
        var player = settings.player;
        var camera = settings.camera;
        var planetData = Utils.worldMap.planetData;
        var planetName = planetData.planetName;
        var token = Bento.saveState.load(planetName + '-tokenTotal', 0);
        var tokenCount = 0;
        var tokenIcon = new Entity({
            z: Utils.Layers.GUI,
            name: 'tokenIcon',
            position: new Vector2(viewport.width / 2 - 26, 0),
            float: true,
            originRelative: new Vector2(0.5, 0.5),
            components: [new Sprite({
                imageName: 'worldmap/tokenicon'
            })]
        }).attach({
            update: function () {
                tokenIcon.position.y = teamPanel.position.y - 2 - 19 - 4;
            }
        });
        var tokenCounter = new Text({
            z: Utils.Layers.GUI,
            name: 'tokenCounter',
            position: new Vector2(viewport.width / 2 - 12, 0),
            float: true,
            text: '0 / 10',
            // fontSettings: Utils.FontSettings.get('font'),
            font: 'volter',
            fontSize: 9,
            fontColor: '#ffffff',
            centerByCanvas: true,
            align: 'left',
            textBaseline: 'bottom',
            lineWidth: 1,
            strokeStyle: '#000000',
            pixelStroke: true // workaround for Cocoon bug
        }).attach({
            update: function () {
                tokenCounter.position.y = teamPanel.position.y - 2 - 8 - 4;
            }
        }).extend({
            updateValue: function () {
                token = Bento.saveState.load(planetName + '-tokenTotal', 0);
                tokenCounter.setText(token + ' / ' + tokenCount);

                new Tween({
                    from: 1.2,
                    to: 1,
                    'in': 10,
                    ease: 'linear',
                    'do': function (v, t) {
                        tokenIcon.scale.x = v;
                        tokenIcon.scale.y = v;
                    }
                });

                // got all?
                if (checkObjectivesCleared()) {
                    // show boss!
                    showBoss();
                }
            }
        });
        var enemyIcon = new Entity({
            z: Utils.Layers.GUI,
            name: 'enemyIcon',
            position: new Vector2(viewport.width / 2 - 24 + 44, 0),
            float: true,
            originRelative: new Vector2(0.5, 0.5),
            components: [new Sprite({
                imageName: 'worldmap/' + planetName + '/enemyicon'
            })]
        }).attach({
            update: function () {
                enemyIcon.position.y = teamPanel.position.y - 2 - 17;
            }
        });
        // load enemy objective from planet json
        var enemy = Bento.saveState.load(planetName + '-enemyTotal', 0);
        var enemyCount = planetData.enemiesToDefeat || 10;
        var enemyCounter = new Text({
            z: Utils.Layers.GUI,
            name: 'enemyCounter',
            position: new Vector2(viewport.width / 2 - 12 + 44, 0),
            float: true,
            text: '0 / ' + enemyCount,
            fontSettings: Utils.FontSettings.get('font'),
            align: 'left',
            textBaseline: 'bottom',
            lineWidth: 1,
            strokeStyle: '#000000',
            pixelStroke: true // workaround for Cocoon bug
        }).attach({
            update: function () {
                enemyCounter.position.y = teamPanel.position.y - 2 - 8;
            }
        }).extend({
            updateValue: function () {
                enemy = Bento.saveState.load(planetName + '-enemyTotal', 0);
                enemyCounter.setText(Math.min(enemyCount, enemy) + ' / ' + enemyCount);

                // got all?
                if (checkObjectivesCleared()) {
                    // show boss!
                    showBoss();
                }
            }
        });
        var showBoss = function () {
            // find map corner of player's position
            var cornerPosX = Math.floor(player.position.x / mapDimension.width) * mapDimension.width;
            var cornerPosY = Math.floor(player.position.y / mapDimension.height) * mapDimension.height;
            var realBossPosition = new Vector2(
                bossPosition.x + cornerPosX - viewport.width / 2,
                bossPosition.y + cornerPosY - viewport.height / 2
            );
            var viewportPos = new Vector2(viewport.x, viewport.y);
            var viewportOriginal = viewportPos.clone();

            // scroll there!
            // Bento.objects.pause(Utils.PauseLevels.SHOWBOSS);
            player.activate(false);
            player.stopSound();
            player.releaseControls();
            player.addToWorld();
            Bento.objects.remove(camera);
            Utils.timeout(5, function () {
                Bento.audio.playSound('sfx_new_critter');
            });
            Utils.timeout(60, function () {
                new Tween({
                    from: viewportOriginal,
                    to: realBossPosition,
                    'in': 60,
                    ease: 'exponential',
                    alpha: -5,
                    // updateWhenPaused: Utils.PauseLevels.SHOWBOSS,
                    'do': function (v, t) {
                        viewport.x = v.x;
                        viewport.y = v.y;

                        viewportPos = v;
                    },
                    onComplete: function () {
                        // there should be an inactive boss here...
                        Bento.objects.get('inactiveBoss', function (boss) {
                            var depth = boss.boss.getComponent('depth');
                            boss.name = 'boss';
                            boss.boss.visible = true;
                            Bento.audio.playSound('sfx_battle_false');
                            new Tween({
                                from: 1,
                                to: 0,
                                'in': 120,
                                ease: 'elastic',
                                alpha: -9,
                                beta: 5,
                                'do': function (v, t) {
                                    depth.scale.x = 1 + 0.5 * v;
                                    depth.scale.y = 1 - 0.5 * v;
                                },
                                onComplete: function () {

                                }
                            });
                        });
                        // stay on the boss
                        new Tween({
                            'in': 120,
                            ease: 'linear',
                            'do': function (v, t) {
                                viewport.x = viewportPos.x;
                                viewport.y = viewportPos.y;
                            },
                            onComplete: function () {
                                // go back
                                new Tween({
                                    from: realBossPosition,
                                    to: viewportOriginal,
                                    'in': 30,
                                    ease: 'exponential',
                                    alpha: -5,
                                    'do': function (v, t) {
                                        viewport.x = v.x;
                                        viewport.y = v.y;
                                    },
                                    onComplete: function () {
                                        player.activate(true);
                                        Bento.objects.attach(camera);
                                        player.removeFromWorld();
                                        Bento.saveState.save(planetName + '-showBoss', true);
                                    }
                                });
                            }
                        });
                    }
                });
            });

        };
        var bossPosition;
        var mapDimension;
        var countTokens = function () {
            var json = Bento.assets.getJson('maps/' + Utils.worldMap.planetData.planetName);
            var i, j, k, l,
                width = json.width,
                height = json.height,
                layers = json.layers.length,
                tilesets = json.tilesets,
                tileset,
                layer,
                object,
                tokenGid,
                bossGid;

            mapDimension = new Rectangle(0, 0, width * 16, height * 16);

            // find the checkpoint gid
            for (i = 0; i < tilesets.length; ++i) {
                tileset = tilesets[i];
                if (tileset.source) {
                    if (tileset.source.indexOf('token.json') >= 0) {
                        tokenGid = tileset.firstgid;
                    }
                    if (tileset.source.indexOf('boss.json') >= 0) {
                        bossGid = tileset.firstgid;
                    }
                }
            }

            for (k = 0; k < layers; ++k) {
                layer = json.layers[k];
                // loop through objects
                if (layer.type === 'objectgroup') {
                    for (i = 0, l = layer.objects.length; i < l; ++i) {
                        object = layer.objects[i];
                        // not saved
                        // checkpoints
                        if (object.gid) {
                            if (object.gid === tokenGid) {
                                tokenCount += 1;
                            }
                            if (object.gid === bossGid) {
                                bossPosition = new Vector2(object.x, object.y);
                            }
                        }
                    }
                }
            }

            // update values (make sure to wait until the map is loaded)
            tokenCounter.updateValue();
            enemyCounter.updateValue();
        };
        var checkObjectivesCleared = function () {
            if (
                Bento.saveState.load(planetName + '-showBoss', false) === false &&
                // enemy >= enemyCount &&
                token >= tokenCount
            ) {
                return true;
            }
            return false;
        };

        Bento.objects.attach(tokenIcon);
        Bento.objects.attach(tokenCounter);
        // Bento.objects.attach(enemyIcon);
        // Bento.objects.attach(enemyCounter);

        return {
            countTokens: countTokens
        };
    };
});