/**
 * Notification scheduler for Facebook
 * Also see the fbig-notifications repo
 * @moduleName FBNotification
 */
bento.define('modules/fbnotification', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween
) {
    'use strict';
    var gameId = "ComboCritters";
    var sendToServer = function (path, json, callback) {
        var xhr = new window.XMLHttpRequest();
        xhr.open("POST", "https://9iwf3q15w5.execute-api.us-east-1.amazonaws.com/latest/" + path);
        xhr.setRequestHeader("Content-Type", "application/json;charset=UTF-8");
        xhr.onreadystatechange = function () {
            if (xhr.readyState === 4) {
                if (xhr.status >= 200 && xhr.status < 300) {
                    try {
                        callback(null, JSON.parse(xhr.responseText));
                    } catch (e) {
                        callback("Could not parse response", xhr.responseText);
                    }
                } else {
                    callback("Error", xhr.responseText);
                }
            }
        };
        xhr.send(JSON.stringify(json));
    };
    return {
        /**
         * Schedule a notification
         * @snippet FBNotification.schedule
FBNotification.schedule(${1:24}, {
    message: '$2',
    title: '$3',
    buttonText: '$4',
    imageUrl: '$5'
}, function (error, messageDataStr) {
    if (error) {
        console.error(error);
        return;
    }
    // save messageData which can be used to delete the notification
    Bento.saveState.save('$6', messageDataStr);
});
         */
        schedule: function (hours, messageData, callback) {
            sendToServer('fbig-schedule', {
                gameId: gameId,
                playerId: window.FBInstant.player.getID(),
                senderId: null,
                message: messageData.message,
                title: messageData.title,
                buttonText: messageData.buttonText,
                imageUrl: messageData.imageUrl,
                hours: hours,
            }, callback);
        },
        /**
         * Remove scheduled notification
         * @snippet FBNotification.remove
FBNotification.remove(Bento.saveState.load('$1', '{}'), function (error, msg) {
    if (error) {
        console.error(msg);
        return;
    }
});
         */
        remove: function (dataStr, callback) {
            var data = {};
            try {
                data = JSON.parse(dataStr);
            } catch (e) {
                callback(e);
                return;
            }
            if (!data.scheduledTimeHour || !data.notificationId) {
                callback("No notification data.");
                return;
            }
            sendToServer('fbig-remove', {
                scheduledTimeHour: data.scheduledTimeHour,
                notificationId: data.notificationId
            }, callback);
        }
    };
});