/**
 * Global FBInstant session data.
 * @moduleName FBSession
 * @snippet FBSession.set
FBSession.set('${1:key}', ${2:value});
 * @snippet FBSession.get
FBSession.get('${1:key}');
 * @snippet FBSession.update
FBSession.update();
 * @snippet FBSession.getEntryPointData
FBSession.getEntryPointData();
 * @snippet FBSession.resetEntryPointData
FBSession.resetEntryPointData();
 */
bento.define('modules/fbsession', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween
) {
    'use strict';
    var sessionData = {};
    var entryPointData = null;
    var cached;
    var FBSession = {
        update: function () {
            FBInstant.setSessionData(sessionData);
        },
        set: function (key, value, skipUpdate) {
            sessionData[key] = value;
            if (!skipUpdate) {
                FBSession.update();
            }
        },
        get: function (key) {
            return sessionData[key];
        },
        getEntryPointData: function () {
            if (!entryPointData) {
                entryPointData = FBInstant.getEntryPointData();
            }
            return entryPointData;

            // simulate data
            // return {dailyMission: "discythrow", playerId: "1580726322012646", score: 1, objective: 18};
        },
        resetEntryPointData: function () {
            entryPointData = {};
        },
        // returns cached canvas with player photo
        getCirclePhoto: function () {
            return cached;
        },
        setCirclePhoto: function (canvas) {
            if (!cached) {
                cached = canvas;                
                if (Bento.isDev()) {
                    Utils.cachedPhoto = cached;
                }
            }
        }
    };

    if (Bento.isDev()) {
        cached = Utils.cachedPhoto;
    }

    return FBSession;
});