bento.define('modules/forcedadtimer', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    Utils,
    Tween
) {
    'use strict';
    var minutesUntilReset = 4;
    var counterBehavior = {
        start: function () {
            console.log('video ad timer started');
        },
        destroy: function () {
            console.log('Warning: attempted to remove video ad timer');
            // put self back??
        },
        update: function (data) {
            if (videoAdTimer.videoAdTimer > 0) {
                videoAdTimer.videoAdTimer -= 1;
            }
            // autosave every 10 seconds
            if (data.entity.videoAdTimer % 600 === 1) {
                Bento.saveState.save('forcedAdTimer', data.entity.videoAdTimer);
            }
        }
    };
    var videoAdTimer = new Entity({
        z: 0,
        name: 'forcedAdTimer',
        global: true,
        updateWhenPaused: Infinity,
        components: [counterBehavior]
    });

    // public functions
    videoAdTimer.extend({
        videoAdTimer: Bento.saveState.load('forcedAdTimer', minutesUntilReset * 60 * 60),
        getTimeString: function () {
            var totalSeconds = Math.floor(this.videoAdTimer / 60),
                seconds = totalSeconds % 60,
                minutes = Math.floor(totalSeconds / 60),
                str = '' + minutes + ':' + ('00' + seconds).slice(-2);
            return str;
        },
        force: function () {
            videoAdTimer.videoAdTimer = 0;
            Bento.saveState.save('forcedAdTimer', 0);
        },
        reset: function () {
            this.videoAdTimer = minutesUntilReset * 60 * 60;
            Bento.saveState.save('forcedAdTimer', minutesUntilReset * 60 * 60);
        },
        canPlay: function () {
            if (!videoAdTimer.isAdded) {
                Bento.objects.attach(videoAdTimer);
            }
            return this.videoAdTimer <= 0;
        },
        setMinutesUntilReset: function (value) {
            if (value) {
                minutesUntilReset = value;
            } else {
                // reset to original
                minutesUntilReset = 4;
            }
        }
    });

    return videoAdTimer;
});