bento.define('modules/playtimer', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    Utils,
    Tween
) {
    'use strict';
    var counterBehavior = {
        start: function (data) {
            console.log('play timer started');
            // save play timer on screen change
            EventSystem.on('screenHidden', function () {
                Bento.saveState.save('playTimer', playTimer.playTimer);
            });
        },
        destroy: function (data) {
            console.log('Warning: attempted to remove play timer');
            // put self back??
        },
        update: function (data) {
            ++playTimer.playTimer;
        }
    };
    var playTimer = new Entity({
        z: 0,
        name: 'playTimer',
        global: true,
        updateWhenPaused: Infinity,
        components: [
            counterBehavior
        ]
    });

    // public functions
    playTimer.extend({
        playTimer: Bento.saveState.load('playTimer', 0),
        getTimeString: function () {
            var totalSeconds = Math.floor(this.playTimer / 60);
            var seconds = totalSeconds % 60;
            var minutes = Math.floor(totalSeconds / 60);
            var hours = Math.floor(totalSeconds / 3600);
            var str = '' + ('00' + hours).slice(-2) + ':' + ('00' + minutes).slice(-2) + ':' + ('00' + seconds).slice(-2);
            return str;
        }
    });

    return playTimer;
});