/**
 * Module that handles watching video for daily gift reroll and giving the reward
 */
bento.define('modules/rerollad', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'ads/heyzap',
    'gui/dialog',
    'modules/videoadtimer',
    'modules/forcedadtimer',
    'gui/rewardpopup'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    ClickButton,
    Counter,
    Text,
    EventSystem,
    Utils,
    Tween,
    HeyZap,
    Dialog,
    VideoAdTimer,
    ForcedAdTimer,
    RewardPopup
) {
    'use strict';

    var oldPauseLevel = 0;
    var module = {
        getReward: function (success, onSuccess, onFail) {
            Bento.objects.pause(oldPauseLevel);
            if (!success) {
                if (onFail) {
                    onFail();
                }
                return;
            }

            if (onSuccess) {
                onSuccess();
            }
            // reset video timer
            VideoAdTimer.reset();
            ForcedAdTimer.reset();
        },
        watchAd: function (onSuccess, onFail) {
            var totalAdsWatched = Bento.saveState.load('totalAdsWatched', 0);
            var rerollAdsWatched = Bento.saveState.load('rerollAdsWatched', 0);
            var forcedAdsWatched = Bento.saveState.load('forcedAdsWatched', 0);
            var coinAdsWatched = Bento.saveState.load('coinAdsWatched', 0);

            oldPauseLevel = Bento.objects.isPaused();
            Bento.objects.pause(oldPauseLevel + 1);
            if (Utils.EveryPlay.isRecording()) {
                Utils.EveryPlay.pause();
            }

            // don't show ads on DGG
            if (Utils.DGG_DEMO) {
                window.alert('You are now watching an ad.');
                module.getReward(true, onSuccess);
                return;
            }

            // watch ad
            if (HeyZap.canShowRewarded()) {
                Bento.audio.muteMusic(true);

                ++totalAdsWatched;
                ++rerollAdsWatched;

                // user started watching ad
                Utils.Analytics.event('Ads', 'startRerollAd_' + rerollAdsWatched);
                Utils.Analytics.event('Ads', 'startTotalAd_' + totalAdsWatched);

                HeyZap.showRewarded(function () {}, function (hasCompleted) {
                    // user finished watching ad
                    Utils.Analytics.event('Ads', 'finishRerollAd_' + rerollAdsWatched);
                    Utils.Analytics.event('Ads', 'finishTotalAd_' + totalAdsWatched);

                    Utils.Analytics.event('Ads', 'forReroll_' + rerollAdsWatched);
                    Utils.Analytics.event('Ads', 'total_' + totalAdsWatched);

                    if (window.tenjin && Utils.isNativeIos()) {
                        window.tenjin.sendEvent('rewardedAd', function () {}, function () {});
                    }

                    Utils.Firebase.logEvent('watched_reroll_ad', {
                        value: 1,
                        forced_ads_watched: forcedAdsWatched,
                        coin_ads_watched: coinAdsWatched,
                        total_ads_watched: totalAdsWatched,
                        reroll_ads_watched: rerollAdsWatched
                    });
                    Utils.Firebase.logEvent('watched_ad', {
                        value: 1,
                        ad_type: 'reroll',
                        forced_ads_watched: forcedAdsWatched,
                        coin_ads_watched: coinAdsWatched,
                        total_ads_watched: totalAdsWatched,
                        reroll_ads_watched: rerollAdsWatched
                    });

                    Bento.saveState.save('totalAdsWatched', totalAdsWatched);
                    Bento.saveState.save('rerollAdsWatched', rerollAdsWatched);

                    Bento.audio.muteMusic(Bento.saveState.load('muteMusic', false), true);
                    bento.require(['audia'], function (Audia) {
                        var audioContext = Audia.audioContext;
                        if (audioContext && audioContext.state === 'suspended') {
                            audioContext.resume();
                        }
                    });
                    module.getReward(hasCompleted, onSuccess);
                    Utils.Analytics.event('UI', 'adForReroll');
                    Utils.GooglePlay.submitEvent('CgkI--zjoo8UEAIQAw', 1);
                    if (Utils.EveryPlay.isPaused()) {
                        Utils.EveryPlay.resume();
                    }
                });
            } else {
                if (Utils.isCocoonJS()) {
                    // alert('Sorry, no ad available');
                    new Dialog({
                        name: 'noAd',
                        text: Utils.Localization.getText("noAd"),
                        type: 'yes',
                        pauseLevel: oldPauseLevel + 1,
                        onClose: function () {
                            module.getReward(false, onSuccess, onFail);
                        }
                    });
                } else {
                    // simulate watching
                    window.alert('You are now watching an ad.');
                    module.getReward(true, onSuccess);
                }
            }
        }
    };

    return module;
});