bento.define('modules/savestate', [
    'bento/utils'
], function (
    Utils
) {
    'use strict';
    var uniqueID = 'grumpycat/',
        storage,
        storageFallBack = {
            data: {},
            setItem: function (key, value) {
                var k,
                    count = 0;
                storageFallBack.data[key] = value;
                // update length
                for (k in storageFallBack.data) {
                    if (storageFallBack.data.hasOwnProperty(k)) {
                        ++count;
                    }
                }
                this.length = count;
            },
            getItem: function (key) {
                var item = storageFallBack.data[key];
                return Utils.isDefined(item) ? item : null;
            },
            removeItem: function (key) {
                delete storageFallBack.data[key];
            },
            clear: function () {
                this.data = {};
                this.length = 0;
            },
            length: 0
        };

    // initialize
    // try {
    //     storage = window.localStorage;
    //     // try saving once
    //     if (window.localStorage) {
    //         window.localStorage.setItem(uniqueID + 'save', '0');
    //     } else {
    //         throw 'No local storage available';
    //     }
    // } catch (e) {
    //     // alert('Warning: you have disabled cookies on your browser. You cannot save progress in your game.');
    //     storage = storageFallBack;
    // }
    storage = storageFallBack;
    return {
        saveKeys: false,
        save: function (elementKey, element) {
            if (typeof elementKey !== 'string') {
                elementKey = JSON.stringify(elementKey);
            }
            storage.setItem(uniqueID + elementKey, JSON.stringify(element));
            // sync with FBInstant
            FBInstant.player.setDataAsync({
                save: storage.data
            }).then(function () {
                // console.log('data is saved');
            });
        },
        add: function (elementKey, element) {
            if (!elementKey) {
                console.warn("ERROR: savestate key is not defined.");
                return;
            }
            var value = this.load(elementKey, 0);
            value += element;
            this.save(elementKey, value);
        },
        load: function (elementKey, defaultValue) {
            var element;
            element = storage.getItem(uniqueID + elementKey);
            if (element === null) {
                return defaultValue;
            }
            return JSON.parse(element);
        },
        remove: function (elementKey) {
            storage.removeItem(uniqueID + elementKey);
        },
        clear: function () {
            storage.clear();

            // sync with FBInstant
            FBInstant.player.setDataAsync({
                save: storage.data
            }).then(function () {
                // console.log('data is saved');
            });
        },
        debug: function () {
            console.log(localStorage);
        },
        isEmpty: function () {
            return storage.length === 0;
        },
        getId: function () {
            return uniqueID.slice(0);
        },
        setId: function (str) {
            uniqueID = str;
        },
        // sync with FBInstant
        setData: function (data) {
            storageFallBack.data = data;
        }
    };
});