/**
 * Share image for catching
 * (note: almost a copy of combine, did not bother making this modular)
 */
bento.define('modules/share/catch', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'bento/canvas',
    'entities/battle/critter'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    ClickButton,
    Counter,
    Text,
    EventSystem,
    Utils,
    Tween,
    Canvas,
    Critter
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
            critter: String // id for left criter
            critterImage: PackedImage // alternative: image
        }*/
        var viewport = Bento.getViewport();
        var sprite = new Sprite({
            imageName: 'share/catch'
        });
        var background = new Entity({
            name: 'background',
            originRelative: new Vector2(0.5, 0.5), // ? not working
            components: [new Sprite({
                imageName: 'share/catch'
            })]
        });
        var critterId = settings.critter;
        var middle = new Vector2(190 * imageScale / 2, 134 * imageScale / 2);
        var imageScale = 4;
        var entity = new Canvas({
            name: 'share',
            width: 190 * imageScale,
            height: 134 * imageScale,
            drawOnce: true,
            originRelative: new Vector2(imageScale / 2, imageScale / 2),
            components: [
                sprite
            ]
        });
        var makeCritterShadow = function (critterId, color) {
            var critter;
            if (settings.critter) {
                critter = new Critter({
                    id: critterId
                });
            } else if (settings.critterImage) {
                critter = new Entity({
                    position: new Vector2(0, 0),
                    originRelative: new Vector2(0.5, 1),
                    components: [new Sprite({
                        image: settings.critterImage
                    })]
                });
            }
            var width = critter.dimension.width;
            var height = critter.dimension.height;
            var canvas = new Canvas({
                width: width,
                height: height,
                drawOnce: true,
                originRelative: new Vector2(0.5, 1)
            });
            var fill = {
                draw: function (data) {
                    data.renderer.fillRect(color || [1, 1, 1, 1], -width / 2, -height, width, height);
                    canvas.getContext().globalCompositeOperation = 'destination-in';
                }
            };

            // fill canvas
            canvas.attach(fill);
            // draw critter
            canvas.attach(critter);
            // draw canvas
            canvas.draw(Bento.getGameData());

            canvas.remove(fill);
            canvas.remove(critter);

            // background.attach(canvas);

            return canvas;
        };
        var makeCritterOutline = function (critter, color) {
            var container = new Entity({});
            var shadow1 = makeCritterShadow(critter, color);
            var shadow2 = makeCritterShadow(critter, color);
            var shadow3 = makeCritterShadow(critter, color);
            var shadow4 = makeCritterShadow(critter, color);
            shadow1.position.x += 1;
            shadow2.position.x -= 1;
            shadow3.position.y += 1;
            shadow4.position.y -= 1;

            container.attach(shadow1);
            container.attach(shadow2);
            container.attach(shadow3);
            container.attach(shadow4);
            return container;
        };
        var makeCritter3 = function () {
            var container = new Entity({});
            var outline = makeCritterOutline(settings.critter);
            var critter;

            if (settings.critter)
                critter = new Critter({
                    id: critterId
                });
            else {
                critter = new Entity({
                    position: new Vector2(0, 0),
                    originRelative: new Vector2(0.5, 1),
                    components: [new Sprite({
                        image: settings.critterImage
                    })]
                });
            }
            critter.position = new Vector2(0, 32);
            outline.position = new Vector2(0, 32);

            // give critter white outline
            container.attach(outline);
            container.attach(critter);

            background.attach(container);
        };

        background.scale.x = imageScale;
        background.scale.y = imageScale;

        makeCritter3();

        entity.attach(background);
        entity.draw(Bento.getGameData());

        // window.open(entity.getCanvas().toDataURL(), '_blank');

        return {
            getBase64: function () {
                return entity.getCanvas().toDataURL();
            }
        };
    };
});