/**
 * Share image for combine
 */
bento.define('modules/share/combine', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'bento/canvas',
    'entities/battle/critter'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    ClickButton,
    Counter,
    Text,
    EventSystem,
    Utils,
    Tween,
    Canvas,
    Critter
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
            critterId1: String // id for left criter
            critterId2: String // id for right criter
            critterId3: String // id for result criter
        }*/
        var viewport = Bento.getViewport();
        var sprite = new Sprite({
            imageName: 'share/combine'
        });
        var background = new Entity({
            name: 'background',
            originRelative: new Vector2(0.5, 0.5), // ? not working
            components: [new Sprite({
                imageName: 'share/combine'
            })]
        });
        var middle = new Vector2(252 * imageScale / 2, 180 * imageScale / 2);
        var imageScale = 4;
        var entity = new Canvas({
            name: 'share',
            width: 252 * imageScale,
            height: 180 * imageScale,
            drawOnce: true,
            originRelative: new Vector2(imageScale / 2, imageScale / 2),
            components: [
                sprite
            ]
        });
        var makeCritterShadow = function (critterId, color) {
            var critter = new Critter({
                id: critterId
            });
            var width = critter.dimension.width;
            var height = critter.dimension.height;
            var canvas = new Canvas({
                width: width,
                height: height,
                drawOnce: true,
                originRelative: new Vector2(0.5, 1)
            });
            var fill = {
                draw: function (data) {
                    data.renderer.fillRect(color || [1, 1, 1, 1], -width / 2, -height, width, height);
                    canvas.getContext().globalCompositeOperation = 'destination-in';
                }
            };

            // fill canvas
            canvas.attach(fill);
            // draw critter
            canvas.attach(critter);
            // draw canvas
            canvas.draw(Bento.getGameData());

            canvas.remove(fill);
            canvas.remove(critter);

            // background.attach(canvas);

            return canvas;
        };
        // this doesnt work on cocoon for whatever reason
        /*var makeCritterOutline = function (critterId, color) {
            var critter = new Critter({
                id: critterId
            });
            var width = critter.dimension.width;
            var height = critter.dimension.height;
            var canvas = new Canvas({
                width: width + 2,
                height: height + 2,
                drawOnce: true,
                originRelative: new Vector2(0.5, 1)
            });
            var context = canvas.getContext();
            var fill = {
                draw: function (data) {
                    data.renderer.fillRect(color || [1, 1, 1, 1], -(width + 2) / 2, -(height + 2), (width + 2), (height + 2));
                    canvas.getContext().globalCompositeOperation = 'destination-in';
                }
            };
            critter.position.y -= 1;

            // fill canvas
            canvas.attach(fill);
            // draw critter
            canvas.attach(critter);
            // draw canvas
            canvas.draw(Bento.getGameData());

            context.globalCompositeOperation = 'source-over';
            context.drawImage(canvas.getCanvas(), 0, 0, width, height, 0, 1, width, height);
            context.drawImage(canvas.getCanvas(), 0, 0, width, height, 0, -1, width, height);
            context.drawImage(canvas.getCanvas(), 0, 0, width, height, 1, 0, width, height);
            context.drawImage(canvas.getCanvas(), 0, 0, width, height, -1, 0, width, height);

            canvas.remove(fill);
            canvas.remove(critter);

            // background.attach(canvas);

            return canvas;
        };*/
        var makeCritterOutline = function (critter, color) {
            var container = new Entity({});
            var shadow1 = makeCritterShadow(critter, color);
            var shadow2 = makeCritterShadow(critter, color);
            var shadow3 = makeCritterShadow(critter, color);
            var shadow4 = makeCritterShadow(critter, color);
            shadow1.position.x += 1;
            shadow2.position.x -= 1;
            shadow3.position.y += 1;
            shadow4.position.y -= 1;
            
            container.attach(shadow1);
            container.attach(shadow2);
            container.attach(shadow3);
            container.attach(shadow4);
            return container;
        };
        var makeCritter1 = function () {
            var container = new Entity({});
            var critter = new Critter({
                id: settings.critter1
            });
            var outline = makeCritterOutline(settings.critter1, [66 / 255, 97 / 255, 255 / 255, 1]);
            var shadow = makeCritterShadow(settings.critter1, [66 / 255, 97 / 255, 255 / 255, 1]);
            var shadow2 = makeCritterShadow(settings.critter1, [66 / 255, 25 / 255, 145 / 255, 1]);
            critter.position = new Vector2(-100, 32);
            outline.position = new Vector2(-100, 32);
            shadow.position = new Vector2(-100 - 4, 32 + 1);
            shadow2.position = new Vector2(-100 - 8, 32 + 1);
            critter.scale.x = 0.75;
            critter.scale.y = 0.75;
            outline.scale.x = 0.75;
            outline.scale.y = 0.75;
            shadow.scale.x = 0.75;
            shadow.scale.y = 0.75;
            shadow2.scale.x = 0.75;
            shadow2.scale.y = 0.75;

            container.attach(shadow2);
            container.attach(shadow);
            container.attach(outline);
            container.attach(critter);

            background.attach(container);
        };
        var makeCritter2 = function () {
            var container = new Entity({});
            var critter = new Critter({
                id: settings.critter2
            });
            var outline = makeCritterOutline(settings.critter2, [66 / 255, 97 / 255, 255 / 255, 1]);
            var shadow = makeCritterShadow(settings.critter2, [66 / 255, 97 / 255, 255 / 255, 1]);
            var shadow2 = makeCritterShadow(settings.critter2, [66 / 255, 25 / 255, 145 / 255, 1]);
            critter.position = new Vector2(100, 32);
            outline.position = new Vector2(100, 32);
            shadow.position = new Vector2(100 + 4, 32 + 1);
            shadow2.position = new Vector2(100 + 8, 32 + 1);
            critter.scale.x = -0.75;
            critter.scale.y = 0.75;
            outline.scale.x = -0.75;
            outline.scale.y = 0.75;
            shadow.scale.x = -0.75;
            shadow.scale.y = 0.75;
            shadow2.scale.x = -0.75;
            shadow2.scale.y = 0.75;
            container.attach(shadow2);
            container.attach(shadow);
            container.attach(outline);
            container.attach(critter);

            background.attach(container);
        };
        var makeCritter3 = function () {
            var container = new Entity({});
            var outline = makeCritterOutline(settings.critter3);
            var critter = new Critter({
                id: settings.critter3
            });
            critter.position = new Vector2(0, 32);
            outline.position = new Vector2(0, 32);

            // give critter white outline
            container.attach(outline);
            container.attach(critter);

            background.attach(container);
        };

        background.scale.x = imageScale;
        background.scale.y = imageScale;

        makeCritter1();
        makeCritter2();
        makeCritter3();
        // makeCritterShadow('baanu');

        entity.attach(background);
        // entity.origin = middle.clone(); // <- bug?? can't set the origin no matter what
        entity.draw(Bento.getGameData());

        // window.open(entity.getCanvas().toDataURL(), '_blank');

        return {
            getBase64: function () {
                return entity.getCanvas().toDataURL();
            }
        };
    };
});