bento.define('modules/zoom', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    Utils,
    Tween
) {
    'use strict';
    var viewport = Bento.getViewport(),
        renderer = Bento.getRenderer(),
        zoomValue = 1,
        zoomCenter = new Vector2(viewport.width / 2, viewport.height / 2),
        rotation = 0,
        module = {
            /**
             * Rotates screen.
             * @function
             * @param {Number} value - Angle in radians
             * @instance
             * @name rotate
             */
            rotate: function (value) {
                rotation = value;
            },
            /**
             * Zooms screen in or out depending on passed value.
             * @function
             * @param {Number} value - 0 < 1 to zoom out, > 1 to zoom in. Keep number positive to prevent point reflection.
             * @instance
             * @name zoom
             */
            zoom: function (value) {
                zoomValue = value;
            },
            getZoom: function () {
                return zoomValue;
            },
            setZoomCenter: function (vector) {
                zoomCenter = vector;
            },
            /**
             * Adds two objects that ensure only certain layers get zoomed.
             */
            init: function () {
                // check if not added yet
                if (Bento.objects.get('zoomStart')) {
                    return;
                }

                Bento.objects.add({
                    z: Utils.Layers.TO_ZOOM,
                    name: 'zoomStart',
                    destroy: function () {
                        zoomValue = 1;
                        rotation = 0;
                    },
                    draw: function (data) {
                        renderer.save();
                        renderer.translate(
                            zoomCenter.x * (1 - zoomValue),
                            zoomCenter.y * (1 - zoomValue)
                        );
                        renderer.scale(zoomValue, zoomValue);
                        // rotate
                        renderer.translate(zoomCenter.x, zoomCenter.y);
                        renderer.rotate(rotation);
                        renderer.translate(-zoomCenter.x, -zoomCenter.y);
                    }
                });
                Bento.objects.add({
                    z: Utils.Layers.STOP_ZOOM,
                    name: 'zoomEnd',
                    draw: function (data) {
                        renderer.restore();
                    }
                });
            }
        };

    return module;
});