/**
 * Battle screen
 */
bento.define('screens/battle', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/screen',
    'bento/tween',
    'entities/battle/manager',
    'entities/battle/grid',
    'gui/fade',
    'gui/coincounter',
    'gui/hudcoincounter',
    'gui/battle/menu'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    Utils,
    Screen,
    Tween,
    BattleManager,
    Grid,
    Fade,
    CoinCounter,
    HudCoinCounter,
    Menu
) {
    'use strict';
    var screenObject;
    var onShow = function (data) {
        // data passed from map screen
        /*{
            planetData: Object // see 'screens/map'
            enemies: Array // see 'components/collidewithtrigger'
            isBossBattle: Boolean // set by collideWithTrigger component
            isTwitterBattle: Boolean
        }*/
        var viewport = Bento.getViewport();
        var background = new Entity({
            z: Utils.Layers.BACKGROUND,
            name: 'background',
            position: new Vector2(viewport.width / 2, 0),
            originRelative: new Vector2(0.5, 0),
            components: [new Sprite({
                imageName: 'worldmap/' + (data || window.cachedData).planetData.planetName + '/background'
            })]
        });
        var backgroundTop = new Entity({
            z: Utils.Layers.BACKGROUND,
            name: 'background',
            position: new Vector2(0, 0),
            originRelative: new Vector2(0.5, 1),
            components: [new Sprite({
                imageName: 'worldmap/' + (data || window.cachedData).planetData.planetName + '/background',
                frameCountY: 184
            })]
        });
        var ground = new Entity({
            z: Utils.Layers.CRITTERS + 1,
            name: 'ground',
            position: new Vector2(viewport.width / 2, viewport.height * 2 / 3 - 17 + 8),
            originRelative: new Vector2(0.5, 0.5),
            family: ['solid'],
            components: [new Sprite({
                imageName: 'worldmap/' + (data || window.cachedData).planetData.planetName + '/ground'
            })]
        });
        var space = new Entity({
            z: Utils.Layers.CRITTERS + 2,
            name: 'background',
            position: new Vector2(viewport.width / 2, viewport.height * 2 / 3),
            originRelative: new Vector2(0.5, 0),
            components: [new Sprite({
                imageName: 'battle/space_background'
            })]
        });
        var gridBorder = new Entity({
            z: Utils.Layers.CRITTERS + 3,
            name: 'background',
            position: new Vector2(viewport.width / 2, viewport.height * 5 / 6),
            originRelative: new Vector2(0.5, 0.5),
            components: [new Sprite({
                imageName: 'battle/grid'
            })]
        });
        var grid = new Grid({
            position: new Vector2(0, -9),
            spacing: 2
        });
        var enemyGrid = new Grid({
            position: new Vector2(viewport.width - 32, 32),
            spacing: 2,
            align: 'right'
        });
        var battle;
        var enemies = (data || window.cachedData).enemies;
        var isBossBattle = (data || window.cachedData).isBossBattle;
        var isTwitterBattle = (data || window.cachedData).isTwitterBattle;
        var buttonMenu = new Menu({
            // TODO: how much does it cost?
            isBossBattle: isBossBattle,
            fleeCost: Bento.assets.getJson('creatures/retrycost')[enemies[enemies.length - 1].rarity],
            isTwitterBattle: isTwitterBattle
        });
        var coinCounter = new HudCoinCounter({});

        gridBorder.attach(grid);

        battle = new BattleManager({
            grid: grid,
            enemyGrid: enemyGrid,
            data: (data || window.cachedData).planetData,
            enemies: (data || window.cachedData).enemies,
            isBossBattle: isBossBattle,
            isTwitterBattle: (data || window.cachedData).isTwitterBattle
        });

        // save to cache for debugging purposes
        window.cachedData = data || window.cachedData;

        background.attach(backgroundTop);
        backgroundTop.scale.y = 100;
        Bento.objects.attach(background);
        Bento.objects.attach(ground);
        Bento.objects.attach(space);
        Bento.objects.attach(gridBorder);
        Bento.objects.attach(enemyGrid);
        Bento.objects.attach(battle);
        if (Bento.saveState.load('sawTutorial2', false) || Bento.saveState.load('healTutorial', false)) {
            // only allow fleeing after tutorials or the free heal
            Bento.objects.attach(buttonMenu);
        }
        Bento.objects.attach(coinCounter);

        if (viewport.width < 170) {
            // move enemies down
            enemyGrid.position.y += 16;
        } else {
            enemyGrid.position.y += 4;
        }

        new Fade({
            fadeIn: true
        });

        if ((data || window.cachedData).isBossBattle) {
            if (!Bento.audio.isPlayingMusic('bgm_boss')) {
                Bento.audio.stopAllMusic();
                Bento.audio.playMusic('bgm_boss');
                Bento.audio.setVolume(1, 'bgm_boss');
            }
        } else {
            if (!Bento.audio.isPlayingMusic('bgm_battle')) {
                Bento.audio.stopAllMusic();
                Bento.audio.playMusic('bgm_battle');
                Bento.audio.setVolume(1, 'bgm_battle');
            }
        }

        Utils.EveryPlay.record();

        Utils.Analytics.trackView('battle');
    };

    screenObject = new Screen({
        onShow: onShow,
        onHide: function () {
            Bento.audio.stopAllMusic();
        }
    });
    return screenObject;
});