bento.define('screens/cloudsave', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/components/fill',
    'bento/entity',
    'bento/utils',
    'bento/tween',
    'bento/screen',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'gui/fade',
    'bento/gui/text',
    'gui/dialog',
    'lzstring'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Fill,
    Entity,
    Utils,
    Tween,
    Screen,
    EventSystem,
    ClickButton,
    Fade,
    Text,
    Dialog,
    LZString
) {
    'use strict';
    var object = new Screen({
        dimension: Bento.getViewport()
    });
    var onShow = function (data) {
        var viewport = Bento.getViewport();
        var CloudModule;
        var hasCloudSave = false;
        var currentSave;
        var cloudSave;
        var background = new Entity({
            z: 0,
            name: 'background',
            position: new Vector2(viewport.width / 2, viewport.height / 2),
            originRelative: new Vector2(0.5, 0.5),
            components: [
                new Sprite({
                    image: Bento.assets.getImage('team/background')
                })
            ]
        });
        var buttonSave = new ClickButton({
            z: Utils.Layers.GUI,
            name: 'buttonSave',
            sfx: 'sfx_button',
            image: Bento.assets.getImage('gui/buttoncloudsave'),
            frameCountX: 3,
            frameCountY: 1,
            animations: {
                up: {
                    frames: [0]
                },
                down: {
                    frames: [1]
                },
                inactive: {
                    frames: [2]
                }
            },
            position: new Vector2(viewport.width / 2 - 30, viewport.height / 2 + 10),
            onClick: function () {

                Bento.objects.pause();

                var dialog = new Dialog({
                    name: 'saveToCloud',
                    type: 'yesno',
                    text: Utils.Localization.getText('saveToCloud'),
                    onYes: saveToCloud,
                    onNo: function () {
                        Bento.objects.resume();
                    },
                    onClose: function () { // callback when closed, after pressing yes, no or close

                    }
                });
            }
        });
        var buttonLoad = new ClickButton({
            z: Utils.Layers.GUI,
            name: 'buttonLoad',
            sfx: 'sfx_button',
            image: Bento.assets.getImage('gui/buttoncloudload'),
            frameCountX: 3,
            frameCountY: 1,
            animations: {
                up: {
                    frames: [0]
                },
                down: {
                    frames: [1]
                },
                inactive: {
                    frames: [2]
                }
            },
            position: new Vector2(viewport.width / 2 + 30, viewport.height / 2 + 10),
            onClick: function () {
                if (!hasCloudSave) {
                    // there's nothing to load
                    Bento.objects.pause();
                    new Dialog({
                        name: 'noCloudSave',
                        canClose: false,
                        type: 'yes',
                        text: Utils.Localization.getText('cloudNoSaveFile'),
                        onClose: function () {
                            Bento.objects.resume();
                        }
                    });
                    return;
                }
                // ask if user really wants to load
                Bento.objects.pause();

                var dialog = new Dialog({
                    name: 'loadFromCloud',
                    type: 'yesno',
                    text: Utils.Localization.getText('loadFromCloud'),
                    onYes: function () {
                        var key;

                        // clear save file on device
                        Bento.saveState.clear();
                        // load save file from cloud
                        for (key in cloudSave) {
                            Bento.saveState.save(key, cloudSave[key]);
                        }

                        Utils.reloadTeam();
                        Utils.reloadCritters();
                        Utils.reloadTwitterCritters();

                        Bento.screens.show('screens/cloudsave', {
                            loaded: true
                        });
                    },
                    onNo: function () {
                        Bento.objects.resume();
                    },
                    onClose: function () { // callback when closed, after pressing yes, no or close

                    }
                });
            }
        });
        var buttonExit = new ClickButton({
            z: Utils.Layers.GUI,
            sfx: 'sfx_button',
            image: Bento.assets.getImage('combiner/buttonexit'),
            frameCountX: 2,
            frameCountY: 1,
            animations: {
                up: {
                    frames: [0]
                },
                down: {
                    frames: [1]
                }
            },
            position: new Vector2(12, 10),
            onClick: function () {
                Bento.screens.show('screens/map');
            }
        }).attach(
            // back button pressed
            new Utils.BackButton(function (data) {
                data.entity.doCallback();
            })
        );
        /**
         * Containers showing save information
         */
        var containerTop = new Entity({
            z: 2,
            name: 'containerTop',
            position: new Vector2(viewport.width / 2, 70),
            originRelative: new Vector2(0.5, 0.5),
            components: [
                new Sprite({
                    imageName: 'gui/grid'
                }),
                new Entity({
                    name: 'phone',
                    position: new Vector2(74 - 24, -32),
                    components: [
                        new Sprite({
                            image: Bento.assets.getImage('gui/phone')
                        })
                    ]
                })
            ]
        });
        var containerBottom = new Entity({
            z: 2,
            name: 'containerBottom',
            position: new Vector2(viewport.width / 2, viewport.height - 50),
            originRelative: new Vector2(0.5, 0.5),
            components: [
                new Sprite({
                    imageName: 'gui/grid'
                }),
                new Entity({
                    name: 'cloud',
                    position: new Vector2(69 - 24, -38),
                    components: [
                        new Sprite({
                            image: Bento.assets.getImage('gui/cloud')
                        })
                    ]
                })
            ]
        });
        var showSave = function (container, saveData) {
            // amount of unique critters
            var collection = saveData['collection'] ? saveData['collection'].length : 0;
            // total amount of unique critters
            var maxCollection = Utils.getKeyLength(Bento.assets.getJson('creatures/creatures')['creatures']);
            var collectionIcon = new Entity({
                name: 'collectionIcon',
                position: new Vector2(-22 + 1, -21),
                originRelative: new Vector2(0.5, 0.5),
                components: [
                    new Sprite({
                        imageName: 'gui/cloudcritters'
                    })
                ]
            });
            var collectionText = new Text({
                position: new Vector2(-10 + 10, -23),
                text: collection + "/" + maxCollection,
                font: 'font',
                fontSize: 16,
                align: 'left',
                textBaseline: 'middle',
                fontColor: '#ffffff',
                pixelStroke: true
            });

            var coins = saveData['myCoins'] || 0;
            var coinsIcon = new Entity({
                name: 'coinsIcon',
                position: new Vector2(-22, 1),
                originRelative: new Vector2(0.5, 0.5),
                components: [
                    new Sprite({
                        imageName: 'gui/cloudcoins'
                    })
                ]
            });
            var coinsText = new Text({
                position: new Vector2(-10 + 10, -2),
                text: '' + coins,
                font: 'font',
                fontSize: 16,
                align: 'left',
                textBaseline: 'middle',
                fontColor: '#ffffff',
                pixelStroke: true
            });

            // amount of planets unlocked
            var planets = Bento.saveState.load('unlockedPlanetNames', ['starti']).length;
            var planetIcon = new Entity({
                name: 'planetIcon',
                position: new Vector2(-22, 23),
                originRelative: new Vector2(0.5, 0.5),
                components: [
                    new Sprite({
                        imageName: 'gui/cloudplanets'
                    })
                ]
            });
            var planetsText = new Text({
                position: new Vector2(-10 + 10, 20),
                text: planets + '/9', // TODO: get amount of planets dynamically, not hardcoded
                font: 'font',
                fontSize: 16,
                align: 'left',
                textBaseline: 'middle',
                fontColor: '#ffffff',
                pixelStroke: true
            });

            container.attach(collectionIcon);
            container.attach(collectionText);
            container.attach(coinsIcon);
            container.attach(coinsText);
            container.attach(planetIcon);
            container.attach(planetsText);
        };
        var getCurrentSave = function () {
            var i;
            var len;
            var storageCopy = {};
            var keys = Bento.saveState.load('_keys');

            for (i = 0, len = keys.length; i < len; ++i) {
                storageCopy[keys[i]] = Bento.saveState.load(keys[i]);
            }

            return storageCopy;
        };
        var saveToCloud = function () {
            var stringifiedSaveFile;
            var compressedSaveFile;

            try {
                stringifiedSaveFile = JSON.stringify(currentSave);
            } catch (e) {
                window.alert('Unable to stringify save file');
                Bento.objects.resume();
                Bento.screens.show('screens/map');
                return;
            }

            try {
                compressedSaveFile = LZString.compressToBase64(stringifiedSaveFile);
            } catch (e) {
                window.alert('Unable to compress save file');
                Bento.objects.resume();
                Bento.screens.show('screens/map');
                return;
            }

            // save data to cloud!
            var snapshot = {
                identifier: 'combocritters',
                title: 'combocritters',
                description: 'combocritters',
                data: compressedSaveFile
            };

            if (snapshot.data) {
                CloudModule.writeSavedGame(
                    snapshot,
                    function () {
                        // success on saving to cloud
                        new Dialog({
                            name: 'cloudSaved',
                            canClose: false,
                            type: 'yes',
                            text: Utils.Localization.getText('cloudSaved'),
                            onClose: function () {
                                Bento.objects.resume();
                                // refresh screen
                                Bento.screens.show('screens/cloudsave', {
                                    saved: true
                                });
                            }
                        });
                    },
                    function (error) {
                        // something went wrong?
                        new Dialog({
                            name: 'cloudSaveError',
                            canClose: false,
                            type: 'yes',
                            text: Utils.Localization.getText('cloudSaveError'),
                            onClose: function () {
                                console.log('onClose');
                                Bento.objects.resume();
                            }
                        });
                    }
                );
            } else {
                // there's nothing to save
                Bento.objects.resume();
            }
        };
        var loadFromCloud = function () {
            // get the savefile
            CloudModule.loadSavedGame(
                'combocritters',
                function (snapshot) {
                    var decompressedSaveFile;

                    // success
                    if (snapshot.data) {
                        try {
                            decompressedSaveFile = LZString.decompressFromBase64(snapshot.data);
                        } catch (e) {
                            window.alert('Savefile corrupted!');
                            return;
                        }

                        try {
                            cloudSave = JSON.parse(decompressedSaveFile);
                        } catch (e) {
                            window.alert('Savefile corrupted!');
                            return;
                        }

                        if (!cloudSave) {
                            console.log('Error parsing compressed data');
                            return;
                        }

                        showSave(containerBottom, cloudSave);
                        hasCloudSave = true;
                    }
                },
                function (error) {
                    // fail
                    console.log('error cloudsave', JSON.stringify(error));
                    // GooglePlay.showSavedGame()
                    // no snapshot available?
                }
            );
        };

        /**
         * Set up
         */
        if (Utils.isNativeAndroid()) {
            CloudModule = Utils.GooglePlay;
            // player should be logged in
            if (!Utils.GooglePlay.isLoggedIn()) {
                new Dialog({
                    name: 'cloudNoLogin',
                    canClose: false,
                    type: 'yes',
                    text: Utils.Localization.getText('cloudNoLogin'),
                    onClose: function () {
                        Utils.GooglePlay.login(
                            function (error) {
                                // wait 30 frames before reloading galaxy screen,
                                // for some reason it doesnt load the background
                                // if there is no delay
                                Utils.timeout(
                                    30,
                                    function () {
                                        // game is still paused from being in menu
                                        Bento.objects.resume();
                                        Bento.screens.show('screens/map');
                                    },
                                    Utils.PauseLevels.EXIT
                                );

                                Utils.checkIfHighSpender();
                            }
                        );
                    }
                });
                // wait for login
                return;
            }
        } else if (Utils.isNativeIos()) {
            CloudModule = Utils.ICloud;
        } else {
            // on web??
            new Dialog({
                canClose: false,
                name: 'cloudNotAvailable',
                type: 'yes',
                text: "Cloud saving not available",
                onClose: function () {
                    // game is still paused from being in menu
                    Bento.objects.resume();
                    // failed to login: back to galaxy screen
                    Bento.screens.show('screens/map');
                }
            });
        }

        loadFromCloud();

        // load current savefile
        currentSave = getCurrentSave();
        showSave(containerTop, currentSave);

        Bento.objects.attach(containerTop);
        Bento.objects.attach(containerBottom);
        Bento.objects.attach(background);
        Bento.objects.attach(buttonSave);
        Bento.objects.attach(buttonLoad);
        Bento.objects.attach(buttonExit);

        new Fade({
            fadeIn: true,
            type: 'fade',
            callback: function () {
                var data = data;
                // show popup after saving/loading
                // game should still be paused, until dialog is dismissed
                if (data) {
                    Bento.objects.pause();
                    new Dialog({
                        name: 'cloudLoaded',
                        canClose: false,
                        type: 'yes',
                        text: data.loaded ? Utils.Localization.getText('cloudLoaded') : Utils.Localization.getText('cloudSaved'),
                        onClose: function () {
                            Bento.objects.resume();
                        }
                    });
                } else {
                    // might be paused from the title screen
                    Bento.objects.resume();
                }
            }
        });
    };
    Utils.extend(object, {
        onShow: onShow
    });
    return object;
});