/**
 * Level select screen
 */
bento.define('screens/map', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/screen',
    'bento/tween',
    'bento/canvas',
    'bento/gui/clickbutton',
    'components/debug',
    'bento/gui/text',
    'entities/camera',
    'entities/battle/caughtpopup',
    'gui/centeredlist',
    'gui/scrollinglist',
    'gui/storydialog',
    'gui/dialog',
    'gui/team/teamedit',
    'entities/card',
    'entities/effect',
    'gui/askrating',
    'gui/dailygift',
    'gui/worldmap/menu',
    'gui/credits/credits',
    'gui/hudcoincounter',
    'gui/fade',
    'gui/pause',
    'gui/shop/buttonrestore',
    'entities/map/planet',
    'subtiled',
    'components/stretchtofill',
    'bento/components/fill'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    Utils,
    Screen,
    Tween,
    Canvas,
    ClickButton,
    Debug,
    Text,
    Camera,
    CaughtPopup,
    CenteredList,
    ScrollingList,
    StoryDialog,
    Dialog,
    TeamEdit,
    Card,
    Effect,
    AskRating,
    DailyGift,
    Menu,
    Credits,
    HudCoinCounter,
    Fade,
    Pause,
    ButtonRestore,
    Planet,
    SubTiled,
    StretchToFill,
    Fill
) {
    'use strict';
    var PauseLevels = Utils.PauseLevels;
    var background;
    var buttonSettings;
    var buttonCombineOuter;
    var buttonEditTeamOuter;
    var creatures = Bento.assets.getJson('creatures/creatures');

    var addHand = function (planet) {
        var select = new Entity({
            name: 'hand',
            position: new Vector2(0, 0),
            originRelative: new Vector2(0.5, 0),
            components: [new Sprite({
                imageName: 'map/hand'
            })]
        }).attach({
            start: function () {
                select.position.y = planet.dimension.height / 2 + 4 * Math.abs(Math.sin(select.timer / 20));
            },
            update: function () {
                select.position.y = planet.dimension.height / 2 + 4 * Math.abs(Math.sin(select.timer / 20));
                // select.scale.y = 1 + 0.01 * Math.sin(select.timer / 20);
            }
        });
        planet.attach(select);
    };
    var addTwitterHand = function (targetPos) {
        var viewport = Bento.getViewport();
        var posY;
        var select = new Entity({
            z: 10,
            name: 'hand',
            position: new Vector2(targetPos.x, 0),
            originRelative: new Vector2(0.5, 0),
            components: [new Sprite({
                imageName: 'map/hand'
            })]
        }).attach({
            update: function () {
                posY = Math.min(targetPos.y - 16, viewport.y + viewport.height);
                select.position.y = posY - 4 * Math.abs(Math.sin(select.timer / 20));
            }
        });
        select.scale.y = -1;
        Bento.objects.attach(select);
    };
    var showTwitterTutorial = function () {
        var dialog;
        // already saw
        if (Bento.saveState.load('twitterTutorial', false) === true) {
            return;
        }
        Bento.objects.pause(1);
        dialog = new Dialog({
            overlayOpacity: 0.8,
            name: 'twitterTutorial',
            text: Utils.Localization.getText("twitterTutorial"),
            type: "yes",
            canClose: true,
            imageWindow: Utils.Localization.isChinese() ? 'map/weibotutorial' : 'map/twittertutorial',
            onClose: function () {
                Bento.objects.resume();
                Bento.saveState.save('twitterTutorial', true);
            }
        });
        dialog.getComponent('buttonYes', function (button) {
            button.position.y += 124;
        });
        dialog.origin.x = 91;
        dialog.origin.y = 40;

        // Bento.objects.get('buttonTwitter', function (buttonTwitter) {
        //     addTwitterHand(buttonTwitter.position);
        // });
    };

    var checkDailyGift = function () {
        // small delay to wait for the fade
        Utils.timeout(30, function () {
            // should give daily gift?
            if (DailyGift.canGive()) {
                new DailyGift({
                    onComplete: function () {
                        // showTwitterTutorial();
                    }
                });
            }
        }, Utils.PauseLevels.ALWAYS);
    };

    var unlockPlanet = function (onComplete) {
        var unlockedPlanets = Bento.saveState.load('unlockedPlanets', 0);
        var planetsToUnlock = Bento.saveState.load('planetsToUnlock', []);

        // remove the hand on starti
        Bento.objects.get('starti', function (starti) {
            starti.getComponent('hand', function (select) {
                starti.remove(select);
            });
        });

        if (planetsToUnlock[0] === 'ending1') {
            if (Bento.saveState.load('showEnding1', false) === false) {
                new StoryDialog({
                    text: Utils.Localization.getText('ending'),
                    type: 'yes',
                    canClose: false,
                    onClose: function () {
                        // NOTE: we do not save over planetsToUnlock,
                        // this can be used when the game is updated with more content
                        // Bento.saveState.save('planetsToUnlock', []);
                        Bento.saveState.save('showEnding1', true);
                    }
                });
            }
            return;
        } else if (planetsToUnlock[0] === 'ending2') {
            if (Bento.saveState.load('showEnding2', false) === false) {
                new StoryDialog({
                    text: Utils.Localization.getText('ending2'),
                    type: 'yes',
                    canClose: false,
                    onClose: function () {
                        // NOTE: we do not save over planetsToUnlock,
                        // this can be used when the game is updated with more content
                        // Bento.saveState.save('planetsToUnlock', []);
                        Bento.saveState.save('showEnding2', true);
                    }
                });
            }
            return;
        }

        // explain what's happening
        new StoryDialog({
            text: Utils.Localization.getText('discovered'),
            type: 'yes',
            canClose: false,
            onClose: function () {
                var unlockedPlanetNames = Bento.saveState.load('unlockedPlanetNames', ['starti']);
                var hasCompleted = false;

                // show an effect on the planet to unlock
                Bento.audio.playSound('sfx_unlock_planet');

                Utils.forEach(planetsToUnlock, function (name) {
                    unlockedPlanetNames.push(name);
                    Bento.objects.get(name, function (planetToUnlock) {
                        planetToUnlock.getComponent('sprite', function (animation) {
                            animation.setAnimation('default');
                        });
                        new Effect({
                            sprite: new Sprite(Bento.assets.getJson('animations/effects/pickup')),
                            position: planetToUnlock.getWorldPosition(),
                            onComplete: function () {
                                if (hasCompleted) {
                                    // make sure not to run onComplete twice
                                    return;
                                }
                                hasCompleted = true;
                                if (onComplete) {
                                    onComplete();
                                }
                            }
                        });
                    });
                });

                // first time: add effect on buttontwitter
                if (planetsToUnlock[0] === 'veggi') {
                    // Bento.objects.get('buttonTwitter', function (buttonTwitter) {
                    //     new Effect({
                    //         sprite: new Sprite(Bento.assets.getJson('animations/effects/pickup')),
                    //         position: buttonTwitter.getWorldPosition(),
                    //         onComplete: function () {}
                    //     });
                    // });
                }

                // done!
                Bento.saveState.save('unlockedPlanetNames', unlockedPlanetNames);
                Bento.saveState.save('planetsToUnlock', []);

            }
        });
    };
    var onShow = function (data) {
        /* Screen starts here */
        var viewport = Bento.getViewport();
        // var camera = new Camera();
        var json = Bento.assets.getJson('maps/galaxy');
        var shipSprite = new Sprite({
            image: Bento.assets.getImage('map/shipicon')
        });
        var buttonTwitter = new ClickButton({
            z: 2,
            name: 'buttonTwitter',
            sfx: 'sfx_button',
            image: Bento.assets.getImage(Utils.Localization.isChinese() ? 'map/planets/weibo' : 'map/planets/twitter'),
            frameCountX: 1,
            frameCountY: 2,
            animations: {
                'down': {
                    frames: [0]
                },
                'up': {
                    frames: [0]
                },
                'inactive': {
                    frames: [1]
                }
            },
            // position: new Vector2(24, viewport.height - 24),
            position: new Vector2(32, (json.height - 2) * 16),
            // float: true,
            updateWhenPaused: 0,
            onClick: function () {
                Bento.screens.show('screens/twitter');
            }
        });
        var twitterText = new Text({
            position: new Vector2(25, 0),
            text: Utils.Localization.getText("twitterPlanet"),
            fontSettings: Utils.FontSettings.get('font'),
            font: 'volter',
            fontSize: 9,
            fontColor: '#ffffff',
            align: 'left',
            textBaseline: 'middle',
            maxWidth: viewport.width - 64
        });
        var comingSoonText = new Text({
            z: 10,
            position: new Vector2(json.width * 16 / 2, -6),
            text: Utils.Localization.getText("comingsoon"),
            fontSettings: Utils.FontSettings.get('font'),
            font: 'volter',
            fontSize: 9,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'bottom',
            maxWidth: viewport.width
        });
        var coinCounter = new HudCoinCounter({});
        var ship = new Entity({
            z: 2,
            name: 'ship',
            position: new Vector2(0, 0),
            originRelative: new Vector2(0.5, 0.5),
            updateWhenPaused: PauseLevels.INFO,
            components: [
                shipSprite
            ]
        }).attach({
            update: function () {
                ship.transform.y = -16 + 4 * Math.sin(ship.timer / 40);
            }
        });
        var onLoad = function () {
            var unlockedPlanets = Bento.saveState.load('unlockedPlanetNames', ['starti']);
            var lastPlanetVisited = Bento.saveState.load('lastPlanetVisited', null);
            if (unlockedPlanets.length === 1) {
                Bento.objects.get('starti', function (starti) {
                    addHand(starti);
                });
            }

            viewport.x = (map.dimension.width - viewport.width) / 2;
            viewport.y = map.dimension.height - viewport.height;

            if (Utils.worldMap.planetData && !Utils.worldMap.planetData.twitterData) {
                // scroll to last planet selected
                Bento.objects.get(Utils.worldMap.planetData.planetName, function (planet) {
                    scroll.forceTo(-planet.position.y + viewport.height / 2);
                    ship.position = planet.position.clone();
                });
            } else if (lastPlanetVisited) {
                // scroll to last planet visited even if game is just booted up
                Bento.objects.get(lastPlanetVisited, function (planet) {
                    scroll.forceTo(-planet.position.y + viewport.height / 2);
                    ship.position = planet.position.clone();
                });
            } else {
                // scroll to last planet unlocked
                Bento.objects.get(unlockedPlanets[unlockedPlanets.length - 1], function (starti) {
                    // camera.centerTo(starti.position);
                    scroll.forceTo(-starti.position.y + viewport.height / 2);
                    ship.position = starti.position.clone();
                });
            }

            // caught boss critter?
            if (data && data.caught) {
                new CaughtPopup({
                    caught: data.caught,
                    onClose: function () {
                        // unlock planet?
                        if (Bento.saveState.load('planetsToUnlock', []).length > 0) {
                            // is set in entities/battle/manager
                            unlockPlanet(checkDailyGift);
                        } else {
                            checkDailyGift();
                        }
                    }
                });
            } else {
                // unlock planet?
                if (Bento.saveState.load('planetsToUnlock', []).length > 0) {
                    unlockPlanet(checkDailyGift);
                } else {
                    checkDailyGift();
                }
            }

            // storydialog intro
            if (Bento.saveState.load('introDialog', false) === false) {
                new StoryDialog({
                    text: Utils.Localization.getText('intro'),
                    onClose: function () {
                        Bento.saveState.save('introDialog', true);
                    }
                });
            }

            // wait 2 extra frames before removing black overlay
            Utils.timeout(2, function () {
                Bento.objects.remove(overlay);
            }, Utils.PauseLevels.ALWAYS);

            // scroll has a high pauselevel to scroll to the right place in the beginning
            Utils.timeout(30, function () {
                scroll.updateWhenPaused = 0;
            }, Utils.PauseLevels.STORY);
        };
        var map = new SubTiled({
            json: json,
            spawn: true,
            rectangle: new Rectangle(0, 0, json.width, json.height),
            onLoad: onLoad
        });
        var overlay = new Entity({
            z: Utils.Layers.FADE,
            name: 'overlay',
            position: new Vector2(viewport.width / 2, viewport.height / 2),
            originRelative: new Vector2(0.5, 0.5),
            components: [
                new Sprite({
                    image: Bento.assets.getImage('fade')
                })
            ]
        });
        var unlockedPlanets = Bento.saveState.load('unlockedPlanets', 0);
        var buttonReset = new ClickButton({
            sfx: 'sfx_button',
            z: Utils.Layers.PAUSE,
            name: 'buttonReset',
            image: Bento.assets.getImage('gui/buttonreset'),
            frameCountX: 1,
            frameCountY: 2,
            updateWhenPaused: Utils.PauseLevels.MENU,
            position: new Vector2(-viewport.width / 2 + 48, viewport.height / 2 - 56),
            onClick: function () {
                var res,
                    removeSaveFile = function () {
                        Utils.reset();
                        Bento.resume();
                        Bento.screens.show('screens/title');
                    };
                if (Utils.isCocoonJS() && Cocoon) {
                    Cocoon.Dialog.confirm({
                        title: "Warning",
                        message: "You are about to delete your savefile! Are you sure?"
                    }, function (accepted) {
                        if (accepted) {
                            removeSaveFile();
                        }
                    });
                } else {
                    res = window.confirm("You are about to delete your savefile! Are you sure?");
                    if (res) {
                        removeSaveFile();
                    }
                }
            }
        });
        var buttonCredits = new ClickButton({
            sfx: 'sfx_button',
            z: Utils.Layers.PAUSE,
            name: 'buttonCredits',
            image: Bento.assets.getImage('gui/buttonluckykat'),
            frameCountX: 2,
            frameCountY: 1,
            updateWhenPaused: Utils.PauseLevels.MENU,
            position: new Vector2(viewport.width / 8 * 3, viewport.height / 2 - 21),
            onClick: function () {
                Bento.objects.attach(new Credits({}));
            }
        });
        var buttonNotif = new ClickButton({
            sfx: 'sfx_button',
            z: Utils.Layers.PAUSE,
            name: 'buttonNotif',
            image: Bento.assets.getImage('gui/buttonnotifications'),
            frameCountX: 2,
            frameCountY: 1,
            updateWhenPaused: Utils.PauseLevels.MENU,
            position: new Vector2(viewport.width / 8 * 3, viewport.height / 2 - 56),
            onClick: function () {
                // Initialize notification system (which will prompt a
                // permission request on iOS) and send a notification
                Utils.Notification.sendNotification(Utils.getNotifications().dailyGift);
            }
        });
        var buttonRestore = new ButtonRestore({
            position: new Vector2(-viewport.width / 8 * 3, viewport.height / 2 - 16)
        });
        var buttonCloud = new ClickButton({
            sfx: 'sfx_button',
            z: Utils.Layers.PAUSE,
            name: 'buttonCloud',
            image: Bento.assets.getImage('gui/buttoncloud'),
            frameCountX: 1,
            frameCountY: 2,
            animations: {
                up: {
                    frames: [0]
                },
                down: {
                    frames: [1]
                },
                inactive: {
                    frames: [0]
                }
            },
            updateWhenPaused: Utils.PauseLevels.MENU,
            position: new Vector2(viewport.width / 8, viewport.height / 2 - 16),
            onClick: function () {
                Bento.screens.show('screens/cloudsave');
            }
        });
        var buttonGameCenter = new ClickButton({
            z: Utils.Layers.PAUSE,
            image: Bento.assets.getImage(Utils.isNativeIos() ? 'gui/buttongamecenter' : 'gui/buttongoogleplay'),
            position: new Vector2(-viewport.width / 8, viewport.height / 2 - 16),
            // position: new Vector2(viewport.width / 2 - 30, viewport.height / 2 - 32 - 35),
            frameCountX: 1,
            frameCountY: 2,
            updateWhenPaused: Utils.PauseLevels.MENU,
            sfx: 'sfx_button',
            onClick: function () {
                // clicked
                if (Utils.isNativeIos()) {
                    Utils.GameCenter.showAchievement(function () {});
                } else {
                    Utils.GooglePlay.showAchievement(function () {});
                }
            }
        });
        var scroll = new ScrollingList({
            spacing: map.dimension.height - viewport.height
        });
        // scroll.updateWhenPaused = Utils.PauseLevels.ALWAYS;
        Bento.objects.attach(scroll);
        scroll.addItem(new Entity({
            name: 'dummyObj'
        }));
        scroll.addItem(new Entity({
            name: 'dummyObj'
        }));
        scroll.attach({
            update: function () {
                viewport.y = -scroll.getScroll();
            }
        });
        scroll.updateWhenPaused = Utils.PauseLevels.STORY;
        screen.dimension.width = map.dimension.width;
        screen.dimension.height = map.dimension.height;

        buttonSettings = new Menu({
            makeTeamCritters: function () {},
            onClose: function () {
                Utils.Analytics.trackView('galaxy');
            },
            newCritter: (data && data.caught) ? data.caught.newCritter : false
        });
        buttonSettings.removeComponent('buttonPlanet');
        buttonSettings.removeComponent('teamPanel');
        if (Utils.BETA_TEST) {
            buttonSettings.attachComponent(buttonReset);
        }
        buttonSettings.attachComponent(buttonCredits);
        // only add cloud save button on Android
        if (Utils.isNativeAndroid() /*|| Utils.isNativeIos()*/ ) {
            buttonSettings.attachComponent(buttonCloud);
        }
        // restore purchases/achievemnts only on iOS and Android
        if (Utils.isNativeAndroid() || Utils.isNativeIos()) {
            buttonSettings.attachComponent(buttonRestore);
            buttonSettings.attachComponent(buttonGameCenter);
        }
        // show a button to ask about notifications if player hasn't yet
        // clicked on 'yes' when asked about notifications, but was asked
        if (!Utils.isFB() &&
            Utils.isCocoonJs() &&
            !Bento.saveState.load('notificationsAllowed', false) &&
            Bento.saveState.load('notificationsAsked', false)
        ) {
            buttonSettings.attachComponent(buttonNotif);
        }

        // show twitter after 1st boss
        if (Utils.hasDefeatedBoss('octoking')) {
            buttonTwitter.attach(twitterText);
        } else {
            buttonTwitter.setActive(false);
        }

        background = new Entity({
            z: -1,
            name: 'background',
            position: new Vector2(viewport.width / 2, viewport.height / 2),
            originRelative: new Vector2(0.5, 0.5),
            float: true,
            components: [
                new StretchToFill({
                    retainRatio: true
                }),
                new Sprite({
                    imageName: 'map/empty'
                })
            ]
        });

        new Fade({
            fadeIn: true,
            callback: function () {}
        });

        // clear cache before going into a planet!
        EventSystem.fire('clearTiledCache');

        Bento.objects.attach(background);
        // Bento.objects.attach(buttonTwitter);
        Bento.objects.attach(coinCounter);
        Bento.objects.attach(buttonSettings);
        Bento.objects.attach(overlay);
        Bento.objects.attach(ship);
        // Bento.objects.attach(comingSoonText);
        // Bento.objects.attach(camera);
        // addPlanets();

        if (!Bento.audio.isPlayingMusic('bgm_title')) {
            Bento.audio.stopAllMusic();
            Bento.audio.playMusic('bgm_title');
            Bento.audio.setVolume(1, 'bgm_title');
        }
        Utils.Analytics.trackView('galaxy');

        // reset the planet playtime tracker and current world
        Bento.saveState.save('prevTimeSample', null);
        Bento.saveState.save('currentPlanet', null);

        // reset tokens picked up on planet
        Utils.tokens.reset();

        // test with 3d touch
        /*EventSystem.on('3Dtouch', function (result) {
            // touched
            var ball = new Entity({
                z: 1000,
                name: 'ball',
                position: result.worldPosition.clone(),
                originRelative: new Vector2(0.5, 0.5),
                components: [new Sprite({
                    imageName: 'minigames/catch/ball'
                })]
            });
            new Tween({
                from: 0,
                to: 1,
                'in': 60,
                ease: 'linear',
                alpha: 0,
                beta: 0,
                'do': function (v, t) {
                    ball.scale.x = v * 2;
                    ball.scale.y = v * 2;
                    ball.alpha = 1 - v;
                },
                onComplete: function () {
                    Bento.objects.remove(ball);
                }
            });
            Bento.objects.attach(ball);
        });*/
    };
    var screen = new Screen({
        onShow: onShow
    });

    return screen;
});