/**
 * Lucky Kat splash screen intro
 */
bento.define('screens/splash', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/screen',
    'bento/tween',
    'gui/combiner/splash',
    'modules/playtimer',
    'gui/fade'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    Utils,
    Screen,
    Tween,
    Splash,
    PlayTimer,
    Fade
) {
    'use strict';
    var onShow = function () {
        /* Screen starts here */
        var isFading = false;
        var fadeOut = function () {
            if (isFading) {
                return;
            }
            isFading = true;
            new Fade({
                callback: function () {
                    Bento.objects.resume();
                    Bento.screens.show('screens/title');
                    // stop any sound if still playing
                    Bento.audio.stopSound('sfx_combine_start');
                    Bento.audio.stopSound('sfx_combine_bump');
                    Bento.audio.stopSound('sfx_combine');
                }
            });
        };
        var combine = new Splash({
            critter1: 'splash/cat',
            critter2: 'splash/phone',
            result: 'splash/logo',
            onComplete: fadeOut
        });
        var changeLoadingCritter = function () {
            var critters = Bento.assets.getJson('creatures/creatures')['creatures'];
            var choice = Utils.getRandom(Utils.getKeyLength(critters));
            var critterId = critters[Object.keys(critters)[choice]]['id'];
            var sprite = new Sprite({
                imageName: 'creatures/' + critterId
            });
            var critterEntity = new Entity({
                z: 0,
                position: new Vector2(0, 0),
                updateWhenPaused: 0,
                components: [
                    sprite
                ]
            });
            var canvas = document.createElement('canvas');
            var context = canvas.getContext('2d');
            var renderer = Bento.getRenderer();

            canvas.width = sprite.spriteImage.width;
            canvas.height = sprite.spriteImage.height;

            if (context.imageSmoothingEnabled)
                context.imageSmoothingEnabled = false;
            if (context.webkitImageSmoothingEnabled)
                context.webkitImageSmoothingEnabled = false;
            if (context.mozImageSmoothingEnabled)
                context.mozImageSmoothingEnabled = false;

            // swap context
            renderer.setContext(context);
            context.save();

            // create white background for cocoon
            context.fillStyle = "#fff";
            context.fillRect(0, 0, canvas.width, canvas.height);

            // draw critter on context
            critterEntity.draw();

            // restore
            context.restore();
            renderer.restoreContext();

            // save new loading critter for next bootup
            Bento.saveState.save('loadingCritterBase64', canvas.toDataURL());
        };

        // user clicks the splash away
        combine.attach(new Clickable({
            onClick: function () {
                fadeOut();
            }
        }));

        // set new loading critter for next bootup
        changeLoadingCritter();

        // reset to prevent time being counted for last planet
        // (would happen if game was quit while on a planet)
        Bento.saveState.save('currentPlanet', null);
        Bento.saveState.save('prevTimeSample', null);

        EventSystem.on('screenHidden', function () {
            var i;
            var currentPlanet = Bento.saveState.load('currentPlanet', null);
            // value of PlayTimer.playTimer when play time was checked last time
            // is reset when going back to the galaxy screen
            var prevTimeSample;
            // current value of PlayTimer.playTimer
            var timeSample;
            // time played since last time sampling
            // equals the difference between timeSample and prevTimeSample
            var timePlayed;
            var totalTimePlayedInWorld;
            // which play time division was sent last time (e.g. played the planet for 2-3 minutes)
            var lastSentTimeThreshold;
            // play time division currently (e.g. played the planet for 3-4 minutes)
            var currentTimeThreshold;

            // stop logging time in world after boss is defeated
            if (Bento.saveState.load('' + currentPlanet + 'BossDefeated', false)) {
                return;
            }

            prevTimeSample = Bento.saveState.load('prevTimeSample', null);

            if (prevTimeSample && currentPlanet) {
                // get the current time
                timeSample = PlayTimer.playTimer;
                // get how much time has user been playing
                timePlayed = timeSample - prevTimeSample;
                totalTimePlayedInWorld = Bento.saveState.load('totalTimePlayedInWorld_' + currentPlanet, 0);
                lastSentTimeThreshold = Bento.saveState.load('lastSentTimeThreshold_' + currentPlanet, -1);
                currentTimeThreshold = Math.floor((timePlayed + totalTimePlayedInWorld) / 60 / 60);

                // if we've been in a screen long enough to pass more than 1 threshold,
                // make sure to also send the events before the current threshold
                for (i = currentTimeThreshold; i > lastSentTimeThreshold; --i) {
                    Utils.Analytics.event('Planets', '' + currentPlanet + 'PlydFr_' + i);
                    Utils.Firebase.logEvent(currentPlanet + '_played_for', {
                        minutes: i
                    });
                }
                // save which threshold we're at for this planet
                Bento.saveState.save('lastSentTimeThreshold_' + currentPlanet, currentTimeThreshold);
                // add the amount of time we've been in the world
                Bento.saveState.save('totalTimePlayedInWorld_' + currentPlanet, totalTimePlayedInWorld + timePlayed);
                // save the current time sample
                Bento.saveState.save('prevTimeSample', timeSample);
            }
        });
    };

    return new Screen({
        onShow: onShow
    });
});