/**
 * Screen description
 */
bento.define('screens/start', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/screen',
    'bento/tween',
    'bento/gui/clickbutton',
    'bento/gui/text',
    'gui/fade',
    'gui/storydialog',
    'entities/battle/critter',
    'entities/card'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    Utils,
    Screen,
    Tween,
    ClickButton,
    Text,
    Fade,
    StoryDialog,
    Critter,
    Card
) {
    'use strict';
    var askCritter = function (settings) {
        var viewport = Bento.getViewport();
        var background = new Entity({
            z: 0,
            name: 'background',
            position: new Vector2(viewport.width / 2, viewport.height / 2),
            originRelative: new Vector2(0.5, 0.5),
            components: [
                new Sprite({
                    imageName: 'team/background'
                })
            ]
        });
        var text = new Text({
            z: 1,
            position: new Vector2(viewport.width / 2, viewport.height / 4 - 16),
            text: Utils.Localization.getText('choose'),
            font: 'volter',
            fontSize: 9,
            fontSettings: Utils.FontSettings.get('font'),
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'bottom'
        });
        var infoPanel = new Entity({
            z: 1,
            name: 'infoPanel',
            position: new Vector2(viewport.width / 2 + 2, viewport.height / 2),
            originRelative: new Vector2(0.5, 0.5),
            components: [new Sprite({
                imageName: 'start/choosewindow'
            })]
        });
        var infoPanelBorder = new Entity({
            z: 100,
            name: 'infoPanelBorder',
            position: new Vector2(viewport.width / 2 + 2, viewport.height / 2),
            originRelative: new Vector2(0.5, 0.5),
            components: [new Sprite({
                imageName: 'start/border'
            })]
        });
        var infoName = new Text({
            position: new Vector2(0, 16 - 2),
            text: '',
            font: 'volter',
            fontSize: 9,
            fontSettings: Utils.FontSettings.get('font'),
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'bottom',
            maxWidth: 96,
            linebreaks: false
        });
        var infoDescription = new Text({
            position: new Vector2(0, -9 + 32 - 4),
            text: Utils.Localization.getText('tapToStop'),
            font: 'volter',
            fontSize: 9,
            fontSettings: Utils.FontSettings.get('font'),
            fontColor: '#ffffff',
            align: 'center',
            maxWidth: 145,
            maxHeight: 32,
            textBaseline: 'top'
        });
        var infoSeperator = new Entity({
            name: 'seperator',
            position: new Vector2(-2, -12 + 32 - 3),
            originRelative: new Vector2(0.5, 0.5),
            components: [new Sprite({
                imageName: 'map/stripe'
            })]
        });
        var buttonChoose = new ClickButton({
            sfx: 'sfx_button',
            image: Bento.assets.getImage('map/buttonattack'),
            frameCountX: 1,
            frameCountY: 2,
            animations: {
                up: {
                    frames: [0]
                },
                down: {
                    frames: [1]
                }
            },
            position: new Vector2(viewport.width / 2, viewport.height * 3 / 4 + 24),
            onClick: function () {
                var team = Utils.getTeam();
                team[0] = selected.getProperties();
                Utils.addToCollection(selected.getProperty('id'));
                Utils.saveTeam();
                Bento.objects.pause();
                new Fade({
                    callback: function () {
                        Bento.objects.resume();
                        Bento.screens.show('screens/map');
                    }
                });
            }
        });
        var selected = null;
        var makeCritter = function (prop) {
            var critter = new Critter(prop);
            var opacity;
            var clicked = false;
            var slots = {
                name: 'slots',
                update: function () {
                    var dist = Math.abs(critter.position.x + 8) / 64;
                    if (!clicked) {
                        critter.position.x += 10;
                        if (critter.position.x > viewport.width / 2 + 64) {
                            critter.position.x -= 64 * critters.length;
                        }
                    }

                    critter.scale.x = 1.5 - Utils.clamp(0.5, dist, 1);
                    critter.scale.y = critter.scale.x;

                    // highlight center
                    // if (Math.abs(critter.position.x) < 64) {
                    //     opacity.setOpacity(1);
                    // } else {
                    //     opacity.setOpacity(0.5);
                    // }
                    critter.alpha = (1.5 - Utils.clamp(0.5, dist, 1));

                }
            };
            // critter.boundingBox = new Rectangle(-24, -48, 48, 48);
            critter.position.x = 64 * critters.length;
            infoPanel.attach(critter);
            critter.attach(new Clickable({
                pointerDown: function () {
                    // stop rotating
                    // critter.remove(slots);
                    if (clicked) {
                        return;
                    }
                    clicked = true;
                    Bento.audio.stopSound('sfx_button');
                    Bento.audio.playSound('sfx_button');

                    // stop moving
                    new Tween({
                        from: critter.position.x,
                        to: Math.floor(critter.position.x / 64) * 64,
                        'in': 50,
                        ease: 'elastic',
                        alpha: -8,
                        beta: 2,
                        'do': function (v, t) {
                            critter.position.x = v;
                        },
                        onComplete: function () {
                            // selected critter is middle
                            if (Math.round(critter.position.x / 64) === 0) {
                                critter.alpha = (1);
                                critter.scale.x = 1;
                                critter.scale.y = 1;
                                selected = critter;
                                // Bento.objects.attach(buttonChoose);
                                infoDescription.setText(Utils.Localization.getName(critter.getProperty('id')));
                                Bento.audio.playSound('sfx_new_critter');

                                Utils.timeout(60, function () {
                                    showTutorial(critter, prop);
                                });
                            }
                        }
                    });

                },
                onClick: function () {
                    /*var i;
                    for (i = 0; i < critters.length; ++i) {
                        critters[i].disable();
                    }
                    Bento.audio.playSound('sfx_button');
                    critter.enable();
                    selected = critter;
                    // set info
                    infoName.setText(critter.getProperty('name'));
                    infoDescription.setText(Bento.assets.getJson('creatures/descriptions/' + 'en')[critter.getProperty('id')] || '');

                    // add button
                    if (!buttonChoose.isAdded) {
                        infoPanel.attach(infoSeperator);
                        Bento.objects.attach(buttonChoose);
                    }*/
                }
            }));

            // wrap
            critter.attach(slots);

            return critter;
        };
        var critterList = ['squid', 'baanu', 'sucker', 'shelly', 'stabbycarrot'];
        var critters = [];
        var critterProps = [];
        var critterNames = [];
        var makeInfoPanel = function () {
            var creatures = Bento.assets.getJson('creatures/creatures');

            Utils.forEach(critterList, function (id, i, l, breakLoop) {
                var properties = Utils.clone(creatures.creatures[id]);
                var critter = makeCritter(properties);
                critterNames.push(properties.name);
                critters.push(critter);
                critterProps.push(properties);
            });

            infoPanel.attach(infoDescription);
        };
        // start the tutorial explaining how cards work
        var showTutorial = function (critter, properties) {
            // turn critter into a card
            var card = new Card(properties);
            var tweenOutCritter = function () {
                new Tween({
                    from: 1,
                    to: 0,
                    'in': 30,
                    ease: 'linear',
                    'do': function (v, t) {
                        critter.scale.x = v;
                    },
                    onComplete: tweenInCard
                });
            };
            var tweenInCard = function () {
                card.position = critter.getWorldPosition().add(new Vector2(0, -20));
                card.z = Utils.Layers.FADE;
                card.getComponent('attack', function (counter) {
                    counter.origin.x = 10;
                    counter.origin.y = 10;
                    counter.scale.x = 0;
                    counter.scale.y = 0;
                });
                card.getComponent('currentHp', function (counter) {
                    counter.scale.x = 0;
                    counter.scale.y = 0;
                });
                // card.getComponent('hpBar', function (bar) {
                //     card.remove(bar);
                // });
                Bento.objects.attach(card);
                critter.visible = false;
                new Tween({
                    from: 0,
                    to: 1,
                    'in': 60,
                    ease: 'cuberoot',
                    'do': function (v) {
                        var oscillations = 2;
                        var t = Math.PI * 2 * oscillations + Math.PI * 0.5;
                        card.scale.x = Math.sin(t * v);
                    },
                    onComplete: function () {
                        card.scale.x = 1;
                        Utils.timeout(60, function () {
                            showText(card);
                            // zoom in to card
                            // new Tween({
                            //     from: 1,
                            //     to: 2,
                            //     'in': 60,
                            //     ease: 'elastic',
                            //     alpha: -5,
                            //     beta: 3,
                            //     'do': function (v, t) {
                            //         card.scale.x = v;
                            //         card.scale.y = v;
                            //     }
                            // });
                        });

                        // highlight attack and hp
                        card.getComponent('attack', function (counter) {
                            new Tween({
                                from: 0,
                                to: 1,
                                'in': 30,
                                ease: 'easeOutBack',
                                'do': function (v, t) {
                                    counter.scale.x = v;
                                    counter.scale.y = v;
                                }
                            });
                        });
                        card.getComponent('currentHp', function (counter) {
                            new Tween({
                                from: 0,
                                to: 1,
                                'in': 30,
                                ease: 'easeOutBack',
                                'do': function (v, t) {
                                    counter.scale.x = v;
                                    counter.scale.y = v;
                                }
                            });
                        });

                        // with arrows and text
                        Utils.timeout(30, function () {
                            showArrows(card);
                        });
                    }
                });

                // fade out the panel?
                new Tween({
                    from: 1,
                    to: 0,
                    'in': 60,
                    ease: 'linear',
                    'do': function (v, t) {
                        infoPanel.alpha = v;
                        infoPanelBorder.alpha = v;
                    }
                });
            };
            var showArrows = function (card) {
                var arrowRight = new Entity({
                    name: 'arrowRight',
                    position: new Vector2(-34, 20),
                    originRelative: new Vector2(0.5, 0.5),
                    components: [new Sprite({
                        imageName: 'gui/arrowright'
                    })]
                });
                var textHp = new Text({
                    position: new Vector2(0, 10),
                    text: Utils.Localization.getText('health'),
                    font: 'volter',
                    fontSize: 9,
                    fontSettings: Utils.FontSettings.get('font'),
                    fontColor: '#ffffff',
                    align: 'center',
                    textBaseline: 'top'
                });
                var arrowLeft = new Entity({
                    name: 'arrowLeft',
                    position: new Vector2(33, 20),
                    originRelative: new Vector2(0.5, 0.5),
                    components: [new Sprite({
                        imageName: 'gui/arrowleft'
                    })]
                });
                var textAttack = new Text({
                    position: new Vector2(0, 10),
                    text: Utils.Localization.getText('attack'),
                    font: 'volter',
                    fontSize: 9,
                    fontSettings: Utils.FontSettings.get('font'),
                    fontColor: '#ffffff',
                    align: 'center',
                    textBaseline: 'top'
                });
                arrowRight.attach(textHp);
                card.attach(arrowRight);
                arrowLeft.attach(textAttack);
                card.attach(arrowLeft);
            };
            var showText = function (card) {
                new StoryDialog({
                    text: Utils.Localization.getText('firstCard')
                        .replace("{HEALTH}", properties.hp)
                        .replace("{ATTACK}", properties.attack),
                    hideCat: true,
                    onClose: function () {
                        buttonChoose.doCallback();
                    }
                });
                Utils.Firebase.logEvent(Utils.Firebase.Events.TUTORIAL_BEGIN);
            };

            tweenOutCritter();
        };

        makeInfoPanel();

        Bento.objects.attach(background);
        Bento.objects.attach(text);
        Bento.objects.attach(infoPanel);
        Bento.objects.attach(infoPanelBorder);
    };
    var onShow = function () {
        /* Screen starts here */
        var viewport = Bento.getViewport();

        askCritter();

    };

    return new Screen({
        onShow: onShow
    });
});