bento.define('screens/title', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/components/fill',
    'bento/entity',
    'bento/utils',
    'bento/tween',
    'bento/screen',
    'bento/eventsystem',
    'gui/fade',
    'entities/crosspromo',
    'bento/gui/text',
    'bento/packedimage',
    'xhr',
    'modules/share/combine',
    'modules/forcedadtimer',
    'modules/videoadtimer',
    'components/stretchtofill'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Fill,
    Entity,
    Utils,
    Tween,
    Screen,
    EventSystem,
    Fade,
    StaticCrossPromo,
    Text,
    PackedImage,
    Xhr,
    ShareCombine,
    ForcedAdTimer,
    VideoAdTimer,
    StretchToFill
) {
    'use strict';
    var object = new Screen({
        dimension: Bento.getViewport()
    });
    Utils.extend(object, {
        onShow: function () {
            var viewport = Bento.getViewport();
            var screenMid = new Vector2(viewport.width / 2, viewport.height / 2);
            var clicked = 0;
            var pointerDown = false;
            var fading = false;
            var ttsYPos = viewport.height * 0.7 + 32;
            var overlay = new Entity({
                z: Utils.Layers.FADE,
                name: 'overlay',
                position: screenMid.clone(),
                alpha: 0.75,
                originRelative: new Vector2(0.5, 0.5),
                components: [new Sprite({
                    imageName: 'fade'
                })]
            });
            var touchToStart = new Text({
                z: Utils.Layers.BACKGROUND + 10,
                position: new Vector2(screenMid.x, ttsYPos),
                float: true,
                text: Utils.Localization.getText('touchToStart'),
                // font: 'font',
                fontSettings: Utils.FontSettings.get('font'),
                fontSize: 16,
                align: 'center',
                textBaseline: 'middle',
                fontColor: '#ffffff',
                strokeStyle: '#000000',
                pixelStroke: true,
                lineWidth: 1
            }).attach({
                update: function (data) {
                    data.entity.position.y = ttsYPos + 2 * Math.sin(data.entity.timer / 40);
                }
            });
            /**
             * Returns title image with correct language
             */
            var getTitleImage = function () {
                var code = Utils.Localization.getLanguage();

                // only titles in these languages
                if (code === 'zh' ||
                    code === 'zt' ||
                    code === 'ja' ||
                    code === 'ko') {
                    return code;
                } else {
                    return 'en';
                }
            };
            var title = new Entity({
                z: Utils.Layers.BACKGROUND,
                name: 'title',
                position: screenMid.clone(),
                originRelative: new Vector2(0.5, 0.5),
                components: [
                    new StretchToFill({
                        retainRatio: true
                    }),
                    new Sprite({
                        imageName: 'title/background'
                    }),
                    new Clickable({
                        onClick: function () {
                            if (fading)
                                return;
                            pointerDown = true;
                            Bento.audio.playSound('sfx_touch_to_start');
                        },
                        pointerUp: function () {
                            if (fading)
                                return;

                            if (Bento.objects.isPaused()) {
                                return;
                            }

                            pointerDown = false;
                            if (clicked < 240 && !StaticCrossPromo.isShowing())
                                startGame();
                        }
                    }), {
                        update: function () {
                            if (pointerDown)
                                clicked++;
                            else
                                clicked = 0;

                            // reset game after holding the title screen for 4 seconds
                            if (clicked === 240) {
                                ask();
                            }
                        }
                    }
                ]
            });
            var logo = new Entity({
                z: 1,
                name: 'logo',
                position: new Vector2(0, -title.dimension.height * 0.18),
                originRelative: new Vector2(0.5, 0.5),
                components: [
                    new Sprite({
                        image: Bento.assets.getImage('title/title/' + getTitleImage())
                    })
                ]
            });
            var startGame = function () {
                fading = true;

                new Fade({
                    callback: function () {
                        var team = Utils.getTeam();
                        // start?
                        if (!team[0] && !team[1] && !team[2]) {
                            Bento.screens.show('screens/start');
                        } else {
                            Bento.screens.show('screens/map');
                        }
                    },
                    type: 'drop'
                });
            };
            var ask = function () {
                var res;
                if (Utils.isCocoonJS() && Cocoon) {
                    Cocoon.Dialog.prompt({
                        title: 'Hello',
                        message: 'What would you like?'
                    }, {
                        success: function (text) {
                            cheat(text);
                        }
                    });
                } else {
                    res = window.prompt("What would you like?");
                    cheat(res);
                }
            };
            var cheat = function (text) {
                // if (Utils.isCocoonJS()) {
                //     // disable cheats
                //     return;
                // }
                if (text === 'reset') {
                    resetGame();
                }
                if (text === 'heyzapmediation') {
                    window.HeyzapAds.showMediationTestSuite();
                }
                if (text === 'forcedAds') {
                    ForcedAdTimer.setMinutesUntilReset(0.1);
                    ForcedAdTimer.force();
                } else if (text === 'coinAds') {
                    VideoAdTimer.setMinutesUntilReset(0.1);
                    VideoAdTimer.force();
                }
                if (!Bento.isDev()) {
                    return;
                }
                if (text === 'unlockAllCritters') {
                    unlockAllCritters();
                } else if (text === '10ofeach') {
                    unlockAllCritters(10);
                } else if (text === 'unlockPlanets') {
                    Bento.saveState.save('sawTutorial', true);
                    Bento.saveState.save('sawTutorial2', true);
                    Bento.saveState.save('firstBossBattle', true);
                    Bento.saveState.save('unlockedPlanetNames', ['starti', 'veggi', 'zaspiri', 'dark', 'hotmountain', 'candy', 'pirate', 'lavalli', 'mechastation']);
                } else if (text === 'unlockPlanet') {
                    var unlockedPlanetNames = Bento.saveState.load('unlockedPlanetNames', ['starti']);
                    var res;
                    if (Utils.isCocoonJS() && Cocoon) {
                        Cocoon.Dialog.prompt({
                            title: 'Which planet?',
                            message: 'Which planet?'
                        }, {
                            success: function (t) {
                                unlockedPlanetNames.push(t);
                                Bento.saveState.save('unlockedPlanetNames', unlockedPlanetNames);
                            }
                        });
                    } else {
                        res = window.prompt("Which planet?");
                        if (res) {
                            unlockedPlanetNames.push(res);
                            Bento.saveState.save('unlockedPlanetNames', unlockedPlanetNames);
                        }
                    }
                } else if (text === 'makeMeRich') {
                    Bento.saveState.save('myCoins', Bento.saveState.load('myCoins', 0) + 1000);
                    EventSystem.fire('updateCoins');
                } else if (text === 'easyGames') {
                    window.easyMinigames = true;
                } else if (text === 'unlockTiers') {
                    Bento.saveState.save('unlockedRarity', 100);
                } else if (text === 'hurryUp') {
                    Bento.setGameSpeed(3);
                } else if (text === 'language') {
                    promptDialog('Enter language code', function (language) {
                        Utils.Localization.setLanguage(language);
                    });
                }
            };
            var promptDialog = function (title, callback) {
                var res;
                if (Utils.isCocoonJS() && Cocoon) {
                    Cocoon.Dialog.prompt({
                        title: 'Input',
                        message: title
                    }, {
                        success: callback
                    });
                } else {
                    res = window.prompt(title);
                    callback(res);
                }
            };
            var unlockAllCritters = function (amount) {
                var critterList = Bento.assets.getJson('creatures/creaturelist').creatures;
                var critterProperties = Bento.assets.getJson('creatures/creatures').creatures;
                var myCritters = Utils.getCritters();
                amount = amount || 1;
                Utils.forEach(critterList, function (id, i, l, breakLoop) {
                    var critter = Utils.clone(critterProperties[id]);
                    Utils.repeat(amount, function () {
                        myCritters.push(critter);
                    });
                });
                Utils.saveCritters();

            };
            var resetGame = function () {
                localStorage.clear();
                Bento.saveState.clear();
                Bento.saveState.save('lastVersion', Utils.version);
                // send a signal
                EventSystem.fire('reset');
                Bento.resume();
                Bento.audio.stopAllMusic();
                Bento.screens.show('screens/title');
                // Bento.audio.playSound('sfx_gen_save');
            };
            var text = new Text({
                z: Utils.Layers.BACKGROUND + 10,
                position: new Vector2(2, viewport.height - 2),
                text: 'v' + window.VERSION + ' BETA\n\nNote: Your save file may be erased on the next version.',
                font: 'volter',
                fontSize: 4,
                fontColor: '#ffffff',
                align: 'left',
                textBaseline: 'bottom',
                lineWidth: 0,
                maxWidth: viewport.width,
                pixelStroke: true // workaround for Cocoon bug
            });
            var showCrossPromo = function () {
                var openedTitleScreen = Bento.saveState.load('openedTitleScreen', 0);

                // don't show crosspromo the first time!!
                if (openedTitleScreen > 0 && Bento.saveState.load('removeAds', false) === false) {
                    // static image as crosspromo
                    new StaticCrossPromo({
                        z: 20,
                        pauseLevel: 1,
                        sfx: 'sfx_button'
                    });
                }

                Bento.saveState.save('openedTitleScreen', openedTitleScreen + 1);

            };
            var test = function () {
                var name = 'luckykatstudios';
                var xhr = new Xhr({
                    // get: 'https://twitter.com/' + name + '/profile_image?size=original',
                    // get: 'https://api.weibo.com/2/users/show.json?source=31641035&screen_name=幸运猫工作室',
                    get: 'https://api.weibo.com/2/users/show.json?source=783190658&screen_name=幸运猫工作室',
                    responseType: 'xhr',
                    onSuccess: function (res) {
                        var doc = res.responseText;
                        console.log(doc);
                        // return;
                        var start = '<script type="application/json" id="init-data">';
                        var end = '</script>';
                        var startIndex = -1;
                        var endIndex = -1;
                        var jsonString;
                        var json;

                        // get the json out!
                        startIndex = doc.indexOf(start);

                        if (startIndex >= 0) {
                            endIndex = doc.indexOf(end, startIndex);
                            if (endIndex >= 0) {
                                startIndex += start.length;
                                jsonString = doc.substr(startIndex, endIndex - startIndex);
                                try {
                                    json = JSON.parse(jsonString);
                                } catch (e) {
                                    console.log('Parsing data failed');
                                }
                            } else {
                                console.log('could not find end');
                            }
                        } else {
                            console.log('could not find start');
                        }

                        return;
                    },
                    onFail: function (res) {
                        console.log(JSON.stringify(res));
                    }
                });
            };

            // fit logo to screen
            // logo.scale.x = viewport.width * 0.8 / logo.dimension.width;
            // logo.scale.y = logo.scale.x;

            // fit background to screen
            // fit to height
            title.scale.y = viewport.height / title.dimension.height;
            title.scale.x = title.scale.y;

            if (Utils.isWideScreen) {
                logo.scale.x = 0.75;
                logo.scale.y = 0.75;
            } else {
                logo.scale.x = 0.8;
                logo.scale.y = 0.8;                
            }

            Bento.objects.attach(title);
            title.attach(logo);
            Bento.objects.attach(touchToStart);
            if (Utils.BETA_TEST) {
                Bento.objects.attach(text);
            }

            Bento.audio.playMusic('bgm_title', true, null, true);

            // TEMP
            window.cheat = cheat;

            new Fade({
                fadeIn: true,
                type: 'drop'
            });

            // test();

            Utils.Analytics.trackView('title');

            if (!Utils.isFB() && !Utils.WEB && !Utils.East2West) {
                showCrossPromo();
            }
        }
    });
    return object;
});