bento.define('screens/twitter', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/components/fill',
    'bento/entity',
    'bento/utils',
    'bento/tween',
    'bento/screen',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'gui/askrating',
    'gui/fade',
    'gui/twitter/twitter',
    'gui/twitter/caught',
    'entities/card',
    'modules/photo',
    'bento/gui/text',
    'bento/packedimage',
    'xhr'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Fill,
    Entity,
    Utils,
    Tween,
    Screen,
    EventSystem,
    ClickButton,
    AskRating,
    Fade,
    Twitter,
    Caught,
    Card,
    Photo,
    Text,
    PackedImage,
    Xhr
) {
    'use strict';
    var onShow = function (data) {
        var viewport = Bento.getViewport();
        var mode = Utils.Localization.isChinese() ? 'weibo' : 'twitter';
        var timeToFirstVisit;
        var amountOfTwitterVisits;
        var text = new Text({
            z: Utils.Layers.TEAMEDIT + 2,
            position: new Vector2(viewport.width / 2, viewport.height / 4),
            text: '',
            font: 'volter',
            fontSize: 9,
            fontSettings: Utils.FontSettings.get('font'),
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'bottom',
            maxWidth: viewport.width
        });
        var currentHandle = '';
        var ask = function () {
            var res;
            var message = Utils.Localization.getText('twitterPopup');
            Bento.objects.pause(Utils.PauseLevels.TWITTER);
            Bento.objects.attach(overlay);
            if (Utils.isCocoonJS() && Cocoon) {
                Cocoon.Dialog.prompt({
                    title: '',
                    message: message
                }, {
                    success: function (text) {
                        if (text) {
                            if (mode === 'twitter') {
                                loadTwitter(text);
                            } else if (mode === 'weibo') {
                                loadWeibo(text);
                            }
                        } else {
                            loadFail();
                        }
                    },
                    cancel: function () {
                        loadFail();
                    }
                });
            } else {
                res = window.prompt(message);
                if (res) {
                    if (mode === 'twitter') {
                        loadTwitter(res);
                    } else if (mode === 'weibo') {
                        loadWeibo(res);
                    }
                } else {
                    loadFail();
                }
            }
        };
        var loadWeibo = function (screenName) {
            if (!navigator.onLine) {
                loadFail('twitterOffline');
                return;
            }

            var xhr = new Xhr({
                get: 'https://api.weibo.com/2/users/show.json?source=783190658&screen_name=' + screenName,
                responseType: 'xhr',
                onSuccess: function (res) {
                    var json;
                    var user = {};
                    try {
                        json = JSON.parse(res.responseText);

                        // convert to a "twitter" json
                        user.avatarUrl = json.avatar_large;
                        user.followingCount = json.friends_count.toString();
                        user.followerCount = json.followers_count.toString();
                        user.displayName = json.screen_name;

                        // max 4 numbers for followingCount and followerCount
                        if (user.followingCount.length > 4) {
                            user.followingCount = user.followingCount.slice(0, 4);
                        }
                        if (user.followerCount.length > 4) {
                            user.followerCount = user.followerCount.slice(0, 4);
                        }

                        json.state = {};
                        json.state.user = user;
                        loadEnemy(json);
                    } catch (e) {
                        console.log('Parsing data failed', e);
                        loadFail();
                    }
                }
            });
        };
        var loadTwitter = function (handleName) {
            // remove @
            handleName = handleName.replace('@', '');

            if (!navigator.onLine) {
                loadFail('twitterOffline');
                return;
            }

            var name = handleName;
            var xhr = new Xhr({
                // get: 'https://twitter.com/' + name + '/profile_image?size=original',
                get: 'https://twitter.com/' + name + '/',
                responseType: 'xhr',
                onSuccess: loadSuccess,
                onFail: loadFail
            });
            text.setText('Loading...');
            currentHandle = handleName;
        };
        var loadSuccess = function (res) {
            var doc = res.responseText;
            var start = '<script type="application/json" id="init-data">';
            var end = '</script>';
            var startIndex = -1;
            var endIndex = -1;
            var jsonString;
            var json;
            var user;

            // get the json out!
            startIndex = doc.indexOf(start);

            if (startIndex >= 0) {
                endIndex = doc.indexOf(end, startIndex);
                if (endIndex >= 0) {
                    startIndex += start.length;
                    jsonString = doc.substr(startIndex, endIndex - startIndex);
                    try {
                        json = JSON.parse(jsonString);
                        loadEnemy(json);
                    } catch (e) {
                        console.log('Parsing data failed', e);
                        Utils.Analytics.event("Twitter", "json_parse_error");
                        Utils.Firebase.logEvent('twitter_json_parse_error');
                        loadFail();
                    }
                } else {
                    console.log('could not find end');
                    Utils.Analytics.event("Twitter", "html_parse_error");
                    Utils.Firebase.logEvent('twitter_html_parse_error');
                    loadFail();
                }
            } else {
                console.log('could not find start');
                // loadFail();
                // try alternative start end
                start = 'class="json-data" value="';
                end = '">';
                startIndex = doc.indexOf(start);
                console.log(doc.length);
                console.log(startIndex);
                if (startIndex >= 0) {
                    endIndex = doc.indexOf(end, startIndex);
                    if (endIndex >= 0) {
                        startIndex += start.length;
                        jsonString = doc.substr(startIndex, endIndex - startIndex);
                        jsonString = jsonString.replace(new RegExp("&quot;", 'g'), '"');
                        try {
                            json = JSON.parse(jsonString);

                            user = json.profile_user;
                            user.avatarUrl = user.profile_image_url;
                            user.followingCount = user.friends_count.toString();
                            user.followerCount = user.followers_count.toString();
                            user.displayName = user.name;

                            // max 4 numbers for followingCount and followerCount
                            if (user.followingCount.length > 4) {
                                user.followingCount = user.followingCount.slice(0, 4);
                            }
                            if (user.followerCount.length > 4) {
                                user.followerCount = user.followerCount.slice(0, 4);
                            }

                            json.state = {};
                            json.state.user = user;
                            loadEnemy(json);
                        } catch (e) {
                            console.log('Parsing data failed', e);
                            Utils.Analytics.event("Twitter", "json_parse_error");
                            Utils.Firebase.logEvent('twitter_json_parse_error');
                            loadFail();
                        }
                    } else {
                        Utils.Analytics.event("Twitter", "html_parse_error");
                        Utils.Firebase.logEvent('twitter_html_parse_error');
                        console.log('could not find end');
                        loadFail();
                    }
                } else {
                    console.log('could not find start');
                    Utils.Analytics.event("Twitter", "html_parse_error");
                    Utils.Firebase.logEvent('twitter_html_parse_error');
                    loadFail();
                }
            }

            return;
        };
        var loadFail = function (textId) {
            text.setText(Utils.Localization.getText(textId || "twitterFail"));
            // TODO: go back?
            Utils.timeout(60, function () {
                Bento.objects.remove(overlay);
                text.setText('');
                Bento.objects.resume();
            }, Utils.PauseLevels.TWITTER);

        };
        var loadEnemy = function (twitterJson) {
            var user = twitterJson.state.user;
            var imageUrl = user.avatarUrl;
            var image = new Image();

            image.setAttribute('crossOrigin', 'anonymous');
            image.src = imageUrl;
            image.onload = function () {
                // critter selection image is 42 x 37
                var canvas = document.createElement('canvas');
                var context = canvas.getContext('2d');
                var packedImage;
                var followingStr = user.followingCount.replace(',', '').replace('.', '');
                var followerStr = user.followerCount.replace(',', '').replace('.', '');
                var followingCount = parseInt(followingStr);
                var followerCount = parseInt(followerStr);
                var hp = Math.max(followerCount, followingCount) || 1;
                canvas.width = 64;
                canvas.height = 64;
                context.drawImage(image, 0, 0, image.width, image.height, 0, 0, 64, 64);

                packedImage = new PackedImage(canvas);


                // copy baanu
                var base = Bento.assets.getJson('creatures/creatures').creatures.baanu;
                base = Utils.clone(base);

                base.twitter = true;
                // base.image = packedImage;
                base.image = canvas.toDataURL();

                // hp: allow numbers in the 1000s
                if (followingCount > followerCount) {
                    followingStr = followingStr.replace('M', 0).replace('k', 0);
                    followingCount = parseInt(followingStr);
                    hp = followingCount;
                } else {
                    followerStr = followerStr.replace('M', 0).replace('k', 0);
                    followerCount = parseInt(followerStr);
                    hp = followerCount;
                }


                // copy twitter info
                base.name = user.displayName;
                base.handle = '@' + currentHandle;
                base.element = '';
                base.catchChance = 80;
                base.turnsToAttack = 3;
                base.hp = hp;
                base.currentHp = base.hp;
                base.coins = 0;
                base.attack = Math.min(followerCount, followingCount) || 1;

                // max attack should be 2 digits
                if (base.attack > 99) {
                    base.attack = Math.floor(base.attack / 10);
                }

                twitterEnemy = base;
                twitterData.image = packedImage;

                card = new Card(base, {
                    image: base.image
                });
                card.position.x = viewport.width / 2;
                card.position.y = viewport.height / 2;
                card.z = Utils.Layers.TEAMEDIT + 2;

                Bento.objects.attach(card);

                text.setText(Utils.Localization.getText('twitterBattle').replace('{NAME}', user.displayName));

                new Tween({
                    from: 0,
                    to: 1,
                    'in': 60,
                    ease: 'cuberoot',
                    updateWhenPaused: Utils.PauseLevels.TWITTER,
                    'do': function (v) {
                        var oscillations = 2;
                        var t = Math.PI * 2 * oscillations + Math.PI * 0.5;
                        card.scale.x = Math.sin(t * v);
                    },
                    onComplete: function () {
                        card.scale.x = 1;
                        Utils.timeout(60, transition, Utils.PauseLevels.TWITTER);
                    }
                });
            };
        };
        var card;
        var twitterEnemy;
        var twitterData = {};
        var background = new Entity({
            z: 0,
            name: 'background',
            position: new Vector2(viewport.width / 2, viewport.height / 2),
            originRelative: new Vector2(0.5, 0.5),
            components: [
                new Sprite({
                    image: Bento.assets.getImage('team/background')
                })
            ]
        });
        /**
         * Transition screen into battle
         */
        var transition = function () {
            var viewport = Bento.getViewport();
            var sprite;
            var entity;
            var scale;
            var fadeSprite = new Sprite({
                imageName: 'fade'
            });
            var fade = new Entity({
                z: 1,
                name: 'fade',
                position: new Vector2(viewport.width / 2, viewport.height / 2),
                originRelative: new Vector2(0.5, 0.5),
                float: true,
                components: [
                    fadeSprite
                ]
            });
            var photo = new Photo({
                scale: 1
            });
            var oldRatio = 0;
            var smear = 0.5;
            Bento.objects.resume();
            photo.snap();
            Bento.objects.removeAll();

            sprite = new Sprite({
                image: new PackedImage(photo.getCanvas())
            });
            entity = new Entity({
                z: 0,
                name: '',
                position: new Vector2(viewport.width / 2, viewport.height / 2),
                originRelative: new Vector2(0.5, 0.5),
                float: true,
                components: [
                    sprite
                ]
            });

            fade.alpha = (0);
            scale = viewport.width / entity.dimension.width;
            entity.scale.x = scale;
            entity.scale.y = scale;

            Bento.objects.attach(entity);
            Bento.objects.attach(fade);

            Bento.audio.playSound('sfx_transition_battle');
            Bento.audio.stopAllMusic();
            Bento.audio.playMusic('bgm_boss');
            Bento.audio.setVolume(1, 'bgm_boss');

            // smearing fade doesnt work on android
            if (Utils.isNativeAndroid()) {
                fade = 1;
            }
            new Tween({
                from: 0,
                to: 1,
                'in': 60,
                ease: 'quadratic',
                updateWhenPaused: Utils.PauseLevels.TWITTER,
                'do': function (v, t) {
                    entity.alpha = (smear - v * smear);
                    entity.rotation = v * Math.PI * 3;
                    // entity.rotation += Math.PI / 60;
                    entity.scale.x = scale * (1 + v * 10);
                    entity.scale.y = scale * (1 + v * 10);
                },
                onComplete: function () {
                    new Tween({
                        from: 0,
                        to: 1,
                        'in': 60,
                        ease: 'linear',
                        'do': function (v, t) {
                            fade.alpha = (v);
                        },
                        updateWhenPaused: Utils.PauseLevels.TWITTER,
                        onComplete: function () {
                            // randomize planet
                            var checksum = Math.abs(Utils.checksum(twitterEnemy.handle || ''));
                            var planets = ['starti', 'veggi', 'zaspiri', 'dark', 'hotmountain', 'starti', 'veggi', 'zaspiri', 'dark', 'hotmountain'];
                            var planetName = planets[checksum % planets.length];

                            Utils.worldMap.planetData = {
                                planetName: planetName,
                                boss: twitterEnemy,
                                twitterData: twitterData
                            };
                            Bento.screens.show('screens/battle', {
                                planetData: Utils.worldMap.planetData,
                                isBossBattle: true,
                                enemies: [twitterEnemy],
                                isTwitterBattle: true
                            }, function () {
                                var amountOfTwitterBattles = Bento.saveState.load('amountOfTwitterBattles', 0);
                                // track amount of twitter battles
                                ++amountOfTwitterBattles;
                                Bento.saveState.save('amountOfTwitterBattles', amountOfTwitterBattles);
                                Utils.Analytics.event('Twitter', 'amntOfBttls_' + amountOfTwitterBattles);

                                // track which twitter account is chosen
                                Utils.Analytics.event('Twitter', 'opponent_' + twitterEnemy.handle);
                                Utils.Firebase.logEvent('twitter_opponent', {
                                    opponent: twitterEnemy.handle
                                });

                                // only check the time difference between first battle and first twitter visit
                                if (Bento.saveState.load('timeFromFirstVisitToFirstBattle', -1) < 0) {
                                    var timeFromFirstVisitToFirstBattle = Math.floor(Bento.saveState.load('playTimer', -1) / 60 / 60) - Bento.saveState.load('firstVisitToTwitterScreen', -1);
                                    Utils.Analytics.event('Twitter', 'time1stVst2Bttl_' + timeFromFirstVisitToFirstBattle);
                                    Utils.Firebase.logEvent('twitter_1st_visit_to_1st_battle', {
                                        time: timeFromFirstVisitToFirstBattle
                                    });
                                    Bento.saveState.save('timeFromFirstVisitToFirstBattle', timeFromFirstVisitToFirstBattle);
                                }
                            });
                        }
                    });
                }
            });
        };
        // search
        var buttonSearch = new ClickButton({
            z: Utils.Layers.TEAMEDIT + 2,
            name: 'buttonSearch',
            sfx: 'sfx_button',
            image: Bento.assets.getImage(mode === 'twitter' ? 'gui/buttonsearch' : 'gui/buttonsearchweibo'),
            frameCountX: 2,
            frameCountY: 1,
            position: new Vector2(viewport.width / 2, viewport.height - 24),
            float: false,
            onClick: function () {
                ask();
            }
        });
        var overlay = new Entity({
            z: Utils.Layers.TEAMEDIT + 1,
            name: 'overlay',
            position: new Vector2(viewport.width / 2, viewport.height / 2),
            originRelative: new Vector2(0.5, 0.5),
            components: [new Sprite({
                imageName: 'fade'
            })],
            init: function () {
                this.alpha = 0.75;
            }
        });
        var collectionList = new Twitter({
            onClick: ask,
            onClose: function () {
                Bento.screens.show('screens/map');
            }
        });


        Bento.objects.attach(background);
        Bento.objects.attach(text);
        // Bento.objects.attach(buttonSearch);
        Bento.objects.attach(collectionList);

        // reload team because the battles here shouldn't matter
        Utils.reloadTeam();

        if (data && data.caught) {
            new Caught(data, function () {
                if (Bento.saveState.load('askedRatingV' + window.VERSION, false) === false) {
                    Utils.timeout(30, function () {
                        new AskRating({});
                    });
                }
            });
        } else if (data && data.battled && data.escaped) {
            // temp fix for crash when defeating, but not catching twitter opponent
            // new Caught(data);
        } else if (data && data.battled) {
            new Caught(data);
        } else {
            if (!Bento.audio.isPlayingMusic('bgm_title')) {
                Bento.audio.stopAllMusic();
                Bento.audio.playMusic('bgm_title');
                Bento.audio.setVolume(1, 'bgm_title');
            }
        }

        Utils.Analytics.trackView('twitter');

        // track minute first visit to twitter screen
        if (Bento.saveState.load('firstVisitToTwitterScreen', -1) < 0) {
            timeToFirstVisit = Math.floor(Bento.saveState.load('playTimer', -1) / 60 / 60);
            Utils.Analytics.event('Twitter', '1stVstTwttrScrn_' + timeToFirstVisit);
            Utils.Firebase.logEvent('twitter_screen_1st_visit', {
                time: timeToFirstVisit
            });
            Bento.saveState.save('firstVisitToTwitterScreen', timeToFirstVisit);
        }

        amountOfTwitterVisits = Bento.saveState.load('amountOfTwitterVisits', 0);
        amountOfTwitterVisits++;
        Bento.saveState.save('amountOfTwitterVisits', amountOfTwitterVisits);
        Utils.Analytics.event('Twitter', 'amntOfVisits_' + amountOfTwitterVisits);
        Utils.Firebase.logEvent('twitter_screen_visit', {
            value: 1,
            amount_of_visits: amountOfTwitterVisits
        });

        if (Utils.EveryPlay.isRecording()) {
            Utils.EveryPlay.stop();
        }

        new Fade({
            fadeIn: true,
            type: 'fade'
        });
    };
    return new Screen({
        onShow: onShow
    });
});