/**
 * World map
 */
bento.define('screens/worldmap', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/gui/clickbutton',
    'bento/gui/text',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/screen',
    'bento/tween',
    'gui/centeredlist',
    'gui/coincounter',
    'entities/simplecard',
    'entities/worldmap/player',
    'entities/camera',
    'modules/arealoader',
    'entities/worldmap/mode7',
    'modules/zoom',
    'entities/paralax',
    'modules/photo',
    'entities/battle/caughtpopup',
    'gui/worldmap/menu',
    'gui/dialog',
    'gui/storydialog',
    'gui/worldmap/objectives',
    'gui/team/teamedit',
    'gui/combiner/combiner',
    'gui/dailygift',
    'components/sort',
    'entities/effect',
    'entities/tilecolliders',
    'modules/videoadtimer',
    'modules/forcedadtimer',
    'ads/mediator',
    'bento/profiler',
    'gui/shop/shop',
    'gui/hudcoincounter',
    'gui/coinsforad',
    'gui/askrating'
], function (
    Bento,
    Vector2,
    Rectangle,
    ClickButton,
    Text,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    Utils,
    Screen,
    Tween,
    CenteredList,
    CoinCounter,
    Card,
    Player,
    Camera,
    AreaLoader,
    Mode7,
    Zoom,
    Paralax,
    Photo,
    CaughtPopup,
    Menu,
    Dialog,
    StoryDialog,
    Objectives,
    TeamEdit,
    Combiner,
    DailyGift,
    Sort,
    Effect,
    TileColliders,
    VideoAdTimer,
    ForcedAdTimer,
    Mediator,
    Profiler,
    Shop,
    HudCoinCounter,
    CoinsForAd,
    AskRating
) {
    'use strict';
    var onShow = function (data) {
        /* Screen starts here */
        var viewport = Bento.getViewport();
        var planetData = Utils.worldMap.planetData;
        var planetName = planetData.planetName;
        var player = new Player({
            position: new Vector2(32, 32),
            planetData: Utils.worldMap.planetData
        });
        var tileColliders = new TileColliders({});
        var camera = new Camera({}).attach({
            position: new Vector2(0, 0),
            start: function () {
                this.position = player.position.clone();
            },
            update: function () {
                camera.ignoreEdge();
                this.position.x += (player.position.x - this.position.x) / 5;
                this.position.y += (player.position.y - this.position.y) / 5;
                camera.centerTo(this.position);
            }
        });
        var teamPanel = new Entity({
            z: Utils.Layers.GUI,
            name: 'teamPanel',
            position: new Vector2(viewport.width / 2, viewport.height - 64),
            float: true,
            components: [

                new Entity({
                    name: 'darkOverlay',
                    originRelative: new Vector2(0.5, 0),
                    components: [
                        new Sprite({
                            imageName: 'fade'
                        })
                    ],
                    init: function () {
                        this.alpha = 0.6;
                    }
                })
            ]
        });
        var coinCounter = new HudCoinCounter({
            z: Utils.Layers.GUI + 11,
            name: 'myCoins',
            position: new Vector2(viewport.width - 2, 2),
        });
        var teamList;
        var healthChecker = {
            name: 'healthChecker',
            wasDead: false,
            update: function () {
                var dialog;
                var tutorialHeal;

                // was played defeated?
                if (isPlaying && player.isActivated()) {
                    // give free health the first time player was defeated
                    if (data && data.wasDefeated) {
                        // Bento.audio.playSound('sfx_error');
                        // player.activate(false);
                        this.wasDead = true;
                        // auto heal first time
                        if (
                            (planetName === 'starti' || planetName === 'veggi') &&
                            Bento.saveState.load('healTutorial', false) === false
                        ) {
                            dialog = new StoryDialog({
                                text: Utils.Localization.getText('healTutorial'),
                                attachments: {
                                    "2": new Entity({
                                        position: new Vector2(0, 20),
                                        originRelative: new Vector2(0.5, 0.5),
                                        components: [
                                            new Sprite({
                                                image: Bento.assets.getImage('gui/tutorialflee'),
                                            })
                                        ]
                                    })
                                },
                                type: 'yes',
                                hideCat: true,
                                canClose: false,
                                onClose: function () {
                                    var teamCritters = Utils.getTeam();
                                    Utils.forEach(teamCritters, function (properties, i, l, breakLoop) {
                                        if (!properties) {
                                            return;
                                        }
                                        properties.currentHp = properties.hp;
                                        totalHealth += properties.currentHp;
                                    });
                                    Utils.saveTeam();
                                    // reset team
                                    createTeam();
                                    new Tween({
                                        from: teamPanel.position.y,
                                        to: viewport.height - 64,
                                        'in': 30,
                                        ease: 'squareroot',
                                        'do': function (v, t) {
                                            teamPanel.position.y = v;
                                        }
                                    });
                                    Bento.saveState.save('healTutorial', true);
                                    Utils.Analytics.event('Game', 'healTutorial');
                                    Utils.Firebase.logEvent('healTutorial');
                                }
                            });
                        }
                    } else if (totalHealth < maxHealth && Bento.saveState.load('healCrittersTutorial', false) === false) {
                        tutorialHeal = new Entity({
                            name: 'tutheal',
                            position: new Vector2(-3, 16),
                            originRelative: new Vector2(0.5, 0.5),
                            components: [
                                new Sprite({
                                    imageName: 'gui/tutorialheal'
                                })
                            ]
                        });
                        dialog = new StoryDialog({
                            text: Utils.Localization.getText('healCrittersTutorial'),
                            type: 'yes',
                            canClose: false,
                            onClose: function () {
                                Bento.saveState.save('healCrittersTutorial', true);
                                Utils.Analytics.event('Game', 'healCrittersTutorial');
                                Utils.Firebase.logEvent('healCrittersTutorial');
                                // find heal buttons and attach hand
                                if (teamList) {
                                    teamList.iterate(function (slot, i) {
                                        slot.getComponent('buttonHeal', function (buttonHeal) {
                                            buttonHeal.attach(getHandDiagonal(-4));
                                        });
                                    });
                                }
                            }
                        });
                        dialog.attach(tutorialHeal);
                    }
                }
                if (!player.isActivated() && this.wasDead && totalHealth > 0) {
                    this.wasDead = false;
                    player.activate(true);
                }
            }
        };
        var adButton = null;
        var hasPlayedTutorial = Bento.saveState.load('sawTutorial2', false);
        var hasSeenBoss = Bento.saveState.load('firstBossBattle', false);
        var retryCost = Bento.assets.getJson('creatures/retrycost');
        var rewardAmount = Bento.assets.getJson('creatures/rewardamount');
        var unlockedRarity = Bento.saveState.load('unlockedRarity', 3);
        var myCoins = Bento.saveState.load('myCoins', 0);
        var adAsGift = (Utils.isFB() && Utils.softlaunch) || Utils.NoAdsVersion; // facebook softlaunch
        var adChecker = {
            name: 'adChecker',
            oldPauseLevel: 0,
            moveIn: function () {
                // move in
                new Tween({
                    from: adButton.position.x,
                    to: viewport.width - 27,
                    'in': 30,
                    ease: 'easeOutExpo',
                    'do': function (v, t) {
                        if (adButton)
                            adButton.position.x = v;
                    }
                });
                // make boundingbox of coin smaller
                coinCounter.setBoundingBox(new Rectangle(-16, -6, 64, 12));
            },
            reset: function () {
                // reset
                Bento.objects.remove(adButton);
                adButton = null;
                // make boundingbox of coin larger again
                coinCounter.setBoundingBox(new Rectangle(-16, -16, 64, 32));
            },
            getReward: function (success) {

            },
            update: function () {
                var canPlay = VideoAdTimer.canPlay() && hasPlayedTutorial /* && hasSeenBoss*/ ;

                // update coins
                myCoins = coinCounter.getValue();

                // emergency: out of coins
                if (myCoins < rewardAmount[unlockedRarity - 1] && totalHealth <= 3) {
                    canPlay = true;
                }

                // show ad option
                if (canPlay && !adButton && player.timer > 60) {
                    adButton = new ClickButton({
                        z: Utils.Layers.GUI + 11,
                        name: 'adButton',
                        sfx: 'sfx_button',
                        image: Bento.assets.getImage(adAsGift ? 'gui/buttoncoinsforgift' : 'gui/buttoncoinsforad'),
                        frameCountX: 2,
                        frameCountY: 2,
                        animations: {
                            down: {
                                frames: [1]
                            },
                            up: {
                                speed: 0.025,
                                frames: [0, 2]
                            }
                        },
                        position: new Vector2(viewport.width + 27, 24),
                        float: true,
                        updateWhenPaused: Utils.PauseLevels.ALWAYS,
                        onClick: function () {
                            this.setActive(false);
                            adChecker.oldPauseLevel = Bento.objects.isPaused();

                            if (Utils.isNativeAndroid()) {
                                buttonMenu.doCallback();
                            }
                            CoinsForAd.watchAd(function () {
                                // clicked on 'ad for coins' from the worldmap
                                Utils.Analytics.event('Ads', 'fromWorldmap');
                                Utils.Firebase.logEvent('clicked_ad_from_worldmap', {
                                    value: 1
                                });
                                adChecker.reset();
                            });
                        }
                    });
                    Bento.objects.attach(adButton);
                    this.moveIn();
                } else if ((!canPlay || player.timer <= 60) && adButton) {
                    adChecker.reset();
                }
            }
        };
        var totalHealth = 0;
        var maxHealth = -1;
        var buttonMenu;
        var sky = new Paralax({
            z: Utils.Layers.SKY,
            imageName: 'worldmap/' + Utils.worldMap.planetData.planetName + '/sky',
            updateWhenPaused: Utils.PauseLevels.ALWAYS,
            scrollFactor: 0.1
        });
        var upperFill = new Paralax({
            z: Utils.Layers.SKY,
            imageName: 'worldmap/' + Utils.worldMap.planetData.planetName + '/sky2',
            updateWhenPaused: Utils.PauseLevels.ALWAYS,
            offsetY: -64,
            scrollFactor: 0.1
        });
        var lowerShade = new Entity({
            z: Utils.Layers.SKY,
            name: 'lowerShade',
            position: new Vector2(viewport.width / 2, viewport.height),
            originRelative: new Vector2(0.5, 1),
            float: true,
            components: [
                new Sprite({
                    imageName: 'worldmap/lowershade'
                })
            ]
        });

        var worldmapBottom = new Entity({
            z: Utils.Layers.SKY,
            name: 'lowerShade',
            position: new Vector2(viewport.width / 2, viewport.height),
            originRelative: new Vector2(0.5, 0),
            float: true,
            components: [
                new Sprite({
                    imageName: 'worldmap/bottom'
                })
            ]
        });
        var mode7 = new Mode7({});
        var areaLoadTrigger = new Entity({
            z: 0,
            name: 'areaLoadTrigger',
            components: [],
            updateWhenPaused: Utils.PauseLevels.ALWAYS
        }).attach({
            start: function () {
                areaLoadTrigger.dimension = new Rectangle(0, 0, 1, 1);
            },
            update: function () {
                // load trigger is near the top of the screen
                areaLoadTrigger.position.x = viewport.x + viewport.width / 2;
                areaLoadTrigger.position.y = viewport.y + 32;
            }
        });
        var loadedMeshes = 0;
        var totalMeshes = 0;
        var hasStarted = false;
        var isPlaying = false; // bugfix for healCrittersTutorial starting before caughtpopup
        var hasLoadedObjects = false;
        var areaLoader = new AreaLoader({
            areaLoadTrigger: areaLoadTrigger,
            json: 'maps/' + Utils.worldMap.planetData.planetName,
            loadRect: new Rectangle(0, 0, 16, 16),
            loadTriggerSize: 2,
            unloadTriggerSize: 2.5,
            cachedLayers: true,
            updateWhenPaused: Utils.PauseLevels.ALWAYS,
            onLoad: function () {
                hasLoadedObjects = true;
                if (hasLoadedObjects && loadedMeshes === totalMeshes) {
                    onLoad();
                }
            },
            onSectionLoaded: function (section, tiled, total) {
                // load corresponding mesh
                // must convert to image due to cocoon bug
                var canvas = tiled.tileLayer.getComponent('sprite').spriteImage.image;
                var image;
                var makeMesh = function () {
                    var mesh = mode7.makeMesh(new Vector2(tiled.tileLayer.position.x, tiled.tileLayer.position.y), image);
                    tiled.mesh = mesh;
                    // store image conversion
                    canvas.image = image;
                };
                // there is a delay between converting the image
                // trigger onload when all images were converted

                totalMeshes = total;

                if (canvas.image) {
                    image = canvas.image;
                    makeMesh();
                    loadedMeshes += 1;
                    if (hasLoadedObjects && loadedMeshes === totalMeshes) {
                        onLoad();
                    }
                } else {
                    image = new Image();
                    image.src = canvas.toDataURL();
                    image.onload = function () {
                        makeMesh();
                        loadedMeshes += 1;
                        if (hasLoadedObjects && loadedMeshes === totalMeshes) {
                            onLoad();
                        }
                    };
                }
                // hide tileLayers
                Utils.forEach(tiled.tileLayers, function (tileLayer, i, l, breakLoop) {
                    tileLayer.entity.draw = null;
                    tileLayer.entity.update = null;
                });
                tiled.tileLayer.visible = false;
                // var mesh = mode7.makeMesh(new Vector2(tiled.tileLayer.position.x, tiled.tileLayer.position.y), canvas);
                // tiled.mesh = mesh;
            },
            onSectionUnloaded: function (tiled) {
                if (tiled && tiled.mesh) {
                    mode7.removeMesh(tiled.mesh);
                } else {
                    // TODO: could miss the oportunity to clear the mesh, because canvas -> image conversion
                }
            }
        });
        var fade = new Entity({
            z: Utils.Layers.FADE,
            name: 'fade',
            position: new Vector2(viewport.width / 2, viewport.height / 2),
            originRelative: new Vector2(0.5, 0.5),
            float: true,
            components: [
                new Sprite({
                    imageName: 'fade'
                })
            ]
        });
        var objectives = new Objectives({
            camera: camera,
            teamPanel: teamPanel,
            player: player
        });
        var showEveryPlayButton = function () {
            var behavior = {
                done: false,
                start: function () {
                    Utils.timeout(60, function () {
                        new Tween({
                            from: -12,
                            to: 12,
                            'in': 30,
                            ease: 'exponential',
                            alpha: -4,
                            // updateWhenPaused: Utils.PauseLevels.PAUSE,
                            'do': function (v, t) {
                                button.position.y = v;
                            }
                        });

                    });
                },
                update: function () {
                    if (this.done === false && button.timer > 10 * 60) {
                        this.done = true;
                        new Tween({
                            from: 12,
                            to: -12,
                            'in': 30,
                            ease: 'exponential',
                            alpha: -4,
                            // updateWhenPaused: Utils.PauseLevels.PAUSE,
                            'do': function (v, t) {
                                button.position.y = v;
                            },
                            onComplete: function () {
                                Bento.objects.remove(button);
                            }
                        });
                    }
                }
            };
            var button = new ClickButton({
                z: Utils.Layers.GUI + 20,
                name: 'everyplayButton',
                sfx: 'sfx_button',
                image: Bento.assets.getImage('gui/buttonmovie'),
                frameCountX: 2,
                frameCountY: 1,
                position: new Vector2(64, -12),
                float: true,
                // updateWhenPaused: Utils.PauseLevels.PAUSE,
                onClick: function () {
                    var shareScore = Utils.Localization.getText('shareText'),
                        link = Utils.getStoreLink();

                    shareScore = shareScore.replace('[LINK]', link);

                    Utils.EveryPlay.setData({
                        text: shareScore
                    });
                    Bento.audio.muteMusic(true);
                    Utils.EveryPlay.replay();
                    Bento.objects.attach(waitForInput);
                }
            });
            // this entity is a workaround for the fact that the music keeps playing in everyplay
            var waitForInput = new Entity({
                name: 'waitForInput',
                components: [
                    new Clickable({
                        pointerDown: function () {
                            // reset music and remove itself
                            Bento.audio.muteMusic(Bento.saveState.load('muteMusic', false), true);
                            bento.require(['audia'], function (Audia) {
                                var audioContext = Audia.audioContext;
                                if (audioContext && audioContext.state === 'suspended') {
                                    audioContext.resume();
                                }
                            });
                            Bento.objects.remove(waitForInput);
                        }
                    })
                ]
            });

            button.attach(behavior);
            Bento.objects.attach(button);
        };
        var askRating = function (watchedAd, battleCount) {
            Utils.timeout(30, function () {
                if (planetData.planetName !== 'starti') {
                    battleCount += 1;
                    Bento.saveState.save('battleCount', battleCount);
                    if (battleCount >= 5 && !watchedAd) {
                        if (Bento.saveState.load('askedRatingV' + window.VERSION, false) === true) {
                            // if asked before, ask again every 10 catches
                            if (battleCount % 10 === 0) {
                                new AskRating({});
                            }
                        } else {
                            new AskRating({});
                        }
                    }
                }
            });
        };
        var showTutorial = function (onFinished) {
            var sawTutorial = Bento.saveState.load('sawTutorial', false);
            var sawTutorial2 = Bento.saveState.load('sawTutorial2', false);
            var text;

            if (sawTutorial && sawTutorial2) {
                return;
            }

            // lock player until tutorial is over
            player.activate(false);

            if (!sawTutorial) {
                // show tutorial 1
                text = Utils.Localization.getText('teamTutorial');
                new StoryDialog({
                    text: [
                        text[0],
                        text[1]
                    ],
                    attachments: {
                        "1": new Entity({
                            position: new Vector2(-4, 16),
                            originRelative: new Vector2(0.5, 0.5),
                            components: [
                                new Sprite({
                                    image: Bento.assets.getImage('gui/tutorialteamedit'),
                                })
                            ]
                        })
                    },
                    onClose: function () {
                        var hand1 = getHand();
                        var hand2 = getHandDiagonal();
                        var hand3 = getHand();
                        var overlay = new Entity({
                            z: Utils.Layers.GUI,
                            name: 'overlay',
                            alpha: 0.5,
                            position: new Vector2(viewport.width / 2, viewport.height / 2),
                            originRelative: new Vector2(0.5, 0.5),
                            float: true,
                            components: [new Sprite({
                                imageName: 'fade'
                            })]
                        });
                        var buttonExit = buttonMenu.getComponentMenu('buttonExit');
                        var buttonDex = buttonMenu.getComponentMenu('buttonDex');
                        var buttonEditTeam = buttonMenu.getComponentMenu('buttonEditTeam');
                        var buttonCombiner = buttonMenu.getComponentMenu('buttonCombine');
                        var buttonPlanet = buttonMenu.getComponentMenu('buttonPlanet');
                        var toggleMusic = buttonMenu.getComponentMenu('toggleMusic');
                        var toggleSound = buttonMenu.getComponentMenu('toggleSound');
                        var onClosedMenu = function () {
                            // remove hand
                            buttonExit.remove(hand3);

                            // lighten
                            Bento.objects.remove(overlay);

                            // enable other buttons
                            buttonDex.setActive(true);
                            buttonDex.getComponent('text', function (text) {
                                text.alpha = 1;
                            });
                            buttonPlanet.setActive(true);
                            buttonPlanet.getComponent('text', function (text) {
                                text.alpha = 1;
                            });
                            toggleSound.setActive(true);
                            toggleSound.visible = true;
                            toggleMusic.setActive(true);
                            toggleMusic.visible = true;

                            // clean event listener
                            EventSystem.off('closedMenu', onClosedMenu);

                            // show final text
                            new StoryDialog({
                                text: [
                                    text[2],
                                    text[3]
                                ],
                                onClose: function () {
                                    // unlock player
                                    player.activate(true);

                                    // save tutorial played
                                    Bento.saveState.save('sawTutorial', true);

                                    if (onFinished) {
                                        onFinished();
                                    }
                                }
                            });
                        };
                        var onCompleteObjective = function () {
                            // remove hands
                            buttonMenu.remove(hand1);
                            buttonEditTeam.remove(hand2);

                            // attach hand to exit
                            buttonExit.attach(hand3);
                            buttonExit.setActive(true);

                            // clean event listner
                            EventSystem.off('closedTeamEdit', onCompleteObjective);
                        };

                        // darken
                        Bento.objects.attach(overlay);

                        // add hand to menu
                        buttonMenu.attach(hand1);
                        buttonEditTeam.attach(hand2);

                        // enable teamedit
                        buttonEditTeam.setActive(true);
                        buttonEditTeam.getComponent('text', function (text) {
                            text.alpha = 1;
                        });

                        // disable other buttons
                        buttonExit.setActive(false);
                        buttonDex.setActive(false);
                        buttonDex.getComponent('text', function (text) {
                            text.alpha = 0.5;
                        });
                        buttonPlanet.setActive(false);
                        buttonPlanet.getComponent('text', function (text) {
                            text.alpha = 0.5;
                        });
                        toggleSound.setActive(false);
                        toggleSound.visible = false;
                        toggleMusic.setActive(false);
                        toggleMusic.visible = false;

                        // wait until player completes the objective
                        // by waiting for the event thats fired by closing the teamedit and menu
                        EventSystem.on('closedTeamEdit', onCompleteObjective);
                        EventSystem.on('closedMenu', onClosedMenu);
                    }
                });
            } else if (!sawTutorial2) {
                text = Utils.Localization.getText('combineTutorial');
                new StoryDialog({
                    text: [
                        text[0],
                        text[1]
                    ],
                    attachments: {
                        "1": new Entity({
                            position: new Vector2(-4, 16),
                            originRelative: new Vector2(0.5, 0.5),
                            components: [
                                new Sprite({
                                    image: Bento.assets.getImage('gui/tutorialcombine'),
                                })
                            ]
                        })
                    },
                    onClose: function () {
                        var hand1 = getHand();
                        var hand2 = getHandDiagonal();
                        var hand3 = getHand();
                        var overlay = new Entity({
                            z: Utils.Layers.GUI,
                            name: 'overlay',
                            alpha: 0.5,
                            position: new Vector2(viewport.width / 2, viewport.height / 2),
                            originRelative: new Vector2(0.5, 0.5),
                            float: true,
                            components: [new Sprite({
                                imageName: 'fade'
                            })]
                        });
                        var buttonExit = buttonMenu.getComponentMenu('buttonExit');
                        var buttonDex = buttonMenu.getComponentMenu('buttonDex');
                        var buttonEditTeam = buttonMenu.getComponentMenu('buttonEditTeam');
                        var buttonCombiner = buttonMenu.getComponentMenu('buttonCombine');
                        var toggleMusic = buttonMenu.getComponentMenu('toggleMusic');
                        var toggleSound = buttonMenu.getComponentMenu('toggleSound');
                        var buttonPlanet = buttonMenu.getComponentMenu('buttonPlanet');
                        var onClosedMenu = function () {
                            // remove hand
                            buttonExit.remove(hand3);

                            // lighten
                            Bento.objects.remove(overlay);

                            // enable other buttons
                            buttonDex.setActive(true);
                            buttonDex.getComponent('text', function (text) {
                                text.alpha = 1;
                            });
                            buttonPlanet.setActive(true);
                            buttonPlanet.getComponent('text', function (text) {
                                text.alpha = 1;
                            });
                            buttonEditTeam.setActive(true);
                            buttonEditTeam.getComponent('text', function (text) {
                                text.alpha = 1;
                            });
                            toggleSound.setActive(true);
                            toggleSound.visible = true;
                            toggleMusic.setActive(true);
                            toggleMusic.visible = true;

                            // clean event listener
                            EventSystem.off('closedMenu', onClosedMenu);

                            // show final text
                            new StoryDialog({
                                text: [
                                    text[2],
                                    text[3]
                                ],
                                onClose: function () {
                                    // unlock player
                                    player.activate(true);

                                    // save tutorial played
                                    Bento.saveState.save('sawTutorial2', true);
                                    Utils.Firebase.logEvent('sawTutorial2', {});
                                    Utils.Firebase.logEvent(Utils.Firebase.Events.TUTORIAL_COMPLETE);

                                    if (onFinished) {
                                        onFinished();
                                    }

                                }
                            });
                        };
                        var onCompleteObjective = function () {
                            // remove hands
                            buttonMenu.remove(hand1);
                            buttonCombiner.remove(hand2);

                            // attach hand to exit
                            buttonExit.attach(hand3);
                            buttonExit.setActive(true);

                            // clean event listner
                            EventSystem.off('closedCombiner', onCompleteObjective);
                        };

                        // darken
                        Bento.objects.attach(overlay);

                        // add hand to menu
                        buttonMenu.attach(hand1);
                        buttonCombiner.attach(hand2);

                        // enable combiner
                        buttonCombiner.setActive(true);
                        buttonCombiner.getComponent('text', function (text) {
                            text.alpha = 1;
                        });

                        // disable other buttons
                        buttonExit.setActive(false);
                        buttonDex.setActive(false);
                        buttonDex.getComponent('text', function (text) {
                            text.alpha = 0.5;
                        });
                        buttonPlanet.setActive(false);
                        buttonPlanet.getComponent('text', function (text) {
                            text.alpha = 0.5;
                        });
                        buttonEditTeam.setActive(false);
                        buttonEditTeam.getComponent('text', function (text) {
                            text.alpha = 0.5;
                        });
                        toggleSound.setActive(false);
                        toggleSound.visible = false;
                        toggleMusic.setActive(false);
                        toggleMusic.visible = false;

                        // wait until player completes the objective
                        // by waiting for the event thats fired by closing the teamedit and menu
                        EventSystem.on('closedCombiner', onCompleteObjective);
                        EventSystem.on('closedMenu', onClosedMenu);
                    }
                });
            }

        };
        /*
         * Callback that waits for tiled objects and three meshes to load
         */
        var onLoad = function () {
            var fadeIn = function () {
                // fade in
                new Tween({
                    from: 1,
                    to: 0,
                    'in': 30,
                    ease: 'linear',
                    updateWhenPaused: Utils.PauseLevels.ALWAYS,
                    'do': function (v, t) {
                        fade.alpha = (v);
                    },
                    onComplete: function () {
                        Bento.objects.remove(fade);
                        start();
                    }
                });
            };
            var battleCount = Bento.saveState.load('battleCount', 0);

            if (hasStarted) {
                return;
            }
            hasStarted = true;
            objectives.countTokens();

            if (data && data.battled) {

                // if (Utils.BETA_TEST && planetData.planetName === 'veggi') {
                //     battleCount += 1;
                //     Bento.saveState.save('battleCount', battleCount);
                //     // if (battleCount > 3) {
                //     //     openForm();
                //     // }
                // }

                // just came back from catching a critter
                Utils.timeout(30, function (watchedAd) {
                    if (data.caught) {
                        new CaughtPopup(data, function () {
                            coinCounter.setValue(Bento.saveState.load('myCoins', 0));

                            // ask for rating
                            askRating(watchedAd, battleCount);

                            // show tutorial
                            showTutorial(function () {
                                isPlaying = true;
                            });
                            checkDailyGift();
                        });
                    } else {
                        Utils.timeout(30, function () {
                            isPlaying = true;
                            checkDailyGift();
                        });
                    }
                    fadeIn();
                }, Infinity);

                // show share button
                // showEveryPlayButton();

            } else {
                // came from planet map
                intro();
            }

        };
        /*
         * Start worldmap
         */
        var start = function (delay) {
            delay = delay || 0;
            // allow player to move
            player.activate(true);
            player.attach(new Sort({}));

            tutorial();
            Utils.timeout(delay, function () {
                if (!Bento.audio.isPlayingMusic(Utils.worldMap.planetData.bgm)) {
                    Bento.audio.stopAllMusic();
                    Bento.audio.playMusic(Utils.worldMap.planetData.bgm);
                }
            });
            // Bento.audio.playSound('sfx_transition');

        };
        /*
         * Show an intro of the alien landing
         */
        var intro = function () {
            var camera = mode7.getCamera();
            var rotation = camera.rotation.x;
            var zoom = camera.position.z;
            var showTriggers = function (value) {
                Utils.forEach(Bento.objects.getByFamily('triggers'), function (trigger, i, l, breakLoop) {
                    trigger.visible = value;
                });
            };
            var fadeIn = function () {
                // fade in
                new Tween({
                    from: 1,
                    to: 0,
                    'in': 30,
                    ease: 'linear',
                    updateWhenPaused: Utils.PauseLevels.ALWAYS,
                    'do': function (v, t) {
                        fade.alpha = (v);
                    },
                    onComplete: function () {
                        Bento.objects.remove(fade);
                    }
                });
            };
            var dropPlayer = function () {
                playSound();
                new Tween({
                    from: viewport.height / 2,
                    to: 0,
                    'in': 120,
                    ease: 'linear',
                    'do': function (v, t) {
                        player.setZ(v);
                        player.z = Utils.Layers.SKY;
                    },
                    onComplete: function () {
                        start(60);
                        showTriggers(true);
                        player.setAnimation('down');
                        stopSound();
                        Bento.audio.playSound(Utils.worldMap.planetData.sfx);
                        isPlaying = true;
                        checkDailyGift();

                        // Bento.objects.attach({
                        //     z: -1000,
                        //     timer: 0,
                        //     update: function () {
                        //         Profiler.start();
                        //         Profiler.reportAfter = 60 * 60;
                        //     }
                        // });
                        // Bento.objects.attach({
                        //     z: 10000000,
                        //     update: function () {
                        //         Profiler.stop();
                        //     }
                        // });
                    }
                });
            };
            var zoomOut = function () {
                // camera.position.z += 64;
                camera.position.z += 0;
                new Tween({
                    from: camera.position.z,
                    to: zoom,
                    'in': 60,
                    ease: 'squareroot',
                    'do': function (v, t) {
                        camera.position.z = v;
                    },
                    onComplete: function () {
                        // dropPlayer();
                        showSpaceShip();
                    }
                });
            };
            var showSpaceShip = function () {
                var spaceShip = new Entity({
                    z: Utils.Layers.SKY + 10,
                    name: 'spaceShip',
                    position: new Vector2(viewport.width / 2, 75),
                    originRelative: new Vector2(0.5, 1),
                    float: true,
                    components: [
                        new Sprite({
                            imageName: 'worldmap/ship',
                        })
                    ]
                });
                var beamSprite = new Sprite(Bento.assets.getJson('animations/worldmap/beamupoverworld'));
                var beam = new Entity({
                    z: Utils.Layers.SKY,
                    name: 'beam',
                    position: new Vector2(0, 0),
                    originRelative: new Vector2(0.5, 0),
                    float: true,
                    components: [
                        beamSprite
                    ]
                });
                var moveIn = function () {
                    // move in
                    new Tween({
                        from: -75,
                        to: spaceShip.getLocalPosition(player.position).y - 80 + Utils.screenSize.y,
                        'in': 60,
                        ease: 'squareroot',
                        'do': function (v, t) {
                            spaceShip.position.y = v;
                        },
                        onComplete: function () {
                            // show beam
                            Bento.objects.attach(beam);
                            beam.position = spaceShip.position.subtract(new Vector2(0, 90 + Utils.screenSize.y));
                            beamSprite.setAnimation('start');
                            beam.alpha = (0.6);
                            dropPlayer();
                            Utils.timeout(120, moveOut);
                        }
                    });
                };
                var moveOut = function () {
                    beamSprite.setAnimation('end', function () {
                        Bento.objects.remove(beam);

                        new Tween({
                            from: spaceShip.position.y,
                            to: spaceShip.position.y - 80,
                            'in': 60,
                            ease: 'easeInBack',
                            'do': function (v, t) {
                                spaceShip.position.y = v;
                            },
                            onComplete: function () {
                                Bento.objects.remove(spaceShip);
                            }
                        });
                        // new Tween({
                        //     from: spaceShip.position.x,
                        //     to: spaceShip.position.x + 16,
                        //     'in': 60,
                        //     ease: 'cubic',
                        //     'do': function (v, t) {
                        //         spaceShip.position.x = v;
                        //     }
                        // });
                        // new Tween({
                        //     from: 0,
                        //     to: Math.PI / 8,
                        //     'in': 60,
                        //     ease: 'cubic',
                        //     'do': function (v, t) {
                        //         spaceShip.rotation = v;
                        //     }
                        // });
                        // new Tween({
                        //     from: 0,
                        //     to: 1,
                        //     'in': 60,
                        //     ease: 'cubic',
                        //     'do': function (v, t) {
                        //         spaceShip.scale.x = 1 + v;
                        //         spaceShip.scale.y = 1 + v;
                        //     }
                        // });
                    });

                };

                moveIn();

                Bento.objects.attach(spaceShip);
            };

            showTriggers(false);

            fadeIn();

            player.setZ(viewport.height / 2);
            // player.setAnimation('swirl');

            zoomOut();

        };
        var isPlayingSound = false;
        var sound = 'sfx_spaceship_beam';
        var playSound = function () {
            if (!isPlayingSound) {
                isPlayingSound = true;
                Bento.audio.playSound(sound, true);
            }
        };
        var stopSound = function () {
            isPlayingSound = false;
            Bento.audio.stopSound(sound);
        };
        var getHand = function () {
            return new Entity({
                name: 'hand',
                position: new Vector2(0, 8),
                originRelative: new Vector2(0.5, 0),
                components: [new Sprite({
                    imageName: 'map/hand'
                })]
            }).attach({
                update: function (data) {
                    var hand = data.entity;
                    hand.position.y = 4 + 4 * Math.abs(Math.sin(hand.timer / 20));
                }
            });
        };
        var getHandDiagonal = function (offsetY) {
            return new Entity({
                name: 'hand',
                position: new Vector2(0, 8),
                originRelative: new Vector2(0.5, 0),
                components: [new Sprite({
                    imageName: 'map/hand2'
                })]
            }).attach({
                offsetX: 0,
                offsetY: offsetY || 4,
                start: function (data) {
                    this.offsetX = data.entity.parent.dimension.width / 2;
                },
                update: function (data) {
                    var hand = data.entity;
                    hand.position.y = this.offsetY + 4 * Math.abs(Math.sin(hand.timer / 20));
                    hand.position.x = this.offsetX + 4 * Math.abs(Math.sin(hand.timer / 20));
                }
            });
        };
        /**
         * Make critters and check total health
         */
        var createTeam = function () {
            var i = 0;
            var makeTeamCritters = function () {
                var teamCritters = Utils.getTeam();
                var slot;
                var card;
                var canHeal = false;

                if (teamList) {
                    // remove old
                    teamPanel.remove(teamList);
                }
                teamList = new CenteredList({
                    position: new Vector2(0, 32),
                    spacing: 2
                });
                totalHealth = 0;
                maxHealth = 0;
                Utils.forEach(teamCritters, function (properties, i, l, breakLoop) {
                    var card;
                    var coinPrice;
                    var buttonHeal = new ClickButton({
                        name: 'buttonHeal',
                        sfx: 'sfx_button',
                        image: Bento.assets.getImage('worldmap/buttonheal'),
                        frameCountX: 2,
                        frameCountY: 2,
                        position: new Vector2(0, 33),
                        animations: {
                            up: {
                                frames: [0]
                            },
                            down: {
                                frames: [1]
                            },
                            inactive: {
                                frames: [2]
                            }
                        },
                        // updateWhenPaused: Utils.PauseLevels.MENU,
                        onClick: function () {
                            // enough cash?
                            var myCoins = Bento.saveState.load('myCoins', 0);
                            var healedFrom0 = Bento.saveState.load('healedFrom0', 0);
                            var currentPlanet;
                            var amountOfHeals;
                            if (myCoins < coinPrice) {
                                // not enough
                                Bento.audio.playSound('sfx_error');
                                new Shop({});
                                // allow video ad when user doesnt have enough coins??
                                // VideoAdTimer.force();
                                return;
                            } else {
                                myCoins -= coinPrice;
                                Utils.trackCoins(-coinPrice, 'heal');
                                Bento.saveState.save('myCoins', myCoins);
                                EventSystem.fire('updateCoins');

                                // track amount of heals per planet
                                currentPlanet = Bento.saveState.load('currentPlanet', null);
                                amountOfHeals = Bento.saveState.load('' + currentPlanet + 'AmountOfHeals', 0);
                                ++amountOfHeals;
                                Bento.saveState.save('' + currentPlanet + 'AmountOfHeals', amountOfHeals);
                                Utils.Analytics.event('Planets', '' + currentPlanet + 'Heal_' + amountOfHeals, coinPrice);
                                // value is amount of health left when healing as a ratio of max health.
                                // From 0 to 1. To get an average divide value by total amount of events
                                Utils.Firebase.logEvent('heal', {
                                    value: coinPrice / properties.hp,
                                    planet: currentPlanet,
                                    amount_of_heals: amountOfHeals,
                                    critter: properties.id
                                });
                            }

                            // Achievement: Heal 10 critters from 0 health
                            if (properties.currentHp <= 0) {
                                if (++healedFrom0 >= 10) {
                                    Utils.postAchievement('CgkI--zjoo8UEAIQDQ', function () {});
                                }
                                Bento.saveState.save('healedFrom0', healedFrom0);
                            }

                            // TODO: ask confirmation to heal?
                            card.remove(buttonHeal);
                            buttonHeal.setActive(false);
                            buttonHeal.visible = false;
                            price.visible = false;
                            // can move now
                            totalHealth += properties.hp - Math.max(properties.currentHp, 0);

                            properties.currentHp = properties.hp;
                            card.setHp(properties.currentHp);
                            card.refresh();
                            Utils.saveTeam();
                            Bento.audio.playSound('sfx_heal');

                            new Effect({
                                z: Utils.Layers.GUI + 1,
                                sprite: new Sprite(Bento.assets.getJson('animations/effects/pickup')),
                                position: card.getWorldPosition()
                            });

                            // healed all?
                            if (totalHealth >= maxHealth) {
                                new Tween({
                                    from: teamPanel.position.y,
                                    to: viewport.height - 64,
                                    'in': 30,
                                    ease: 'squareroot',
                                    'do': function (v, t) {
                                        teamPanel.position.y = v;
                                    }
                                });
                            }
                        }
                    });
                    var price = new CoinCounter({
                        position: new Vector2(0, 9),
                        align: 'center',
                        value: '$'
                    });

                    buttonHeal.attach(price);

                    if (properties) {
                        coinPrice = properties.hp - Math.max(properties.currentHp, 0);
                        price.setValue('$' + coinPrice);
                        card = new Card(properties);
                        totalHealth += Math.max(properties.currentHp, 0);
                        maxHealth += properties.hp;

                        // attach a clickbutton if critter needs healing
                        if (properties.currentHp < properties.hp) {
                            // if (Bento.saveState.load('healCrittersTutorial', false) === false) {
                            //     buttonHeal.attach(getHandDiagonal(-4));
                            // }
                            buttonHeal.setActive(true);
                            buttonHeal.visible = true;
                            canHeal = true;
                            card.attach(buttonHeal);
                        } else {
                            buttonHeal.setActive(false);
                            buttonHeal.visible = false;
                            price.visible = false;
                        }

                        // card 3d touch:
                        card.on3DTouch(function (details, pauseLevel) {
                            // attach buttonCombine and buttonTeamEdit
                            var buttonCombine = new ClickButton({
                                name: 'buttonCombine',
                                sfx: 'sfx_button',
                                image: Bento.assets.getImage('map/buttoncombine'),
                                frameCountX: 2,
                                frameCountY: 2,
                                animations: {
                                    up: {
                                        frames: [0]
                                    },
                                    down: {
                                        frames: [1]
                                    },
                                    inactive: {
                                        frames: [2]
                                    }
                                },
                                position: new Vector2(-32, 72),
                                updateWhenPaused: Utils.PauseLevels.DETAILS,
                                onClick: function () {
                                    var combiner;

                                    // close details asap
                                    details.close();

                                    // go to team select screen
                                    combiner = new Combiner({
                                        onClose: function () {
                                            setVisible(true);

                                            // reload team
                                            createTeam();
                                        }
                                    });
                                    Bento.objects.attach(combiner);
                                    // make everything else invisible
                                    setVisible(false);
                                }
                            });
                            var buttonEditTeam = new ClickButton({
                                name: 'buttonEditTeam',
                                sfx: 'sfx_button',
                                image: Bento.assets.getImage('map/buttonteamedit'),
                                frameCountX: 2,
                                frameCountY: 2,
                                animations: {
                                    up: {
                                        frames: [0]
                                    },
                                    down: {
                                        frames: [1]
                                    },
                                    inactive: {
                                        frames: [2]
                                    }
                                },
                                position: new Vector2(32, 72),
                                updateWhenPaused: Utils.PauseLevels.DETAILS,
                                onClick: function () {
                                    var teamEdit;

                                    // close details asap
                                    details.close();

                                    // go to team select screen
                                    teamEdit = new TeamEdit({
                                        onClose: function () {
                                            setVisible(true);

                                            // reload team
                                            createTeam();
                                        }
                                    });
                                    Bento.objects.attach(teamEdit);
                                    // make everything else invisible
                                    setVisible(false);
                                }
                            });

                            // quick fix: it's possible to 3d touch on this card
                            // while the screen is in the combiner/team edit
                            // passed pauseLevel before the critterDetails appeared
                            if (pauseLevel !== 0) {
                                details.close();
                                return;
                            }


                            if (Bento.saveState.load('sawTutorial2', false) === true) {
                                details.attachToWindow(buttonCombine);
                            }
                            if (Bento.saveState.load('sawTutorial', false) === true) {
                                details.attachToWindow(buttonEditTeam);
                            }
                        });
                    } else {
                        card = new Entity({
                            name: 'empty',
                            originRelative: new Vector2(0.5, 0.5),
                            components: [new Sprite({
                                imageName: 'gui/emptyslot'
                            })]
                        });
                        card.dimension.width = 45;
                        buttonHeal.setActive(false);
                        buttonHeal.visible = false;
                        price.visible = false;
                    }
                    teamList.addItem(card);

                });
                // add self to teamPanel
                teamPanel.attach(teamList);

                // move up
                if (canHeal) {
                    new Tween({
                        from: teamPanel.position.y,
                        to: viewport.height - 64 - 24,
                        'in': 30,
                        ease: 'squareroot',
                        'do': function (v, t) {
                            teamPanel.position.y = v;
                        }
                    });
                }
            };
            var team = Utils.getTeam();

            makeTeamCritters();

            // perform a save here(?)
            // Utils.saveTeam();
            // Utils.saveCritters();

            return teamList;
        };
        var setVisible = function (value) {
            Bento.objects.get('mode7', function (entity) {
                entity.visible = value;
            });
            Bento.objects.get('player', function (entity) {
                entity.visible = value;
            });
            Bento.objects.getByFamily('triggers', function (array) {
                Utils.forEach(array, function (item, i, l, breakLoop) {
                    item.visible = value;
                });
            });
        };
        /*
         * Tutorial entering the worldmap for the first time
         */
        var tutorial = function () {
            var dialog;
            var hand;
            if (Bento.saveState.load('tutorial-worldmap', false) === false) {
                dialog = new StoryDialog({
                    text: Utils.Localization.getText('swipeToMove'),
                    attachments: {
                        '0': new Entity({
                            name: 'swipe',
                            position: new Vector2(0, 0),
                            originRelative: new Vector2(0.5, 0.5),
                            components: [
                                new Sprite({
                                    imageName: 'gui/swipetutorial'
                                })
                            ]
                        })
                    },
                    onClose: function () {
                        Bento.saveState.save('tutorial-worldmap', true);
                    }
                });
            }
        };
        var createMenu = function () {
            if (buttonMenu) {
                Bento.objects.remove(buttonMenu);
            }
            buttonMenu = new Menu({
                makeTeamCritters: createTeam,
                onClick: function () {
                    player.stopSound();
                    stopSound();
                    // reset menu
                    // createMenu();
                },
                onClose: function () {
                    myCoins = Bento.saveState.load('myCoins', 0);
                    coinCounter.setValue(myCoins);
                    Utils.Analytics.trackView('overworld');
                },
                newCritter: (data && data.caught) ? data.caught.newCritter : false
            });
            // buttonMenu.position.y = 32;
            Bento.objects.attach(buttonMenu);
        };
        var openForm = function () {
            var openedForm = Bento.saveState.load('openedForm', false);

            if (!openedForm && !Utils.DGG_DEMO) {
                Bento.saveState.save('openedForm', true);
                if (Utils.isCocoonJS() && Cocoon) {
                    Cocoon.Dialog.confirm({
                        message: 'Thanks for playing! Please take a moment to fill in this form. Your feedback is appreciated!'
                    }, function () {
                        Cocoon.App.openURL('http://goo.gl/forms/2T2UNNrtw4');
                    });
                } else {
                    window.alert('Thanks for playing! Please take a moment to fill in this form. Your feedback is appreciated!');
                    window.open('http://goo.gl/forms/2T2UNNrtw4');
                }
            }
        };
        /*
         * Daily gift
         */
        var checkDailyGift = function () {
            // small delay to wait for the fade
            Utils.timeout(30, function () {
                // should give daily gift?
                if (DailyGift.canGive()) {
                    new DailyGift({});
                }
            }, Utils.PauseLevels.ALWAYS);
        };

        createMenu();
        // set screen size to size of all sections combined
        screenObject.dimension = areaLoader.getFullLevelSize();
        // screenObject.dimension = new Rectangle(0, 0, Infinity, Infinity);

        // set to start position?
        if (Utils.worldMap.position) {
            player.position = Utils.worldMap.position.clone();
        } else {
            player.position.x = 50 * 16;
            player.position.y = 50 * 16;
            camera.forceTo(player.position);
            viewport.x = player.position.x - viewport.width / 2;
            viewport.y = player.position.y - viewport.height / 2;
            areaLoadTrigger.position.x = viewport.x + viewport.width / 2;
            areaLoadTrigger.position.y = viewport.y + 32;

        }
        // debug
        // mode7.getComponent('opacity', function (component) {
        //     component.setOpacity(0.5);
        // });

        teamPanel.extend({
            makeTeamCritters: createTeam
        });

        sky.scale.x = 0.5;
        sky.scale.y = 0.5;

        upperFill.scale.x = 0.5;
        upperFill.scale.y = 0.5;

        Bento.objects.attach(tileColliders);
        Bento.objects.attach(player);
        Bento.objects.attach(areaLoadTrigger);
        Bento.objects.attach(camera);
        Bento.objects.attach(sky);
        Bento.objects.attach(upperFill);
        Bento.objects.attach(lowerShade);
        Bento.objects.attach(areaLoader);
        Bento.objects.attach(mode7);
        Bento.objects.attach(fade);
        // gui
        Bento.objects.attach(coinCounter);
        Bento.objects.attach(teamPanel);
        Bento.objects.attach(healthChecker);
        Bento.objects.attach(adChecker);
        Bento.objects.attach(worldmapBottom);
        createTeam();

        // if (!Bento.audio.isPlayingMusic('bgm_map')) {
        //     Bento.audio.stopAllMusic();
        //     Bento.audio.playMusic('bgm_map');
        // }

        Bento.audio.stopAllMusic();

        // test
        // Zoom.init();
        // Zoom.zoom(0.5);

        Utils.EveryPlay.stop();

        Utils.Analytics.trackView('overworld');

        // save the last planet visited so that we can load it in at the next bootup
        Bento.saveState.save('lastPlanetVisited', planetName);

    };
    var screenObject = new Screen({
        onShow: onShow
    });

    return screenObject;
});