/**
 * Module description
 */
bento.define('singletons/3dtouch', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    ClickButton,
    Counter,
    Text,
    EventSystem,
    Utils,
    Tween
) {
    'use strict';
    var module = {
        on3DTouch: function (result) {
            //???
        }
    };
    // callback for watchForceTouches
    var on3DTouch = function (result) {
        var viewport = Bento.getViewport();

        // result.x and y are relative to window.innerWidth and innerHeight it seems
        if (!result.position || !result.position.isVector2) {
            result.position = new Vector2(
                result.x / window.innerWidth * viewport.width,
                result.y / window.innerHeight * viewport.height
            );
        }
        result.localPosition = result.position.clone();
        result.worldPosition = result.position.clone();
        result.worldPosition.x += viewport.x;
        result.worldPosition.y += viewport.y;

        if (module.on3DTouch) {
            module.on3DTouch(result);
        }

        EventSystem.fire('3Dtouch', result);

        // should send bento events???
        // console.log("force touch % " + result.force); // 84
        // console.log("force touch timestamp " + result.timestamp); // 1449908744.706419
        // console.log("force touch x coordinate " + result.x); // 213
        // console.log("force touch y coordinate " + result.y); // 41
    };

    if (window.ThreeDeeTouch) {
        window.ThreeDeeTouch.isAvailable(function (avail) {
            // 'avail' is a boolean
            console.log("3D touch available? " + avail);
        });

        /* You can get a notification when the user force touches the webview. 
        The plugin defines a Force Touch when at least 75% of the maximum force 
        is applied to the screen. Your app will receive the x and y coordinates, 
        so you have to figure out which UI element was touched.*/
        window.ThreeDeeTouch.watchForceTouches(on3DTouch);
    }

    // for testing purposes!
    // EventSystem.on('pointerDown', function (evt) {
    //     // right click (on the mobile emulator, right click does not exist)
    //     if (evt.which === 3) {
    //         on3DTouch(evt);
    //     }
    // });

    return module;
});