bento.define('singletons/googleplay', [
    'bento'
], function (
    Bento
) {
    'use strict';
    var gp;
    var clientId = '691294238331-tqpnplsp0kjnltgph0l5pfn3n8sc4uaf.apps.googleusercontent.com';
    var initialized = false;
    var nothing = function () {};
    var init = function (callback) {
        // set default player stats if none are available (eg. on first bootup or iOS)
        if (!Bento.saveState.load('playerStats', null)) {
            Bento.saveState.save('playerStats', {
                averageSessionLength: -1, // -1 is the default value when there's not enough data
                churnProbability: -1,
                daysSinceLastPlayed: -1,
                highSpenderProbability: -1,
                numberOfPurchases: -1,
                numberOfSessions: -1,
                sessionPercentile: -1,
                spendPercentile: -1,
                spendProbability: -1,
                totalSpendNext28Days: -1
            });
        }

        gp = Cocoon.Social.GooglePlayGames;
        gp.init({
            clientId: clientId,
            // defaultLeaderboard: "CgkI_P7xs4EREAIQBg",
            showAchievementNotifications: true,
            /**
             * WARNING: Be sure to also enable the scopes on console.developers.google.com,
             * otherwise you will get an 'INVALID_SCOPE' login error.
             */
            scopes: [
                // 'https://www.googleapis.com/auth/drive.appdata',
                // 'appfolder',
                'drive.appfolder',
                // 'drive.appdata'
            ]
        }, function () {
            if (callback) {
                callback();
            }
        });
        initialized = true;
    };
    var isLoggedIn = function () {
        if (!initialized) {
            return false;
        }
        var socialService = gp.getSocialInterface();
        console.log('Google Play: is logged in', socialService.isLoggedIn());
        return socialService.isLoggedIn();
    };
    var login = function (callback) {
        var socialService;
        callback = callback || function () {};
        socialService = gp.getSocialInterface();
        socialService.login(callback);
    };
    if (!window.Cocoon || !window.Cocoon.Social || !window.Cocoon.Social.GooglePlayGames) {
        return {
            init: nothing,
            isLoggedIn: nothing,
            login: nothing,
            post: nothing,
            show: nothing,
            postAchievement: nothing,
            showAchievement: nothing,
            submitEvent: nothing,
            loadSavedGame: nothing,
            writeSavedGame: nothing,
            showSavedGame: nothing,
            loadPlayerStats: nothing
        };
    }

    return {
        init: function (callback) {
            if (!initialized) {
                init(callback);
            }
        },
        isLoggedIn: isLoggedIn,
        login: function (callback) {
            var socialService,
                self = this;

            if (!initialized) {
                init(function () {
                    self.login(callback);
                });
                return;
            }

            socialService = gp.getSocialInterface();
            if (socialService.isLoggedIn()) {
                // dont log in twice
                if (callback) {
                    callback(false);
                }
                return;
            }
            socialService.login(function (loggedIn, error) {
                if (error) {
                    if (callback) {
                        callback(error);
                    }
                    console.error("login error: " + error.message + " " + error.code);
                } else {
                    if (callback) {
                        callback(loggedIn);
                    }
                }
            });
        },
        post: function (v, leaderboardId, callback) {
            if (!initialized) {
                return;
            }
            var socialService = gp.getSocialInterface();
            if (isLoggedIn()) {
                socialService.submitScore(v, callback, {
                    leaderboardID: leaderboardId
                });
            }
        },
        show: function (callback, param) {
            if (!initialized) {
                return;
            }
            var socialService = gp.getSocialInterface();

            if (!isLoggedIn()) {
                login(function () {
                    console.log('logged in');
                    socialService.showLeaderboard(function (err) {
                        console.log('show leaderboard', JSON.stringify(err));
                    }, param);
                });
            } else {
                socialService.showLeaderboard(function (err) {
                    console.log('show leaderboard', JSON.stringify(err));
                }, param);
            }

        },
        postAchievement: function (id, callback) {
            if (!initialized) {
                return;
            }
            var socialService = gp.getSocialInterface();
            if (isLoggedIn()) {
                socialService.submitAchievement(id, function (response) {
                    console.log('achievement unlocked', response);
                    if (response && response.newlyUnlocked && callback) {
                        // show message
                        callback();
                    }
                });
            }
        },
        showAchievement: function (callback) {
            if (!initialized) {
                return;
            }
            var socialService = gp.getSocialInterface();
            if (!isLoggedIn()) {
                login(function () {
                    socialService.showAchievements(function (err) {
                        console.log('show leaderboard', JSON.stringify(err));
                    });
                });
            } else {
                socialService.showAchievements(function (err) {
                    console.log('show leaderboard', JSON.stringify(err));
                });

            }
        },
        submitEvent: function (name, amount) {
            if (!initialized) {
                return;
            }
            if (Cocoon.Social.GooglePlayGames.submitEvent) {
                Cocoon.Social.GooglePlayGames.submitEvent(name, amount);
            } else {
                console.log('submitEvent not available (cocoon devapp?)');
            }
        },
        loadSavedGame: function (name, onSuccess, onFail) {
            if (!initialized) {
                return;
            }
            Cocoon.Social.GooglePlayGames.loadSavedGame(name, function (snapshot, error) {
                if (error && onFail) {
                    onFail(error); // error should be a json object
                    return;
                }
                if (onSuccess) {
                    // console.log('load snapshot', JSON.stringify(snapshot));
                    if (snapshot.data) {
                        onSuccess(snapshot);
                    } else {
                        onFail(snapshot);
                    }
                }
            });
        },
        writeSavedGame: function (snapshot, onSuccess, onFail) {
            if (!initialized) {
                return;
            }
            /*snapshot {
                identifier: string,
                title: string,
                description: string,
                data: string //this is the JSON you want to load/store
            }*/
            Cocoon.Social.GooglePlayGames.writeSavedGame(snapshot, function (error) {
                if (error && onFail) {
                    onFail(error); // error should be a json object
                    return;
                }
                if (onSuccess) {
                    onSuccess();
                }
            });
        },
        showSavedGame: function (snapshot) {
            if (!initialized) {
                return;
            }
            Cocoon.Social.GooglePlayGames.showSavedGames(snapshot, function (error) {

            });
        },
        loadPlayerStats: function (onComplete) {
            if (!initialized) {
                return;
            }
            Cocoon.Social.GooglePlayGames.loadPlayerStats(function (stats, error) {
                if (stats) {
                    Bento.saveState.save('playerStats', stats);

                    if (onComplete) {
                        onComplete(stats);
                    }
                }
                if (error) {
                    console.error(error);

                    if (onComplete) {
                        onComplete(null);
                    }
                }
            });
        }
    };
});