/**
 * An iCloud compatible storage object for the Bento saveState manager
 */
bento.define('singletons/icloud', [
    'bento',
    'bento/utils',
    'singletons/file'
], function (
    Bento,
    Utils,
    File
) {
    'use strict';
    var isSaving = false;
    var retry = false;
    var nothing = function () {};
    var saveToCloud = function (data) {
        // is already saving
        if (isSaving) {
            retry = true; // try again later
            return;
        }
        isSaving = true;
        File.write('saveState.sav', data, function () {
            console.log('success');
            isSaving = false;
            // should retry? 
            if (retry) {
                retry = false;
                saveToCloud();
            }
        }, function () {
            console.log('failed');
            isSaving = false;
        });
    };
    var defaultErrorCallback = function (error) {
        console.log('File error:');
        console.log(error);
    };
    var module = {
        writeSavedGame: function (snapshot, onSuccess, onFail) {
            File.write('save.sav', snapshot, function () {
                onSuccess();
            }, function () {
                onFail("File load failed");
            });
        },
        loadSavedGame: function (name, onSuccess, onFail) {
            // name is ignored, used in googleplay saved games

            File.setErrorCallback(onFail);

            File.read('save.sav', function (result) {
                onSuccess(result);
            });
        }
    };

    if (!window.cordova || !window.cordova.file) {
        console.log('file plugin not loaded.');
        module.writeSavedGame = nothing;
        module.loadSavedGame = nothing;
    } else {
        // File.setDataDirectory(window.cordova.file.documentsDirectory);
        File.setDataDirectory(window.cordova.file.syncedDataDirectory); // or should it be this for iCloud??

        File.setErrorCallback(defaultErrorCallback);
    }


    return module;
});