bento.define('utils', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/eventsystem',
    'bento/entity',
    'bento/utils',
    'bento/tween',
    'components/backbutton',
    'components/debug',
    'modules/appavailability',
    'modules/tapticengine',
    'singletons/3dtouch',
    'singletons/everyplay',
    'singletons/icloud',
    'singletons/localization',
    'singletons/fontsettings',
    'singletons/analytics',
    'singletons/gamecenter',
    'singletons/googleplay',
    'singletons/notification',
    'singletons/firebase',
    'singletons/cocoon'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    EventSystem,
    Entity,
    Utils,
    Tween,
    BackButton,
    Debug,
    AppAvailability,
    TapticEngine,
    ThreeDeeTouch,
    EveryPlay,
    ICloud,
    Localization,
    FontSettings,
    Analytics,
    GameCenter,
    GooglePlay,
    Notification,
    Firebase,
    Cocoon
) {
    'use strict';
    var twitterCritters = null;
    var critters = null;
    var team = null;
    var utils = {
        version: window.VERSION, // defined in game.js
        /**
         * Singletons
         */
        ICloud: ICloud,
        ThreeDeeTouch: ThreeDeeTouch,
        EveryPlay: EveryPlay,
        Localization: Localization,
        FontSettings: FontSettings,
        Analytics: Analytics,
        GameCenter: GameCenter,
        GooglePlay: GooglePlay,
        Notification: Notification,
        Firebase: Firebase,
        TapticEngine: TapticEngine,
        Cocoon: Cocoon,
        /**
         * Flags
         */
        WEB: false,
        East2West: false,
        NoAdsVersion: false,
        DGG_DEMO: false,
        BETA_TEST: false, // beta testers have their save file erased on every new version
        /**
         * @snippet Utils.isWideScreen|snippet
                    Utils.isWideScreen
         */
        isWideScreen: false, // larger than normal resolution
        /**
         * @snippet Utils.screenSize|snippet
                    Utils.screenSize
         */
        screenSize: new Rectangle(0, 0, 1, 1),
        Layers: {
            CAMERA: -1000,
            TO_ZOOM: -2000,
            BACKGROUND: 0,
            MODE7: 1000,
            CRITTERS: 1000,
            SKY: 2000,
            EFFECTS: 2000,
            GUI: 3000,
            TEAMEDIT: 4000,
            COMBINERESULT: 4000,
            BUY: 4000,
            STOP_ZOOM: 5000,
            PAUSE: 9000,
            DETAILS: 9100,
            SHOP: 9500,
            MINIGAME: 9600,
            DIALOG: 9800,
            DAILYGIFT: 9800,
            SYSTEM: 9900,
            FADE: 10000
        },
        PauseLevels: {
            NONE: 0,
            // planet select screen
            INFO: 1,
            // worldmap menu
            MENU: 3,
            // catch result
            CAUGHT: 4,
            // combiner and team screen
            TEAMEDIT: 5,
            COMBINERESULT: 6,
            TWITTER: 7,
            // other
            SHOWBOSS: 94,
            PAUSE: 95,
            SHOP: 96,
            MINIGAME: 97, // battle minigame goes over the shop and pause button
            STORY: 98,
            SHOPRESPONSE: 99,
            ALWAYS: 100,
            DETAILS: 102,
            DAILYGIFT: 103, // should go over everything?
            VIDEOAD: 104,
            EXIT: 1000 // "Are you sure you want to exit the app" dialog
        },
        IAP: [
            'coinpack1',
            'coinpack2',
            'coinpack3',
            'coinpack4new',
            'coinpack5',
            'coinpack6',
            'removeads'
        ],
        rewardsIAP: {
            coinpack1: 1000,
            coinpack2: 3000,
            coinpack3: 6000,
            coinpack4new: 10000, // this id has 'new' added because the one without it is already taken
            coinpack5: 21000,
            coinpack6: 50000
        },
        Debug: Debug,
        BackButton: BackButton,
        /**
         * Save state
         */
        reset: function () {
            localStorage.clear();
            team = undefined;
            critters = undefined;
        },
        getCritters: function () {
            if (!critters) {
                critters = Bento.saveState.load('critters', []);
            }
            return critters;
        },
        getTeam: function () {
            if (!team) {
                team = Bento.saveState.load('team', [null, null, null]);
            }
            return team;
        },
        removeCritter: function (critter) {
            var index = -1;
            // find critter
            index = critters.indexOf(critter);
            if (index >= 0) {
                critters.splice(index, 1);
                return true;
            }
            return false;
        },
        saveCritters: function () {
            if (critters) {
                Bento.saveState.save('critters', critters);
            }
        },
        saveTeam: function () {
            if (team) {
                Bento.saveState.save('team', team);
            }
        },
        reloadTeam: function () {
            team = Bento.saveState.load('team', [null, null, null]);
        },
        reloadCritters: function () {
            critters = Bento.saveState.load('critters', []);
        },
        reloadTwitterCritters: function () {
            twitterCritters = Bento.saveState.load('twitterCritters', []);
        },
        getTwitterCritters: function () {
            if (!twitterCritters) {
                twitterCritters = Bento.saveState.load('twitterCritters', []);
            }
            return twitterCritters;
        },
        saveTwitterCritters: function () {
            if (twitterCritters) {
                Bento.saveState.save('twitterCritters', twitterCritters);
            }
        },
        addToTwitterCritters: function (properties) {
            var hasDefeatedBefore = false;
            if (twitterCritters) {
                // check if already defeated before
                Utils.forEach(twitterCritters, function (critter, i, l, breakLoop) {
                    if (critter.name === properties.name) {
                        hasDefeatedBefore = true;
                        breakLoop();
                        return;
                    }
                });
                if (hasDefeatedBefore) {
                    return;
                }

                twitterCritters.push(properties);
                Bento.saveState.save('twitterCritters', twitterCritters);
            }
        },
        addToCollection: function (id) {
            var collection = Bento.saveState.load('collection', []);
            var vegCritters = ['bananaman', 'broccotree', 'captaincarrot', 'crawlson', 'crybaman', 'cryman', 'punchycarrot', 'purplehaze', 'stabbycarrot'];
            var vegCaught = 0;
            var i;

            if (!utils.hasInCollection(id)) {
                collection.push(id);
                Bento.saveState.save('collection', collection);

                // track how much unique critters a user has
                Utils.Analytics.event('Critters', 'amntOfUnqCrttrs_' + collection.length);
                Utils.Firebase.logEvent('unique_critters', {
                    unique_critter_amount: collection.length
                });

                // Achievement: Collect 10 unique critters
                if (collection.length >= 10)
                    utils.postAchievement('CgkI--zjoo8UEAIQBA', function () {});

                // Achievement: Collect all possible critters
                if (collection.length === Utils.getKeyLength(Bento.assets.getJson('creatures/creatures')['creatures']))
                    utils.postAchievement('CgkI--zjoo8UEAIQDg', function () {});

                // Achievement: Collect all fruit and vegetable based critters
                for (i = 0; i < vegCritters.length; ++i) {
                    if (collection.indexOf(vegCritters[i]) > -1) {
                        ++vegCaught;
                    }
                }
                if (vegCaught === vegCritters.length) {
                    utils.postAchievement('CgkI--zjoo8UEAIQDA', function () {});
                }
            }
        },
        hasInCollection: function (id) {
            var collection = Bento.saveState.load('collection', []);
            var index = collection.indexOf(id);
            return index >= 0;
        },
        defeatedBoss: function (id) {
            var defeatedBosses = Bento.saveState.load('defeatedBosses', []);
            if (!utils.hasDefeatedBoss(id)) {
                defeatedBosses.push(id);
                Bento.saveState.save('defeatedBosses', defeatedBosses);
            }
        },
        hasDefeatedBoss: function (id) {
            var defeatedBosses = Bento.saveState.load('defeatedBosses', []);
            var index = defeatedBosses.indexOf(id);
            return index >= 0;
        },
        screenShake: function (strength, duration) {
            var viewport = Bento.getViewport();
            new Tween({
                from: strength,
                to: 0,
                'in': duration,
                ease: 'linear',
                'do': function (v, t) {
                    viewport.x = Utils.getRandomRange(-v, v);
                    viewport.y = Utils.getRandomRange(-v, v);
                },
                onComplete: function () {
                    viewport.x = 0;
                    viewport.y = 0;
                }
            });
        },
        tokens: {
            collection: {},
            reset: function () {
                this.collection = {};
            },
            add: function (id, value) {
                this.collection[id] = value;
            },
            has: function (id) {
                return this.collection[id];
            }
        },
        /*
         * Amount negative means its a sink
         */
        trackCoins: function (amount, itemName) {
            if (amount > 0) {
                // source
                GooglePlay.submitEvent('CgkI--zjoo8UEAIQAQ', amount);
                Analytics.event('Game', 'source', amount);
                Firebase.logEvent('source', {
                    value: amount
                });
            } else if (amount < 0) {
                // sink
                GooglePlay.submitEvent('CgkI--zjoo8UEAIQAg', -amount);
                Analytics.event('Game', 'sink', -amount);
                Firebase.logEvent('sink', {
                    value: -amount
                });
                // Standard Firebase event for tracking in-game spenditure
                Firebase.logEvent(Firebase.Events.SPEND_VIRTUAL_CURRENCY, {
                    item_name: itemName,
                    value: -amount,
                    virtual_currency_name: 'coins'
                });
            }
        },
        // remember data about the world map
        worldMap: {
            planetData: null,
            position: null, // last position in  worldmap
            collected: {}, // triggers touched
            collectedMecha: {}, // which mechabosses collected
            reset: function () {
                this.planetData = null;
                this.position = null;
                this.collected = {};
                this.collectedMecha = {};
            }
        },
        lastScreenshot: null,
        tutorialEnemy: 'blazingturtle',
        tutorialEnemy2: 'usaseal',
        //
        coinSettings: [{
            amount: 23,
            animationKey: 'animations/coinpickup',
            scale: 1.2
        }, {
            amount: 7,
            animationKey: 'animations/coinpickup',
            scale: 1
        }, {
            amount: 1,
            animationKey: 'animations/coinpickup',
            scale: 0.8
        }],
        /**
         * Useful functions
         */
        isMobile: function () {
            var check = false;
            (function (a) {
                if (/(android|bb\d+|meego|android|ipad|playbook|silk).+mobile|avantgo|bada\/|blackberry|blazer|compal|elaine|fennec|hiptop|iemobile|ip(hone|od)|iris|kindle|lge |maemo|midp|mmp|mobile.+firefox|netfront|opera m(ob|in)i|palm( os)?|phone|p(ixi|re)\/|plucker|pocket|psp|series(4|6)0|symbian|treo|up\.(browser|link)|vodafone|wap|windows (ce|phone)|xda|xiino/i.test(a) || /1207|6310|6590|3gso|4thp|50[1-6]i|770s|802s|a wa|abac|ac(er|oo|s\-)|ai(ko|rn)|al(av|ca|co)|amoi|an(ex|ny|yw)|aptu|ar(ch|go)|as(te|us)|attw|au(di|\-m|r |s )|avan|be(ck|ll|nq)|bi(lb|rd)|bl(ac|az)|br(e|v)w|bumb|bw\-(n|u)|c55\/|capi|ccwa|cdm\-|cell|chtm|cldc|cmd\-|co(mp|nd)|craw|da(it|ll|ng)|dbte|dc\-s|devi|dica|dmob|do(c|p)o|ds(12|\-d)|el(49|ai)|em(l2|ul)|er(ic|k0)|esl8|ez([4-7]0|os|wa|ze)|fetc|fly(\-|_)|g1 u|g560|gene|gf\-5|g\-mo|go(\.w|od)|gr(ad|un)|haie|hcit|hd\-(m|p|t)|hei\-|hi(pt|ta)|hp( i|ip)|hs\-c|ht(c(\-| |_|a|g|p|s|t)|tp)|hu(aw|tc)|i\-(20|go|ma)|i230|iac( |\-|\/)|ibro|idea|ig01|ikom|im1k|inno|ipaq|iris|ja(t|v)a|jbro|jemu|jigs|kddi|keji|kgt( |\/)|klon|kpt |kwc\-|kyo(c|k)|le(no|xi)|lg( g|\/(k|l|u)|50|54|\-[a-w])|libw|lynx|m1\-w|m3ga|m50\/|ma(te|ui|xo)|mc(01|21|ca)|m\-cr|me(rc|ri)|mi(o8|oa|ts)|mmef|mo(01|02|bi|de|do|t(\-| |o|v)|zz)|mt(50|p1|v )|mwbp|mywa|n10[0-2]|n20[2-3]|n30(0|2)|n50(0|2|5)|n7(0(0|1)|10)|ne((c|m)\-|on|tf|wf|wg|wt)|nok(6|i)|nzph|o2im|op(ti|wv)|oran|owg1|p800|pan(a|d|t)|pdxg|pg(13|\-([1-8]|c))|phil|pire|pl(ay|uc)|pn\-2|po(ck|rt|se)|prox|psio|pt\-g|qa\-a|qc(07|12|21|32|60|\-[2-7]|i\-)|qtek|r380|r600|raks|rim9|ro(ve|zo)|s55\/|sa(ge|ma|mm|ms|ny|va)|sc(01|h\-|oo|p\-)|sdk\/|se(c(\-|0|1)|47|mc|nd|ri)|sgh\-|shar|sie(\-|m)|sk\-0|sl(45|id)|sm(al|ar|b3|it|t5)|so(ft|ny)|sp(01|h\-|v\-|v )|sy(01|mb)|t2(18|50)|t6(00|10|18)|ta(gt|lk)|tcl\-|tdg\-|tel(i|m)|tim\-|t\-mo|to(pl|sh)|ts(70|m\-|m3|m5)|tx\-9|up(\.b|g1|si)|utst|v400|v750|veri|vi(rg|te)|vk(40|5[0-3]|\-v)|vm40|voda|vulc|vx(52|53|60|61|70|80|81|83|85|98)|w3c(\-| )|webc|whit|wi(g |nc|nw)|wmlb|wonu|x700|yas\-|your|zeto|zte\-/i.test(a.substr(0, 4))) {
                    check = true;
                }
            })(navigator.userAgent || navigator.vendor || window.opera);
            return check;
        },
        isNativeIos: function () {
            if (navigator.isCocoonJS && window.Cocoon && window.Cocoon.getPlatform() === 'ios') {
                return true;
            } else if (window.device) {
                if (window.device && window.device.platform) {
                    return window.device.platform.toLowerCase() === 'ios';
                }
            }
            return false;
        },
        isNativeAndroid: function () {
            var platform;
            if (navigator.isCocoonJS && window.Cocoon) {
                platform = window.Cocoon.getPlatform();
                if (platform === 'android' || platform === 'amazon') {
                    return true;
                }
            } else if (window.device) {
                if (window.device && window.device.platform) {
                    return window.device.platform.toLowerCase() === 'android';
                }
            }
            return false;
        },
        timeout: function (time, callback, updateWhenPaused) {
            var tween = new Tween({
                from: 0,
                to: 1,
                'in': time,
                ease: 'linear',
                updateWhenPaused: updateWhenPaused,
                onComplete: callback
            });
            return tween;
        },
        isInViewport: function (entity, edgeSize) {
            var viewport = Bento.getViewport().clone();
            return viewport.grow(edgeSize || 0).intersect(entity.getBoundingBox());
        },
        clamp: function (min, value, max) {
            return Math.max(min, Math.min(max, value));
        },
        /**
         * Copies a json object
         */
        clone: function (jsonObj) {
            var out;
            try {
                out = JSON.parse(JSON.stringify(jsonObj));
            } catch (e) {
                out = {};
                console.log('WARNING: object cloning failed');
            }
            return out;
        },
        forEach: function (array, callback) {
            var obj;
            var i;
            var l;
            var stop = false;
            var breakLoop = function () {
                stop = true;
            };
            if (Utils.isArray(array)) {
                for (i = 0, l = array.length; i < l; ++i) {
                    callback(array[i], i, l, breakLoop, array[i + 1]);
                    if (stop) {
                        return;
                    }
                }
            } else {
                l = Utils.getKeyLength(array);
                for (i in array) {
                    if (!array.hasOwnProperty(i)) {
                        continue;
                    }
                    callback(array[i], i, l, breakLoop);
                    if (stop) {
                        return;
                    }
                }
            }
        },
        isBetween: function (min, value, max, includeEdge) {
            if (includeEdge) {
                return min <= value && value <= max;
            }
            return min < value && value < max;
        },
        pickRandom: function () {
            return arguments[Utils.getRandom(arguments.length)];
        },
        getStoreLink: function () {
            if (this.isNativeIos()) {
                return 'https://itunes.apple.com/us/app/combo-critters/id1075920778?ls=1&mt=8';
            } else {
                return 'https://play.google.com/store/apps/details?id=com.luckykat.combocritters';
            }
        },
        /**
         * General achievement setting, also saves achievement in Bento.saveState
         * Make sure achievement id is the same in GameCenter as in Google Play!
         * The callback is only called when achieving the first time
         */
        submitted: {},
        postAchievement: function (id, callback) {
            var Utils = utils;

            // this.submitted is a simple check to prevent submitting the same achievements in a row
            if (!this.submitted[id])
                this.submitted[id] = true;
            else
                return;

            if (Utils.isNativeAndroid() && Utils.GooglePlay.isLoggedIn() && !Bento.saveState.load('GP-' + id, false)) {
                Utils.GooglePlay.postAchievement(
                    id,
                    function () {
                        Bento.saveState.save('GP-' + id, true);
                        if (callback)
                            callback();
                    }
                );
            } else if (Utils.isNativeIos() && Utils.GameCenter.isLoggedIn() && !Bento.saveState.load('GC-' + id, false)) {
                id = id.replace('-', ''); // GC disallows characters such as -, while GP can generate these -> delete these characters!
                Utils.GameCenter.postAchievement(
                    id,
                    function () {
                        Bento.saveState.save('GC-' + id, true);
                        if (callback)
                            callback();
                    }
                );
            }
        },
        /**
         * Get all possible notifications.
         * Has to be provided using this function because Localization is
         * not initialized right at the start, leaving the properties
         * that use it blank
         * @return {Object} Object containing all possible notifications
         */
        getNotifications: function () {
            return {
                dailyGift: {
                    title: Localization.getText('title'), // ios title text
                    message: Utils.isNativeIos() ? Localization.getText('dailyGift') : // ios body text
                        Localization.getText('title'), // but android title text
                    soundEnabled: true,
                    badgeNumber: 1,
                    userData: {
                        dailyGift: true
                    },
                    contentBody: Utils.isNativeIos() ? "" : // android body text
                        Localization.getText('dailyGift'),
                    contentTitle: "",
                    // notify again in 24 hours
                    date: new Date().valueOf() + 24 * 60 * 60 * 1000,
                    icon: 'mynoticon'
                }
            };
        },
        checkIfHighSpender: function () {
            Utils.GooglePlay.loadPlayerStats(
                function (playerStats) {
                    if (playerStats) {
                        if (!Bento.saveState.load('highSpender', false) && playerStats.spendProbability && playerStats.spendProbability > 0.8) {
                            Bento.saveState.save('highSpender', true);
                            Utils.Firebase.logEvent('high_spender', {
                                spend_probability: playerStats.spendProbability
                            });
                        }
                        // send player stats the first time we get them to get a feel for the user base
                        if (!Bento.saveState.load('sentPlayerStats', false)) {
                            Utils.Firebase.logEvent('player_stats', {
                                averageSessionLength: playerStats.averageSessionLength,
                                churnProbability: playerStats.churnProbability,
                                daysSinceLastPlayed: playerStats.daysSinceLastPlayed,
                                highSpenderProbability: playerStats.highSpenderProbability,
                                numberOfPurchases: playerStats.numberOfPurchases,
                                numberOfSessions: playerStats.numberOfSessions,
                                sessionPercentile: playerStats.sessionPercentile,
                                spendPercentile: playerStats.spendPercentile,
                                spendProbability: playerStats.spendProbability,
                                totalSpendNext28Days: playerStats.totalSpendNext28Days
                            });
                            Bento.saveState.save('sentPlayerStats', true);
                        }
                    }
                }
            );
        },
        /**
         * Checks if url can be opened, always true for android
         * Uses https://github.com/ohh2ahh/AppAvailability
         */
        canOpenUrl: function (name) {
            if (this.isNativeIos()) {
                return AppAvailability[name] || false;
            } else {
                return true;
            }
        },
        openUrl: function (url) {
            if (window.Cocoon) {
                window.Cocoon.App.openURL(url);
            } else if (window.open) {
                // web?
                window.open(url);
            }
        },
        log: function (msg) {
            console.error(msg);
        },
        // fb softlaunch
        softlaunch: false,
        /**
         * Returns truthy if global FBInstant exists
         */
        isFB: function () {
            // return true;
            return window.FBInstant;
        },
        /**
         * Returns either the provided value, or the provided fallback value in case the provided value was undefined
         * @function
         * @instance
         * @name getDefault
         * @snippet Utils.getDefault|snippet
        Utils.getDefault(${1:value}, ${2:default})
         * @param {Anything} value - any type
         * @param {Anything} value - any type
         */
        getDefault: function (param, fallback) {
            return (param !== void(0)) ? param : fallback;
        },
    };

    // triple touch reload
    // if (!Utils.isCocoonJs() && Utils.isMobile()) {
    //     EventSystem.on('pointerDown', function () {
    //         // check if there are 3 pointers down
    //         if (Bento.input.getPointers().length === 3) {
    //             Bento.reload();
    //         }
    //     });
    // }

    Utils.extend(Utils, utils);
    return Utils;
});