bento.define('xhr', [], function () {
    'use strict';
    return function (settings) {
        /*settings = {
            get: String, // url to get
            post: String, // url to post
            params: {}, // additional parameters
            onSuccess: Function (res),
            onFail: Function (res),
            responseType: String // 'json'
        }*/
        var responseType = settings.responseType || 'json';
        var xhr = new XMLHttpRequest(),
            getPostBody = function () {
                // turns the params string into a url encoded form
                var param,
                    params = settings.params,
                    first = true,
                    url = '';
                for (param in params) {
                    if (!params.hasOwnProperty(param)) {
                        continue;
                    }

                    if (!first) {
                        url += '&';
                    }
                    url += param + '=' + params[param];
                    first = false;
                }
                return url;
            },
            addHeaders = function () {
                var key,
                    headers = settings.headers;
                for (key in headers) {
                    if (!headers.hasOwnProperty(key)) {
                        continue;
                    }
                    xhr.setRequestHeader(key, headers[key]);
                }
            };

        if (settings.get) {
            xhr.open('GET', settings.get, true);
        } else if (settings.post) {
            xhr.open('POST', settings.post, true);
            if (settings.json) {
                xhr.setRequestHeader('Content-Type', 'text/plain');
                xhr.setRequestHeader('X-Requested-With', 'XMLHttpRequest');
            } else {
                xhr.setRequestHeader('Content-type', 'application/x-www-form-urlencoded');
            }
        }

        if (settings.headers) {
            addHeaders();
        }

        xhr.onreadystatechange = function () {
            if (xhr.readyState === 4) {
                if (xhr.status >= 200 && xhr.status < 300) {
                    if (!settings.onSuccess) {
                        return;
                    }
                    if (responseType === 'json') {
                        try {
                            settings.onSuccess(JSON.parse(xhr.responseText));
                        } catch (e) {
                            settings.onSuccess({
                                response: xhr.responseText
                            });
                        }
                    } else if (responseType === 'xhr') {
                        settings.onSuccess(xhr);                        
                    } else {
                        settings.onSuccess(xhr.responseText);                        
                    }
                } else {
                    if (!settings.onFail) {
                        return;
                    }
                    try {
                        settings.onFail(JSON.parse(xhr.responseText));
                    } catch (e) {
                        settings.onFail({
                            response: xhr.responseText
                        });
                    }
                }
            }
        };

        //console.log('success', xhr.responseText);
        if (settings.json) {
            xhr.send(JSON.stringify(settings.params));
        } else if (settings.post) {
            xhr.send(getPostBody());
        } else {
            xhr.send();
        }

        return xhr;
    };
});